@@ -27,7 +27,7 @@ import Darcs.Patch.Merge ( Merge(..) )
 import Darcs.Patch.Patchy ( Invert(..), Commute(..) )
 import Darcs.Patch.Prim ( PrimOf, PrimPatch, PrimPatchBase, FromPrim(..), PrimConstruct( anIdentity ) )
 import Darcs.Patch.Prim.V1 ()
-import Darcs.Patch.V2 ( RealPatch ) -- XXX this is more or less a hack
+import Darcs.Patch.V2 ( RepoPatchV2 ) -- XXX this is more or less a hack
 --import Darcs.ColorPrinter ( errorDoc )
 --import Darcs.ColorPrinter ( traceDoc )
 import Darcs.Patch.Witnesses.Show
@@ -119,7 +119,7 @@ class MightBeEmptyHunk p where
 instance MightBeEmptyHunk (FL p)
 
 class MightHaveDuplicate p where
-  -- |"duplicates" in V2 patches (RealPatch) have lots of bugs
+  -- |"duplicates" in V2 patches (RepoPatchV2) have lots of bugs
   -- that break various commute/merge properties.
   hasDuplicate :: p wX wY -> Bool
   hasDuplicate _ = False
@@ -261,6 +261,6 @@ instance (ArbitraryPrim prim, RepoModel (ModelOf prim), model ~ ModelOf prim,
                     else do n <- choose (0, num - 1)
                             return $ Sealed $ WithStartState rm $ TWFP n t
                     where -- just used to get the length. In principle this should be independent of the patch type.
-                          flattenOneRP :: Tree prim wX -> Sealed (FL (RealPatch prim) wX)
+                          flattenOneRP :: Tree prim wX -> Sealed (FL (RepoPatchV2 prim) wX)
                           flattenOneRP = flattenOne
 
