@@ -35,7 +35,7 @@ import Darcs.Patch
      , adddir, addfile, hunk, binary, rmdir, rmfile, tokreplace )
 import Darcs.Patch.Prim ( PrimOf, FromPrim )
 import Darcs.Patch.Prim.V1 ( Prim )
-import qualified Darcs.Patch.V1 as V1 ( Patch )
+import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
 import Darcs.Test.Patch.Properties.Check( checkAPatch )
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( unsafeUnseal )
@@ -43,7 +43,7 @@ import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
 
 #include "impossible.h"
 
-type Patch = V1.Patch Prim
+type Patch = V1.RepoPatchV1 Prim
 
 -- The unit tester function is really just a glorified map for functions that
 -- return lists, in which the lists get concatenated (where map would end up
@@ -132,68 +132,68 @@ primitiveCommutePairs =
 -- ----------------------------------------------------------------------
 
 -- | Here we provide a set of known interesting commutes.
-knownCommutes :: [((FL Patch:<FL Patch) wX wY,(FL Patch:<FL Patch) wX wY)]
+knownCommutes :: [((FL Patch :> FL Patch) wX wY,(FL Patch :> FL Patch) wX wY)]
 knownCommutes = [
-                  (testhunk 1 [] ["A"]:<
-                   testhunk 2 [] ["B"],
-                   testhunk 3 [] ["B"]:<
-                   testhunk 1 [] ["A"]),
-                  (fromPrim (tokreplace "test" "A-Za-z_" "old" "new"):<
-                   testhunk 2
+                  (testhunk 2 [] ["B"]:>
+                   testhunk 1 [] ["A"],
+                   testhunk 1 [] ["A"]:>
+                   testhunk 3 [] ["B"]),
+                  (testhunk 2
                    ["hello world all that is old is good old_"]
-                   ["I don't like old things"],
+                   ["I don't like old things"]:>
+                   fromPrim (tokreplace "test" "A-Za-z_" "old" "new"),
+                   fromPrim (tokreplace "test" "A-Za-z_" "old" "new"):>
                    testhunk 2
                    ["hello world all that is new is good old_"]
-                   ["I don't like new things"]:<
-                   fromPrim (tokreplace "test" "A-Za-z_" "old" "new")),
-                  (testhunk 1 ["A"] ["B"]:<
-                   testhunk 2 ["C"] ["D"],
-                   testhunk 2 ["C"] ["D"]:<
-                   testhunk 1 ["A"] ["B"]),
-                  (fromPrim (rmfile "NwNSO"):<
-                   (quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello"))),
-                   (quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello"))):<
-                   fromPrim (rmfile "NwNSO")),
+                   ["I don't like new things"]),
+                  (testhunk 2 ["C"] ["D"]:>
+                   testhunk 1 ["A"] ["B"],
+                   testhunk 1 ["A"] ["B"]:>
+                   testhunk 2 ["C"] ["D"]),
+                  ((quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello"))):>
+                   fromPrim (rmfile "NwNSO"),
+                   fromPrim (rmfile "NwNSO"):>
+                   (quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello")))),
 
-                  (quickmerge (testhunk 3 ["o"] ["n"]:\/:
-                               testhunk 3 ["o"] ["v"]):<
-                   testhunk 1 [] ["a"],
-                   testhunk 1 [] ["a"]:<
+                  (testhunk 1 [] ["a"]:>
+                   quickmerge (testhunk 3 ["o"] ["n"]:\/:
+                               testhunk 3 ["o"] ["v"]),
                    quickmerge (testhunk 2 ["o"] ["n"]:\/:
-                               testhunk 2 ["o"] ["v"])),
+                               testhunk 2 ["o"] ["v"]):>
+                   testhunk 1 [] ["a"]),
 
-                  (testhunk 1 ["A"] []:<
-                   testhunk 3 ["B"] [],
-                   testhunk 2 ["B"] []:<
-                   testhunk 1 ["A"] []),
+                  (testhunk 3 ["B"] []:>
+                   testhunk 1 ["A"] [],
+                   testhunk 1 ["A"] []:>
+                   testhunk 2 ["B"] []),
 
-                  (testhunk 1 ["A"] ["B"]:<
-                   testhunk 2 ["B"] ["C"],
-                   testhunk 2 ["B"] ["C"]:<
-                   testhunk 1 ["A"] ["B"]),
+                  (testhunk 2 ["B"] ["C"]:>
+                   testhunk 1 ["A"] ["B"],
+                   testhunk 1 ["A"] ["B"]:>
+                   testhunk 2 ["B"] ["C"]),
 
-                  (testhunk 1 ["A"] ["B"]:<
-                   testhunk 3 ["B"] ["C"],
-                   testhunk 3 ["B"] ["C"]:<
-                   testhunk 1 ["A"] ["B"]),
+                  (testhunk 3 ["B"] ["C"]:>
+                   testhunk 1 ["A"] ["B"],
+                   testhunk 1 ["A"] ["B"]:>
+                   testhunk 3 ["B"] ["C"]),
 
-                  (testhunk 1 ["A"] ["B","C"]:<
-                   testhunk 2 ["B"] ["C","D"],
-                   testhunk 3 ["B"] ["C","D"]:<
-                   testhunk 1 ["A"] ["B","C"])]
+                  (testhunk 2 ["B"] ["C","D"]:>
+                   testhunk 1 ["A"] ["B","C"],
+                   testhunk 1 ["A"] ["B","C"]:>
+                   testhunk 3 ["B"] ["C","D"])]
   where testhunk l o n = fromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
 
-knownCantCommutes :: [(FL Patch:< FL Patch) wX wY]
+knownCantCommutes :: [(FL Patch :> FL Patch) wX wY]
 knownCantCommutes = [
-                      (testhunk 2 ["o"] ["n"]:<
+                      (testhunk 1 [] ["A"]:>
+                       testhunk 2 ["o"] ["n"]),
+                      (testhunk 1 ["o"] ["n"]:>
                        testhunk 1 [] ["A"]),
-                      (testhunk 1 [] ["A"]:<
-                       testhunk 1 ["o"] ["n"]),
-                      (quickmerge (testhunk 2 ["o"] ["n"]:\/:
-                                   testhunk 2 ["o"] ["v"]):<
-                       testhunk 1 [] ["a"]),
-                      (fromPrim (hunk "test" 1 ([BC.pack "a"]) ([BC.pack "b"])):<
-                       fromPrim (addfile "test"))]
+                      (testhunk 1 [] ["a"]:>
+                       quickmerge (testhunk 2 ["o"] ["n"]:\/:
+                                   testhunk 2 ["o"] ["v"])),
+                      (fromPrim (addfile "test"):>
+                       fromPrim (hunk "test" 1 ([BC.pack "a"]) ([BC.pack "b"])))]
   where testhunk l o n = fromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
 
 -- ----------------------------------------------------------------------
