@@ -22,8 +22,8 @@
 module Darcs.Test.Patch.Examples.Set2Unwitnessed
        ( primPermutables, primPatches
        , commutables, commutablesFL
-       , realCommutables , realMergeables, realTriples
-       , realNonduplicateTriples, realPatches, realPatchLoopExamples
+       , repov2Commutables , repov2Mergeables, repov2Triples
+       , repov2NonduplicateTriples, repov2Patches, repov2PatchLoopExamples
        ) where
 
 import Data.Maybe ( catMaybes )
@@ -33,12 +33,12 @@ import Darcs.Patch ( invert, hunk )
 import Darcs.Patch.Patchy ( Invert(..) )
 import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Prim.V1 ( Prim )
-import Darcs.Patch.V2 ( RealPatch )
-import Darcs.Patch.V2.Real ( prim2real )
+import Darcs.Patch.V2 ( RepoPatchV2 )
+import Darcs.Patch.V2.RepoPatch ( prim2repopatchV2 )
 -- import Darcs.Test.Patch.Test () -- for instance Eq Patch
 -- import Darcs.Test.Patch.Examples.Set2Unwitnessed
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
-import qualified Darcs.Test.Patch.Arbitrary.Real as W ( notDuplicatestriple )
+import qualified Darcs.Test.Patch.Arbitrary.RepoPatchV2 as W ( notDuplicatestriple )
 --import Darcs.Util.Printer ( greenText )
 --import Darcs.Util.Printer.Color ( traceDoc )
 --import Darcs.Util.Printer.Color ( errorDoc )
@@ -48,10 +48,10 @@ import Darcs.Test.Patch.WSub
 import qualified Darcs.Patch.Witnesses.Ordered as W ( (:>), (:\/:) )
 import qualified Data.ByteString as B ( ByteString )
 import Darcs.Test.Patch.V1Model ( V1Model, Content
-                                , makeRepo, makeName, makeFile)
+                                , makeRepo, makeFile)
 import Darcs.Test.Patch.WithState ( WithStartState(..) )
 import Darcs.Patch.Prim.V1.Core ( Prim(FP), FilePatchType(Hunk) )
-import Darcs.Util.Path ( FileName, fp2fn )
+import Darcs.Util.Path ( FileName, fp2fn, makeName )
 import Darcs.Patch.Prim ( PrimPatchBase(..), FromPrim )
 import Darcs.Patch.Merge ( Merge )
 import Darcs.Test.Patch.Arbitrary.Generic
@@ -237,8 +237,8 @@ w_commuteExamples = [
 packStringLetters :: String -> [B.ByteString]
 packStringLetters = map (BC.pack . (:[]))
 
-w_realPatchLoopExamples :: [Sealed (WithStartState V1Model (Tree Prim))]
-w_realPatchLoopExamples =
+w_repov2PatchLoopExamples :: [Sealed (WithStartState V1Model (Tree Prim))]
+w_repov2PatchLoopExamples =
     [Sealed (WithStartState (makeSimpleRepo fx_name [])
      $ canonizeTree
      (ParTree
@@ -304,19 +304,19 @@ w_realPatchLoopExamples =
       fx = fp2fn "./F"
 
 
-mergeExamples :: [Sealed2 (RealPatch Prim :\/: RealPatch Prim)]
+mergeExamples :: [Sealed2 (RepoPatchV2 Prim :\/: RepoPatchV2 Prim)]
 mergeExamples = map (mapSeal2 fromW) w_mergeExamples
 
-realPatchLoopExamples :: [Sealed (WithStartState V1Model (Tree Prim))]
-realPatchLoopExamples = w_realPatchLoopExamples
+repov2PatchLoopExamples :: [Sealed (WithStartState V1Model (Tree Prim))]
+repov2PatchLoopExamples = w_repov2PatchLoopExamples
 
-commuteExamples :: [Sealed2 (RealPatch Prim :> RealPatch Prim)]
+commuteExamples :: [Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim)]
 commuteExamples = map (mapSeal2 fromW) w_commuteExamples
 
-tripleExamples :: [Sealed2 (RealPatch Prim :> RealPatch Prim :> RealPatch Prim)]
+tripleExamples :: [Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim :> RepoPatchV2 Prim)]
 tripleExamples = map (mapSeal2 fromW) w_tripleExamples
 
-notDuplicatestriple :: (RealPatch Prim :> RealPatch Prim :> RealPatch Prim) wX wY -> Bool
+notDuplicatestriple :: (RepoPatchV2 Prim :> RepoPatchV2 Prim :> RepoPatchV2 Prim) wX wY -> Bool
 notDuplicatestriple = W.notDuplicatestriple . toW
 
 quickhunk :: PrimPatch prim => Int -> String -> String -> prim wX wY
@@ -355,45 +355,45 @@ primPatches :: [Prim wX wY]
 primPatches = concatMap mergeable2patches mergeables
     where mergeable2patches (x:\/:y) = [x,y]
 
-realPatches :: [RealPatch Prim wX wY]
-realPatches = concatMap commutable2patches realCommutables
+repov2Patches :: [RepoPatchV2 Prim wX wY]
+repov2Patches = concatMap commutable2patches repov2Commutables
     where commutable2patches (x:>y) = [x,y]
 
-realTriples :: [(RealPatch Prim :> RealPatch Prim :> RealPatch Prim) wX wY]
-realTriples = [ob' :> oa2 :> a2'',
-                oa' :> oa2 :> a2'']
+repov2Triples :: [(RepoPatchV2 Prim :> RepoPatchV2 Prim :> RepoPatchV2 Prim) wX wY]
+repov2Triples = [ob' :> oa2 :> a2'',
+                 oa' :> oa2 :> a2'']
                ++ map unsafeUnseal2 tripleExamples
-               ++ map unsafeUnseal2 (concatMap getTriples realFLs)
-    where oa = prim2real $ quickhunk 1 "o" "aa"
+               ++ map unsafeUnseal2 (concatMap getTriples repov2FLs)
+    where oa = prim2repopatchV2 $ quickhunk 1 "o" "aa"
           oa2 = oa
-          a2 = prim2real $ quickhunk 2 "a34" "2xx"
-          ob = prim2real $ quickhunk 1 "o" "bb"
+          a2 = prim2repopatchV2 $ quickhunk 2 "a34" "2xx"
+          ob = prim2repopatchV2 $ quickhunk 1 "o" "bb"
           ob' :/\: oa' = merge (oa :\/: ob)
           a2' :/\: _ = merge (ob' :\/: a2)
           a2'' :/\: _ = merge (oa2 :\/: a2')
 
-realNonduplicateTriples :: [(RealPatch Prim :> RealPatch Prim :> RealPatch Prim) wX wY]
-realNonduplicateTriples = filter (notDuplicatestriple) realTriples
+repov2NonduplicateTriples :: [(RepoPatchV2 Prim :> RepoPatchV2 Prim :> RepoPatchV2 Prim) wX wY]
+repov2NonduplicateTriples = filter (notDuplicatestriple) repov2Triples
 
-realFLs :: [FL (RealPatch Prim) wX wY]
-realFLs = [oa :>: invert oa :>: oa :>: invert oa :>: ps +>+ oa :>: invert oa :>: NilFL]
-    where oa = prim2real $ quickhunk 1 "o" "a"
+repov2FLs :: [FL (RepoPatchV2 Prim) wX wY]
+repov2FLs = [oa :>: invert oa :>: oa :>: invert oa :>: ps +>+ oa :>: invert oa :>: NilFL]
+    where oa = prim2repopatchV2 $ quickhunk 1 "o" "a"
           ps :/\: _ = merge (oa :>: invert oa :>: NilFL :\/: oa :>: invert oa :>: NilFL)
 
-realCommutables :: [(RealPatch Prim :> RealPatch Prim) wX wY]
-realCommutables = map unsafeUnseal2 commuteExamples++
-                   map mergeable2commutable realMergeables++
-                   [invert oa :> ob'] ++ map unsafeUnseal2 (concatMap getPairs realFLs)
-    where oa = prim2real $ quickhunk 1 "o" "a"
-          ob = prim2real $ quickhunk 1 "o" "b"
+repov2Commutables :: [(RepoPatchV2 Prim :> RepoPatchV2 Prim) wX wY]
+repov2Commutables = map unsafeUnseal2 commuteExamples++
+                     map mergeable2commutable repov2Mergeables++
+                     [invert oa :> ob'] ++ map unsafeUnseal2 (concatMap getPairs repov2FLs)
+    where oa = prim2repopatchV2 $ quickhunk 1 "o" "a"
+          ob = prim2repopatchV2 $ quickhunk 1 "o" "b"
           _ :/\: ob' = mergeFL (ob :\/: oa :>: invert oa :>: NilFL)
 
-realMergeables :: [(RealPatch Prim :\/: RealPatch Prim) wX wY]
-realMergeables = map (\ (x :\/: y) -> prim2real x :\/: prim2real y) mergeables
-                        ++ realIglooMergeables
-                        ++ realQuickcheckMergeables
+repov2Mergeables :: [(RepoPatchV2 Prim :\/: RepoPatchV2 Prim) wX wY]
+repov2Mergeables = map (\ (x :\/: y) -> prim2repopatchV2 x :\/: prim2repopatchV2 y) mergeables
+                        ++ repov2IglooMergeables
+                        ++ repov2QuickcheckMergeables
                         ++ map unsafeUnseal2 mergeExamples
-                        ++ catMaybes (map pair2m (concatMap getPairs realFLs))
+                        ++ catMaybes (map pair2m (concatMap getPairs repov2FLs))
                         ++ [(oa :\/: od),
                             (oa :\/: a2'),
                             (ob' :\/: od''),
@@ -410,16 +410,16 @@ realMergeables = map (\ (x :\/: y) -> prim2real x :\/: prim2real y) mergeables
                             (ob'' :\/: og''),
                             (ob'' :\/: oc''),
                             (oc' :\/: od'')]
-    where oa = prim2real $ quickhunk 1 "o" "aa"
-          a2 = prim2real $ quickhunk 2 "a34" "2xx"
-          og = prim2real $ quickhunk 3 "4" "g"
-          ob = prim2real $ quickhunk 1 "o" "bb"
-          b2 = prim2real $ quickhunk 2 "b" "2"
-          oc = prim2real $ quickhunk 1 "o" "cc"
-          od = prim2real $ quickhunk 7 "x" "d"
-          oe = prim2real $ quickhunk 7 "x" "e"
-          pf = prim2real $ quickhunk 7 "x" "f"
-          od'' = prim2real $ quickhunk 8 "x" "d"
+    where oa = prim2repopatchV2 $ quickhunk 1 "o" "aa"
+          a2 = prim2repopatchV2 $ quickhunk 2 "a34" "2xx"
+          og = prim2repopatchV2 $ quickhunk 3 "4" "g"
+          ob = prim2repopatchV2 $ quickhunk 1 "o" "bb"
+          b2 = prim2repopatchV2 $ quickhunk 2 "b" "2"
+          oc = prim2repopatchV2 $ quickhunk 1 "o" "cc"
+          od = prim2repopatchV2 $ quickhunk 7 "x" "d"
+          oe = prim2repopatchV2 $ quickhunk 7 "x" "e"
+          pf = prim2repopatchV2 $ quickhunk 7 "x" "f"
+          od'' = prim2repopatchV2 $ quickhunk 8 "x" "d"
           ob' :>: b2' :>: NilFL :/\: _ = mergeFL (oa :\/: ob :>: b2 :>: NilFL)
           a2' :/\: _ = merge (ob' :\/: a2)
           ob'' :/\: _ = merge (a2 :\/: ob')
@@ -431,13 +431,13 @@ realMergeables = map (\ (x :\/: y) -> prim2real x :\/: prim2real y) mergeables
           oc''' :/\: _ = merge (ob' :\/: oc')
           oe' :/\: _ = merge (od :\/: oe)
           of' :/\: _ = merge (od :\/: pf)
-          pair2m :: Sealed2 (RealPatch Prim :> RealPatch Prim)
-                 -> Maybe ((RealPatch Prim :\/: RealPatch Prim) wX wY)
+          pair2m :: Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim)
+                 -> Maybe ((RepoPatchV2 Prim :\/: RepoPatchV2 Prim) wX wY)
           pair2m (Sealed2 (xx :> y)) = do y' :> _ <- commute (xx :> y)
                                           return $ unsafeCoerceP (xx :\/: y')
 
-realIglooMergeables :: [(RealPatch Prim :\/: RealPatch Prim) wX wY]
-realIglooMergeables = [(a :\/: b),
+repov2IglooMergeables :: [(RepoPatchV2 Prim :\/: RepoPatchV2 Prim) wX wY]
+repov2IglooMergeables = [(a :\/: b),
                     (b :\/: c),
                     (a :\/: c),
                     (x :\/: a),
@@ -447,18 +447,18 @@ realIglooMergeables = [(a :\/: b),
                     (z' :\/: y'),
                     (x' :\/: z'),
                     (a :\/: a)]
-    where a = prim2real $ quickhunk 1 "1" "A"
-          b = prim2real $ quickhunk 2 "2" "B"
-          c = prim2real $ quickhunk 3 "3" "C"
-          x = prim2real $ quickhunk 1 "1BC" "xbc"
-          y = prim2real $ quickhunk 1 "A2C" "ayc"
-          z = prim2real $ quickhunk 1 "AB3" "abz"
+    where a = prim2repopatchV2 $ quickhunk 1 "1" "A"
+          b = prim2repopatchV2 $ quickhunk 2 "2" "B"
+          c = prim2repopatchV2 $ quickhunk 3 "3" "C"
+          x = prim2repopatchV2 $ quickhunk 1 "1BC" "xbc"
+          y = prim2repopatchV2 $ quickhunk 1 "A2C" "ayc"
+          z = prim2repopatchV2 $ quickhunk 1 "AB3" "abz"
           x' :/\: _ = merge (a :\/: x)
           y' :/\: _ = merge (b :\/: y)
           z' :/\: _ = merge (c :\/: z)
 
-realQuickcheckMergeables :: [(RealPatch Prim :\/: RealPatch Prim) wX wY]
-realQuickcheckMergeables = [-- invert k1 :\/: n1
+repov2QuickcheckMergeables :: [(RepoPatchV2 Prim :\/: RepoPatchV2 Prim) wX wY]
+repov2QuickcheckMergeables = [-- invert k1 :\/: n1
                              --, invert k2 :\/: n2
                                hb :\/: k
                              , b' :\/: b'
@@ -467,25 +467,25 @@ realQuickcheckMergeables = [-- invert k1 :\/: n1
                              , k' :\/: k'
                              , k3 :\/: k3
                              ] ++ catMaybes (map pair2m pairs)
-    where hb = prim2real $ quickhunk 0 "" "hb"
-          k = prim2real $ quickhunk 0 "" "k"
-          n = prim2real $ quickhunk 0 "" "n"
-          b = prim2real $ quickhunk 1 "b" ""
-          d = prim2real $ quickhunk 2 "" "d"
+    where hb = prim2repopatchV2 $ quickhunk 0 "" "hb"
+          k = prim2repopatchV2 $ quickhunk 0 "" "k"
+          n = prim2repopatchV2 $ quickhunk 0 "" "n"
+          b = prim2repopatchV2 $ quickhunk 1 "b" ""
+          d = prim2repopatchV2 $ quickhunk 2 "" "d"
           d':/\:_ = merge (b :\/: d)
           --k1 :>: n1 :>: NilFL :/\: _ = mergeFL (hb :\/: k :>: n :>: NilFL)
           --k2 :>: n2 :>: NilFL :/\: _ =
           --    merge (hb :>: b :>: NilFL :\/: k :>: n :>: NilFL)
           k' :>: n' :>: NilFL :/\: _ :>: b' :>: _ = merge (hb :>: b :>: d' :>: NilFL :\/: k :>: n :>: NilFL)
           pairs = getPairs (hb :>: b :>: d' :>: k' :>: n' :>: NilFL)
-          pair2m :: Sealed2 (RealPatch Prim :> RealPatch Prim)
-                 -> Maybe ((RealPatch Prim :\/: RealPatch Prim) wX wY)
+          pair2m :: Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim)
+                 -> Maybe ((RepoPatchV2 Prim :\/: RepoPatchV2 Prim) wX wY)
           pair2m (Sealed2 (xx :> y)) = do y' :> _ <- commute (xx :> y)
                                           return $ unsafeCoerceP (xx :\/: y')
 
-          i = prim2real $ quickhunk 0 "" "i"
-          x = prim2real $ quickhunk 0 "" "x"
-          xi = prim2real $ quickhunk 0 "xi" ""
+          i = prim2repopatchV2 $ quickhunk 0 "" "i"
+          x = prim2repopatchV2 $ quickhunk 0 "" "x"
+          xi = prim2repopatchV2 $ quickhunk 0 "xi" ""
           d3 :/\: _ = merge (xi :\/: d)
           _ :/\: k3 = mergeFL (k :\/: i :>: x :>: xi :>: d3 :>: NilFL)
 
