@@ -9,7 +9,7 @@ import Test.HUnit ( assertFailure )
 
 import Darcs.Patch
 import Darcs.Patch.Conflict
-import Darcs.Patch.Rebase
+import Darcs.Patch.Rebase.Fixup
 import Darcs.Patch.Rebase.Viewing
 import Darcs.Patch.Type
 import Darcs.Patch.Witnesses.Ordered
@@ -17,7 +17,7 @@ import Darcs.Patch.Witnesses.Show
 
 import Darcs.Test.Patch.Arbitrary.Generic
 
-testSuite :: forall p . (RepoPatch p, ArbitraryPrim (PrimOf p), Show2 (PrimOf p)) => PatchType p -> [Test]
+testSuite :: forall rt p . (RepoPatch p, ArbitraryPrim (PrimOf p), Show2 (PrimOf p)) => PatchType rt p -> [Test]
 testSuite pt =
     if hasPrimConstruct (undefined :: PrimOf p WX WX)
         then
@@ -29,7 +29,7 @@ testSuite pt =
 
 data WX
 
-duplicateConflictedEffect :: forall p . (RepoPatch p, Show2 (PrimOf p)) => PatchType p -> Test
+duplicateConflictedEffect :: forall rt p . (RepoPatch p, Show2 (PrimOf p)) => PatchType rt p -> Test
 duplicateConflictedEffect _ =
     testCase "duplicate in rebase fixup has a conflicted effect" $
         unless (all (/= Okay) cStatuses) $
