@@ -1,5 +1,9 @@
 {-# OPTIONS_GHC -fno-warn-missing-methods -fno-warn-orphans #-}
-{-# LANGUAGE TypeSynonymInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE TypeSynonymInstances, MultiParamTypeClasses
+           , ConstraintKinds, UndecidableInstances, CPP #-}
+#if __GLASGOW_HASKELL__ >= 800
+{-# LANGUAGE UndecidableSuperClasses #-}
+#endif
 -- Copyright (C) 2010, 2011 Petr Rockai
 --
 -- Permission is hereby granted, free of charge, to any person
@@ -21,14 +25,21 @@
 -- ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 -- CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 -- SOFTWARE.
-module Darcs.Patch.ApplyMonad( ApplyMonad(..), ApplyMonadTrans(..), withFileNames, withFiles, ToTree(..) ) where
+module Darcs.Patch.ApplyMonad
+  ( ApplyMonad(..), ApplyMonadTrans(..), ApplyMonadState(..)
+  , withFileNames, withFiles, ToTree(..)
+  , ApplyMonadTree(..)
+  ) where
+
+import Prelude ()
+import Darcs.Prelude
 
 import qualified Data.ByteString      as B
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.Map             as M
-import qualified Storage.Hashed.Monad as HSM
+import qualified Darcs.Util.Tree.Monad as TM
+import Darcs.Util.Tree ( Tree )
 import Data.Maybe ( fromMaybe )
-import Storage.Hashed.Tree ( Tree )
 import Darcs.Util.ByteString( linesPS, unlinesPS )
 import Darcs.Util.Path ( FileName, movedirfilename, fn2fp, isParentOrEqOf,
                     floatPath, AnchoredPath )
@@ -36,8 +47,7 @@ import Control.Monad.State.Strict
 import Control.Monad.Identity( Identity )
 import Darcs.Patch.MonadProgress
 
--- TODO should UUID/Object live somewhere more central?
-import Darcs.Patch.Prim.V3.ObjectMap ( UUID, ObjectMap, DirContent )
+import GHC.Exts ( Constraint )
 
 fn2ap :: FileName -> AnchoredPath
 fn2ap = floatPath . fn2fp
@@ -48,19 +58,46 @@ class ToTree s where
 instance ToTree Tree where
   toTree = id
 
-class (Functor m, Monad m, ApplyMonad (ApplyMonadOver m state) state)
-      => ApplyMonadTrans m (state :: (* -> *) -> *) where
-  type ApplyMonadOver m state :: * -> *
-  runApplyMonad :: (ApplyMonadOver m state) x -> state m -> m (x, state m)
+class (Functor m, Monad m, ApplyMonad state (ApplyMonadOver state m))
+      => ApplyMonadTrans (state :: (* -> *) -> *) m where
+  type ApplyMonadOver state m :: * -> *
+  runApplyMonad :: (ApplyMonadOver state m) x -> state m -> m (x, state m)
 
-instance (Functor m, Monad m) => ApplyMonadTrans m Tree where
-  type ApplyMonadOver m Tree = HSM.TreeMonad m
-  runApplyMonad = HSM.virtualTreeMonad
+instance (Functor m, Monad m) => ApplyMonadTrans Tree m where
+  type ApplyMonadOver Tree m = TM.TreeMonad m
+  runApplyMonad = TM.virtualTreeMonad
 
-class (Functor m, Monad m, Functor (ApplyMonadBase m), Monad (ApplyMonadBase m), ToTree state)
+class ApplyMonadState (state :: (* -> *) -> *) where
+  type ApplyMonadStateOperations state :: (* -> *) -> Constraint
+
+class (Functor m, Monad m) => ApplyMonadTree m where
+    -- a semantic, Tree-based interface for patch application
+    mDoesDirectoryExist ::  FileName -> m Bool
+    mDoesFileExist ::  FileName -> m Bool
+    mReadFilePS ::  FileName -> m B.ByteString
+    mReadFilePSs ::  FileName -> m [B.ByteString]
+    mReadFilePSs f = linesPS `fmap` mReadFilePS f
+    mCreateDirectory ::  FileName -> m ()
+    mRemoveDirectory ::  FileName -> m ()
+    mCreateFile ::  FileName -> m ()
+    mCreateFile f = mModifyFilePS f $ \_ -> return B.empty
+    mRemoveFile ::  FileName -> m ()
+    mRename ::  FileName -> FileName -> m ()
+    mModifyFilePS ::  FileName -> (B.ByteString -> m B.ByteString) -> m ()
+    mModifyFilePSs ::  FileName -> ([B.ByteString] -> m [B.ByteString]) -> m ()
+    mModifyFilePSs f j = mModifyFilePS f (fmap unlinesPS . j . linesPS)
+    mChangePref ::  String -> String -> String -> m ()
+    mChangePref _ _ _ = return ()
+
+instance ApplyMonadState Tree where
+    type ApplyMonadStateOperations Tree = ApplyMonadTree
+
+class ( Functor m, Monad m, Functor (ApplyMonadBase m), Monad (ApplyMonadBase m)
+      , ApplyMonadStateOperations state m, ToTree state
+      )
        -- ApplyMonadOver (ApplyMonadBase m) ~ m is *not* required in general,
        -- since ApplyMonadBase is not injective
-       => ApplyMonad m (state :: (* -> *) -> *) where
+       => ApplyMonad (state :: (* -> *) -> *) m where
     type ApplyMonadBase m :: * -> *
 
     nestedApply :: m x -> state (ApplyMonadBase m) -> m (x, state (ApplyMonadBase m))
@@ -68,50 +105,27 @@ class (Functor m, Monad m, Functor (ApplyMonadBase m), Monad (ApplyMonadBase m),
                  -> m (x, state (ApplyMonadBase m))
 
     getApplyState :: m (state (ApplyMonadBase m))
-    putApplyState :: state m -> m ()
 
-    -- a semantic, ObjectMap-based interface for patch application
-    editFile :: (state ~ ObjectMap) => UUID -> (B.ByteString -> B.ByteString) -> m ()
-    editDirectory :: (state ~ ObjectMap) => UUID -> (DirContent -> DirContent) -> m ()
-
-    -- a semantic, Tree-based interface for patch application
-    mDoesDirectoryExist :: (state ~ Tree) => FileName -> m Bool
-    mDoesFileExist :: (state ~ Tree) => FileName -> m Bool
-    mReadFilePS :: (state ~ Tree) => FileName -> m B.ByteString
-    mReadFilePSs :: (state ~ Tree) => FileName -> m [B.ByteString]
-    mReadFilePSs f = linesPS `fmap` mReadFilePS f
-    mCreateDirectory :: (state ~ Tree) => FileName -> m ()
-    mRemoveDirectory :: (state ~ Tree) => FileName -> m ()
-    mCreateFile :: (state ~ Tree) => FileName -> m ()
-    mCreateFile f = mModifyFilePS f $ \_ -> return B.empty
-    mRemoveFile :: (state ~ Tree) => FileName -> m ()
-    mRename :: (state ~ Tree) => FileName -> FileName -> m ()
-    mModifyFilePS :: (state ~ Tree) => FileName -> (B.ByteString -> m B.ByteString) -> m ()
-    mModifyFilePSs :: (state ~ Tree) => FileName -> ([B.ByteString] -> m [B.ByteString]) -> m ()
-    mModifyFilePSs f j = mModifyFilePS f (fmap unlinesPS . j . linesPS)
-    mChangePref :: (state ~ Tree) => String -> String -> String -> m ()
-    mChangePref _ _ _ = return ()
-
-instance (Functor m, Monad m) => ApplyMonad (HSM.TreeMonad m) Tree where
-    type ApplyMonadBase (HSM.TreeMonad m) = m
-    getApplyState = gets HSM.tree
+instance (Functor m, Monad m) => ApplyMonad Tree (TM.TreeMonad m) where
+    type ApplyMonadBase (TM.TreeMonad m) = m
+    getApplyState = gets TM.tree
     nestedApply a start = lift $ runApplyMonad a start
-    liftApply a start = do x <- gets HSM.tree
+    liftApply a start = do x <- gets TM.tree
                            lift $ runApplyMonad (lift $ a x) start
 
-    -- putApplyState needs some support from HSM
+instance (Functor m, Monad m) => ApplyMonadTree (TM.TreeMonad m) where
 
-    mDoesDirectoryExist d = HSM.directoryExists (fn2ap d)
-    mDoesFileExist d = HSM.fileExists (fn2ap d)
-    mReadFilePS p = B.concat `fmap` BL.toChunks `fmap` HSM.readFile (fn2ap p)
-    mModifyFilePS p j = do have <- HSM.fileExists (fn2ap p)
-                           x <- if have then B.concat `fmap` BL.toChunks `fmap` HSM.readFile (fn2ap p)
+    mDoesDirectoryExist d = TM.directoryExists (fn2ap d)
+    mDoesFileExist d = TM.fileExists (fn2ap d)
+    mReadFilePS p = B.concat `fmap` BL.toChunks `fmap` TM.readFile (fn2ap p)
+    mModifyFilePS p j = do have <- TM.fileExists (fn2ap p)
+                           x <- if have then B.concat `fmap` BL.toChunks `fmap` TM.readFile (fn2ap p)
                                         else return B.empty
-                           HSM.writeFile (fn2ap p) . BL.fromChunks . (:[]) =<< j x
-    mCreateDirectory p = HSM.createDirectory (fn2ap p)
-    mRename from to = HSM.rename (fn2ap from) (fn2ap to)
-    mRemoveDirectory = HSM.unlink . fn2ap
-    mRemoveFile = HSM.unlink . fn2ap
+                           TM.writeFile (fn2ap p) . BL.fromChunks . (:[]) =<< j x
+    mCreateDirectory p = TM.createDirectory (fn2ap p)
+    mRename from to = TM.rename (fn2ap from) (fn2ap to)
+    mRemoveDirectory = TM.unlink . fn2ap
+    mRemoveFile = TM.unlink . fn2ap
 
 -- Latest name, current original name.
 type OrigFileNameOf = (FileName, FileName)
@@ -138,8 +152,11 @@ withFileNames :: Maybe [OrigFileNameOf] -> [FileName] -> FilePathMonad a
 withFileNames mbofnos fps x = execState x ([], fps, ofnos) where
     ofnos = fromMaybe (map (\y -> (y, y)) fps) mbofnos
 
-instance ApplyMonad FilePathMonad Tree where
+instance ApplyMonad Tree FilePathMonad where
     type ApplyMonadBase FilePathMonad = Identity
+
+
+instance ApplyMonadTree FilePathMonad where
     -- We can't check it actually is a directory here
     mDoesDirectoryExist d = gets $ \(_, fs, _) -> d `elem` fs
 
@@ -158,8 +175,10 @@ instance MonadProgress FilePathMonad where
 
 type RestrictedApply = State (M.Map FileName B.ByteString)
 
-instance ApplyMonad RestrictedApply Tree where
+instance ApplyMonad Tree RestrictedApply where
   type ApplyMonadBase RestrictedApply = Identity
+
+instance ApplyMonadTree RestrictedApply where
   mDoesDirectoryExist _ = return True
   mCreateDirectory _ = return ()
   mRemoveFile f = modify $ M.delete f
