@@ -12,8 +12,8 @@ import Darcs.Patch.MonadProgress ( MonadProgress, ProgressAction(..), runProgres
 import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL )
 import Darcs.Util.Printer ( text, ($$) )
 
-applyPatches :: (MonadProgress m, ApplyMonad m (ApplyState p), Patchy p)
-             => FL (PatchInfoAnd p) wX wY -> m ()
+applyPatches :: (MonadProgress m, ApplyMonad (ApplyState p) m, Patchy p)
+             => FL (PatchInfoAnd rt p) wX wY -> m ()
 applyPatches ps = runProgressActions "Applying patch" (mapFL doApply ps)
   where
     doApply hp = ProgressAction { paAction = apply (hopefully hp)
