@@ -30,15 +30,18 @@ module Darcs.Patch.Bundle
     , parseBundle
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.Char ( isAlpha, toLower, isDigit, isSpace )
 import qualified Data.ByteString as B ( ByteString, length, null, drop,
                                         isPrefixOf )
 import qualified Data.ByteString.Char8 as BC ( unpack, break, pack )
 
-import Storage.Hashed.Tree( Tree )
-import Storage.Hashed.Monad( virtualTreeIO )
+import Darcs.Util.Tree( Tree )
+import Darcs.Util.Tree.Monad( virtualTreeIO )
 
-import Darcs.Patch ( RepoPatch, Named, showPatch, showContextPatch,
+import Darcs.Patch ( RepoPatch, showPatch, showContextPatch,
                      readPatchPartial )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Bracketed ( Bracketed, unBracketedFL )
@@ -48,9 +51,13 @@ import Darcs.Patch.Depends ( slightlyOptimizePatchset )
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Info ( PatchInfo, readPatchInfo, showPatchInfo,
                           showPatchInfoUI, isTag )
+import Darcs.Patch.Named.Wrapped ( WrappedNamed )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, piap, fmapFLPIAP, info,
-                                  patchInfoAndPatch, unavailable, hopefully )
+                                  patchInfoAndPatch, unavailable, hopefully,
+                                  generaliseRepoTypePIAP
+                                )
 import Darcs.Patch.ReadMonads ( parseStrictly )
+import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, Origin )
 import Darcs.Patch.Show ( ShowPatchBasic )
 import Darcs.Patch.Witnesses.Ordered
@@ -67,16 +74,16 @@ import Darcs.Util.Printer ( Doc, renderPS, newline, text, ($$),
 -- |hashBundle creates a SHA1 string of a given a FL of named patches. This
 -- allows us to ensure that the patches in a received patchBundle have not been
 -- modified in transit.
-hashBundle :: (PatchListFormat p, ShowPatchBasic p) => FL (Named p) wX wY
+hashBundle :: (PatchListFormat p, ShowPatchBasic p) => FL (WrappedNamed rt p) wX wY
            -> String
 hashBundle to_be_sent =
     show $ sha1PS $ renderPS Standard $ vcat (mapFL showPatch to_be_sent) <> newline
 
 makeBundleN :: (ApplyState p ~ Tree, RepoPatch p) => Maybe (Tree IO)
-            -> PatchSet p wStart wX -> FL (Named p) wX wY -> IO Doc
-makeBundleN the_s (PatchSet ps (Tagged t _ _ :<: _)) to_be_sent =
-    makeBundle2 the_s (ps +<+ (t :<: NilRL)) to_be_sent to_be_sent
-makeBundleN the_s (PatchSet ps NilRL) to_be_sent =
+            -> PatchSet rt p wStart wX -> FL (WrappedNamed rt p) wX wY -> IO Doc
+makeBundleN the_s (PatchSet (_ :<: Tagged t _ _) ps) to_be_sent =
+    makeBundle2 the_s ((NilRL :<: t) +<+ ps) to_be_sent to_be_sent
+makeBundleN the_s (PatchSet NilRL ps) to_be_sent =
     makeBundle2 the_s ps to_be_sent to_be_sent
 
 -- | In makeBundle2, it is presumed that the two patch sequences are
@@ -85,8 +92,8 @@ makeBundleN the_s (PatchSet ps NilRL) to_be_sent =
 -- generated, which is not the end of the world, but isn't very useful
 -- either.
 makeBundle2 :: (ApplyState p ~ Tree, RepoPatch p) => Maybe (Tree IO)
-            -> RL (PatchInfoAnd p) wStart wX -> FL (Named p) wX wY
-            -> FL (Named p) wX wY -> IO Doc
+            -> RL (PatchInfoAnd rt p) wStart wX -> FL (WrappedNamed rt p) wX wY
+            -> FL (WrappedNamed rt p) wX wY -> IO Doc
 makeBundle2 the_s common' to_be_sent to_be_sent2 = do
     patches <- case the_s of
                    Just tree -> fst `fmap` virtualTreeIO (showContextPatch to_be_sent) tree
@@ -106,9 +113,9 @@ makeBundle2 the_s common' to_be_sent to_be_sent2 = do
                      $$ text ""
     common = mapRL info common'
 
-parseBundle :: forall p. RepoPatch p => B.ByteString
+parseBundle :: forall rt p. RepoPatch p => B.ByteString
             -> Either String
-                      (Sealed ((PatchSet p :> FL (PatchInfoAnd p)) Origin))
+                      (Sealed ((PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin))
 parseBundle input | B.null input = Left "Bad patch bundle!"
 parseBundle input = case sillyLex input of
     ("New patches:", rest) -> case getPatches rest of
@@ -147,28 +154,34 @@ parseBundle input = case sillyLex input of
     sealCtxAndPs ctx ps = Right $ sealContextWithPatches ctx ps
 
     sealContextWithPatches :: RepoPatch p => [PatchInfo]
-                           -> FL (PatchInfoAnd (Bracketed p)) wX wY
+                           -> FL (PatchInfoAnd ('RepoType 'NoRebase) (Bracketed p)) wX wY
                            -> Sealed
-                                  ((PatchSet p :> FL (PatchInfoAnd p)) Origin)
+                                  ((PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin)
     sealContextWithPatches context bracketedPatches =
-        let patches = mapFL_FL (fmapFLPIAP unBracketedFL) bracketedPatches in
+        let -- witness to fmapFLPIAP that the bundle won't contain stash/rebase patches
+            -- TODO use EmptyCase with GHC 7.8+
+            notRebasing _
+              = error "internal error: unreachable case (Darcs.Patch.Bundle.parseBundle.notRebasing)"
+            patches = mapFL_FL (generaliseRepoTypePIAP . fmapFLPIAP unBracketedFL notRebasing)
+                               bracketedPatches
+        in
         case reverse context of
             (x : ry) | isTag x ->
                   let ps = unavailablePatches (reverse ry)
                       t = Tagged (piUnavailable x) Nothing NilRL in
-                  Sealed $ PatchSet ps (t :<: NilRL) :> patches
-            _ -> let ps = PatchSet (unavailablePatches context) NilRL in
+                  Sealed $ PatchSet (NilRL :<: t) ps :> patches
+            _ -> let ps = PatchSet NilRL (unavailablePatches context) in
                  Sealed $ ps :> patches
                  -- The above NilRLs aren't quite right, because ther *are*
                  -- earlier patches, but we can't set this to undefined
                  -- because there are situations where we look at the rest.
                  -- :{
 
-scanBundle :: forall p . RepoPatch p => B.ByteString
-           -> Either String (SealedPatchSet p Origin)
+scanBundle :: forall rt p . RepoPatch p => B.ByteString
+           -> Either String (SealedPatchSet rt p Origin)
 scanBundle bundle = do
-  Sealed (PatchSet recent tagged :> ps) <- parseBundle bundle
-  return . Sealed $ PatchSet (reverseFL ps +<+ recent) tagged
+  Sealed (PatchSet tagged recent :> ps) <- parseBundle bundle
+  return . Sealed $ PatchSet tagged (recent +<+ reverseFL ps)
 
 -- |filterGpgDashes unescapes a clearsigned patch, which will have had any
 -- lines starting with dashes escaped with a leading "- ".
@@ -188,12 +201,12 @@ filterGpgDashes ps =
 
 -- |unavailablePatches converts a list of PatchInfos into a RL of PatchInfoAnd
 -- Unavailable patches. This is used to represent the Context of a patchBundle.
-unavailablePatches :: RepoPatch p => [PatchInfo] -> RL (PatchInfoAnd p) wX wY
-unavailablePatches = foldr ((:<:) . piUnavailable) (unsafeCoerceP NilRL)
+unavailablePatches :: RepoPatch p => [PatchInfo] -> RL (PatchInfoAnd rt p) wX wY
+unavailablePatches = foldr (flip (:<:) . piUnavailable) (unsafeCoerceP NilRL)
 
 -- |piUnavailable returns an Unavailable within a PatchInfoAnd given a
 -- PatchInfo.
-piUnavailable :: RepoPatch p => PatchInfo -> PatchInfoAnd p wX wY
+piUnavailable :: RepoPatch p => PatchInfo -> PatchInfoAnd rt p wX wY
 piUnavailable i = patchInfoAndPatch i . unavailable $
     "Patch not stored in patch bundle:\n" ++ renderString Encode (showPatchInfoUI i)
 
@@ -214,7 +227,7 @@ p -:- (Sealed ps, r) = (Sealed (p :>: ps), r)
 -- returning the FL of as many patches-with-info as were successfully parsed,
 -- along with any unconsumed input.
 getPatches :: RepoPatch p => B.ByteString
-           -> (Sealed (FL (PatchInfoAnd (Bracketed p)) wX), B.ByteString)
+           -> (Sealed (FL (PatchInfoAnd ('RepoType 'NoRebase) (Bracketed p)) wX), B.ByteString)
 getPatches ps = case parseStrictly readPatchInfo ps of
     Nothing -> (Sealed NilFL, ps)
     Just (pinfo, _) -> case readPatchPartial ps of
@@ -229,19 +242,19 @@ sillyLex ps = (BC.unpack a, b)
   where
     (a, b) = BC.break (== '\n') (dropSpace ps)
 
-contextPatches :: RepoPatch p => PatchSet p Origin wX
-               -> (PatchSet p :> RL (PatchInfoAnd p)) Origin wX
+contextPatches :: RepoPatch p => PatchSet rt p Origin wX
+               -> (PatchSet rt p :> RL (PatchInfoAnd rt p)) Origin wX
 contextPatches set = case slightlyOptimizePatchset set of
-    PatchSet ps (Tagged t _ ps' :<: ts) ->
-        PatchSet ps' ts :> (ps +<+ (t :<: NilRL))
-    PatchSet ps NilRL -> PatchSet NilRL NilRL :> ps
+    PatchSet (ts :<: Tagged t _ ps') ps ->
+        PatchSet ts ps' :> ((NilRL :<: t) +<+ ps)
+    PatchSet NilRL ps -> PatchSet NilRL NilRL :> ps
 
 -- |'scanContextFile' scans the context in the file of the given name.
-scanContextFile :: RepoPatch p => FilePath -> IO (PatchSet p Origin wX)
+scanContextFile :: RepoPatch p => FilePath -> IO (PatchSet rt p Origin wX)
 scanContextFile filename = scanContext `fmap` mmapFilePS filename
   where
     -- are the type witnesses sensible?
-    scanContext :: RepoPatch p => B.ByteString -> PatchSet p Origin wX
+    scanContext :: RepoPatch p => B.ByteString -> PatchSet rt p Origin wX
     scanContext input
         | B.null input = error "Bad context!"
         | otherwise = case sillyLex input of
@@ -249,8 +262,8 @@ scanContextFile filename = scanContext `fmap` mmapFilePS filename
                 (cont@(_ : _), _) | isTag (last cont) ->
                     let ps = unavailablePatches $ init cont
                         t = Tagged (piUnavailable $ last cont) Nothing NilRL in
-                    PatchSet ps (t :<: NilRL)
-                (cont, _) -> PatchSet (unavailablePatches cont) NilRL
+                    PatchSet (NilRL :<: t) ps
+                (cont, _) -> PatchSet NilRL (unavailablePatches cont)
             ("-----BEGIN PGP SIGNED MESSAGE-----",rest) ->
                 scanContext $ filterGpgDashes rest
             (_, rest) -> scanContext rest
@@ -258,20 +271,20 @@ scanContextFile filename = scanContext `fmap` mmapFilePS filename
 -- | Minimize the context of a bundle to be sent, taking into account
 --   the patches selected to be sent 
 minContext :: (RepoPatch p)
-          => PatchSet p wStart wB
-          -> FL (PatchInfoAnd p) wB wC
-          -> Sealed ((PatchSet p :> FL (PatchInfoAnd p)) wStart)
-minContext (PatchSet topCommon behindTag) to_be_sent =
+          => PatchSet rt p wStart wB
+          -> FL (PatchInfoAnd rt p) wB wC
+          -> Sealed ((PatchSet rt p :> FL (PatchInfoAnd rt p)) wStart)
+minContext (PatchSet behindTag topCommon) to_be_sent =
   case go topCommon NilFL to_be_sent of
-    Sealed (c :> to_be_sent') -> seal (PatchSet c behindTag :> to_be_sent') 
+    Sealed (c :> to_be_sent') -> seal (PatchSet behindTag c :> to_be_sent') 
   where
     go :: (RepoPatch p)
-       => RL (PatchInfoAnd p) wA wB -- context we attempt to minimize
-       -> FL (PatchInfoAnd p) wB wC -- patches we cannot remove from context
-       -> FL (PatchInfoAnd p) wC wD -- patches to be included in the bundle
-       -> Sealed (( RL (PatchInfoAnd p) :> FL (PatchInfoAnd p) ) wA )
+       => RL (PatchInfoAnd rt p) wA wB -- context we attempt to minimize
+       -> FL (PatchInfoAnd rt p) wB wC -- patches we cannot remove from context
+       -> FL (PatchInfoAnd rt p) wC wD -- patches to be included in the bundle
+       -> Sealed (( RL (PatchInfoAnd rt p) :> FL (PatchInfoAnd rt p) ) wA )
     go NilRL necessary to_be_sent' = seal (reverseFL necessary :> to_be_sent')
-    go (candidate :<: rest) necessary to_be_sent' =
+    go (rest :<: candidate) necessary to_be_sent' =
       let fl1 = (candidate :>: NilFL) in
       case commute (fl1 :> necessary) of
         Nothing                   -> go rest (candidate :>: necessary) to_be_sent'
