@@ -54,11 +54,14 @@ module Darcs.Patch.Choices ( PatchChoices, patchChoices, patchChoicesLps,
                       forceMatchingFirst, forceMatchingLast,
                       selectAllMiddles,
                       makeUncertain, makeEverythingLater, makeEverythingSooner,
-                      LabelledPatch, Label, label, lpPatch,
+                      LabelledPatch, Label, label, lpPatch, getLabelInt,
                              Slot(..),
                       substitute
                     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Monad.Identity ( Identity )
 import Control.Monad.State ( StateT(..) )
 
@@ -114,6 +117,9 @@ data Slot = InFirst | InMiddle | InLast
 label :: LabelledPatch p wX wY -> Label
 label (LP tg _) = tg
 
+getLabelInt :: Label -> Integer
+getLabelInt (Label _ i) = i
+
 lpPatch :: LabelledPatch p wX wY -> p wX wY
 lpPatch (LP _ p) = p
 
@@ -256,11 +262,11 @@ patchSlot (LP t _) pc@(PCs f l) =
                                     +>+ PC lp True
                                     :>: ls})
     psLast firsts middles bubble (PC lp True :>: ls) =
-      psLast firsts middles (lp :<: bubble) ls
+      psLast firsts middles (bubble :<: lp) ls
     psLast firsts middles bubble (PC lp False :>: ls) =
       case commuteRL (bubble :> lp) of
-        Just (lp' :> bubble') -> psLast firsts (lp' :<: middles) bubble' ls
-        Nothing -> psLast firsts middles (lp :<: bubble) ls
+        Just (lp' :> bubble') -> psLast firsts (middles :<: lp') bubble' ls
+        Nothing -> psLast firsts middles (bubble :<: lp) ls
     psLast _ _ _ NilFL = impossible
     settleM middles = mapFL_FL (\lp -> PC lp False) $ reverseRL middles
     settleB bubble = mapFL_FL (\lp -> PC lp True) $ reverseRL bubble
@@ -287,7 +293,7 @@ forceMatchingFirst pred (PCs fn l) =
                 let
                   f' = f +>+ mapFL_FL pcPatch (reverseRL deps) +>+ (pcPatch a' :>: NilFL)
                 in fmfLasts f' l1' l2
-      fmfLasts f l1 (a :>: l2) = fmfLasts f (a :<: l1) l2
+      fmfLasts f l1 (a :>: l2) = fmfLasts f (l1 :<: a) l2
       fmfLasts f l1 NilFL = PCs { pcsFirsts = f
                                 , pcsLasts = reverseRL l1 }
       pred_pc :: forall wX wY . PatchChoice p wX wY -> Bool
@@ -316,9 +322,9 @@ selectAllMiddles False (PCs f l) = samf f NilRL NilRL l
            PatchChoices p wX wY
     samf f1 f2 l1 (pc@(PC lp False) :>: l2) =
       case commuteRL (l1 :> pc) of
-        Nothing -> samf f1 f2 (PC lp True :<: l1) l2
-        Just ((PC lp' _) :> l1') -> samf f1 (lp' :<: f2) l1' l2
-    samf f1 f2 l1 (PC lp True :>: l2) = samf f1 f2 (PC lp True :<: l1) l2
+        Nothing -> samf f1 f2 (l1 :<: PC lp True) l2
+        Just ((PC lp' _) :> l1') -> samf f1 (f2 :<: lp') l1' l2
+    samf f1 f2 l1 (PC lp True :>: l2) = samf f1 f2 (l1 :<: PC lp True) l2
     samf f1 f2 l1 NilFL = PCs (f1 +>+ reverseRL f2) (reverseRL l1)
 
 forceMatchingLast :: Patchy p => (forall wX wY . LabelledPatch p wX wY -> Bool)
@@ -341,7 +347,7 @@ fmlFirst pred b f1 (a :>: f2) l
                 l' = mapFL_FL (\lp -> PC lp b) (a' :>: deps) +>+ l
               in
               fmlFirst pred b f1 f2' l'
-fmlFirst pred b f1 (a :>: f2) l = fmlFirst pred b (a :<: f1) f2 l
+fmlFirst pred b f1 (a :>: f2) l = fmlFirst pred b (f1 :<: a) f2 l
 fmlFirst pred b f1 NilFL l = PCs { pcsFirsts = reverseRL f1
                                  , pcsLasts = mapFL_FL ch l}
   where ch (PC lp c) = (PC lp (if pred lp then b else c) )
@@ -376,11 +382,11 @@ makeEverythingSooner (PCs f l) =
             RL (LabelledPatch p) wM2 wM3 ->
             FL (PatchChoice p) wM3 wY ->
             (FL (LabelledPatch p) :> FL (PatchChoice p)) wM1 wY
-      mes middle bubble (PC lp True :>: ls) = mes middle (lp :<: bubble) ls
+      mes middle bubble (PC lp True :>: ls) = mes middle (bubble :<: lp) ls
       mes middle bubble (PC lp False :>: ls) =
         case commuteRL (bubble :> lp) of
-          Nothing -> mes middle (lp :<: bubble) ls
-          Just (lp' :> bubble') -> mes (lp' :<: middle) bubble' ls
+          Nothing -> mes middle (bubble :<: lp) ls
+          Just (lp' :> bubble') -> mes (middle :<: lp') bubble' ls
       mes middle bubble NilFL = (reverseRL middle) :> mapFL_FL (\lp -> PC lp False) (reverseRL bubble)
 
 -- | 'substitute' @(a :||: bs)@ @pcs@ replaces @a@ with @bs@ in @pcs@ preserving the choice
