@@ -30,25 +30,36 @@ module Darcs.Patch.Depends
     , splitOnTag
     , newsetUnion
     , newsetIntersection
-    , commuteToEnd
     , findUncommon
     , merge2FL
+    , getDeps
+    , SPatchAndDeps
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 #include "impossible.h"
 
 import Prelude hiding ( pi )
 import Data.List ( delete, intersect, (\\) )
 import Data.Maybe ( fromMaybe )
+import qualified Data.ByteString.Char8 as BC ( unpack )
+import Control.Arrow ( (&&&) )
 
-import Darcs.Patch ( Patchy, getdeps, commute, commuteFLorComplain,
-                     commuteRL )
-import Darcs.Patch.Info ( PatchInfo, isTag, showPatchInfoUI )
+import Darcs.Patch ( RepoPatch )
+import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Patchy ( Patchy )
+import Darcs.Patch.Named ( Named (..), patch2patchinfo )
+import Darcs.Patch.Named.Wrapped ( getdeps )
+import Darcs.Patch.Choices ( Label, patchChoices, forceFirst
+                           , PatchChoices, lpPatch, getChoices
+                           , LabelledPatch, label )
+import Darcs.Patch.Commute ( commute, commuteFL, commuteRL )
+import Darcs.Patch.Info ( PatchInfo, isTag, showPatchInfoUI, _piName )
 import Darcs.Patch.Merge ( Merge, mergeFL )
-import Darcs.Patch.Permutations ( partitionFL, partitionRL,
-                                  removeSubsequenceRL )
+import Darcs.Patch.Permutations ( partitionFL, partitionRL )
 import Darcs.Patch.PatchInfoAnd( PatchInfoAnd, hopefully, hopefullyM, info )
-import Darcs.Patch.Rebase.NameHack ( NameHack )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL,
                          appendPSFL )
 import Darcs.Patch.Progress ( progressRL )
@@ -57,11 +68,12 @@ import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePStart )
 import Darcs.Patch.Witnesses.Ordered
     ( (:\/:)(..), (:/\:)(..), (:>)(..), Fork(..),
     (+>+), mapFL, RL(..), FL(..), isShorterThanRL,
-    (+<+), reverseFL, reverseRL, mapRL )
+    (+<+), reverseFL, reverseRL, mapRL, lengthFL, splitAtFL )
 import Darcs.Patch.Witnesses.Sealed
-    ( Sealed(..), FlippedSeal(..), flipSeal, seal )
+    ( Sealed(..), FlippedSeal(..), flipSeal, seal, Sealed2(..), seal2 )
 
 import Darcs.Util.Printer ( renderString, vcat, RenderMode(..) )
+import Darcs.Util.Tree ( Tree )
 
 {-
  - This module uses the following definitions:
@@ -82,6 +94,54 @@ import Darcs.Util.Printer ( renderString, vcat, RenderMode(..) )
  - since they explicitly depend on all uncovered patches.
  -}
 
+-- | S(ealed) Patch and his dependencies.
+type SPatchAndDeps p = ( Sealed2 (LabelledPatch (Named p))
+                       , Sealed2 (FL (LabelledPatch (Named p)))
+                       )
+
+-- | Searchs dependencies in @repoFL@ of the patches in @getDepsFL@.
+getDeps :: (RepoPatch p, ApplyState p ~ Tree) =>
+            FL (Named p) wA wR -> FL (PatchInfoAnd rt p) wX wY -> [SPatchAndDeps p]
+getDeps repoFL getDepsFL =
+        let repoChoices   = patchChoices repoFL
+            getDepsFL'    = mapFL (BC.unpack . _piName . info) getDepsFL
+            labelledDeps  = getLabelledDeps getDepsFL' repoChoices
+        in
+            map (deps repoChoices) labelledDeps
+    where
+        -- Search dependencies for the patch with label @l@ in @repoChoices@.
+        deps :: (Patchy (Named p)) => PatchChoices (Named p) wX wY ->
+                (String,Label) -> SPatchAndDeps p
+        deps repoChoices (_,l) =
+            case getChoices $ forceFirst l repoChoices of
+                (ds :> _) -> let i = lengthFL ds
+                             in case splitAtFL (i-1) ds of
+                                    -- Separate last patch in list
+                                    ds' :> (r :>: NilFL) -> (seal2 r, seal2 ds')
+                                    _ -> impossible -- Because deps at least
+                                                    -- has r, which is the patch
+                                                    -- that we are looking at
+                                                    -- dependencies.
+        getLabelledDeps :: Patchy (Named p) => [String] ->
+                           PatchChoices (Named p) x y -> [(String, Label)]
+        getLabelledDeps patchnames repoChoices =
+            case getChoices repoChoices of
+                 a :> (b :> c) -> filterDepsFL patchnames a ++
+                                  filterDepsFL patchnames b ++
+                                  filterDepsFL patchnames c
+        filterDepsFL :: [String] -> FL (LabelledPatch (Named p)) wX wY ->
+                        [(String, Label)]
+        filterDepsFL _ NilFL = []
+        filterDepsFL patchnames (lp :>: lps) =
+                                if fst dep `elem` patchnames
+                                    then dep : filterDepsFL patchnames lps
+                                    else filterDepsFL patchnames lps
+            where
+                lpTostring :: LabelledPatch (Named p) wA wB -> String
+                lpTostring = BC.unpack . _piName . patch2patchinfo . lpPatch
+                dep :: (String, Label)
+                dep = lpTostring &&& label $ lp
+
 {-|
 taggedIntersection takes two 'PatchSet's and splits them into a /common/
 intersection portion and two sets of patches.  The intersection, however,
@@ -95,21 +155,21 @@ the repository that is more cheaply accessed (i.e. local), as
 taggedIntersection does its best to reduce the number of
 inventories that are accessed from its rightmost argument.
 -}
-taggedIntersection :: forall p wStart wX wY . (Patchy p, NameHack p) =>
-                      PatchSet p wStart wX -> PatchSet p wStart wY ->
-                      Fork (RL (Tagged p))
-                           (RL (PatchInfoAnd p))
-                           (RL (PatchInfoAnd p)) wStart wX wY
-taggedIntersection (PatchSet ps1 NilRL) s2 = Fork NilRL ps1 (newset2RL s2)
-taggedIntersection s1 (PatchSet ps2 NilRL) = Fork NilRL (newset2RL s1) ps2
-taggedIntersection s1 (PatchSet ps2 (Tagged t _ _ :<: _))
-    | Just (PatchSet ps1 ts1) <- maybeSplitSetOnTag (info t) s1 =
+taggedIntersection :: forall rt p wStart wX wY . Patchy p =>
+                      PatchSet rt p wStart wX -> PatchSet rt p wStart wY ->
+                      Fork (RL (Tagged rt p))
+                           (RL (PatchInfoAnd rt p))
+                           (RL (PatchInfoAnd rt p)) wStart wX wY
+taggedIntersection (PatchSet NilRL ps1) s2 = Fork NilRL ps1 (newset2RL s2)
+taggedIntersection s1 (PatchSet NilRL ps2) = Fork NilRL (newset2RL s1) ps2
+taggedIntersection s1 (PatchSet (_ :<: Tagged t _ _) ps2)
+    | Just (PatchSet ts1 ps1) <- maybeSplitSetOnTag (info t) s1 =
     Fork ts1 ps1 (unsafeCoercePStart ps2)
-taggedIntersection s1 s2@(PatchSet ps2 (Tagged t _ p :<: ts2)) =
+taggedIntersection s1 s2@(PatchSet (ts2 :<: Tagged t _ p) ps2) =
     case hopefullyM t of
-        Just _ -> taggedIntersection s1 (PatchSet (ps2 +<+ t :<: p) ts2)
+        Just _ -> taggedIntersection s1 (PatchSet ts2 (p :<: t +<+ ps2))
         Nothing -> case splitOnTag (info t) s1 of
-                       Just (PatchSet NilRL com :> us) ->
+                       Just (PatchSet com NilRL :> us) ->
                            Fork com us (unsafeCoercePStart ps2)
                        Just _ -> impossible
                        Nothing -> Fork NilRL (newset2RL s1) (newset2RL s2)
@@ -119,20 +179,20 @@ taggedIntersection s1 s2@(PatchSet ps2 (Tagged t _ p :<: ts2)) =
 -- found, the PatchSet is split up, on that tag, such that all later patches
 -- are in the "since last tag" patch list. If the tag is not found, 'Nothing'
 -- is returned.
-maybeSplitSetOnTag :: PatchInfo -> PatchSet p wStart wX
-                   -> Maybe (PatchSet p wStart wX)
-maybeSplitSetOnTag t0 origSet@(PatchSet ps (Tagged t _ pst :<: ts))
+maybeSplitSetOnTag :: PatchInfo -> PatchSet rt p wStart wX
+                   -> Maybe (PatchSet rt p wStart wX)
+maybeSplitSetOnTag t0 origSet@(PatchSet (ts :<: Tagged t _ pst) ps)
     | t0 == info t = Just origSet
     | otherwise = do
-        PatchSet ps' ts' <- maybeSplitSetOnTag t0 (PatchSet (t :<: pst) ts)
-        Just $ PatchSet (ps +<+ ps') ts'
+        PatchSet ts' ps' <- maybeSplitSetOnTag t0 (PatchSet ts (pst :<: t))
+        Just $ PatchSet ts' (ps' +<+ ps)
 maybeSplitSetOnTag _ _ = Nothing
 
-getPatchesBeyondTag :: (Patchy p, NameHack p) => PatchInfo -> PatchSet p wStart wX
-                    -> FlippedSeal (RL (PatchInfoAnd p)) wX
-getPatchesBeyondTag t (PatchSet ps (Tagged hp _ _ :<: _)) | info hp == t =
+getPatchesBeyondTag :: Patchy p => PatchInfo -> PatchSet rt p wStart wX
+                    -> FlippedSeal (RL (PatchInfoAnd rt p)) wX
+getPatchesBeyondTag t (PatchSet (_ :<: Tagged hp _ _) ps) | info hp == t =
     flipSeal ps
-getPatchesBeyondTag t patchset@(PatchSet (hp :<: ps) ts) =
+getPatchesBeyondTag t patchset@(PatchSet ts (ps :<: hp)) =
     if info hp == t
         then if getUncovered patchset == [info hp]
                  -- special case to avoid looking at redundant patches
@@ -140,63 +200,63 @@ getPatchesBeyondTag t patchset@(PatchSet (hp :<: ps) ts) =
                  else case splitOnTag t patchset of
                      Just (_ :> e) -> flipSeal e
                      _ -> impossible
-        else case getPatchesBeyondTag t (PatchSet ps ts) of
-                 FlippedSeal xxs -> FlippedSeal (hp :<: xxs)
+        else case getPatchesBeyondTag t (PatchSet ts ps) of
+                 FlippedSeal xxs -> FlippedSeal (xxs :<: hp)
 getPatchesBeyondTag t (PatchSet NilRL NilRL) =
     bug $ "tag\n" ++ renderString Encode (showPatchInfoUI t)
           ++ "\nis not in the patchset in getPatchesBeyondTag."
-getPatchesBeyondTag t0 (PatchSet NilRL (Tagged t _ ps :<: ts)) =
-    getPatchesBeyondTag t0 (PatchSet (t :<: ps) ts)
+getPatchesBeyondTag t0 (PatchSet (ts :<: Tagged t _ ps) NilRL) =
+    getPatchesBeyondTag t0 (PatchSet ts (ps :<: t))
 
 -- |splitOnTag takes a tag's 'PatchInfo', and a 'PatchSet', and attempts to
 -- find the tag in the PatchSet, returning a pair: the clean PatchSet "up to"
 -- the tag, and a RL of patches after the tag; If the tag is not in the
 -- PatchSet, we return Nothing.
-splitOnTag :: (Patchy p, NameHack p) => PatchInfo -> PatchSet p wStart wX
-           -> Maybe ((PatchSet p :> RL (PatchInfoAnd p)) wStart wX)
+splitOnTag :: Patchy p => PatchInfo -> PatchSet rt p wStart wX
+           -> Maybe ((PatchSet rt p :> RL (PatchInfoAnd rt p)) wStart wX)
 -- If the tag we are looking for is the first Tagged tag of the patchset, just
 -- separate out the patchset's patches.
-splitOnTag t (PatchSet ps ts@(Tagged hp _ _ :<: _)) | info hp == t =
-    Just $ PatchSet NilRL ts :> ps
+splitOnTag t (PatchSet ts@(_ :<: Tagged hp _ _) ps) | info hp == t =
+    Just $ PatchSet ts NilRL :> ps
 -- If the tag is the most recent patch in the set, we check if the patch is the
 -- only non-depended-on patch in the set (i.e. it is a clean tag); creating a
 -- new Tagged out of the patches and tag, and adding it to the patchset, if
 -- this is the case. Otherwise, we try to make the tag clean.
-splitOnTag t patchset@(PatchSet hps@(hp :<: ps) ts) | info hp == t =
+splitOnTag t patchset@(PatchSet ts hps@(ps :<: hp)) | info hp == t =
     if getUncovered patchset == [t]
-        then Just $ PatchSet NilRL (Tagged hp Nothing ps :<: ts) :> NilRL
+        then Just $ PatchSet (ts :<: Tagged hp Nothing ps) NilRL :> NilRL
         else case partitionRL ((`notElem` (t : ds)) . info) hps of
             -- Partition hps by those that are the tag and its explicit deps.
-            tagAndDeps@(hp' :<: ds') :> nonDeps ->
+            tagAndDeps@(ds' :<: hp') :> nonDeps ->
                 -- If @ds@ doesn't contain the tag of the first Tagged, that
                 -- tag will also be returned by the call to getUncovered - so
                 -- we need to unwrap the next Tagged in order to expose it to
                 -- being partitioned out in the recursive call to splitOnTag.
-                if getUncovered (PatchSet tagAndDeps ts) == [t]
+                if getUncovered (PatchSet ts tagAndDeps) == [t]
                     then let tagged = Tagged hp' Nothing ds' in
-                         return $ PatchSet NilRL (tagged :<: ts) :> nonDeps
+                         return $ PatchSet (ts :<: tagged) NilRL :> nonDeps
                     else do
-                        unfolded <- unwrapOneTagged $ PatchSet tagAndDeps ts
+                        unfolded <- unwrapOneTagged $ PatchSet ts tagAndDeps
                         xx :> yy <- splitOnTag t unfolded
-                        return $ xx :> (nonDeps +<+ yy)
+                        return $ xx :> (yy +<+ nonDeps)
             _ -> impossible
   where
     ds = getdeps (hopefully hp)
 -- We drop the leading patch, to try and find a non-Tagged tag.
-splitOnTag t (PatchSet (p :<: ps) ts) = do
-    ns :> x <- splitOnTag t (PatchSet ps ts)
-    return $ ns :> (p :<: x)
+splitOnTag t (PatchSet ts (ps :<: p)) = do
+    ns :> x <- splitOnTag t (PatchSet ts ps)
+    return $ ns :> (x :<: p)
 -- If there are no patches left, we "unfold" the next Tagged, and try again.
-splitOnTag t0 patchset@(PatchSet NilRL (Tagged _ _ _s :<: _)) =
+splitOnTag t0 patchset@(PatchSet (_ :<: Tagged _ _ _s) NilRL) =
     unwrapOneTagged patchset >>= splitOnTag t0
 -- If we've checked all the patches, but haven't found the tag, return Nothing.
 splitOnTag _ (PatchSet NilRL NilRL) = Nothing
 
 -- |'unwrapOneTagged' unfolds a single Tagged object in a PatchSet, adding the
 -- tag and patches to the PatchSet's patch list.
-unwrapOneTagged :: (Monad m) => PatchSet p wX wY -> m (PatchSet p wX wY)
-unwrapOneTagged (PatchSet ps (Tagged t _ tps :<: ts)) =
-    return $ PatchSet (ps +<+ t :<: tps) ts
+unwrapOneTagged :: (Monad m) => PatchSet rt p wX wY -> m (PatchSet rt p wX wY)
+unwrapOneTagged (PatchSet (ts :<: Tagged t _ tps) ps) =
+    return $ PatchSet ts (tps :<: t +<+ ps)
 unwrapOneTagged _ = fail "called unwrapOneTagged with no Tagged's in the set"
 
 -- | @getUncovered ps@ returns the 'PatchInfo' for all the patches in
@@ -207,11 +267,11 @@ unwrapOneTagged _ = fail "called unwrapOneTagged with no Tagged's in the set"
 --   Keep in mind that in a typical repository with a lot of tags, only a small
 --   fraction of tags would be returned as they would be at least indirectly
 --   depended on by the topmost ones.
-getUncovered :: PatchSet p wStart wX -> [PatchInfo]
+getUncovered :: PatchSet rt p wStart wX -> [PatchInfo]
 getUncovered patchset = case patchset of
-    (PatchSet ps NilRL) -> findUncovered (mapRL infoAndExplicitDeps ps)
-    (PatchSet ps (Tagged t _ _ :<: _)) ->
-        findUncovered (mapRL infoAndExplicitDeps (ps +<+ t :<: NilRL))
+    (PatchSet NilRL ps) -> findUncovered (mapRL infoAndExplicitDeps ps)
+    (PatchSet (_ :<: Tagged t _ _) ps) ->
+        findUncovered (mapRL infoAndExplicitDeps (NilRL :<: t +<+ ps))
   where
     findUncovered :: [(PatchInfo, Maybe [PatchInfo])] -> [PatchInfo]
     findUncovered [] = []
@@ -234,7 +294,7 @@ getUncovered patchset = case patchset of
 
     -- |infoAndExplicitDeps returns the patch info and (for tags only) the list
     -- of explicit dependencies of a patch.
-    infoAndExplicitDeps :: PatchInfoAnd p wX wY
+    infoAndExplicitDeps :: PatchInfoAnd rt p wX wY
                         -> (PatchInfo, Maybe [PatchInfo])
     infoAndExplicitDeps p
         | isTag (info p) = (info p, getdeps `fmap` hopefullyM p)
@@ -244,59 +304,46 @@ getUncovered patchset = case patchset of
 --   (see 'optimizePatchset') and only optimises at most one tag in
 --   there, going for the most recent tag which has no non-depended
 --   patch after it. Older tags won't be 'clean', which means the
---   PatchSet will not be in 'unclean :< clean' state.
-slightlyOptimizePatchset :: PatchSet p wStart wX -> PatchSet p wStart wX
+--   PatchSet will not be in 'clean :> unclean' state.
+slightlyOptimizePatchset :: PatchSet rt p wStart wX -> PatchSet rt p wStart wX
 slightlyOptimizePatchset (PatchSet ps0 ts0) =
     sops $ PatchSet (prog ps0) ts0
   where
     prog = progressRL "Optimizing inventory"
-    sops :: PatchSet p wStart wY -> PatchSet p wStart wY
-    sops patchset@(PatchSet NilRL _) = patchset
-    sops patchset@(PatchSet (hp :<: ps) ts)
+    sops :: PatchSet rt p wStart wY -> PatchSet rt p wStart wY
+    sops patchset@(PatchSet _ NilRL) = patchset
+    sops patchset@(PatchSet ts (ps :<: hp))
         | isTag (info hp) =
             if getUncovered patchset == [info hp]
                 -- exactly one tag and it depends on everything not already
                 -- archived
-                then PatchSet NilRL (Tagged hp Nothing ps :<: ts)
+                then PatchSet (ts :<: Tagged hp Nothing ps) NilRL
                 -- other tags or other top-level patches too (so move past hp)
-                else let ps' = sops $ PatchSet (prog ps) ts in
+                else let ps' = sops $ PatchSet ts (prog ps) in
                      appendPSFL ps' (hp :>: NilFL)
-        | otherwise = appendPSFL (sops $ PatchSet ps ts) (hp :>: NilFL)
+        | otherwise = appendPSFL (sops $ PatchSet ts ps) (hp :>: NilFL)
 
-commuteToEnd :: forall p wStart wX wY
-              . (Patchy p, NameHack p)
-             => RL (PatchInfoAnd p) wX wY
-               -> PatchSet p wStart wY
-               -> (PatchSet p :> RL (PatchInfoAnd p)) wStart wX
-commuteToEnd NilRL (PatchSet ps ts) = PatchSet NilRL ts :> ps
-commuteToEnd (p :<: ps) (PatchSet xs ts) | info p `elem` mapRL info xs =
-    case fastRemoveRL p xs of
-        Just xs' -> commuteToEnd ps (PatchSet xs' ts)
-        Nothing -> impossible -- "Nothing is impossible"
-commuteToEnd ps (PatchSet xs (Tagged t _ ys :<: ts)) =
-    commuteToEnd ps (PatchSet (xs +<+ t :<: ys) ts)
-commuteToEnd _ _ = impossible
+removeFromPatchSet :: Patchy p => FL (PatchInfoAnd rt p) wX wY
+                   -> PatchSet rt p wStart wY -> Maybe (PatchSet rt p wStart wX)
+removeFromPatchSet bad (PatchSet ts ps) | all (`elem` mapRL info ps) (mapFL info bad) = do
+    ps' <- fastRemoveSubsequenceRL (reverseFL bad) ps
+    return (PatchSet ts ps')
+removeFromPatchSet _ (PatchSet NilRL _) = Nothing
+removeFromPatchSet bad (PatchSet (ts :<: Tagged t _ tps) ps) =
+    removeFromPatchSet bad (PatchSet ts (tps :<: t +<+ ps))
 
-removeFromPatchSet :: (Patchy p, NameHack p) => FL (PatchInfoAnd p) wX wY
-                   -> PatchSet p wStart wY -> Maybe (PatchSet p wStart wX)
-removeFromPatchSet bad0 = rfns (reverseFL bad0)
-  where
-    rfns :: (Patchy p, NameHack p)
-         => RL (PatchInfoAnd p) wX wY -> PatchSet p wStart wY
-         -> Maybe (PatchSet p wStart wX)
-    rfns bad (PatchSet ps ts)
-        | all (`elem` mapRL info ps) (mapRL info bad) = do
-            ps' <- removeSubsequenceRL bad ps
-            Just $ PatchSet ps' ts
-    rfns _ (PatchSet _ NilRL) = Nothing
-    rfns bad (PatchSet ps (Tagged t _ tps :<: ts)) =
-        rfns bad (PatchSet (ps +<+ t :<: tps) ts)
+fastRemoveSubsequenceRL :: Patchy p
+                        => RL (PatchInfoAnd rt p) wY wZ
+                        -> RL (PatchInfoAnd rt p) wX wZ
+                        -> Maybe (RL (PatchInfoAnd rt p) wX wY)
+fastRemoveSubsequenceRL NilRL ys = Just ys
+fastRemoveSubsequenceRL (xs:<:x) ys = fastRemoveRL x ys >>= fastRemoveSubsequenceRL xs
 
-findCommonAndUncommon :: forall p wStart wX wY . (Patchy p, NameHack p)
-                      => PatchSet p wStart wX -> PatchSet p wStart wY
-                      -> Fork (PatchSet p)
-                              (FL (PatchInfoAnd p))
-                              (FL (PatchInfoAnd p)) wStart wX wY
+findCommonAndUncommon :: forall rt p wStart wX wY . Patchy p
+                      => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
+                      -> Fork (PatchSet rt p)
+                              (FL (PatchInfoAnd rt p))
+                              (FL (PatchInfoAnd rt p)) wStart wX wY
 findCommonAndUncommon us them = case taggedIntersection us them of
     Fork common us' them' ->
         case partitionFL (infoIn them') $ reverseRL us' of
@@ -311,15 +358,15 @@ findCommonAndUncommon us them = case taggedIntersection us them of
                             ++ renderString Encode (vcat $
                                 mapRL (showPatchInfoUI . info) $ reverseFL bad)
                     _ :> NilFL :> only_theirs ->
-                        Fork (PatchSet (reverseFL common2) common)
+                        Fork (PatchSet common (reverseFL common2))
                             only_ours (unsafeCoercePStart only_theirs)
   where
     infoIn inWhat = (`elem` mapRL info inWhat) . info
 
-findCommonWithThem :: (Patchy p, NameHack p)
-                   => PatchSet p wStart wX
-                   -> PatchSet p wStart wY
-                   -> (PatchSet p :> FL (PatchInfoAnd p)) wStart wX
+findCommonWithThem :: Patchy p
+                   => PatchSet rt p wStart wX
+                   -> PatchSet rt p wStart wY
+                   -> (PatchSet rt p :> FL (PatchInfoAnd rt p)) wStart wX
 findCommonWithThem us them = case taggedIntersection us them of
     Fork common us' them' ->
         case partitionFL ((`elem` mapRL info them') . info) $ reverseRL us' of
@@ -328,19 +375,19 @@ findCommonWithThem us them = case taggedIntersection us them of
                       ++ renderString Encode
                           (vcat $ mapRL (showPatchInfoUI . info) $ reverseFL bad)
             common2 :> _nilfl :> only_ours ->
-                PatchSet (reverseFL common2) common :> unsafeCoerceP only_ours
+                PatchSet common (reverseFL common2) :> unsafeCoerceP only_ours
 
-findUncommon :: (Patchy p, NameHack p)
-             => PatchSet p wStart wX -> PatchSet p wStart wY
-             -> (FL (PatchInfoAnd p) :\/: FL (PatchInfoAnd p)) wX wY
+findUncommon :: Patchy p
+             => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
+             -> (FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wX wY
 findUncommon us them =
     case findCommonWithThem us them of
         _common :> us' -> case findCommonWithThem them us of
             _ :> them' -> unsafeCoercePStart us' :\/: them'
 
-countUsThem :: (Patchy p, NameHack p)
-            => PatchSet p wStart wX
-            -> PatchSet p wStart wY
+countUsThem :: Patchy p
+            => PatchSet rt p wStart wX
+            -> PatchSet rt p wStart wY
             -> (Int, Int)
 countUsThem us them =
     case taggedIntersection us them of
@@ -348,18 +395,18 @@ countUsThem us them =
                                 tt = mapRL info them' in
                             (length $ uu \\ tt, length $ tt \\ uu)
 
-mergeThem :: (Patchy p, Merge p, NameHack p)
-          => PatchSet p wStart wX -> PatchSet p wStart wY
-          -> Sealed (FL (PatchInfoAnd p) wX)
+mergeThem :: (Patchy p, Merge p)
+          => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
+          -> Sealed (FL (PatchInfoAnd rt p) wX)
 mergeThem us them =
     case taggedIntersection us them of
         Fork _ us' them' ->
             case merge2FL (reverseRL us') (reverseRL them') of
                them'' :/\: _ -> Sealed them''
 
-newsetIntersection :: (Patchy p, NameHack p)
-                   => [SealedPatchSet p wStart]
-                   -> SealedPatchSet p wStart
+newsetIntersection :: Patchy p
+                   => [SealedPatchSet rt p wStart]
+                   -> SealedPatchSet rt p wStart
 newsetIntersection [] = seal $ PatchSet NilRL NilRL
 newsetIntersection [x] = x
 newsetIntersection (Sealed y : ys) =
@@ -368,38 +415,35 @@ newsetIntersection (Sealed y : ys) =
             Fork common a b -> case mapRL info a `intersect` mapRL info b of
                 morecommon ->
                     case partitionRL (\e -> info e `notElem` morecommon) a of
-                        commonps :> _ -> seal $ PatchSet commonps common
+                        commonps :> _ -> seal $ PatchSet common commonps
 
-newsetUnion :: (Patchy p, Merge p, NameHack p)
-            => [SealedPatchSet p wStart]
-            -> SealedPatchSet p wStart
+newsetUnion :: (Patchy p, Merge p)
+            => [SealedPatchSet rt p wStart]
+            -> SealedPatchSet rt p wStart
 newsetUnion [] = seal $ PatchSet NilRL NilRL
 newsetUnion [x] = x
-newsetUnion (Sealed y@(PatchSet psy tsy) : Sealed y2 : ys) =
+newsetUnion (Sealed y@(PatchSet tsy psy) : Sealed y2 : ys) =
     case mergeThem y y2 of
         Sealed p2 ->
-            newsetUnion $ seal (PatchSet (reverseFL p2 +<+ psy) tsy) : ys
+            newsetUnion $ seal (PatchSet tsy (psy +<+ reverseFL p2)) : ys
 
 -- | Merge two FLs (say L and R), starting in a common context. The result is a
 -- FL starting in the original end context of L, going to a new context that is
 -- the result of applying all patches from R on top of patches from L.
 --
--- While this function is similar to 'mergeFL', there are three important
+-- While this function is similar to 'mergeFL', there are some important
 -- differences to keep in mind:
 --
 -- * 'mergeFL' does not correctly deal with duplicate patches whereas this one
 --   does
---   (Question from Eric Kow: in what sense? Why not fix the mergeFL instance?)
---
--- * The conventional order we use in this function is reversed from
---   'mergeFL' (so @mergeFL r l@ vs. @merge2FL l r@. This does not
---   matter so much for the former since you get both paths.
---   (Question from Eric Kow: should we flip merge2FL for more uniformity in
---    the code?)
-merge2FL :: (Patchy p, Merge p, NameHack p)
-         => FL (PatchInfoAnd p) wX wY
-         -> FL (PatchInfoAnd p) wX wZ
-         -> (FL (PatchInfoAnd p) :/\: FL (PatchInfoAnd p)) wY wZ
+--   (Question from Eric Kow: in what sense? Why not fix 'mergeFL'?)
+--   (bf: I guess what was meant here is that 'merge2FL' works in the
+--    the way it does because it considers patch meta data whereas
+--    'mergeFL' cannot since it must work for primitive patches, too.
+merge2FL :: (Patchy p, Merge p)
+         => FL (PatchInfoAnd rt p) wX wY
+         -> FL (PatchInfoAnd rt p) wX wZ
+         -> (FL (PatchInfoAnd rt p) :/\: FL (PatchInfoAnd rt p)) wY wZ
 merge2FL xs NilFL = NilFL :/\: xs
 merge2FL NilFL ys = ys :/\: NilFL
 merge2FL xs (y :>: ys) | Just xs' <- fastRemoveFL y xs = merge2FL xs' ys
@@ -410,14 +454,14 @@ merge2FL (x :>: xs) ys | Just ys' <- fastRemoveFL x ys = merge2FL xs ys'
                                                 ys'' :/\: xs' ->
                                                    ys'' :/\: (x' :>: xs')
 
-areUnrelatedRepos :: (Patchy p, NameHack p)
-                  => PatchSet p wStart wX
-                  -> PatchSet p wStart wY -> Bool
+areUnrelatedRepos :: Patchy p
+                  => PatchSet rt p wStart wX
+                  -> PatchSet rt p wStart wY -> Bool
 areUnrelatedRepos us them =
     case taggedIntersection us them of
         Fork c u t -> checkit c u t
   where
-    checkit (Tagged{} :<: _) _ _ = False
+    checkit (_ :<: Tagged{}) _ _ = False
     checkit _ u t | t `isShorterThanRL` 5 = False
                   | u `isShorterThanRL` 5 = False
                   | otherwise = null $ intersect (mapRL info u) (mapRL info t)
@@ -427,9 +471,21 @@ areUnrelatedRepos us them =
 -- in the sequence at all or any commutation fails, we get Nothing. First two
 -- cases are optimisations for the common cases where the head of the list is
 -- the patch to remove, or the patch is not there at all.
-fastRemoveFL :: (Patchy p, NameHack p)
-             => PatchInfoAnd p wX wY
-             -> FL (PatchInfoAnd p) wX wZ -> Maybe (FL (PatchInfoAnd p) wY wZ)
+--
+-- A note on the witness types: the patch to be removed is typed as if it had
+-- to be the first in the list, since it has the same pre-context as the list.
+-- The types fit together (internally, in this module) because we commute the
+-- patch to the front before removing it and commutation inside a sequence does
+-- not change the sequence's contexts.
+--
+-- However, the use sites outside this module are something different. We
+-- usually need coercions to get the patch(es) to be removed in shape. This is
+-- not very nice but probably unavoidable given the approximative nature of
+-- context witnesses.
+fastRemoveFL :: Patchy p
+             => PatchInfoAnd rt p wX wY -- this type assumes element is at the front
+             -> FL (PatchInfoAnd rt p) wX wZ
+             -> Maybe (FL (PatchInfoAnd rt p) wY wZ)
 fastRemoveFL _ NilFL = Nothing
 fastRemoveFL a (b :>: bs) | IsEq <- a =\/= b = Just bs
                           | info a `notElem` mapFL info bs = Nothing
@@ -440,38 +496,37 @@ fastRemoveFL a (b :>: bs) = do
     Just (b' :>: bs')
   where
     i = info a
-    pullout :: (Patchy p, NameHack p)
-            => RL (PatchInfoAnd p) wA0 wA
-            -> FL (PatchInfoAnd p) wA wB
-            -> Maybe ((PatchInfoAnd p :> FL (PatchInfoAnd p)) wA0 wB)
+    pullout :: Patchy p
+            => RL (PatchInfoAnd rt p) wA wB
+            -> FL (PatchInfoAnd rt p) wB wC
+            -> Maybe ((PatchInfoAnd rt p :> FL (PatchInfoAnd rt p)) wA wC)
     pullout _ NilFL = Nothing
     pullout acc (x :>: xs)
         | info x == i = do x' :> acc' <- commuteRL (acc :> x)
                            Just (x' :> reverseRL acc' +>+ xs)
-        | otherwise = pullout (x :<: acc) xs
+        | otherwise = pullout (acc :<: x) xs
 
-fastRemoveRL :: (Patchy p, NameHack p)
-             => PatchInfoAnd p wY wZ
-             -> RL (PatchInfoAnd p) wX wZ -> Maybe (RL (PatchInfoAnd p) wX wY)
+-- | Same as 'fastRemoveFL' only for 'RL'.
+fastRemoveRL :: Patchy p
+             => PatchInfoAnd rt p wY wZ -- this type assumes element is at the back
+             -> RL (PatchInfoAnd rt p) wX wZ
+             -> Maybe (RL (PatchInfoAnd rt p) wX wY)
 fastRemoveRL _ NilRL = Nothing
-fastRemoveRL a (b :<: bs) | IsEq <- a =/\= b = Just bs
+fastRemoveRL a (bs :<: b) | IsEq <- b =/\= a = Just bs
                           | info a `notElem` mapRL info bs = Nothing
-fastRemoveRL a (b :<: bs) = do
-    bs' :> a' <- pullout NilFL bs
+fastRemoveRL a (bs :<: b) = do
+    bs' :> a' <- pullout bs NilFL
     b' :> a'' <- commute (a' :> b)
     IsEq <- return (a'' =/\= a)
-    Just (b' :<: bs')
+    Just (bs' :<: b')
   where
     i = info a
-    pullout :: (Patchy p, NameHack p)
-            => FL (PatchInfoAnd p) wB wC
-            -> RL (PatchInfoAnd p) wA wB
-            -> Maybe ((RL (PatchInfoAnd p) :> PatchInfoAnd p) wA wC)
-    pullout _ NilRL = Nothing
-    pullout acc (x :<: xs)
-        | info x == i = do
-            acc' :> x' <- either (const Nothing)
-                                 Just
-                                 (commuteFLorComplain (x :> acc))
-            Just (reverseFL acc' +<+ xs :> x')
-        | otherwise = pullout (x :>: acc) xs
+    pullout :: Patchy p
+            => RL (PatchInfoAnd rt p) wA wB
+            -> FL (PatchInfoAnd rt p) wB wC
+            -> Maybe ((RL (PatchInfoAnd rt p) :> PatchInfoAnd rt p) wA wC)
+    pullout NilRL _ = Nothing
+    pullout (xs :<: x) acc
+        | info x == i = do acc' :> x' <- commuteFL (x :> acc)
+                           Just (xs +<+ reverseFL acc' :> x')
+        | otherwise = pullout xs (x :>: acc)
