@@ -3,6 +3,9 @@ module Darcs.Patch.FileHunk
     )
     where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Util.Path ( FileName )
 import Darcs.Patch.Format ( FileNameFormat )
 import Darcs.Patch.Show ( formatFileName )
