@@ -3,6 +3,9 @@ module Darcs.Patch.Invert
        )
        where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Witnesses.Ordered ( FL(..), RL(..), reverseFL, reverseRL )
 
 
@@ -11,11 +14,11 @@ class Invert p where
 
 invertFL :: Invert p => FL p wX wY -> RL p wY wX
 invertFL NilFL = NilRL
-invertFL (x:>:xs) = invert x :<: invertFL xs
+invertFL (x:>:xs) = invertFL xs :<: invert x
 
 invertRL :: Invert p => RL p wX wY -> FL p wY wX
 invertRL NilRL = NilFL
-invertRL (x:<:xs) = invert x :>: invertRL xs
+invertRL (xs:<:x) = invert x :>: invertRL xs
 
 instance Invert p => Invert (FL p) where
     invert = reverseRL . invertFL
