@@ -4,10 +4,8 @@
 
 module Darcs.Patch.Matchable ( Matchable ) where
 
-import Darcs.Patch.MaybeInternal ( MaybeInternal )
 import Darcs.Patch.Inspect ( PatchInspect )
 import Darcs.Patch.Patchy ( Patchy )
-import Darcs.Patch.Rebase.NameHack ( NameHack )
 
-class (Patchy p, PatchInspect p, MaybeInternal p, NameHack p)
+class (Patchy p, PatchInspect p)
     => Matchable p
