@@ -22,6 +22,9 @@
 
 module Darcs.Patch.OldDate ( readUTCDate, showIsoDateTime ) where
 
+import Prelude ( (^) )
+import Darcs.Prelude
+
 import Text.ParserCombinators.Parsec
 import System.Time
 import Data.Char ( toUpper, isDigit )
