@@ -21,11 +21,14 @@
 module Darcs.Patch.PatchInfoAnd ( Hopefully(..), SimpleHopefully(..), PatchInfoAnd(..),
                          WPatchInfo, unWPatchInfo, compareWPatchInfo,
                          piap, n2pia, patchInfoAndPatch,
-                         fmapPIAP, fmapFLPIAP,
+                         fmapFLPIAP, generaliseRepoTypePIAP,
                          conscientiously, hopefully, info, winfo,
                          hopefullyM, createHashed, extractHash,
                          actually, unavailable, patchDesc ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import System.IO.Unsafe ( unsafeInterleaveIO )
 
 import Darcs.Util.SignalHandler ( catchNonSignal )
@@ -33,19 +36,21 @@ import Darcs.Util.Printer
     ( Doc, renderString, errorDoc, text, ($$), vcat
     , RenderMode(..) )
 import Darcs.Patch.Info ( PatchInfo, showPatchInfoUI, justName )
-import Darcs.Patch ( Named, patch2patchinfo )
 import Darcs.Patch.Conflict ( Conflict, CommuteNoConflicts )
 import Darcs.Patch.Debug ( PatchDebug(..) )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Merge ( Merge(..) )
-import Darcs.Patch.Named ( fmapNamed, fmapFL_Named )
-import Darcs.Patch.Prim ( PrimPatchBase(..) )
+import Darcs.Patch.Named.Wrapped
+    ( WrappedNamed, patch2patchinfo, fmapFL_WrappedNamed, (:~:), (:~~:)
+    , generaliseRepoTypeWrapped
+    )
+import Darcs.Patch.Prim ( PrimPatchBase(..), FromPrim )
 import Darcs.Patch.Patchy ( Patchy, ReadPatch(..), Apply(..), Invert(..),
                             ShowPatch(..), Commute(..), PatchInspect(..) )
-import Darcs.Patch.Rebase.NameHack ( NameHack )
 import Darcs.Patch.Repair ( Repair(..), RepairToFL )
+import Darcs.Patch.RepoType ( RepoType(..), IsRepoType, RebaseTypeOf, RebaseType(..) )
 import Darcs.Patch.Show ( ShowPatchBasic(..) )
 import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
@@ -53,9 +58,7 @@ import Darcs.Patch.Witnesses.Ordered ( (:>)(..), (:\/:)(..), (:/\:)(..), FL, map
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, mapSeal )
 import Darcs.Patch.Witnesses.Show ( Show1(..), Show2(..), ShowDict(ShowDictClass) )
 import Darcs.Util.Exception ( prettyException )
-import Storage.Hashed.Tree( Tree )
-
-import Control.Applicative ( (<$>) )
+import Darcs.Util.Tree( Tree )
 
 -- | @'Hopefully' p C@ @(x y)@ is @'Either' String (p C@ @(x y))@ in a
 -- form adapted to darcs patches. The @C@ @(x y)@ represents the type
@@ -76,17 +79,17 @@ data SimpleHopefully a wX wY = Actually (a wX wY) | Unavailable String
 -- | @'PatchInfoAnd' p wA wB@ represents a hope we have to get a
 -- patch through its info. We're not sure we have the patch, but we
 -- know its info.
-data PatchInfoAnd p wA wB = PIAP !PatchInfo (Hopefully (Named p) wA wB)
+data PatchInfoAnd rt p wA wB = PIAP !PatchInfo (Hopefully (WrappedNamed rt p) wA wB)
     deriving Show
 
-instance Show2 p => Show1 (PatchInfoAnd p wX) where
+instance Show2 p => Show1 (PatchInfoAnd rt p wX) where
     showDict1 = ShowDictClass
 
-instance Show2 p => Show2 (PatchInfoAnd p) where
+instance Show2 p => Show2 (PatchInfoAnd rt p) where
     showDict2 = ShowDictClass
 
-instance PrimPatchBase p => PrimPatchBase (PatchInfoAnd p) where
-   type PrimOf (PatchInfoAnd p) = PrimOf p
+instance PrimPatchBase p => PrimPatchBase (PatchInfoAnd rt p) where
+   type PrimOf (PatchInfoAnd rt p) = PrimOf p
 
 -- | @'WPatchInfo' wA wB@ represents the info of a patch, marked with
 -- the patch's witnesses.
@@ -94,7 +97,7 @@ newtype WPatchInfo wA wB = WPatchInfo { unWPatchInfo :: PatchInfo }
 
 -- This is actually unsafe if we ever commute patches and then compare them
 -- using this function. TODO: consider adding an extra existential to WPatchInfo
--- (as with TaggedPatch in Darcs.Patch.Choices)
+-- (as with LabelledPatch in Darcs.Patch.Choices)
 compareWPatchInfo :: WPatchInfo wA wB -> WPatchInfo wC wD -> EqCheck (wA, wB) (wC, wD)
 compareWPatchInfo (WPatchInfo x) (WPatchInfo y) = if x == y then unsafeCoerceP IsEq else NotEq
 
@@ -109,44 +112,51 @@ fmapH f (Hashed h sh) = Hashed h (ff sh)
     where ff (Actually a) = Actually (f a)
           ff (Unavailable e) = Unavailable e
 
-info :: PatchInfoAnd p wA wB -> PatchInfo
+info :: PatchInfoAnd rt p wA wB -> PatchInfo
 info (PIAP i _) = i
 
-patchDesc :: forall p wX wY . PatchInfoAnd p wX wY -> String
+patchDesc :: forall rt p wX wY . PatchInfoAnd rt p wX wY -> String
 patchDesc p = justName $ info p
 
-winfo :: PatchInfoAnd p wA wB -> WPatchInfo wA wB
+winfo :: PatchInfoAnd rt p wA wB -> WPatchInfo wA wB
 winfo (PIAP i _) = WPatchInfo i
 
 -- | @'piap' i p@ creates a PatchInfoAnd containing p with info i.
-piap :: PatchInfo -> Named p wA wB -> PatchInfoAnd p wA wB
+piap :: PatchInfo -> WrappedNamed rt p wA wB -> PatchInfoAnd rt p wA wB
 piap i p = PIAP i (Hopefully $ Actually p)
 
 -- | @n2pia@ creates a PatchInfoAnd representing a @Named@ patch.
-n2pia :: Named p wX wY -> PatchInfoAnd p wX wY
+n2pia :: WrappedNamed rt p wX wY -> PatchInfoAnd rt p wX wY
 n2pia x = patch2patchinfo x `piap` x
 
-patchInfoAndPatch :: PatchInfo -> Hopefully (Named p) wA wB -> PatchInfoAnd p wA wB
+patchInfoAndPatch :: PatchInfo -> Hopefully (WrappedNamed rt p) wA wB -> PatchInfoAnd rt p wA wB
 patchInfoAndPatch =  PIAP
 
-fmapPIAP :: (forall wA wB . p wA wB -> q wA wB) -> PatchInfoAnd p wX wY -> PatchInfoAnd q wX wY
-fmapPIAP f (PIAP i hp) = PIAP i (fmapH (fmapNamed f) hp)
+fmapFLPIAP
+  :: (FL p wX wY -> FL q wX wY)
+  -> (RebaseTypeOf rt :~~: 'IsRebase -> p :~: q)
+  -> PatchInfoAnd rt p wX wY
+  -> PatchInfoAnd rt q wX wY
+fmapFLPIAP f whenRebase (PIAP i hp)
+  = PIAP i (fmapH (fmapFL_WrappedNamed f whenRebase) hp)
 
-fmapFLPIAP :: (FL p wX wY -> FL q wX wY) -> PatchInfoAnd p wX wY -> PatchInfoAnd q wX wY
-fmapFLPIAP f (PIAP i hp) = PIAP i (fmapH (fmapFL_Named f) hp)
+generaliseRepoTypePIAP
+    :: PatchInfoAnd ('RepoType 'NoRebase) p wA wB
+    -> PatchInfoAnd rt p wA wB
+generaliseRepoTypePIAP (PIAP i hp) = PIAP i (fmapH generaliseRepoTypeWrapped hp)
 
 -- | @'hopefully' hp@ tries to get a patch from a 'PatchInfoAnd'
 -- value. If it fails, it outputs an error \"failed to read patch:
 -- \<description of the patch>\". We get the description of the patch
 -- from the info part of 'hp'
-hopefully :: PatchInfoAnd p wA wB -> Named p wA wB
+hopefully :: PatchInfoAnd rt p wA wB -> WrappedNamed rt p wA wB
 hopefully = conscientiously $ \e -> text "failed to read patch:" $$ e
 
 -- | @'conscientiously' er hp@ tries to extract a patch from a 'PatchInfoAnd'.
 -- If it fails, it applies the error handling function @er@ to a description
 -- of the patch info component of @hp@.
 conscientiously :: (Doc -> Doc)
-                -> PatchInfoAnd p wA wB -> Named p wA wB
+                -> PatchInfoAnd rt p wA wB -> WrappedNamed rt p wA wB
 conscientiously er (PIAP pinf hp) =
     case hopefully2either hp of
       Right p -> p
@@ -154,7 +164,7 @@ conscientiously er (PIAP pinf hp) =
 
 -- | @hopefullyM@ is a version of @hopefully@ which calls @fail@ in a
 -- monad instead of erroring.
-hopefullyM :: Monad m => PatchInfoAnd p wA wB -> m (Named p wA wB)
+hopefullyM :: Monad m => PatchInfoAnd rt p wA wB -> m (WrappedNamed rt p wA wB)
 hopefullyM (PIAP pinf hp) = case hopefully2either hp of
                               Right p -> return p
                               Left e -> fail $ renderString Encode
@@ -177,7 +187,7 @@ createHashed h f = mapSeal (Hashed h) `fmap` unsafeInterleaveIO (f' `catchNonSig
           return (Sealed (Actually x))
   handler e = return $ seal $ Unavailable $ prettyException e
 
-extractHash :: PatchInfoAnd p wA wB -> Either (Named p wA wB) String
+extractHash :: PatchInfoAnd rt p wA wB -> Either (WrappedNamed rt p wA wB) String
 extractHash (PIAP _ (Hashed s _)) = Right s
 extractHash hp = Left $ conscientiously (\e -> text "unable to read patch:" $$ e) hp
 
@@ -187,21 +197,21 @@ unavailable = Hopefully . Unavailable
 -- Equality on PatchInfoAnd is solely determined by the PatchInfo
 -- It is a global invariant of darcs that once a patch is recorded,
 -- it should always have the same representation in the same context.
-instance MyEq (PatchInfoAnd p) where
+instance MyEq (PatchInfoAnd rt p) where
     unsafeCompare (PIAP i _) (PIAP i2 _) = i == i2
 
-instance Invert p => Invert (PatchInfoAnd p) where
+instance Invert p => Invert (PatchInfoAnd rt p) where
     invert (PIAP i p) = PIAP i (invert `fmapH` p)
 
-instance PatchListFormat (PatchInfoAnd p)
+instance PatchListFormat (PatchInfoAnd rt p)
 
-instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (PatchInfoAnd p) where
+instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (PatchInfoAnd rt p) where
     showPatch (PIAP n p) = case hopefully2either p of
                            Right x -> showPatch x
                            Left _ -> showPatchInfoUI n
 
 instance (Apply p, Conflict p, CommuteNoConflicts p, IsHunk p, PatchListFormat p, PrimPatchBase p,
-          ShowPatch p, ApplyState p ~ Tree) => ShowPatch (PatchInfoAnd p) where
+          ShowPatch p, ApplyState p ~ Tree) => ShowPatch (PatchInfoAnd rt p) where
     showContextPatch (PIAP n p) = case hopefully2either p of
                                     Right x -> showContextPatch x
                                     Left _ -> return $ showPatchInfoUI n
@@ -214,39 +224,42 @@ instance (Apply p, Conflict p, CommuteNoConflicts p, IsHunk p, PatchListFormat p
                             Right x -> showNicely x
                             Left _ -> showPatchInfoUI n
 
-instance (Commute p, NameHack p) => Commute (PatchInfoAnd p) where
+instance Commute p => Commute (PatchInfoAnd rt p) where
     commute (x :> y) = do y' :> x' <- commute (hopefully x :> hopefully y)
                           return $ (info y `piap` y') :> (info x `piap` x')
 
-instance (Merge p, NameHack p) => Merge (PatchInfoAnd p) where
+instance Merge p => Merge (PatchInfoAnd rt p) where
     merge (x :\/: y) = case merge (hopefully x :\/: hopefully y) of
                        y' :/\: x' -> (info y `piap` y') :/\: (info x `piap` x')
 
-instance PatchInspect p => PatchInspect (PatchInfoAnd p) where
+instance PatchInspect p => PatchInspect (PatchInfoAnd rt p) where
     listTouchedFiles = listTouchedFiles . hopefully
-    hunkMatches _ _ = error "hunkmatches not implemented for PatchInfoAnd"
+    hunkMatches f = hunkMatches f . hopefully
 
-instance Apply p => Apply (PatchInfoAnd p) where
-    type ApplyState (PatchInfoAnd p) = ApplyState p
+instance Apply p => Apply (PatchInfoAnd rt p) where
+    type ApplyState (PatchInfoAnd rt p) = ApplyState p
     apply p = apply $ hopefully p
 
-instance RepairToFL p => Repair (PatchInfoAnd p) where
+instance RepairToFL p => Repair (PatchInfoAnd rt p) where
     applyAndTryToFix p = do mp' <- applyAndTryToFix $ hopefully p
                             case mp' of
                               Nothing -> return Nothing
                               Just (e,p') -> return $ Just (e, n2pia p')
 
-instance (ReadPatch p, PatchListFormat p) => ReadPatch (PatchInfoAnd p) where
+instance ( ReadPatch p, PatchListFormat p, PrimPatchBase p, Effect p, FromPrim p
+         , IsRepoType rt
+         ) => ReadPatch (PatchInfoAnd rt p) where
+
     readPatch' = mapSeal n2pia <$> readPatch'
 
-instance Effect p => Effect (PatchInfoAnd p) where
+instance Effect p => Effect (PatchInfoAnd rt p) where
     effect = effect . hopefully
     effectRL = effectRL . hopefully
 
-instance IsHunk (PatchInfoAnd p) where
+instance IsHunk (PatchInfoAnd rt p) where
     isHunk _ = Nothing
 
-instance PatchDebug p => PatchDebug (PatchInfoAnd p)
+instance PatchDebug p => PatchDebug (PatchInfoAnd rt p)
 
-instance (Patchy p, NameHack p, ApplyState p ~ Tree) => Patchy (PatchInfoAnd p)
+instance (Patchy p, ApplyState p ~ Tree) => Patchy (PatchInfoAnd rt p)
 
