@@ -17,7 +17,6 @@
 
 {-# LANGUAGE CPP #-}
 
-#include "gadts.h"
 
 module Darcs.Patch.Patchy
      ( Patchy,
@@ -31,8 +30,7 @@ import Darcs.Patch.Commute ( Commute(..) )
 import Darcs.Patch.Invert ( Invert(..) )
 import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.Read ( ReadPatch(..) )
-import Darcs.Patch.Show ( showPatch, ShowPatch(..), ShowPatchBasic )
-import Darcs.Witnesses.Eq ( MyEq )
+import Darcs.Patch.Show ( showPatch, ShowPatch(..) )
 
-class (MyEq p, Apply p, Commute p, PatchInspect p, ShowPatch p, ReadPatch p, Invert p) => Patchy p
+class (Apply p, Commute p, Invert p) => Patchy p
 
