@@ -1,6 +1,9 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.V1 ( Prim ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Prim.V1.Apply ()
 import Darcs.Patch.Prim.V1.Coalesce ()
 import Darcs.Patch.Prim.V1.Commute ()
