@@ -1,7 +1,10 @@
-{-# OPTIONS_GHC -fno-warn-orphans -fno-warn-incomplete-patterns #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE CPP #-}
 module Darcs.Patch.Prim.V1.Apply () where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.Repair ( RepairToFL(..) )
 
@@ -15,8 +18,8 @@ import Darcs.Util.Path ( fn2fp )
 import Darcs.Patch.Format ( FileNameFormat(..) )
 import Darcs.Patch.TokenReplace ( tryTokInternal )
 
-import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Patch.ApplyMonad ( ApplyMonad(..), ApplyMonadTree(..) )
+import Darcs.Util.Tree( Tree )
 
 import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL_FL, spanFL, (:>)(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoercePStart )
@@ -95,7 +98,7 @@ instance PrimApply Prim where
                   applyPrimFL ps'
         where f_hunk (FP f' (Hunk{})) | f == f' = True
               f_hunk _ = False
-              hunkmod :: ApplyMonad m Tree => FL FilePatchType wX wY
+              hunkmod :: ApplyMonad Tree m => FL FilePatchType wX wY
                       -> B.ByteString -> m B.ByteString
               hunkmod NilFL ps = return ps
               hunkmod (Hunk line old new:>:hs) ps
