@@ -6,20 +6,23 @@ module Darcs.Patch.Prim.V1.Commute
     )
     where
 
-import Prelude hiding ( pi )
+import Prelude ()
+import Darcs.Prelude
+
+import Prelude hiding ( pi, Applicative(..) )
 import Control.Monad ( MonadPlus, msum, mzero, mplus )
-import Control.Applicative ( Applicative(..), Alternative(..) )
+import Control.Applicative ( Alternative(..) )
 
 import qualified Data.ByteString as B (ByteString, concat)
 import qualified Data.ByteString.Char8 as BC (pack)
 
 import Darcs.Util.Path ( FileName, fn2fp, movedirfilename )
-import Darcs.Patch.Witnesses.Ordered ( (:<)(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Ordered ( (:>)(..) )
 import Darcs.Patch.Prim.V1.Core
      ( Prim(..), FilePatchType(..) )
 import Darcs.Patch.Invert ( Invert(..) )
-import Darcs.Patch.Commute ( Commute(..), toFwdCommute, toRevCommute )
+import Darcs.Patch.Commute ( Commute(..) )
 import Darcs.Patch.Permutations () -- for Invert instance of FL
 import Darcs.Patch.TokenReplace ( tryTokInternal )
 
@@ -75,12 +78,12 @@ toPerhaps (Just x) = Succeeded x
 toPerhaps Nothing = Failed
 
 cleverCommute :: CommuteFunction -> CommuteFunction
-cleverCommute c (p1:<p2) =
-    case c (p1 :< p2) of
+cleverCommute c (p1:>p2) =
+    case c (p1 :> p2) of
     Succeeded x -> Succeeded x
     Failed -> Failed
-    Unknown -> case c (invert p2 :< invert p1) of
-               Succeeded (p1' :< p2') -> Succeeded (invert p2' :< invert p1')
+    Unknown -> case c (invert p2 :> invert p1) of
+               Succeeded (p1' :> p2') -> Succeeded (invert p2' :> invert p1')
                Failed -> Failed
                Unknown -> Unknown
 --cleverCommute c (p1,p2) = c (p1,p2) `mplus`
@@ -91,17 +94,17 @@ cleverCommute c (p1:<p2) =
 
 speedyCommute :: CommuteFunction  -- Deal with common cases quickly!
     -- Two file-patches modifying different files trivially commute.
-speedyCommute (p2@(FP f' _) :< p1@(FP f _))
-  | f /= f' = Succeeded (unsafeCoerceP p1 :< unsafeCoerceP p2)
+speedyCommute (p1@(FP f1 _) :> p2@(FP f2 _))
+  | f1 /= f2 = Succeeded (unsafeCoerceP p2 :> unsafeCoerceP p1)
 speedyCommute _other = Unknown
 
 everythingElseCommute :: CommuteFunction
 everythingElseCommute = eec
-    where
+  where
     eec :: CommuteFunction
-    eec (ChangePref p f t :<p1) = Succeeded (unsafeCoerceP p1 :< ChangePref p f t)
-    eec (p2 :<ChangePref p f t) = Succeeded (ChangePref p f t :< unsafeCoerceP p2)
-    eec xx = cleverCommute commuteFiledir                xx
+    eec (p1 :> ChangePref p f t) = Succeeded (ChangePref p f t :> unsafeCoerceP p1)
+    eec (ChangePref p f t :> p2) = Succeeded (unsafeCoerceP p2 :> ChangePref p f t)
+    eec xx = cleverCommute commuteFiledir xx
 
 {-
 Note that it must be true that
@@ -115,8 +118,8 @@ then commutex (B^-1, A^-1) == Just (A'^-1, B'^-1)
 -}
 
 instance Commute Prim where
-    commute x = toMaybe $ msum [toFwdCommute speedyCommute x,
-                                toFwdCommute everythingElseCommute x
+    commute x = toMaybe $ msum [speedyCommute x,
+                                everythingElseCommute x
                                ]
 
 isSuperdir :: FileName -> FileName -> Bool
@@ -125,40 +128,40 @@ isSuperdir d1 d2 = isd (fn2fp d1) (fn2fp d2)
               length s2 >= length s1 + 1 && take (length s1 + 1) s2 == s1 ++ "/"
 
 commuteFiledir :: CommuteFunction
-commuteFiledir (FP f1 p1 :< FP f2 p2) =
-  if f1 /= f2 then Succeeded ( FP f2 (unsafeCoerceP p2) :< FP f1 (unsafeCoerceP p1) )
-  else commuteFP f1 (p1 :< p2)
-commuteFiledir (DP d1 p1 :< DP d2 p2) =
+commuteFiledir (FP f1 p1 :> FP f2 p2) =
+  if f1 /= f2 then Succeeded ( FP f2 (unsafeCoerceP p2) :> FP f1 (unsafeCoerceP p1) )
+  else commuteFP f1 (p1 :> p2)
+commuteFiledir (DP d1 p1 :> DP d2 p2) =
   if not (isInDirectory d1 d2 || isInDirectory d2 d1) && d1 /= d2
-  then Succeeded ( DP d2 (unsafeCoerceP p2) :< DP d1 (unsafeCoerceP p1) )
+  then Succeeded ( DP d2 (unsafeCoerceP p2) :> DP d1 (unsafeCoerceP p1) )
   else Failed
-commuteFiledir (DP d dp :< FP f fp) =
+commuteFiledir (FP f fp :> DP d dp) =
     if not $ isInDirectory d f
-    then Succeeded (FP f (unsafeCoerceP fp) :< DP d (unsafeCoerceP dp))
+    then Succeeded (DP d (unsafeCoerceP dp) :> FP f (unsafeCoerceP fp))
     else Failed
 
-commuteFiledir (Move d d' :< FP f2 p2)
-    | f2 == d' = Failed
-    | (p2 == AddFile || p2 == RmFile) && d == f2 = Failed
-    | otherwise = Succeeded (FP (movedirfilename d d' f2) (unsafeCoerceP p2) :< Move d d')
-commuteFiledir (Move d d' :< DP d2 p2)
-    | isSuperdir d2 d' || isSuperdir d2 d = Failed
-    | d == d2 = Failed  -- The exact guard is p2 == AddDir && d == d2
-                        -- but note d == d2 suffices because we know p2 != RmDir
-                        -- (and hence p2 == AddDir) since patches must be sequential.
-    | d2 == d' = Failed
-    | otherwise = Succeeded (DP (movedirfilename d d' d2) (unsafeCoerceP p2) :< Move d d')
-commuteFiledir (Move d d' :< Move f f')
+commuteFiledir (FP f1 p1 :> Move d d')
+    | f1 == d' = Failed
+    | (p1 == AddFile || p1 == RmFile) && d == f1 = Failed
+    | otherwise = Succeeded (Move d d' :> FP (movedirfilename d d' f1) (unsafeCoerceP p1))
+commuteFiledir (DP d1 p1 :> Move d d')
+    | isSuperdir d1 d' || isSuperdir d1 d = Failed
+    | d == d1 = Failed  -- The exact guard is p1 == AddDir && d == d1
+                        -- but note d == d1 suffices because we know p1 != RmDir
+                        -- (and hence p1 == AddDir) since patches must be sequential.
+    | d1 == d' = Failed
+    | otherwise = Succeeded (Move d d' :> DP (movedirfilename d d' d1) (unsafeCoerceP p1))
+commuteFiledir (Move f f' :> Move d d')
     | f == d' || f' == d = Failed
     | f == d || f' == d' = Failed
     | d `isSuperdir` f && f' `isSuperdir` d' = Failed
     | otherwise =
-        Succeeded (Move (movedirfilename d d' f) (movedirfilename d d' f') :<
-                   Move (movedirfilename f' f d) (movedirfilename f' f d'))
+        Succeeded (Move (movedirfilename f' f d) (movedirfilename f' f d') :>
+                   Move (movedirfilename d d' f) (movedirfilename d d' f'))
 
 commuteFiledir _ = Unknown
 
-type CommuteFunction = forall wX wY . (Prim :< Prim) wX wY -> Perhaps ((Prim :< Prim) wX wY)
+type CommuteFunction = forall wX wY . (Prim :> Prim) wX wY -> Perhaps ((Prim :> Prim) wX wY)
 newtype WrappedCommuteFunction = WrappedCommuteFunction { runWrappedCommuteFunction :: CommuteFunction }
 
 subcommutes :: [(String, WrappedCommuteFunction)]
@@ -166,56 +169,56 @@ subcommutes =
     [("speedyCommute", WrappedCommuteFunction speedyCommute),
      ("commuteFiledir", WrappedCommuteFunction (cleverCommute commuteFiledir)),
      ("commuteFilepatches", WrappedCommuteFunction (cleverCommute commuteFilepatches)),
-     ("commutex", WrappedCommuteFunction (toPerhaps . toRevCommute commute))
+     ("commutex", WrappedCommuteFunction (toPerhaps . commute))
     ]
 
 commuteFilepatches :: CommuteFunction
-commuteFilepatches (FP f1 p1 :< FP f2 p2) | f1 == f2 = commuteFP f1 (p1 :< p2)
+commuteFilepatches (FP f1 p1 :> FP f2 p2) | f1 == f2 = commuteFP f1 (p1 :> p2)
 commuteFilepatches _ = Unknown
 
-commuteFP :: FileName -> (FilePatchType :< FilePatchType) wX wY
-          -> Perhaps ((Prim :< Prim) wX wY)
-commuteFP f (Hunk line1 [] [] :< p2) =
-    seq f $ Succeeded (FP f (unsafeCoerceP p2) :< FP f (Hunk line1 [] []))
-commuteFP f (p2 :< Hunk line1 [] []) =
-    seq f $ Succeeded (FP f (Hunk line1 [] []) :< FP f (unsafeCoerceP p2))
-commuteFP f (Hunk line1 old1 new1 :< Hunk line2 old2 new2) = seq f $
-  toPerhaps $ commuteHunk f (Hunk line1 old1 new1 :< Hunk line2 old2 new2)
-commuteFP f (TokReplace t o n :< Hunk line2 old2 new2) = seq f $
-    case tryTokReplace t o n old2 of
+commuteFP :: FileName -> (FilePatchType :> FilePatchType) wX wY
+          -> Perhaps ((Prim :> Prim) wX wY)
+commuteFP f (p1 :> Hunk line1 [] []) =
+    seq f $ Succeeded (FP f (Hunk line1 [] []) :> FP f (unsafeCoerceP p1))
+commuteFP f (Hunk line1 [] [] :> p2) =
+    seq f $ Succeeded (FP f (unsafeCoerceP p2) :> FP f (Hunk line1 [] []))
+commuteFP f (Hunk line1 old1 new1 :> Hunk line2 old2 new2) = seq f $
+  toPerhaps $ commuteHunk f (Hunk line1 old1 new1 :> Hunk line2 old2 new2)
+commuteFP f (Hunk line1 old1 new1 :> TokReplace t o n) = seq f $
+    case tryTokReplace t o n old1 of
     Nothing -> Failed
-    Just old2' ->
-      case tryTokReplace t o n new2 of
-      Nothing -> Failed
-      Just new2' -> Succeeded (FP f (Hunk line2 old2' new2') :<
-                               FP f (TokReplace t o n))
-commuteFP f (TokReplace t o n :< TokReplace t2 o2 n2)
-    | seq f $ t /= t2 = Failed
-    | o == o2 = Failed
-    | n == o2 = Failed
-    | o == n2 = Failed
-    | n == n2 = Failed
-    | otherwise = Succeeded (FP f (TokReplace t2 o2 n2) :<
-                             FP f (TokReplace t o n))
+    Just old1' ->
+      case tryTokReplace t o n new1 of
+        Nothing -> Failed
+        Just new1' -> Succeeded (FP f (TokReplace t o n) :>
+                                 FP f (Hunk line1 old1' new1'))
+commuteFP f (TokReplace t1 o1 n1 :> TokReplace t2 o2 n2)
+    | seq f $ t1 /= t2 = Failed
+    | o1 == o2 = Failed
+    | n1 == o2 = Failed
+    | o1 == n2 = Failed
+    | n1 == n2 = Failed
+    | otherwise = Succeeded (FP f (TokReplace t2 o2 n2) :>
+                             FP f (TokReplace t1 o1 n1))
 commuteFP _ _ = Unknown
 
-commuteHunk :: FileName -> (FilePatchType :< FilePatchType) wX wY
-            -> Maybe ((Prim :< Prim) wX wY)
-commuteHunk f (Hunk line2 old2 new2 :< Hunk line1 old1 new1)
+commuteHunk :: FileName -> (FilePatchType :> FilePatchType) wX wY
+            -> Maybe ((Prim :> Prim) wX wY)
+commuteHunk f (Hunk line1 old1 new1 :> Hunk line2 old2 new2)
   | seq f $ line1 + lengthnew1 < line2 =
-      Just (FP f (Hunk line1 old1 new1) :<
-            FP f (Hunk (line2 - lengthnew1 + lengthold1) old2 new2))
+      Just (FP f (Hunk (line2 - lengthnew1 + lengthold1) old2 new2) :>
+            FP f (Hunk line1 old1 new1))
   | line2 + lengthold2 < line1 =
-      Just (FP f (Hunk (line1+ lengthnew2 - lengthold2) old1 new1) :<
-            FP f (Hunk line2 old2 new2))
+      Just (FP f (Hunk line2 old2 new2) :>
+            FP f (Hunk (line1+ lengthnew2 - lengthold2) old1 new1))
   | line1 + lengthnew1 == line2 &&
     lengthold2 /= 0 && lengthold1 /= 0 && lengthnew2 /= 0 && lengthnew1 /= 0 =
-      Just (FP f (Hunk line1 old1 new1) :<
-            FP f (Hunk (line2 - lengthnew1 + lengthold1) old2 new2))
+      Just (FP f (Hunk (line2 - lengthnew1 + lengthold1) old2 new2) :>
+            FP f (Hunk line1 old1 new1))
   | line2 + lengthold2 == line1 &&
     lengthold2 /= 0 && lengthold1 /= 0 && lengthnew2 /= 0 && lengthnew1 /= 0 =
-      Just (FP f (Hunk (line1 + lengthnew2 - lengthold2) old1 new1) :<
-            FP f (Hunk line2 old2 new2))
+      Just (FP f (Hunk line2 old2 new2) :>
+            FP f (Hunk (line1 + lengthnew2 - lengthold2) old1 new1))
   | otherwise = seq f Nothing
   where lengthnew1 = length new1
         lengthnew2 = length new2
