@@ -26,6 +26,9 @@ module Darcs.Patch.Prim.V1.Core
        )
        where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( pi )
 
 import qualified Data.ByteString as B (ByteString)
