@@ -1,6 +1,9 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.V1.Read () where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Prim.Class ( PrimRead(..), hunk, binary )
 import Darcs.Patch.Prim.V1.Core
     ( Prim(..),
