@@ -4,13 +4,16 @@ module Darcs.Patch.Prim.V1.Show
     ( showHunk )
     where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( pi )
 
 import Darcs.Util.ByteString ( fromPS2Hex )
 import qualified Data.ByteString as B (ByteString, length, take, drop)
 import qualified Data.ByteString.Char8 as BC (head)
 
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 import Darcs.Patch.FileHunk ( FileHunk(..), IsHunk(..), showFileHunk )
 import Darcs.Util.Path ( FileName )
@@ -42,7 +45,7 @@ instance (ApplyState Prim ~ Tree) => ShowPatch Prim where
     showContextPatch (isHunk -> Just fh) = showContextHunk fh
     showContextPatch p = return $ showPatch p
     summary = plainSummaryPrim
-    summaryFL = plainSummaryPrims
+    summaryFL = plainSummaryPrims False []
     thing _ = "change"
 
 instance Show (Prim wX wY) where
