@@ -193,9 +193,6 @@ checkConsumes parser = do
 class (Functor m, Applicative m, Alternative m, Monad m, MonadPlus m) => ParserM m where
     -- | Applies a parsing function inside the 'ParserM' monad.
     work :: (B.ByteString -> Maybe (ParserState a)) -> m a
-    -- | Applies a parsing function, that can return 'Nothing',
-    -- inside the 'ParserM' monad.
-    maybeWork :: (B.ByteString -> Maybe (ParserState a)) -> m (Maybe a)
     -- | Allows for the inspection of the input that is yet to be parsed.
     peekInput :: m B.ByteString
     -- | Run the parser
@@ -244,9 +241,6 @@ instance Monad SM where
 
 instance ParserM SM where
     work = SM
-    maybeWork f = SM $ \s -> case f s of
-                                  Just (x :*: s') -> Just (Just x :*: s')
-                                  Nothing -> Just (Nothing :*: s)
     peekInput = SM $ \s -> Just (s :*: s)
     parse = parseStrictly
 
