@@ -11,6 +11,9 @@ module Darcs.Patch.Rebase.Viewing
     , RebaseChange(..), toRebaseChanges
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Commute ( selfCommuter )
 import Darcs.Patch.CommuteFn ( CommuteFn, commuterIdFL, commuterRLId, MergeFn
                              , totalCommuterIdFL
@@ -26,14 +29,19 @@ import Darcs.Patch.Format ( PatchListFormat(..) )
 import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Patch.Invert ( invertFL, invertRL )
 import Darcs.Patch.Matchable ( Matchable )
-import Darcs.Patch.MaybeInternal ( MaybeInternal(..) )
 import Darcs.Patch.Merge ( Merge(..), selfMerger )
 import Darcs.Patch.Named
     ( Named(..), namepatch, infopatch
     , mergerIdNamed
-    , adddeps, getdeps
+    , getdeps
     , patch2patchinfo, patchcontents
     )
+import Darcs.Patch.Named.Wrapped
+    ( WrappedNamed(..)
+    )
+import qualified Darcs.Patch.Named.Wrapped as Wrapped
+    ( infopatch, adddeps
+    )
 
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
 import Darcs.Patch.Patchy ( Invert(..), Commute(..), Patchy, Apply(..),
@@ -43,16 +51,15 @@ import Darcs.Patch.Patchy ( Invert(..), Commute(..), Patchy, Apply(..),
 import Darcs.Patch.Prim
     ( PrimPatch, PrimPatchBase, PrimOf, FromPrim(..), FromPrims(..)
     )
-import Darcs.Patch.Rebase
-    ( Rebasing(..), RebaseItem(..)
-    )
+import Darcs.Patch.Rebase.Container ( Suspended(..) )
 import Darcs.Patch.Rebase.Fixup
     ( RebaseFixup(..)
     , commuteFixupNamed, commuteNamedFixups
     , flToNamesPrims
     )
+import Darcs.Patch.Rebase.Item ( RebaseItem(..) )
 import Darcs.Patch.Rebase.Name ( RebaseName(..) )
-import Darcs.Patch.Rebase.NameHack ( NameHack(..) )
+import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
 import Darcs.Patch.Show ( ShowPatchBasic(..) )
 import Darcs.Patch.Summary ( plainSummary )
 import Darcs.Patch.Witnesses.Eq
@@ -67,8 +74,6 @@ import Darcs.Util.IsoDate ( getIsoDateTime )
 import Darcs.Util.Printer ( ($$), redText, empty, vcat )
 import Darcs.Util.Show ( appPrec )
 
-import Prelude hiding ( pi )
-import Control.Applicative ( (<$>) )
 import Data.List ( nub, (\\) )
 import Data.Maybe ( fromMaybe )
 
@@ -130,14 +135,14 @@ instance (Show2 p, Show2 (PrimOf p)) => Show (RebaseChange p wX wY) where
             showString " " . showsPrec2 (appPrec + 1) changes
 
 -- |Get hold of the 'PatchInfoAnd' patch inside a 'RebaseSelect'.
-rsToPia :: RebaseSelect p wX wY -> Sealed2 (PatchInfoAnd p)
-rsToPia (RSFwd _ toEdit) = Sealed2 (n2pia toEdit)
-rsToPia (RSRev _ toEdit) = Sealed2 (n2pia toEdit)
+rsToPia :: RebaseSelect p wX wY -> Sealed2 (PatchInfoAnd ('RepoType 'NoRebase) p)
+rsToPia (RSFwd _ toEdit) = Sealed2 (n2pia (NormalP toEdit))
+rsToPia (RSRev _ toEdit) = Sealed2 (n2pia (NormalP toEdit))
 
 instance PrimPatchBase p => PrimPatchBase (RebaseSelect p) where
    type PrimOf (RebaseSelect p) = PrimOf p
 
-instance (PrimPatchBase p, PatchListFormat p, Conflict p, FromPrim p, Effect p, CommuteNoConflicts p, IsHunk p, Patchy p, ApplyState p ~ ApplyState (PrimOf p), NameHack p)
+instance (PrimPatchBase p, PatchListFormat p, Conflict p, FromPrim p, Effect p, CommuteNoConflicts p, IsHunk p, Patchy p, ApplyState p ~ ApplyState (PrimOf p))
     => Patchy (RebaseSelect p)
 
 instance ( PrimPatchBase p, Apply p, ApplyState p ~ ApplyState (PrimOf p)
@@ -167,17 +172,15 @@ instance (PrimPatchBase p, FromPrim p, Conflict p, CommuteNoConflicts p, Invert
    resolveConflicts (RSFwd _ toedit) = resolveConflicts toedit
    resolveConflicts (RSRev{}) = impossible
 
+   conflictedEffect (RSFwd _ toedit) = conflictedEffect toedit
+   conflictedEffect (RSRev{}) = impossible
+
 -- newtypes to help the type-checker with the 'changeAsMerge' abstraction
 newtype ResolveConflictsResult p wY =
     ResolveConflictsResult {
         getResolveConflictsResult :: [[Sealed (FL (PrimOf p) wY)]]
     }
 
-newtype ListConflictedFilesResult (p :: * -> * -> *) wY =
-    ListConflictedFilesResult {
-        getListConflictedFilesResult :: [FilePath]
-    }
-
 newtype ConflictedEffectResult p wY =
     ConflictedEffectResult {
         getConflictedEffectResult :: [IsConflictedPrim (PrimOf p)]
@@ -208,8 +211,6 @@ instance
     resolveConflicts =
         getResolveConflictsResult . changeAsMerge (ResolveConflictsResult . resolveConflicts)
 
-    listConflictedFiles =
-        getListConflictedFilesResult . changeAsMerge (ListConflictedFilesResult . listConflictedFiles)
 
     conflictedEffect =
         getConflictedEffectResult . changeAsMerge (ConflictedEffectResult . conflictedEffect)
@@ -227,7 +228,7 @@ instance (PrimPatchBase p, Invert p, Effect p) => Effect (RebaseChange p) where
 
 instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (RebaseSelect p) where
    showPatch (RSFwd fixups toedit) =
-         showPatch (Suspended (mapFL_FL Fixup fixups +>+ ToEdit toedit :>: NilFL))
+         showPatch (Items (mapFL_FL Fixup fixups +>+ ToEdit toedit :>: NilFL))
    showPatch (RSRev {}) = impossible
 
 instance (PrimPatchBase p, PatchListFormat p, ShowPatchBasic p)
@@ -298,20 +299,20 @@ toRebaseSelect (ToEdit te :>: ps) = RSFwd NilFL te :>: toRebaseSelect ps
 toRebaseChanges
     :: PrimPatchBase p
     => FL (RebaseItem p) wX wY
-    -> FL (PatchInfoAnd (RebaseChange p)) wX wY
+    -> FL (PatchInfoAnd ('RepoType 'IsRebase) (RebaseChange p)) wX wY
 toRebaseChanges = mapFL_FL toChange . toRebaseSelect
 
-toChange :: RebaseSelect p wX wY -> PatchInfoAnd (RebaseChange p) wX wY
+toChange :: RebaseSelect p wX wY -> PatchInfoAnd rt (RebaseChange p) wX wY
 toChange (RSFwd fixups named) =
     n2pia $
-    flip adddeps (getdeps named) $
-    infopatch (patch2patchinfo named) $
+    flip Wrapped.adddeps (getdeps named) $
+    Wrapped.infopatch (patch2patchinfo named) $
     (:>: NilFL) $
     RCFwd fixups (patchcontents named)
 toChange (RSRev fixups named) =
     n2pia $
-    flip adddeps (getdeps named) $
-    infopatch (patch2patchinfo named) $
+    flip Wrapped.adddeps (getdeps named) $
+    Wrapped.infopatch (patch2patchinfo named) $
     (:>: NilFL) $
     RCRev fixups (patchcontents named)
 
@@ -335,7 +336,7 @@ instance (PrimPatchBase p, Commute p, MyEq p) => MyEq (RebaseSelect p) where
 
    _ =\/= _ = impossible
 
-instance (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p, NameHack p) => Commute (RebaseSelect p) where
+instance (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p) => Commute (RebaseSelect p) where
    commute (RSFwd {} :> RSRev {}) = impossible
    commute (RSRev {} :> RSFwd {}) = impossible
    commute (RSRev fixups1 edit1 :> RSRev fixups2 edit2) =
@@ -370,12 +371,12 @@ instance (PrimPatchBase p, PatchInspect p) => PatchInspect (RebaseChange p) wher
 -- |Split a list of rebase patches into those that will
 -- have conflicts if unsuspended and those that won't.
 partitionUnconflicted
-    :: (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p, NameHack p)
+    :: (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p)
     => FL (RebaseSelect p) wX wY
     -> (FL (RebaseSelect p) :> RL (RebaseSelect p)) wX wY
 partitionUnconflicted = partitionUnconflictedAcc NilRL
 
-partitionUnconflictedAcc :: (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p, NameHack p)
+partitionUnconflictedAcc :: (PrimPatchBase p, FromPrim p, Effect p, Commute p, Invert p)
                          => RL (RebaseSelect p) wX wY -> FL (RebaseSelect p) wY wZ
                          -> (FL (RebaseSelect p) :> RL (RebaseSelect p)) wX wZ
 partitionUnconflictedAcc right NilFL = NilFL :> right
@@ -384,7 +385,7 @@ partitionUnconflictedAcc right (p :>: ps) =
      Just (p'@(RSFwd NilFL _) :> right')
        -> case partitionUnconflictedAcc right' ps of
             left' :> right'' -> (p' :>: left') :> right''
-     _ -> partitionUnconflictedAcc (p :<: right) ps
+     _ -> partitionUnconflictedAcc (right :<: p) ps
 
 -- | A patch, together with a list of patch names that it used to depend on,
 -- but were lost during the rebasing process. The UI can use this information
@@ -552,14 +553,11 @@ mkDummy (Rename _ _) = error "internal error: can't make a dummy patch from a re
 instance CommuteNoConflicts (RebaseChange p) where
     commuteNoConflicts _ = impossible
 
-instance MaybeInternal (RebaseChange p)
-
 instance IsHunk p => IsHunk (RebaseChange p) where
     -- RebaseChange is a compound patch, so it doesn't really make sense to
     -- ask whether it's a hunk. TODO: get rid of the need for this.
     isHunk _ = Nothing
 
-instance NameHack (RebaseChange p)
 instance PatchListFormat (RebaseChange p)
 
 instance ( PrimPatchBase p, Apply p, Invert p
