@@ -24,6 +24,9 @@
 
 module Darcs.Patch.Split ( Splitter(..), rawSplitter, noSplitter, primSplitter, reversePrimSplitter ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.List ( intersperse )
 
 import Darcs.Patch.Witnesses.Ordered
@@ -62,7 +65,7 @@ import qualified Data.ByteString.Char8 as BC
 -- splitters for Prim etc, and the generality doesn't cost anything.
 data Splitter p
   = Splitter {
-              applySplitter :: forall wX wY . D.DiffAlgorithm -> p wX wY
+              applySplitter :: forall wX wY . p wX wY
                               -> Maybe (B.ByteString,
                                         B.ByteString -> Maybe (FL p wX wY))
               -- canonization is needed to undo the effects of splitting
@@ -74,7 +77,7 @@ data Splitter p
               -- we might record (or whatever) a rather strange looking patch.
               -- This hook allows the splitter to provide an appropriate
               -- function for doing this.
-             ,canonizeSplit :: forall wX wY . D.DiffAlgorithm -> FL p wX wY
+             ,canonizeSplit :: forall wX wY . FL p wX wY
                               -> FL p wX wY
              }
 
@@ -96,19 +99,19 @@ withEditedHead p res = res :>: invert res :>: p :>: NilFL
 rawSplitter :: (ShowPatch p, ReadPatch p, Invert p) => Splitter p
 rawSplitter = Splitter {
                   applySplitter =
-                     \_ p -> Just (renderPS Standard . showPatch $ p,
+                     \p -> Just (renderPS Standard . showPatch $ p,
                                  \str -> case parseStrictly readPatch' str of
                                           Just (Sealed res, _) -> Just (withEditedHead p res)
                                           _ -> Nothing
                                 )
-                 ,canonizeSplit = \_ b -> id b
+                 ,canonizeSplit = id
                 }
 
 -- |Never splits. In other code we normally pass around Maybe Splitter instead of using this
 -- as the default, because it saves clients that don't care about splitting from having to
 -- import this module just to get noSplitter.
 noSplitter :: Splitter p
-noSplitter = Splitter { applySplitter = \_ -> const Nothing, canonizeSplit = \_ b -> id b }
+noSplitter = Splitter { applySplitter = const Nothing, canonizeSplit = id }
 
 
 doPrimSplit :: PrimPatch prim => D.DiffAlgorithm -> prim wX wY
@@ -119,7 +122,7 @@ doPrimSplit da = doPrimSplit_ da True explanation
                    [ "Interactive hunk edit:"
                    , " - Edit the section marked 'AFTER'"
                    , " - Arbitrary editing is supported"
-                   , " - This will only affect the patch, not your working copy"
+                   , " - This will only affect the patch, not your working tree"
                    , " - Hints:"
                    , "   - To split added text, delete the part you want to postpone"
                    , "   - To split removed text, copy back the part you want to retain"
@@ -161,9 +164,9 @@ doPrimSplit_ _ _ _ _ = Nothing
 -- |Split a primitive hunk patch up
 -- by allowing the user to edit both the before and after lines, then insert fixup patches
 -- to clean up the mess.
-primSplitter :: PrimPatch p => Splitter p
-primSplitter  = Splitter { applySplitter = doPrimSplit
-                         , canonizeSplit = canonizeFL }
+primSplitter :: PrimPatch p => D.DiffAlgorithm -> Splitter p
+primSplitter da = Splitter { applySplitter = doPrimSplit da
+                           , canonizeSplit = canonizeFL da }
 
 doReversePrimSplit :: PrimPatch prim => D.DiffAlgorithm -> prim wX wY
                    -> Maybe (B.ByteString, B.ByteString -> Maybe (FL prim wX wY))
@@ -178,7 +181,7 @@ doReversePrimSplit da prim = do
       map BC.pack [ "Interactive hunk edit:"
                   , " - Edit the section marked 'AFTER' (representing the state to which you'll revert)"
                   , " - Arbitrary editing is supported"
-                  , " - Your working copy will be returned to the 'AFTER' state"
+                  , " - Your working tree will be returned to the 'AFTER' state"
                   , " - Do not touch the 'BEFORE' section"
                   , " - Hints:"
                   , "   - To revert only a part of a text addition, delete the part you want to get rid of"
@@ -186,6 +189,6 @@ doReversePrimSplit da prim = do
                   , ""
                                  ]
 
-reversePrimSplitter :: PrimPatch prim => Splitter prim
-reversePrimSplitter = Splitter { applySplitter = doReversePrimSplit
-                               , canonizeSplit = canonizeFL}
+reversePrimSplitter :: PrimPatch prim => D.DiffAlgorithm -> Splitter prim
+reversePrimSplitter da = Splitter { applySplitter = doReversePrimSplit da
+                                  , canonizeSplit = canonizeFL da }
