@@ -3,8 +3,13 @@ module Darcs.Patch.TokenReplace
       tryTokInternal
     , forceTokReplace
     , breakOutToken
+    , breakToTokens
+    , defaultToks
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 import Data.Maybe ( isNothing )
@@ -55,3 +60,13 @@ forceTokReplace tokChars oldToken newToken = forceReplaceLines
 
     replaceMatchingToken input | input == oldToken = newToken
                                | otherwise = input
+
+-- break a single bytestring into tokens
+breakToTokens :: BC.ByteString -> [BC.ByteString]
+breakToTokens input | B.null input = []
+breakToTokens input =
+  let (_, tok, remaining) = breakOutToken defaultToks input in
+    tok : breakToTokens remaining
+
+defaultToks :: String
+defaultToks = "A-Za-z_0-9"
