@@ -1,7 +1,9 @@
 module Darcs.Patch.Type ( PatchType(..), patchType ) where
 
--- |Used for indicating a patch type without having a concrete patch
-data PatchType (p :: * -> * -> *) = PatchType
+import Darcs.Patch.RepoType ( RepoType )
 
-patchType :: p wX wY -> PatchType p
+-- |Used for indicating a patch type without having a concrete patch
+data PatchType (rt :: RepoType) (p :: * -> * -> *) = PatchType
+
+patchType :: p wX wY -> PatchType rt p
 patchType _ = PatchType
