@@ -15,7 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -fno-warn-orphans -fno-warn-incomplete-patterns #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE CPP #-}
 
 
@@ -27,21 +27,26 @@ module Darcs.Patch.V1.Commute
     )
        where
 
-import Control.Monad ( MonadPlus, mplus, msum, mzero, guard )
-import Control.Applicative ( Applicative(..), Alternative(..) )
-import Data.Maybe ( isJust )
+import Prelude ()
+import Darcs.Prelude
 
-import Darcs.Patch.Commute ( toFwdCommute, selfCommuter )
+import Control.Monad ( MonadPlus, mplus, msum, mzero, guard )
+import Control.Applicative ( Alternative(..) )
+
+import Darcs.Patch.Commute ( selfCommuter )
 import Darcs.Patch.CommuteFn ( commuterIdFL, commuterFLId )
-import Darcs.Patch.ConflictMarking ( mangleUnravelled )
 import Darcs.Util.Path ( FileName )
 import Darcs.Patch.Invert ( invertRL )
 import Darcs.Patch.Merge ( Merge(..) )
 import Darcs.Patch.Patchy ( Commute(..), PatchInspect(..), Invert(..) )
-import Darcs.Patch.V1.Core ( Patch(..),
+import Darcs.Patch.V1.Core ( RepoPatchV1(..),
                              isMerger,
                              mergerUndo )
-import Darcs.Patch.Conflict ( Conflict(..), CommuteNoConflicts(..) )
+import Darcs.Patch.Conflict
+    ( Conflict(..), listConflictedFiles
+    , IsConflictedPrim(..), ConflictState(..), CommuteNoConflicts(..)
+    , mangleUnravelled
+    )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Prim ( FromPrim(..), PrimPatch,
@@ -61,9 +66,9 @@ import Darcs.Patch.Witnesses.Unsafe
     ( unsafeCoerceP, unsafeCoercePStart
     , unsafeCoercePEnd )
 import Darcs.Patch.Witnesses.Ordered
-    ( mapFL_FL,
+    ( mapFL_FL, mapFL,
     FL(..), RL(..),
-    (:/\:)(..), (:<)(..), (:\/:)(..), (:>)(..),
+    (:/\:)(..), (:\/:)(..), (:>)(..),
     lengthFL, mapRL,
     reverseFL, reverseRL, concatFL
     )
@@ -117,34 +122,31 @@ toPerhaps Nothing = Failed
 -- TODO: when can the first attempt fail, but the second not? What's so clever
 -- in this function?
 cleverCommute :: Invert prim => CommuteFunction prim -> CommuteFunction prim
-cleverCommute c (p1:<p2) =
-    case c (p1 :< p2) of
+cleverCommute c (p1 :> p2) = case c (p1 :> p2) of
     Succeeded x -> Succeeded x
     Failed -> Failed
-    Unknown -> case c (invert p2 :< invert p1) of
-               Succeeded (p1' :< p2') -> Succeeded (invert p2' :< invert p1')
-               Failed -> Failed
-               Unknown -> Unknown
+    Unknown -> case c (invert p2 :> invert p1) of
+                 Succeeded (ip1' :> ip2') -> Succeeded (invert ip2' :> invert ip1')
+                 Failed -> Failed
+                 Unknown -> Unknown
 
 -- | If we have two Filepatches which modify different files, we can return a
 -- result early, since the patches trivially commute.
 speedyCommute :: PrimPatch prim => CommuteFunction prim
-speedyCommute (p1 :< p2)
-    | isJust p1_modifies && isJust p2_modifies &&
-      p1_modifies /= p2_modifies = Succeeded (unsafeCoerceP p2 :< unsafeCoerceP p1)
+speedyCommute (p1 :> p2)
+    | Just m1 <- isFilepatchMerger p1
+    , Just m2 <- isFilepatchMerger p2
+    , m1 /= m2 = Succeeded (unsafeCoerceP p2 :> unsafeCoerceP p1)
     | otherwise = Unknown
-    where p1_modifies = isFilepatchMerger p1
-          p2_modifies = isFilepatchMerger p2
 
 everythingElseCommute :: forall prim . PrimPatch prim => CommuteFunction prim
-everythingElseCommute (PP px :< PP py) = toPerhaps $ do
-    x' :> y' <- commute (py :> px)
-    return (PP y' :< PP x')
-everythingElseCommute _xx =
-        msum [
-              cleverCommute commuteRecursiveMerger       _xx
-             ,cleverCommute otherCommuteRecursiveMerger _xx
-             ]
+everythingElseCommute (PP p1 :> PP p2) = toPerhaps $ do
+    p2' :> p1' <- commute (p1 :> p2)
+    return (PP p2' :> PP p1')
+everythingElseCommute ps =
+    msum [ cleverCommute commuteRecursiveMerger      ps
+         , cleverCommute otherCommuteRecursiveMerger ps
+         ]
 
 {-
 Note that it must be true that
@@ -157,24 +159,23 @@ if commutex (A, B) == Just (B', A')
 then commutex (B^-1, A^-1) == Just (A'^-1, B'^-1)
 -}
 
-unsafeMerger :: PrimPatch prim => String -> Patch prim wX wY -> Patch prim wX wZ -> Patch prim wA wB
+unsafeMerger :: PrimPatch prim => String -> RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> RepoPatchV1 prim wA wB
 unsafeMerger x p1 p2 = unsafeCoercePStart $ unsafeUnseal $ merger x p1 p2
 
 -- | Attempt to commute two patches, the first of which is a Merger patch.
-mergerCommute :: PrimPatch prim => (Patch prim :< Patch prim) wX wY -> Perhaps ((Patch prim :< Patch prim) wX wY)
-mergerCommute (Merger _ _ p1 p2 :< pA)
-    | unsafeCompare pA p1 = Succeeded (unsafeMerger "0.0" p2 p1 :< unsafeCoercePStart p2)
+mergerCommute :: PrimPatch prim
+              => (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
+mergerCommute (pA :> Merger _ _ p1 p2)
+    | unsafeCompare pA p1 = Succeeded (unsafeCoercePStart p2 :> unsafeMerger "0.0" p2 p1)
     | unsafeCompare pA (invert (unsafeMerger "0.0" p2 p1)) = Failed
-mergerCommute (Merger _ _
-                (Merger _ _ c b)
-                (Merger _ _ c' a) :<
-                Merger _ _ b' c'')
+mergerCommute (Merger _ _ b' c'' :> Merger _ _ (Merger _ _ c b) (Merger _ _ c' a))
     | unsafeCompare b' b && unsafeCompare c c' && unsafeCompare c c'' =
-        Succeeded (unsafeMerger "0.0" (unsafeMerger "0.0" b (unsafeCoercePStart a)) (unsafeMerger "0.0" b c) :<
-                   unsafeMerger "0.0" b (unsafeCoercePStart a))
+        Succeeded ( unsafeMerger "0.0" b (unsafeCoercePStart a) :>
+                    unsafeMerger "0.0" (unsafeMerger "0.0" b (unsafeCoercePStart a)) (unsafeMerger "0.0" b c)
+                  )
 mergerCommute _ = Unknown
 
-instance PrimPatch prim => Merge (Patch prim) where
+instance PrimPatch prim => Merge (RepoPatchV1 prim) where
     merge (y :\/: z) =
         case actualMerge (y:\/:z) of
         -- actualMerge returns one "arm" of a merge result (@y'@, which applies
@@ -187,14 +188,14 @@ instance PrimPatch prim => Merge (Patch prim) where
                                    $$ showPatch_ y'
                          Just (_ :> z') -> unsafeCoercePStart z' :/\: y'
 
-instance PrimPatch prim => Commute (Patch prim) where
+instance PrimPatch prim => Commute (RepoPatchV1 prim) where
     commute x = toMaybe $ msum
-                  [toFwdCommute speedyCommute x,
-                   toFwdCommute (cleverCommute mergerCommute) x,
-                   toFwdCommute everythingElseCommute x
+                  [speedyCommute x,
+                   (cleverCommute mergerCommute) x,
+                   everythingElseCommute x
                   ]
 
-instance PrimPatch prim => PatchInspect (Patch prim) where
+instance PrimPatch prim => PatchInspect (RepoPatchV1 prim) where
     -- Recurse on everything, these are potentially spoofed patches
     listTouchedFiles (Merger _ _ p1 p2) = nubSort $ listTouchedFiles p1
                                             ++ listTouchedFiles p2
@@ -207,10 +208,11 @@ instance PrimPatch prim => PatchInspect (Patch prim) where
 
 commuteNoMerger :: PrimPatch prim => MaybeCommute prim
 commuteNoMerger x =
-    toMaybe $ msum [speedyCommute x,
-                    everythingElseCommute x]
+    toMaybe $ msum [ speedyCommute x
+                   , everythingElseCommute x
+                   ]
 
-isFilepatchMerger :: PrimPatch prim => Patch prim wX wY -> Maybe FileName
+isFilepatchMerger :: PrimPatch prim => RepoPatchV1 prim wX wY -> Maybe FileName
 isFilepatchMerger (PP p) = is_filepatch p
 isFilepatchMerger (Merger _ _ p1 p2) = do
      f1 <- isFilepatchMerger p1
@@ -219,8 +221,9 @@ isFilepatchMerger (Merger _ _ p1 p2) = do
 isFilepatchMerger (Regrem und unw p1 p2)
     = isFilepatchMerger (Merger und unw p1 p2)
 
-commuteRecursiveMerger :: PrimPatch prim => (Patch prim :< Patch prim) wX wY -> Perhaps ((Patch prim :< Patch prim) wX wY)
-commuteRecursiveMerger (p@(Merger _ _ p1 p2) :< pA) = toPerhaps $
+commuteRecursiveMerger :: PrimPatch prim
+    => (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
+commuteRecursiveMerger (pA :> p@(Merger _ _ p1 p2)) = toPerhaps $
   do (_ :> pA') <- commuterIdFL selfCommuter (pA :> undo)
      _ <- commuterIdFL selfCommuter (pA' :> invert undo)
      (_ :> pAmid) <- commute (pA :> unsafeCoercePStart (invert p1))
@@ -235,13 +238,13 @@ commuteRecursiveMerger (p@(Merger _ _ p1 p2) :< pA) = toPerhaps $
          undo' = mergerUndo p'
      (pAo :> _) <- commuterFLId selfCommuter (undo' :> pA')
      guard (pAo `unsafeCompare` pA)
-     return (pA' :< p')
+     return (p' :> pA')
     where undo = mergerUndo p
 commuteRecursiveMerger _ = Unknown
 
-otherCommuteRecursiveMerger :: PrimPatch prim => (Patch prim :< Patch prim) wX wY -> Perhaps ((Patch prim :< Patch prim) wX wY)
-otherCommuteRecursiveMerger (pA':< p_old@(Merger _ _ p1' p2')) =
-  toPerhaps $
+otherCommuteRecursiveMerger :: PrimPatch prim
+    => (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
+otherCommuteRecursiveMerger (p_old@(Merger _ _ p1' p2') :> pA') = toPerhaps $
   do (pA :> _) <- commuterFLId selfCommuter (mergerUndo p_old :> pA')
      (pAmid :> p1) <- commute (unsafeCoercePEnd p1' :> pA)
      (_ :> pAmido) <- commute (pA :> invert p1)
@@ -256,24 +259,24 @@ otherCommuteRecursiveMerger (pA':< p_old@(Merger _ _ p1' p2')) =
      guard (not $ pA `unsafeCompare` p1) -- special case here...
      (_ :> pAo') <- commuterIdFL selfCommuter (pA :> undo)
      guard (pAo' `unsafeCompare` pA')
-     return (p :< pA)
+     return (pA :> p)
 otherCommuteRecursiveMerger _ = Unknown
 
-type CommuteFunction prim = forall wX wY . (Patch prim :< Patch prim) wX wY -> Perhaps ((Patch prim :< Patch prim) wX wY)
-type MaybeCommute prim = forall wX wY . (Patch prim :< Patch prim) wX wY -> Maybe ((Patch prim :< Patch prim) wX wY)
+type CommuteFunction prim = forall wX wY . (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
+type MaybeCommute prim = forall wX wY . (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Maybe ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
 
-revCommuteFLId :: MaybeCommute prim -> (FL (Patch prim) :< Patch prim) wX wY -> Maybe ((Patch prim :< FL (Patch prim)) wX wY)
-revCommuteFLId _        (NilFL :< p) = return (p :< NilFL)
-revCommuteFLId commuter ((q :>: qs) :< p) = do
-   p' :< q' <- commuter (q :< p)
-   p'' :< qs' <- revCommuteFLId commuter (qs :< p')
-   return (p'' :< (q' :>: qs'))
+commuteFLId :: MaybeCommute prim -> (RepoPatchV1 prim :> FL (RepoPatchV1 prim)) wX wY -> Maybe ((FL (RepoPatchV1 prim) :> RepoPatchV1 prim) wX wY)
+commuteFLId _        (p :> NilFL) = return (NilFL :> p)
+commuteFLId commuter (p :> (q :>: qs)) = do
+   q' :> p' <- commuter (p :> q)
+   qs' :> p'' <- commuteFLId commuter (p' :> qs)
+   return ((q' :>: qs') :> p'')
 
 -- | elegantMerge attempts to perform the "intuitive" merge of two patches,
 -- from a common starting context @wX@.
 elegantMerge :: PrimPatch prim
-              => (Patch prim :\/: Patch prim) wX wY
-              -> Maybe ((Patch prim :/\: Patch prim) wX wY)
+             => (RepoPatchV1 prim :\/: RepoPatchV1 prim) wX wY
+             -> Maybe ((RepoPatchV1 prim :/\: RepoPatchV1 prim) wX wY)
 elegantMerge (p1 :\/: p2) = do
   p1' :> ip2' <- commute (invert p2 :> p1)
   p1o :> _    <- commute (p2 :> p1')
@@ -308,43 +311,45 @@ these constructors.
 --
 -- TODO: why does this code throw away the other branch, only for merge to
 -- rebuild it?
-actualMerge :: PrimPatch prim => (Patch prim :\/: Patch prim) wX wY -> Sealed (Patch prim wY)
+actualMerge :: PrimPatch prim
+    => (RepoPatchV1 prim :\/: RepoPatchV1 prim) wX wY -> Sealed (RepoPatchV1 prim wY)
 actualMerge (p1 :\/: p2) = case elegantMerge (p1:\/:p2) of
                              Just (_ :/\: p1') -> Sealed p1'
                              Nothing -> merger "0.0" p2 p1
 
 -- Recreates a patch history in reverse.
-unwind :: Patch prim wX wY -> Sealed (RL (Patch prim) wX)
+unwind :: RepoPatchV1 prim wX wY -> Sealed (RL (RepoPatchV1 prim) wX)
 unwind (Merger _ unwindings _ _) = Sealed unwindings
-unwind p = Sealed (p :<: NilRL)
+unwind p = Sealed (NilRL :<: p)
 
 -- Recreates a patch history in reverse. The patch being unwound is always at
 -- the start of the list of patches.
-trueUnwind :: PrimPatch prim => Patch prim wX wY -> Sealed (RL (Patch prim) wX)
+trueUnwind :: PrimPatch prim
+    => RepoPatchV1 prim wX wY -> Sealed (RL (RepoPatchV1 prim) wX)
 trueUnwind p@(Merger _ _ p1 p2) =
     case (unwind p1, unwind p2) of
-    (Sealed (_:<:p1s),Sealed (_:<:p2s)) ->
-         Sealed (p :<: unsafeCoerceP p1 :<: unsafeUnsealFlipped (reconcileUnwindings p p1s (unsafeCoercePEnd p2s)))
+    (Sealed (p1s:<:_),Sealed (p2s:<:_)) ->
+         Sealed (unsafeUnsealFlipped (reconcileUnwindings p p1s (unsafeCoercePEnd p2s)) :<: unsafeCoerceP p1 :<: p)
     _ -> impossible
 trueUnwind _ = impossible
 
 reconcileUnwindings :: PrimPatch prim
-                    => Patch prim wA wB -> RL (Patch prim) wX wZ -> RL (Patch prim) wY wZ -> FlippedSeal (RL (Patch prim)) wZ
+    => RepoPatchV1 prim wA wB -> RL (RepoPatchV1 prim) wX wZ -> RL (RepoPatchV1 prim) wY wZ -> FlippedSeal (RL (RepoPatchV1 prim)) wZ
 reconcileUnwindings _ NilRL p2s = FlippedSeal p2s
 reconcileUnwindings _ p1s NilRL = FlippedSeal p1s
-reconcileUnwindings p (p1:<:p1s) p2s@(p2:<:tp2s) =
+reconcileUnwindings p (p1s:<:p1) p2s@(tp2s:<:p2) =
     case [(p1s', p2s')|
-          p1s'@(hp1s':<:_) <- headPermutationsRL (p1:<:p1s),
-          p2s'@(hp2s':<:_) <- headPermutationsRL p2s,
+          p1s'@(_:<:hp1s') <- headPermutationsRL (p1s:<:p1),
+          p2s'@(_:<:hp2s') <- headPermutationsRL p2s,
           hp1s' `unsafeCompare` hp2s'] of
-    ((p1':<:p1s', _:<:p2s'):_) ->
-        mapFlipped (p1' :<:) $ reconcileUnwindings p p1s' (unsafeCoercePEnd p2s')
+    ((p1s':<:p1', p2s':<:_):_) ->
+        mapFlipped (:<:p1') $ reconcileUnwindings p p1s' (unsafeCoercePEnd p2s')
     [] -> case reverseFL `fmap` putBefore p1 (reverseRL p2s) of
-          Just p2s' -> mapFlipped (p1 :<:) $ reconcileUnwindings p p1s p2s'
+          Just p2s' -> mapFlipped (:<:p1) $ reconcileUnwindings p p1s p2s'
           Nothing ->
               case fmap reverseFL $ putBefore p2 $
-                   reverseRL (p1:<:p1s) of
-              Just p1s' -> mapFlipped (p2 :<:) $
+                   reverseRL (p1s:<:p1) of
+              Just p1s' -> mapFlipped (:<:p2) $
                            reconcileUnwindings p p1s' tp2s
               Nothing ->
                   bugDoc $ text "in function reconcileUnwindings"
@@ -356,44 +361,52 @@ reconcileUnwindings p (p1:<:p1s) p2s@(p2:<:tp2s) =
 -- it seems to have been this way forever:
 -- Fri May 23 10:27:04 BST 2003  droundy@abridgegame.org
 --    * fix bug in unwind and add docs on unwind algorithm.
-putBefore :: PrimPatch prim => Patch prim wY wZ -> FL (Patch prim) wX wZ -> Maybe (FL (Patch prim) wY wW)
+putBefore :: PrimPatch prim
+    => RepoPatchV1 prim wY wZ -> FL (RepoPatchV1 prim) wX wZ -> Maybe (FL (RepoPatchV1 prim) wY wW)
 putBefore p1 (p2:>:p2s) =
     do p1' :> p2' <- commute (unsafeCoerceP p2 :> invert p1)
        _ <- commute (p2' :> p1)
        (unsafeCoerceP p2' :>:) `fmap` putBefore p1' (unsafeCoerceP p2s)
 putBefore _ NilFL = Just (unsafeCoerceP NilFL)
 
-instance PrimPatch prim => CommuteNoConflicts (Patch prim) where
-  commuteNoConflicts (x:>y) =   do x' :< y' <- commuteNoMerger (y :< x)
-                                   return (y':>x')
+instance PrimPatch prim => CommuteNoConflicts (RepoPatchV1 prim) where
+  commuteNoConflicts (x :> y) = do y' :> x' <- commuteNoMerger (x :> y)
+                                   return (y' :> x')
 
-instance PrimPatch prim => Conflict (Patch prim) where
-  resolveConflicts patch = rcs NilFL (patch :<: NilRL)
-    where rcs :: FL (Patch prim) wY wW -> RL (Patch prim) wX wY -> [[Sealed (FL prim wW)]]
+instance PrimPatch prim => Conflict (RepoPatchV1 prim) where
+  resolveConflicts patch = rcs NilFL (NilRL :<: patch)
+    where rcs :: FL (RepoPatchV1 prim) wY wW -> RL (RepoPatchV1 prim) wX wY -> [[Sealed (FL prim wW)]]
           rcs _ NilRL = []
-          rcs passedby (p@(Merger{}):<:ps) =
-              case revCommuteFLId commuteNoMerger (passedby:<p) of
-              Just (p'@(Merger _ _ p1 p2):<_) ->
+          rcs passedby (ps:<:p@(Merger{})) =
+              case commuteFLId commuteNoMerger (p :> passedby) of
+              Just (_ :> p'@(Merger _ _ p1 p2)) ->
                   map Sealed (nubBy unsafeCompare $
                         effect (unsafeCoercePStart $ unsafeUnseal (glump09 p1 p2)) : map (unsafeCoercePStart . unsafeUnseal) (unravel p'))
                   : rcs (p :>: passedby) ps
               Nothing -> rcs (p :>: passedby) ps
               _ -> impossible
-          rcs passedby (p:<:ps) = seq passedby $
+          rcs passedby (ps:<:p) = seq passedby $
                                   rcs (p :>: passedby) ps
 
+  conflictedEffect x =
+    case listConflictedFiles x of
+      [] -> mapFL (IsC Okay) $ effect x
+      _ -> mapFL (IsC Conflicted) $ effect x
+
+
 -- This type seems wrong - the most natural type for the result would seem to be
 -- [Sealed (FL Prim wX)], given the type of unwind.
 -- However downstream code in darcs convert assumes the wY type, and I was unable
 -- to figure out whether this could/should reasonably be changed -- Ganesh 13/4/10
-publicUnravel :: PrimPatch prim => Patch prim wX wY -> [Sealed (FL prim wY)]
+publicUnravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wY)]
 publicUnravel = map (mapSeal unsafeCoercePStart) . unravel
 
-unravel :: PrimPatch prim => Patch prim wX wY -> [Sealed (FL prim wX)]
+unravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wX)]
 unravel p = nub $ map (mapSeal (sortCoalesceFL . concatFL . mapFL_FL effect)) $
             getSupers $ map (mapSeal reverseRL) $ unseal (newUr p) $ unwind p
 
-getSupers :: PrimPatch prim => [Sealed (FL (Patch prim) wX)] -> [Sealed (FL (Patch prim) wX)]
+getSupers :: PrimPatch prim
+    => [Sealed (FL (RepoPatchV1 prim) wX)] -> [Sealed (FL (RepoPatchV1 prim) wX)]
 getSupers (x:xs) =
     case filter (not.(x `isSuperpatchOf`)) xs of
     xs' -> if any (`isSuperpatchOf` x) xs'
@@ -401,10 +414,11 @@ getSupers (x:xs) =
            else x : getSupers xs'
 getSupers [] = []
 
-isSuperpatchOf :: PrimPatch prim => Sealed (FL (Patch prim) wX) -> Sealed (FL (Patch prim) wX) -> Bool
+isSuperpatchOf :: PrimPatch prim
+    => Sealed (FL (RepoPatchV1 prim) wX) -> Sealed (FL (RepoPatchV1 prim) wX) -> Bool
 Sealed x `isSuperpatchOf` Sealed y | lengthFL y > lengthFL x = False -- should be just an optimisation
 Sealed x `isSuperpatchOf` Sealed y = x `iso` y
-    where iso :: PrimPatch prim => FL (Patch prim) wX wY -> FL (Patch prim) wX wZ -> Bool
+    where iso :: PrimPatch prim => FL (RepoPatchV1 prim) wX wY -> FL (RepoPatchV1 prim) wX wZ -> Bool
           _ `iso` NilFL = True
           NilFL `iso` _ = False
           a `iso` (b:>:bs) =
@@ -414,7 +428,8 @@ Sealed x `isSuperpatchOf` Sealed y = x `iso` y
 -- constructs a Merger patch to represent the conflict. @p1@ is considered to
 -- be conflicting with @p2@ (@p1@ is the "first" patch in the repo ordering),
 -- the resulting Merger is therefore a representation of @p2@.
-merger :: PrimPatch prim => String -> Patch prim wX wY -> Patch prim wX wZ -> Sealed (Patch prim wY)
+merger :: PrimPatch prim
+    => String -> RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> Sealed (RepoPatchV1 prim wY)
 merger "0.0" p1 p2 = Sealed $ Merger undoit unwindings p1 p2
     where fake_p = Merger NilFL NilRL p1 p2
           unwindings = unsafeUnseal (trueUnwind fake_p)
@@ -422,7 +437,7 @@ merger "0.0" p1 p2 = Sealed $ Merger undoit unwindings p1 p2
           undoit =
               case (isMerger p1, isMerger p2) of
               (True ,True ) -> case unwind p of
-                                 Sealed (_:<:t) -> unsafeCoerceP $ invertRL t
+                                 Sealed (t:<:_) -> unsafeCoerceP $ invertRL t
                                  _ -> impossible
               (False,False) -> unsafeCoerceP $ invert p1 :>: NilFL
               (True ,False) -> unsafeCoerceP NilFL
@@ -431,22 +446,23 @@ merger g _ _ =
     error $ "Cannot handle mergers other than version 0.0\n"++g
     ++ "\nPlease use darcs optimize --modernize with an older darcs."
 
-glump09 :: PrimPatch prim => Patch prim wX wY -> Patch prim wX wZ -> Sealed (FL (Patch prim) wY)
+glump09 :: PrimPatch prim => RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> Sealed (FL (RepoPatchV1 prim) wY)
 glump09 p1 p2 = mapSeal (mapFL_FL fromPrim) $ mangleUnravelled $ unseal unravel $ merger "0.0" p1 p2
 
-instance PrimPatch prim => Effect (Patch prim) where
+instance PrimPatch prim => Effect (RepoPatchV1 prim) where
     effect p@(Merger{}) = sortCoalesceFL $ effect $ mergerUndo p
     effect p@(Regrem{}) = invert $ effect $ invert p
     effect (PP p) = p :>: NilFL
 
-instance IsHunk prim => IsHunk (Patch prim) where
+instance IsHunk prim => IsHunk (RepoPatchV1 prim) where
     isHunk p = do PP p' <- return p
                   isHunk p'
 
-newUr :: PrimPatch prim => Patch prim wA wB -> RL (Patch prim) wX wY -> [Sealed (RL (Patch prim) wX)]
-newUr p (Merger _ _ p1 p2 :<: ps) =
-   case filter (\(pp:<:_) -> pp `unsafeCompare` p1) $ headPermutationsRL ps of
-   ((_:<:ps'):_) -> newUr p (unsafeCoercePStart p1:<:ps') ++ newUr p (unsafeCoercePStart p2:<:ps')
+newUr :: PrimPatch prim
+    => RepoPatchV1 prim wA wB -> RL (RepoPatchV1 prim) wX wY -> [Sealed (RL (RepoPatchV1 prim) wX)]
+newUr p (ps :<: Merger _ _ p1 p2) =
+   case filter (\(_:<:pp) -> pp `unsafeCompare` p1) $ headPermutationsRL ps of
+   ((ps':<:_):_) -> newUr p (ps':<:unsafeCoercePStart p1) ++ newUr p (ps':<:unsafeCoercePStart p2)
    _ -> bugDoc $ text "in function newUr"
               $$ text "Original patch:"
               $$ showPatch_ p
@@ -454,24 +470,24 @@ newUr p (Merger _ _ p1 p2 :<: ps) =
               $$ vcat (unseal (mapRL showPatch_) $ unwind p)
 
 newUr op ps =
-    case filter (\(p:<:_) -> isMerger p) $ headPermutationsRL ps of
+    case filter (\(_:<:p) -> isMerger p) $ headPermutationsRL ps of
     [] -> [Sealed ps]
     (ps':_) -> newUr op ps'
 
-instance Invert prim => Invert (Patch prim) where
+instance Invert prim => Invert (RepoPatchV1 prim) where
     invert (Merger undo unwindings p1 p2)
         = Regrem undo unwindings p1 p2
     invert (Regrem undo unwindings p1 p2)
         = Merger undo unwindings p1 p2
     invert (PP p) = PP (invert p)
 
-instance MyEq prim => MyEq (Patch prim) where
+instance MyEq prim => MyEq (RepoPatchV1 prim) where
     unsafeCompare = eqPatches
 
-instance MyEq prim => Eq (Patch prim wX wY) where
+instance MyEq prim => Eq (RepoPatchV1 prim wX wY) where
     (==) = unsafeCompare
 
-eqPatches :: MyEq prim => Patch prim wX wY -> Patch prim wW wZ -> Bool
+eqPatches :: MyEq prim => RepoPatchV1 prim wX wY -> RepoPatchV1 prim wW wZ -> Bool
 eqPatches (PP p1) (PP p2) = unsafeCompare p1 p2
 eqPatches (Merger _ _ p1a p1b) (Merger _ _ p2a p2b)
  = eqPatches p1a p2a &&
