@@ -23,15 +23,18 @@ module Darcs.Patch.Viewing
     , showContextSeries
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Applicative( (<$>) )
 import qualified Data.ByteString as BS ( null )
 import Prelude hiding ( pi, readFile )
-import Storage.Hashed.Monad ( virtualTreeMonad )
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
+import Darcs.Util.Tree.Monad ( virtualTreeMonad )
 
 import Darcs.Patch.Apply ( Apply(..) )
-import Darcs.Patch.ApplyMonad ( ApplyMonadTrans, getApplyState,
-                                ApplyMonad(..), toTree )
+import Darcs.Patch.ApplyMonad ( getApplyState,
+                                ApplyMonad(..), ApplyMonadTree(..), toTree )
 import Darcs.Patch.FileHunk ( IsHunk(..), FileHunk(..), showFileHunk )
 import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..),
                             FileNameFormat(..) )
@@ -43,15 +46,11 @@ import Darcs.Util.Printer ( Doc, empty, vcat, text, blueText, Color(Cyan, Magent
                  lineColor, ($$), (<+>), prefix, userchunkPS )
 
 showContextSeries :: forall p m wX wY . (Apply p, ShowPatch p, IsHunk p,
-                                         ApplyMonadTrans m (ApplyState p),
-                                         ApplyMonad m (ApplyState p))
+                                         ApplyMonad (ApplyState p) m)
                   => FL p wX wY -> m Doc
 showContextSeries = scs Nothing
   where
-    scs :: forall m' wWw wXx wYy . (ApplyMonadTrans m' (ApplyState p),
-                                    ApplyMonad m' (ApplyState p),
-                                    ApplyMonadBase m ~ ApplyMonadBase m')
-        => Maybe (FileHunk wWw wXx) -> FL p wXx wYy -> m' Doc
+    scs :: forall wWw wXx wYy . Maybe (FileHunk wWw wXx) -> FL p wXx wYy -> m Doc
     scs pold (p :>: ps) = do
         (_, s') <- nestedApply (apply p) =<< getApplyState
         case isHunk p of
@@ -72,10 +71,10 @@ showContextSeries = scs Nothing
     cool pold fh ps s =
         fst <$> virtualTreeMonad (coolContextHunk pold fh ps) (toTree s)
 
-showContextHunk :: (ApplyMonad m Tree) => FileHunk wX wY -> m Doc
+showContextHunk :: (ApplyMonad Tree m) => FileHunk wX wY -> m Doc
 showContextHunk h = coolContextHunk Nothing h Nothing
 
-coolContextHunk :: (ApplyMonad m Tree, ApplyMonadTrans m Tree)
+coolContextHunk :: (ApplyMonad Tree m)
                 => Maybe (FileHunk wA wB) -> FileHunk wB wC
                 -> Maybe (FileHunk wC wD) -> m Doc
 coolContextHunk prev fh@(FileHunk f l o n) next = do
@@ -127,8 +126,7 @@ instance (Apply p, IsHunk p, PatchListFormat p, ShowPatch p)
         => ShowPatch (FL p) where
     showContextPatch = showContextPatchInternal patchListFormat
       where
-        showContextPatchInternal :: (ApplyMonadTrans m (ApplyState p),
-                                     ApplyMonad m (ApplyState (FL p)))
+        showContextPatchInternal :: (ApplyMonad (ApplyState (FL p)) m)
                                  => ListFormat p -> FL p wX wY -> m Doc
         showContextPatchInternal ListFormatV1 (p :>: NilFL) =
             showContextPatch p
