@@ -15,6 +15,9 @@ module Darcs.Patch.Witnesses.Show
     , appPrec
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Util.Show ( appPrec )
 
 data ShowDict a where
