@@ -44,13 +44,13 @@ import System.Posix.Files ( createLink, linkCount, getSymbolicLinkStatus )
 
 import Darcs.Util.ByteString ( gzWriteFilePS, linesPS )
 import Darcs.Util.Global ( darcsdir, addBadSource, isBadSource, addReachableSource,
-                      isReachableSource, getBadSourcesList )
-import Darcs.Repository.External ( gzFetchFilePS, fetchFilePS,
-                                   speculateFileOrUrl, copyFileOrUrl,
-                                   Cachable( Cachable ) )
-import Darcs.Repository.Flags ( Compression(..), RemoteDarcs(..) )
-import Darcs.Repository.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS,
-                               withTemp )
+                      isReachableSource, getBadSourcesList, defaultRemoteDarcsCmd )
+import Darcs.Util.External ( gzFetchFilePS, fetchFilePS
+                           , speculateFileOrUrl, copyFileOrUrl
+                           , Cachable( Cachable ) )
+import Darcs.Repository.Flags ( Compression(..) )
+import Darcs.Util.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS,
+                         withTemp )
 import Darcs.Util.SignalHandler ( catchNonSignal )
 import Darcs.Util.URL ( isValidLocalPath, isHttpUrl, isSshUrl )
 import Darcs.Util.File ( withCurrentDirectory )
@@ -318,7 +318,7 @@ copyFileUsingCache oos (Ca cache) subdir f = do
                      `catchNonSignal`
                      \e -> checkCacheReachability (show e) c
                 else do debugMessage $ "Copying from " ++ show cacheFile ++ " to  " ++ show out
-                        copyFileOrUrl DefaultRemoteDarcs cacheFile out Cachable
+                        copyFileOrUrl defaultRemoteDarcsCmd cacheFile out Cachable
                      `catchNonSignal`
                      (\e -> do checkCacheReachability (show e) c
                                sfuc out cs) -- try another read-only location
@@ -382,7 +382,7 @@ filterBadSources cache = do
 checkHashedInventoryReachability :: CacheLoc -> IO Bool
 checkHashedInventoryReachability cache = withTemp $ \tempout -> do
     let f = cacheSource cache </> darcsdir </> "hashed_inventory"
-    copyFileOrUrl DefaultRemoteDarcs f tempout Cachable
+    copyFileOrUrl defaultRemoteDarcsCmd f tempout Cachable
     return True
     `catchNonSignal` const (return False)
 
