@@ -13,6 +13,7 @@ import Darcs.Prelude
 import Control.Exception ( catch, SomeException )
 import Control.Monad ( when, void )
 import qualified Data.ByteString.Char8 as BS
+import Data.List( intercalate )
 import Data.Maybe( catMaybes, isJust )
 import System.FilePath( (</>) )
 import System.Directory
@@ -184,6 +185,9 @@ createRepository patchfmt withWorkingDir createPatchIndex = do
       NoPatchIndex -> return () -- default
       YesPatchIndex -> createOrUpdatePatchIndexDisk repo
 
+joinUrl :: [String] -> String
+joinUrl = intercalate "/"
+
 cloneRepository ::
     String    -- origin repository path
     -> String -- new repository name (for relative path)
@@ -218,7 +222,7 @@ cloneRepository repodir mysimplename v uc cloneKind um rdarcs sse remoteRepos
         Just e ->  fail $ "Incompatibility with repository " ++ fromDir ++ ":\n" ++ e
         Nothing -> return ()
       debugMessage "Copying prefs"
-      copyFileOrUrl (remoteDarcs rdarcs) (fromDir </> darcsdir </> "prefs" </> "prefs")
+      copyFileOrUrl (remoteDarcs rdarcs) (joinUrl [fromDir, darcsdir, "prefs", "prefs"])
         (darcsdir </> "prefs/prefs") (MaxAge 600) `catchall` return ()
       -- prepare sources and cache
       (Repo toDir toFormat toPristine toCache) <- identifyRepository uc "."
@@ -305,9 +309,9 @@ copyBasicRepoPacked ::
   -> WithWorkingDir
   -> IO ()
 copyBasicRepoPacked r@(Repo fromDir _ _ _) toRepo verb rdarcs withWorkingDir =
-  do let hashURL = fromDir </> darcsdir </> packsDir </> "pristine"
+  do let hashURL = joinUrl [fromDir, darcsdir, packsDir, "pristine"]
      mPackHash <- (Just <$> gzFetchFilePS hashURL Uncachable) `catchall` (return Nothing)
-     let hiURL = fromDir </> darcsdir </> "hashed_inventory"
+     let hiURL = joinUrl [fromDir, darcsdir, "hashed_inventory"]
      i <- gzFetchFilePS hiURL Uncachable
      let currentHash = BS.pack $ inv2pris i
      let copyNormally = copyBasicRepoNotPacked r toRepo verb rdarcs withWorkingDir
