@@ -19,6 +19,9 @@ module Darcs.Repository.Format
     , removeFromFormat
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 #include "impossible.h"
 
 import Control.Monad ( mplus, (<=<) )
@@ -27,13 +30,13 @@ import qualified Data.ByteString  as B ( null, empty )
 import Data.List ( partition, intercalate, (\\) )
 import Data.Maybe ( isJust, mapMaybe )
 
-import Darcs.Repository.External
+import Darcs.Util.External
     ( fetchFilePS
     , Cachable( Cachable )
     )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Repository.Lock ( writeBinFile )
-import qualified Darcs.Repository.Flags as F ( WithWorkingDir (..) )
+import Darcs.Util.Lock ( writeBinFile )
+import qualified Darcs.Repository.Flags as F ( WithWorkingDir (..), PatchFormat (..)  )
 import Darcs.Util.SignalHandler ( catchNonSignal )
 import Darcs.Util.Exception ( catchall, prettyException )
 
@@ -148,17 +151,18 @@ tryIdentifyRepoFormat repo = do
 writeRepoFormat :: RepoFormat -> FilePath -> IO ()
 writeRepoFormat rf loc = writeBinFile loc $ show rf
 
--- | @createRepoFormat useFormat1 useNoWorkingDir@ create a repo format
-createRepoFormat :: Bool -> F.WithWorkingDir -> RepoFormat
-createRepoFormat useFormat1 wwd = RF $ (HashedInventory : flags2wd wwd) : flags2format
+-- | Create a repo format. The first argument is whether to use the old (darcs-1)
+-- format; the second says whether the repo has a working tree.
+createRepoFormat :: F.PatchFormat -> F.WithWorkingDir -> RepoFormat
+createRepoFormat fmt wwd = RF $ (HashedInventory : flags2wd wwd) : flags2format fmt
   where
-    flags2format = if useFormat1 then [] else [[Darcs2]]
-    flags2wd F.NoWorkingDir = [NoWorkingDir]
-    flags2wd _              = []
+    flags2format F.PatchFormat1 = []
+    flags2format F.PatchFormat2 = [[Darcs2]]
+    flags2wd F.NoWorkingDir   = [NoWorkingDir]
+    flags2wd F.WithWorkingDir = []
 
--- | @writeProblem form@ tells if we can write to a repo in format @form@,
--- first checking if we can read that repo It returns @Nothing@ if there's no
--- problem writing to such a repository.
+-- | @'writeProblem' source@ returns 'Just' an error message if we cannot write
+-- to a repo in format @source@, or 'Nothing' if there's no such problem.
 writeProblem :: RepoFormat -> Maybe String
 writeProblem target = readProblem target `mplus` findProblems target wp
   where
