@@ -25,21 +25,23 @@ module Darcs.Repository.HashedIO ( HashedIO,
                                  , pathsAndContents
                                  ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Util.Global ( darcsdir )
 import qualified Data.Set as Set
 import System.Directory ( getDirectoryContents, createDirectoryIfMissing )
 import Control.Monad.State ( StateT, runStateT, modify, get, put, gets, lift, evalStateT )
 import Control.Monad ( when, void, unless )
-import Control.Applicative ( (<$>) )
 import Data.Maybe ( isJust )
 import System.IO.Unsafe ( unsafeInterleaveIO )
 
 import Darcs.Repository.Cache ( Cache(..), fetchFileUsingCache, writeFileUsingCache,
                                 peekInCache, speculateFileUsingCache,
                                 okayHash, cleanCachesWithHint, HashedDir(..), hashedDir )
-import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
+import Darcs.Patch.ApplyMonad ( ApplyMonad(..), ApplyMonadTree(..) )
 import Darcs.Repository.Flags ( Compression( .. ), WithWorkingDir (..) )
-import Darcs.Repository.Lock ( writeAtomicFilePS, removeFileMayNotExist )
+import Darcs.Util.Lock ( writeAtomicFilePS, removeFileMayNotExist )
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Progress ( debugMessage, tediousSize, finishedOneIO )
 import Darcs.Util.Path
@@ -61,9 +63,9 @@ import Darcs.Util.ByteString ( linesPS, unlinesPS )
 import qualified Data.ByteString       as B  (ByteString, length, empty)
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 
-import Storage.Hashed.Darcs( readDarcsHashedDir, darcsLocation,
+import Darcs.Util.Tree.Hashed( readDarcsHashedDir, darcsLocation,
                              decodeDarcsHash, decodeDarcsSize )
-import Storage.Hashed.Tree( ItemType(..), Tree )
+import Darcs.Util.Tree( ItemType(..), Tree )
 
 -- | @readHashFile c subdir hash@ reads the file with hash @hash@ in dir subdir,
 -- fetching it from 'Cache' @c@ if needed.
@@ -119,8 +121,10 @@ mInCurrentDirectory fn j | fn' == fp2fn "" = j
                                                    Just h -> inh h $ mInCurrentDirectory fn'' j
     where fn' = normPath fn
 
-instance ApplyMonad (HashedIO p) Tree where
+instance ApplyMonad Tree (HashedIO p) where
     type ApplyMonadBase (HashedIO p) = IO
+
+instance ApplyMonadTree (HashedIO p) where
     mDoesDirectoryExist fn = do thing <- identifyThing fn
                                 case thing of Just (D,_) -> return True
                                               _ -> return False
@@ -332,4 +336,4 @@ getHashedFiles hashdir hashroots = do
                              hashes = h : [ fst $ darcsLocation "" (s, h') | (_, _, s, h') <- x ]
                          (hashes++) . concat <$> mapM listone subs
       hs <- concat <$> mapM listone hashroots
-      return hs
\ No newline at end of file
+      return hs
