@@ -41,7 +41,6 @@ module Darcs.Repository.HashedRepo
     , readHashedPristineRoot
     , pris2inv
     , inv2pris
-    , copySources
     , listInventories
     , listInventoriesLocal
     , listInventoriesRepoDir
@@ -55,9 +54,9 @@ module Darcs.Repository.HashedRepo
 
 #include "impossible.h"
 
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
 
-import Control.Applicative ( (<$>) )
 import Control.Arrow ( (&&&) )
 import Control.Exception ( catch, IOException )
 import Control.Monad ( unless )
@@ -66,13 +65,12 @@ import qualified Data.ByteString as B ( null, length, empty ,tail, drop,
                                         ByteString, splitAt )
 import qualified Data.ByteString.Char8 as BC ( unpack, dropWhile, break, pack,
                                                 ByteString )
-import Data.List ( delete )
 import qualified Data.Set as Set
-import Storage.Hashed.Darcs( hashedTreeIO, readDarcsHashedNosize,
+import Darcs.Util.Hash( encodeBase16, Hash(..) )
+import Darcs.Util.Tree( treeHash, Tree )
+import Darcs.Util.Tree.Hashed( hashedTreeIO, readDarcsHashedNosize,
                              readDarcsHashed, writeDarcsHashed,
                              decodeDarcsHash, decodeDarcsSize )
-import Storage.Hashed.Tree( treeHash, Tree )
-import Storage.Hashed.Hash( encodeBase16, Hash(..) )
 import System.Directory ( createDirectoryIfMissing, getDirectoryContents
                         , doesFileExist, doesDirectoryExist )
 import System.FilePath.Posix( (</>) )
@@ -80,16 +78,16 @@ import System.IO.Unsafe ( unsafeInterleaveIO )
 import System.IO ( stderr, hPutStrLn )
 
 import Darcs.Util.Printer.Color ( showDoc )
-import Darcs.Repository.External
+import Darcs.Util.External
     ( copyFileOrUrl
     , cloneFile
     , fetchFilePS
     , gzFetchFilePS
     , Cachable( Uncachable )
     )
-import Darcs.Repository.Flags ( Compression, RemoteDarcs, WithWorkingDir )
+import Darcs.Repository.Flags ( Compression, RemoteDarcs, remoteDarcs, WithWorkingDir )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Repository.Lock
+import Darcs.Util.Lock
     ( writeBinFile
     , writeDocBinFile
     , writeAtomicFilePS
@@ -101,27 +99,24 @@ import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, Origin )
 
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, patchInfoAndPatch, info,
                                   extractHash, createHashed )
-import Darcs.Patch ( RepoPatch, Patchy, showPatch, readPatch, apply )
+import Darcs.Patch ( IsRepoType, RepoPatch, Patchy, showPatch, readPatch, apply )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Read ( ReadPatch )
 import Darcs.Patch.ReadMonads ( parseStrictly )
-import Darcs.Patch.Depends ( commuteToEnd, slightlyOptimizePatchset )
+import Darcs.Patch.Depends ( removeFromPatchSet, slightlyOptimizePatchset )
 import Darcs.Patch.Info ( PatchInfo, showPatchInfo, showPatchInfoUI,
                           readPatchInfo )
 import Darcs.Util.Path ( FilePathLike, ioAbsoluteOrRemote, toPath )
-import Darcs.Repository.Cache ( Cache(..), CacheLoc(..), fetchFileUsingCache,
+import Darcs.Repository.Cache ( Cache(..), fetchFileUsingCache,
                                 speculateFilesUsingCache, writeFileUsingCache,
-                                unionCaches, repo2cache, okayHash, takeHash,
+                                okayHash, takeHash,
                                 HashedDir(..), hashedDir, peekInCache, bucketFolder )
-import qualified Darcs.Repository.Cache as DarcsCache
 import Darcs.Repository.HashedIO ( copyHashed, copyPartialsHashed,
                                    cleanHashdir )
-import Darcs.Repository.InternalTypes ( Repository(..), extractCache,
-                                        modifyCache )
+import Darcs.Repository.InternalTypes ( Repository(..), extractCache )
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Patch.Witnesses.Ordered
-    ( reverseRL, reverseFL, (+<+), FL(..), RL(..),
-    (:>)(..), mapRL, mapFL )
+    ( (+<+), FL(..), RL(..), mapRL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), seal, unseal, mapSeal )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 
@@ -224,8 +219,8 @@ revertTentativeChanges = do
 -- then writes out the new tentative inventory, and finally does the atomic
 -- swap. In general, we can't clean the pristine cache at the same time, since
 -- a simultaneous get might be in progress.
-finalizeTentativeChanges :: (RepoPatch p, ApplyState p ~ Tree)
-                         => Repository p wR wU wT -> Compression -> IO ()
+finalizeTentativeChanges :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                         => Repository rt p wR wU wT -> Compression -> IO ()
 finalizeTentativeChanges r compr = do
     debugMessage "Optimizing the inventory..."
     -- Read the tentative patches
@@ -240,7 +235,7 @@ finalizeTentativeChanges r compr = do
 
 -- |readHashedPristineRoot attempts to read the pristine hash from the current
 -- inventory, returning Nothing if it cannot do so.
-readHashedPristineRoot :: Repository p wR wU wT -> IO (Maybe String)
+readHashedPristineRoot :: Repository rt p wR wU wT -> IO (Maybe String)
 readHashedPristineRoot (Repo d _ _ _) = withCurrentDirectory d $ do
     i <- (Just <$> gzReadFilePS hashedInventoryPath)
          `catch` (\(_ :: IOException) -> return Nothing)
@@ -248,7 +243,7 @@ readHashedPristineRoot (Repo d _ _ _) = withCurrentDirectory d $ do
 
 -- |cleanPristine removes any obsolete (unreferenced) entries in the pristine
 -- cache.
-cleanPristine :: Repository p wR wU wT -> IO ()
+cleanPristine :: Repository rt p wR wU wT -> IO ()
 cleanPristine r@(Repo d _ _ _) = withCurrentDirectory d $ do
     debugMessage "Cleaning out the pristine cache..."
     i <- gzReadFilePS hashedInventoryPath
@@ -278,7 +273,7 @@ unset = map BC.unpack . Set.toList
 
 -- |cleanInventories removes any obsolete (unreferenced) files in the
 -- inventories directory.
-cleanInventories :: Repository p wR wU wT -> IO ()
+cleanInventories :: Repository rt p wR wU wT -> IO ()
 cleanInventories _ = do
     debugMessage "Cleaning out inventories..."
     hs <- listInventoriesLocal
@@ -293,7 +288,7 @@ specialPatches = ["unrevert", "pending", "pending.tentative"]
 
 -- |cleanPatches removes any obsolete (unreferenced) files in the
 -- patches directory.
-cleanPatches :: Repository p wR wU wT -> IO ()
+cleanPatches :: Repository rt p wR wU wT -> IO ()
 cleanPatches _ = do
     debugMessage "Cleaning out patches..."
     hs <- listPatchesLocal darcsdir
@@ -305,7 +300,7 @@ cleanPatches _ = do
 -- |addToSpecificInventory adds a patch to a specific inventory file, and
 -- returns the FilePath whichs corresponds to the written-out patch.
 addToSpecificInventory :: RepoPatch p => String -> Cache -> Compression
-                       -> PatchInfoAnd p wX wY -> IO FilePath
+                       -> PatchInfoAnd rt p wX wY -> IO FilePath
 addToSpecificInventory invPath c compr p = do
     let invFile = darcsdir </> invPath
     hash <- snd <$> writePatchIfNecessary c compr p
@@ -314,44 +309,28 @@ addToSpecificInventory invPath c compr p = do
     return $ darcsdir </> "patches" </> hash
 
 addToTentativeInventory :: RepoPatch p => Cache -> Compression
-                        -> PatchInfoAnd p wX wY -> IO FilePath
+                        -> PatchInfoAnd rt p wX wY -> IO FilePath
 addToTentativeInventory = addToSpecificInventory tentativeHashedInventory
 
--- |removeFromTentativeInventory attempts to remove an FL of patches from the
--- tentative inventory. This is used for commands that wish to modify
--- already-recorded patches.
-removeFromTentativeInventory :: (RepoPatch p, ApplyState p ~ Tree)
-                             => Repository p wR wU wT -> Compression
-                             -> FL (PatchInfoAnd p) wX wT -> IO ()
+-- | Attempt to remove an FL of patches from the tentative inventory.
+-- This is used for commands that wish to modify already-recorded patches.
+--
+-- Precondition: it must be possible to remove the patches, i.e.
+--
+-- * the patches are in the repository
+--
+-- * any necessary commutations will succeed
+removeFromTentativeInventory :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                             => Repository rt p wR wU wT -> Compression
+                             -> FL (PatchInfoAnd rt p) wX wT -> IO ()
 removeFromTentativeInventory repo compr to_remove = do
-    -- FIXME: This algorithm should be *far* simpler.  All we need do is to to
-    -- remove the patches from a patchset and then write that patchset.  The
-    -- commutation behavior of PatchInfoAnd should track which patches need to
-    -- be rewritten for us.
+    debugMessage $ "Start removeFromTentativeInventory"
     allpatches <- readTentativeRepo repo "."
-    _ :> skipped <- return $ commuteToEnd (reverseFL to_remove) allpatches
-    okay <- simpleRemoveFromTentativeInventory $
-                mapFL info to_remove ++ mapRL info skipped
-    unless okay $ bug "bug in HashedRepo.removeFromTentativeInventory"
-    sequence_ $ mapFL (addToTentativeInventory (extractCache repo) compr)
-                    (reverseRL skipped)
-  where
-    simpleRemoveFromTentativeInventory :: [PatchInfo] -> IO Bool
-    simpleRemoveFromTentativeInventory pis = do
-        inv <- readTentativeRepo repo "."
-        case cut_inv pis inv of
-            Nothing -> return False
-            Just (Sealed inv') -> do
-                writeTentativeInventory (extractCache repo) compr inv'
-                return True
-    cut_inv :: [PatchInfo] -> PatchSet p wStart wX
-            -> Maybe (SealedPatchSet p wStart)
-    cut_inv [] x = Just $ seal x
-    cut_inv x (PatchSet NilRL (Tagged t _ ps :<: ts)) =
-        cut_inv x (PatchSet (t :<: ps) ts)
-    cut_inv xs (PatchSet (hp:<:r) ts) | info hp `elem` xs =
-        cut_inv (info hp `delete` xs) (PatchSet r ts)
-    cut_inv _ _ = Nothing
+    remaining <- case removeFromPatchSet to_remove allpatches of
+        Nothing -> bug "HashedRepo.removeFromTentativeInventory: precondition violated"
+        Just r -> return r
+    writeTentativeInventory (extractCache repo) compr remaining
+    debugMessage $ "Done removeFromTentativeInventory"
 
 -- |writeHashFile takes a Doc and writes it as a hash-named file, returning the
 -- filename that the contents were written to.
@@ -361,21 +340,21 @@ writeHashFile c compr subdir d = do
     writeFileUsingCache c compr subdir $ renderPS Standard d
 
 -- |readRepo returns the "current" repo patchset.
-readRepo :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT
-         -> String -> IO (PatchSet p Origin wR)
+readRepo :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT
+         -> String -> IO (PatchSet rt p Origin wR)
 readRepo = readRepoUsingSpecificInventory hashedInventory
 
 -- |readRepo returns the tentative repo patchset.
-readTentativeRepo :: (RepoPatch p, ApplyState p ~ Tree)
-                  => Repository p wR wU wT -> String
-                  -> IO (PatchSet p Origin wT)
+readTentativeRepo :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                  => Repository rt p wR wU wT -> String
+                  -> IO (PatchSet rt p Origin wT)
 readTentativeRepo = readRepoUsingSpecificInventory tentativeHashedInventory
 
 -- |readRepoUsingSpecificInventory uses the inventory at @invPath@ to read the
 -- repository @repo@.
-readRepoUsingSpecificInventory :: (RepoPatch p, ApplyState p ~ Tree)
-                               => String -> Repository p wR wU wT
-                               -> String -> IO (PatchSet p Origin wS)
+readRepoUsingSpecificInventory :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                               => String -> Repository rt p wR wU wT
+                               -> String -> IO (PatchSet rt p Origin wS)
 readRepoUsingSpecificInventory invPath repo dir = do
     realdir <- toPath <$> ioAbsoluteOrRemote dir
     Sealed ps <- readRepoPrivate (extractCache repo) realdir invPath
@@ -384,8 +363,8 @@ readRepoUsingSpecificInventory invPath repo dir = do
                      ioError e
     return $ unsafeCoerceP ps
   where
-    readRepoPrivate :: (RepoPatch p, ApplyState p ~ Tree) => Cache -> FilePath
-                    -> FilePath -> IO (SealedPatchSet p Origin)
+    readRepoPrivate :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Cache -> FilePath
+                    -> FilePath -> IO (SealedPatchSet rt p Origin)
     readRepoPrivate cache d iname = do
       inventory <- readInventoryPrivate (d </> darcsdir) iname
       readRepoFromInventoryList cache inventory
@@ -393,35 +372,35 @@ readRepoUsingSpecificInventory invPath repo dir = do
 -- |readRepoFromInventoryList allows the caller to provide an optional "from
 -- inventory" hash, and a list of info/hash pairs that identify a list of
 -- patches, returning a patchset of the resulting repo.
-readRepoFromInventoryList :: (RepoPatch p, ApplyState p ~ Tree) => Cache
+readRepoFromInventoryList :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Cache
                           -> (Maybe String, [(PatchInfo, String)])
-                          -> IO (SealedPatchSet p Origin)
+                          -> IO (SealedPatchSet rt p Origin)
 readRepoFromInventoryList cache = parseinvs
   where
     speculateAndParse h is i = speculate h is >> parse i h
 
-    read_patches :: (RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
-                 -> IO (Sealed (RL (PatchInfoAnd p) wX))
+    read_patches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
+                 -> IO (Sealed (RL (PatchInfoAnd rt p) wX))
     read_patches [] = return $ seal NilRL
     read_patches allis@((i1, h1) : is1) =
-        lift2Sealed (\p rest -> i1 `patchInfoAndPatch` p :<: rest) (rp is1)
+        lift2Sealed (\p rest -> rest :<: i1 `patchInfoAndPatch` p) (rp is1)
                     (createHashed h1 (const $ speculateAndParse h1 allis i1))
       where
-        rp :: (RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
-           -> IO (Sealed (RL (PatchInfoAnd p) wX))
+        rp :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => [(PatchInfo, String)]
+           -> IO (Sealed (RL (PatchInfoAnd rt p) wX))
         rp [] = return $ seal NilRL
         rp [(i, h), (il, hl)] =
-            lift2Sealed (\p rest -> i `patchInfoAndPatch` p :<: rest)
+            lift2Sealed (\p rest -> rest :<: i `patchInfoAndPatch` p)
                         (rp [(il, hl)])
                         (createHashed h
                             (const $ speculateAndParse h (reverse allis) i))
         rp ((i, h) : is) =
-            lift2Sealed (\p rest -> i `patchInfoAndPatch` p :<: rest)
+            lift2Sealed (\p rest -> rest :<: i `patchInfoAndPatch` p)
                         (rp is)
                         (createHashed h (parse i))
 
-    read_tag :: (RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String)
-             -> IO (Sealed (PatchInfoAnd p wX))
+    read_tag :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String)
+             -> IO (Sealed (PatchInfoAnd rt p wX))
     read_tag (i, h) =
         mapSeal (patchInfoAndPatch i) <$> createHashed h (parse i)
 
@@ -441,21 +420,21 @@ readRepoFromInventoryList cache = parseinvs
                                       , "which is patch"
                                       , renderString Encode $ showPatchInfoUI i ]
 
-    parseinvs :: (RepoPatch p, ApplyState p ~ Tree)
+    parseinvs :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
               => (Maybe String, [(PatchInfo, String)])
-              -> IO (SealedPatchSet p Origin)
+              -> IO (SealedPatchSet rt p Origin)
     parseinvs (Nothing, ris) =
-        mapSeal (flip PatchSet NilRL) <$> read_patches (reverse ris)
+        mapSeal (PatchSet NilRL) <$> read_patches (reverse ris)
     parseinvs (Just h, []) =
         bug $ "bad inventory " ++ h ++ " (no tag) in parseinvs!"
     parseinvs (Just h, t : ris) = do
         Sealed ts <- unseal seal <$> unsafeInterleaveIO (read_ts t h)
         Sealed ps <- unseal seal <$>
                         unsafeInterleaveIO (read_patches $ reverse ris)
-        return $ seal $ PatchSet ps ts
+        return $ seal $ PatchSet ts ps
 
-    read_ts :: (RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String)
-            -> String -> IO (Sealed (RL (Tagged p) Origin))
+    read_ts :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => (PatchInfo, String)
+            -> String -> IO (Sealed (RL (Tagged rt p) Origin))
     read_ts tag0 h0 = do
         contents <- unsafeInterleaveIO $ readTaggedInventoryFromHash h0
         let is = reverse $ case contents of
@@ -470,7 +449,7 @@ readRepoFromInventoryList cache = parseinvs
                                  (Nothing, _) -> return $ seal NilRL)
         Sealed ps <- unseal seal <$> unsafeInterleaveIO (read_patches is)
         Sealed tag00 <- read_tag tag0
-        return $ seal $ Tagged tag00 (Just h0) ps :<: ts
+        return $ seal $ ts :<: Tagged tag00 (Just h0) ps
 
     readTaggedInventoryFromHash :: String
                                 -> IO (Maybe String, [(PatchInfo, String)])
@@ -515,33 +494,18 @@ readInventoryFromContent fileName pristineAndInventory = do
 
 -- |copyRepo copies the hashed inventory of @repo@ to the repository located at
 -- @remote@.
-copyHashedInventory :: RepoPatch p => Repository p wR wU wT -> RemoteDarcs -> String -> IO ()
-copyHashedInventory (Repo outr _ _ _) remote inr = do
+copyHashedInventory :: RepoPatch p => Repository rt p wR wU wT -> RemoteDarcs -> String -> IO ()
+copyHashedInventory (Repo outr _ _ _) rdarcs inr | remote <- remoteDarcs rdarcs = do
     createDirectoryIfMissing False (outr ++ "/" ++ inventoriesDirPath)
     copyFileOrUrl remote (inr </> darcsdir </> hashedInventory)
                          (outr </> darcsdir </> hashedInventory)
                   Uncachable -- no need to copy anything but hashed_inventory!
     debugMessage "Done copying hashed inventory."
 
--- |'copySources' copies the prefs/sources file to the local repo, from the
--- remote, having first filtered the local filesystem sources.
-copySources :: RepoPatch p => Repository p wR wU wT -> String -> IO ()
-copySources repo@(Repo outr _ _ _) inr = do
-    let repoCache = extractCache $ modifyCache repo dropNonRepos
-    appendBinFile (outr ++ "/" ++ darcsdir ++ "/prefs/sources")
-                  (show $ repo2cache inr `unionCaches` repoCache )
-    debugMessage "Done copying and filtering pref/sources."
-  where
-    dropNonRepos (Ca cache) = Ca $ filter notRepo cache
-    notRepo xs = case xs of
-        Cache DarcsCache.Directory _ _ -> False
-        Cache _ DarcsCache.Writable _  -> False
-        _                              -> True
-
 -- |writeAndReadPatch makes a patch lazy, by writing it out to disk (thus
 -- forcing it), and then re-reads the patch lazily.
-writeAndReadPatch :: RepoPatch p => Cache -> Compression
-                  -> PatchInfoAnd p wX wY -> IO (PatchInfoAnd p wX wY)
+writeAndReadPatch :: (IsRepoType rt, RepoPatch p) => Cache -> Compression
+                  -> PatchInfoAnd rt p wX wY -> IO (PatchInfoAnd rt p wX wY)
 writeAndReadPatch c compr p = do
     (i, h) <- writePatchIfNecessary c compr p
     unsafeInterleaveIO $ readp h i
@@ -560,7 +524,7 @@ writeAndReadPatch c compr p = do
 
 -- | writeTentativeInventory writes @patchSet@ as the tentative inventory.
 writeTentativeInventory :: RepoPatch p => Cache -> Compression
-                        -> PatchSet p Origin wX -> IO ()
+                        -> PatchSet rt p Origin wX -> IO ()
 writeTentativeInventory cache compr patchSet = do
     debugMessage "in writeTentativeInventory..."
     createDirectoryIfMissing False inventoriesDirPath
@@ -575,20 +539,20 @@ writeTentativeInventory cache compr patchSet = do
             writeAtomicFilePS (darcsdir </> tentativeHashedInventory) content
   where
     tediousName = "Writing inventory"
-    writeInventoryPrivate :: RepoPatch p => PatchSet p Origin wX
+    writeInventoryPrivate :: RepoPatch p => PatchSet rt p Origin wX
                           -> IO (Maybe String)
     writeInventoryPrivate (PatchSet NilRL NilRL) = return Nothing
-    writeInventoryPrivate (PatchSet ps NilRL) = do
+    writeInventoryPrivate (PatchSet NilRL ps) = do
         inventory <- sequence $ mapRL (writePatchIfNecessary cache compr) ps
         let inventorylist = hcat (map pihash $ reverse inventory)
         hash <- writeHashFile cache compr HashedInventoriesDir inventorylist
         return $ Just hash
     writeInventoryPrivate
-        (PatchSet x xs@(Tagged t _ _ :<: _)) = do
+        (PatchSet xs@(_ :<: Tagged t _ _) x) = do
         resthash <- write_ts xs
         finishedOneIO tediousName $ fromMaybe "" resthash
         inventory <- sequence $ mapRL (writePatchIfNecessary cache compr)
-                                    (x +<+ t :<: NilRL)
+                                    (NilRL :<: t +<+ x)
         let inventorylist = hcat (map pihash $ reverse inventory)
             inventorycontents =
                 case resthash of
@@ -600,18 +564,18 @@ writeTentativeInventory cache compr patchSet = do
       where
         -- | write_ts writes out a tagged patchset. If it has already been
         -- written, we'll have the hash, so we can immediately return it.
-        write_ts :: RepoPatch p => RL (Tagged p) Origin wX
+        write_ts :: RepoPatch p => RL (Tagged rt p) Origin wX
                  -> IO (Maybe String)
-        write_ts (Tagged _ (Just h) _ :<: _) = return (Just h)
-        write_ts (Tagged _ Nothing pps :<: tts) =
-            writeInventoryPrivate $ PatchSet pps tts
+        write_ts (_ :<: Tagged _ (Just h) _) = return (Just h)
+        write_ts (tts :<: Tagged _ Nothing pps) =
+            writeInventoryPrivate $ PatchSet tts pps
         write_ts NilRL = return Nothing
 
 -- |writeHashIfNecessary writes the patch and returns the resulting info/hash,
 -- if it has not already been written. If it has been written, we have the hash
 -- in the PatchInfoAnd, so we extract and return the info/hash.
 writePatchIfNecessary :: RepoPatch p => Cache -> Compression
-                      -> PatchInfoAnd p wX wY -> IO (PatchInfo, String)
+                      -> PatchInfoAnd rt p wX wY -> IO (PatchInfo, String)
 writePatchIfNecessary c compr hp = infohp `seq`
     case extractHash hp of
         Right h -> return (infohp, h)
