@@ -21,10 +21,13 @@ module Darcs.Repository.InternalTypes ( Repository(..), Pristine(..)
                                       , extractCache, modifyCache
                                       ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.List ( nub, sortBy )
 import Darcs.Repository.Cache ( Cache (..) , compareByLocality )
 import Darcs.Repository.Format ( RepoFormat )
-import Darcs.Patch ( RepoPatch )
+import Darcs.Patch ( RepoPatch, RepoType )
 
 data Pristine
   = NoPristine
@@ -38,15 +41,15 @@ data Pristine
 -- the unrecorded state (what's in the working directory now),
 -- and the tentative state, which represents work in progress that will
 -- eventually become the new recorded state unless something goes wrong.
-data Repository (p :: * -> * -> *) wRecordedstate wUnrecordedstate wTentativestate =
+data Repository (rt :: RepoType) (p :: * -> * -> *) wRecordedstate wUnrecordedstate wTentativestate =
   Repo !String !RepoFormat !Pristine Cache deriving ( Show )
 
-extractCache :: Repository p wR wU wT -> Cache
+extractCache :: Repository rt p wR wU wT -> Cache
 extractCache (Repo _ _ _ c) = c
 
 -- | 'modifyCache' @repository function@ modifies the cache of
 --   @repository@ with @function@, remove duplicates and sort the results with 'compareByLocality'.
-modifyCache :: forall p wR wU wT . (RepoPatch p)  => Repository p wR wU wT -> (Cache -> Cache) -> Repository p wR wU wT
+modifyCache :: forall rt p wR wU wT . (RepoPatch p)  => Repository rt p wR wU wT -> (Cache -> Cache) -> Repository rt p wR wU wT
 modifyCache (Repo dir rf pristine cache) f
    = Repo dir rf pristine $ cmap ( sortBy compareByLocality . nub ) $ f cache
   where cmap g (Ca c) = Ca (g c)
