@@ -20,11 +20,13 @@
 module Darcs.Repository.Match
     (
       getNonrangeMatch
-    , getPartialNonrangeMatch
     , getFirstMatch
     , getOnePatchset
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Match
     ( getNonrangeMatchS
     , getFirstMatchS
@@ -40,7 +42,7 @@ import Darcs.Patch.Match
 import Darcs.Patch.Bundle ( scanContextFile )
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
 import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch ( RepoPatch )
+import Darcs.Patch ( RepoPatch, IsRepoType )
 import Darcs.Patch.Set ( PatchSet(..), SealedPatchSet, Origin )
 import Darcs.Patch.Witnesses.Sealed ( seal )
 
@@ -50,14 +52,14 @@ import Darcs.Repository.ApplyPatches ( DefaultIO, runDefault )
 import Darcs.Repository.Internal
     ( Repository, readRepo, createPristineDirectoryTree )
 
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 
-import Darcs.Util.Path ( FileName, toFilePath )
+import Darcs.Util.Path ( toFilePath )
 
 #include "impossible.h"
 
-getNonrangeMatch :: (ApplyMonad DefaultIO (ApplyState p), RepoPatch p, ApplyState p ~ Tree)
-                 => Repository p wR wU wT
+getNonrangeMatch :: (ApplyMonad (ApplyState p) DefaultIO, IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                 => Repository rt p wR wU wT
                  -> [MatchFlag]
                  -> IO ()
 getNonrangeMatch r = withRecordedMatch r . getMatch where
@@ -66,24 +68,16 @@ getNonrangeMatch r = withRecordedMatch r . getMatch where
                 | otherwise -> fail "Index range is not allowed for this command."
     _ -> getNonrangeMatchS fs
 
-getPartialNonrangeMatch :: (RepoPatch p, ApplyMonad DefaultIO (ApplyState p), ApplyState p ~ Tree)
-                        => Repository p wR wU wT
-                        -> [MatchFlag]
-                        -> [FileName]
-                        -> IO ()
-getPartialNonrangeMatch r fs _ =
-    withRecordedMatch r (getNonrangeMatchS fs)
-
-getFirstMatch :: (ApplyMonad DefaultIO (ApplyState p), RepoPatch p, ApplyState p ~ Tree)
-              => Repository p wR wU wT
+getFirstMatch :: (ApplyMonad (ApplyState p) DefaultIO, IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+              => Repository rt p wR wU wT
               -> [MatchFlag]
               -> IO ()
 getFirstMatch r fs = withRecordedMatch r (getFirstMatchS fs)
 
-getOnePatchset :: (RepoPatch p, ApplyState p ~ Tree)
-               => Repository p wR wU wT
+getOnePatchset :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+               => Repository rt p wR wU wT
                -> [MatchFlag]
-               -> IO (SealedPatchSet p Origin)
+               -> IO (SealedPatchSet rt p Origin)
 getOnePatchset repository fs =
     case nonrangeMatcher fs of
         Just m -> do ps <- readRepo repository
@@ -95,9 +89,9 @@ getOnePatchset repository fs =
           context_f (Context f:_) = f
           context_f (_:xs) = context_f xs
 
-withRecordedMatch :: (RepoPatch p, ApplyState p ~ Tree)
-                  => Repository p wR wU wT
-                  -> (PatchSet p Origin wR -> DefaultIO ())
+withRecordedMatch :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                  => Repository rt p wR wU wT
+                  -> (PatchSet rt p Origin wR -> DefaultIO ())
                   -> IO ()
 withRecordedMatch r job
     = do createPristineDirectoryTree r "." WithWorkingDir
