@@ -27,7 +27,6 @@ module Darcs.Repository.PatchIndex (
   createOrUpdatePatchIndexDisk,
   deletePatchIndex,
   dumpPatchIndex,
-  dumpPatchIndexFiles,
   filterPatches,
   PatchFilter,
   maybeFilterPatches,
@@ -36,7 +35,7 @@ module Darcs.Repository.PatchIndex (
   attemptCreatePatchIndex
 ) where
 
-import Prelude hiding ( catch, pi )
+import Prelude hiding ( pi, (<$>) )
 import Data.Binary ( encodeFile, decodeFile )
 import Data.Word ( Word32 )
 import Data.Int ( Int8 )
@@ -53,12 +52,12 @@ import Control.Applicative ( (<$>) )
 import System.Directory ( createDirectory, renameDirectory, doesFileExist, doesDirectoryExist )
 import Darcs.Repository.Format ( formatHas, RepoProperty( HashedInventory ) )
 import Darcs.Repository.InternalTypes ( Repository(..) )
-import Darcs.Repository.Read ( readRepo )
+import Darcs.Repository.HashedRepo ( readRepo )
 import Darcs.Patch.Witnesses.Ordered ( mapFL, RL(..), FL(..), reverseRL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), Sealed(..), seal, seal2, unsafeUnseal )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd(..), info )
-import Darcs.Repository.Lock ( withPermDir, rmRecursive )
-import Darcs.Patch ( RepoPatch, listTouchedFiles )
+import Darcs.Util.Lock ( withPermDir, rmRecursive )
+import Darcs.Patch ( RepoPatch, IsRepoType, listTouchedFiles )
 import Darcs.Util.Path ( FileName, fp2fn, fn2fp, toFilePath )
 import Darcs.Patch.Apply ( applyToFileMods, ApplyState(..) )
 import Darcs.Patch.Set ( newset2FL, Origin, newset2FL )
@@ -72,7 +71,7 @@ import System.IO (openFile, IOMode(WriteMode), hClose)
 import qualified Data.ByteString as B
 import Darcs.Util.Crypt.SHA256 (sha256sum )
 import Darcs.Util.Crypt.SHA1 ( SHA1(..), showAsHex )
-import Storage.Hashed.Tree ( Tree(..) )
+import Darcs.Util.Tree ( Tree(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Util.SignalHandler ( catchInterrupt )
 #include "impossible.h"
@@ -318,15 +317,17 @@ nubSeq = map head . group
 -- Create/Update patch-index on disk
 
 -- | create patch index that corresponds to all patches in repo
-createPatchIndexDisk :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
-createPatchIndexDisk repository = do
-  rawpatches <- newset2FL `fmap` readRepo repository
+createPatchIndexDisk
+  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+  => Repository rt p wR wU wT -> IO ()
+createPatchIndexDisk repository@(Repo r _ _ _) = do
+  rawpatches <- newset2FL `fmap` readRepo repository r
   let patches = mapFL Sealed2 rawpatches
   createPatchIndexFrom repository $ patches2patchMods patches S.empty
 
 -- | convert patches to patchmods
 patches2patchMods :: (Apply p, Commute p, PatchInspect p, ApplyState p ~ Tree)
-                  => [Sealed2 (PatchInfoAnd p)] -> Set FileName -> [(PatchId, [PatchMod FileName])]
+                  => [Sealed2 (PatchInfoAnd rt p)] -> Set FileName -> [(PatchId, [PatchMod FileName])]
 patches2patchMods patches fns = snd $ mapAccumL go fns patches
   where
     go filenames (Sealed2 p) = (filenames', (pid, pmods_effect ++ pmods_dup))
@@ -385,18 +386,20 @@ removePidSuffix pid2idx oldpids@(oldpid:_) (PatchIndex pids fidspans fpspans inf
           | otherwise = []
 
 -- | update the patch index to the current state of the repository
-updatePatchIndexDisk :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+updatePatchIndexDisk
+    :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+    => Repository rt p wR wU wT -> IO ()
 updatePatchIndexDisk repo@(Repo repodir _ _ _) = do
     (_,_,pid2idx,pindex) <- loadPatchIndex repodir
     -- check that patch index is up to date
-    patches <- newset2FL `fmap` readRepo repo
+    patches <- newset2FL `fmap` readRepo repo repodir
     let pidsrepo = mapFL (makePatchID . info) patches
         (oldpids,_,len_common) = uncommon (reverse $ pids pindex) pidsrepo
         pindex' = removePidSuffix pid2idx oldpids pindex
         filenames = fpSpans2fileNames (fpspans pindex')
         cdir = repodir </> indexDir
     -- reread to prevent holding onto patches for too long
-    rawpatches <- newset2FL `fmap` readRepo repo
+    rawpatches <- newset2FL `fmap` readRepo repo repodir
     let newpatches = drop len_common $ mapFL seal2 rawpatches
         newpmods = patches2patchMods newpatches filenames
     inv_hash <- getInventoryHash repodir
@@ -411,7 +414,7 @@ updatePatchIndexDisk repo@(Repo repodir _ _ _) = do
 
 -- | 'createPatchIndexFrom repo pmods' creates a patch index from the given
 --   patchmods.
-createPatchIndexFrom :: RepoPatch p => Repository p wR wU wT
+createPatchIndexFrom :: RepoPatch p => Repository rt p wR wU wT
                      -> [(PatchId, [PatchMod FileName])] -> IO ()
 createPatchIndexFrom (Repo repodir _ _ _) pmods = do
     inv_hash <- getInventoryHash repodir
@@ -438,8 +441,8 @@ loadPatchIndex repodir = do
 -- | ensuring that whenever loaded, the patch-index
 -- | can actually be read by the current version of darcs,
 -- | and up to date.
-loadSafePatchIndex :: (RepoPatch p, ApplyState p ~ Tree)
-                   => Repository p wR wU wT
+loadSafePatchIndex :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                   => Repository rt p wR wU wT
                    -> IO (Map PatchId Int, PatchIndex)
 loadSafePatchIndex repo@(Repo repodir _ _ _) = do
    can_use <- isPatchIndexInSync repo
@@ -466,8 +469,8 @@ isPatchIndexDisabled :: FilePath -> IO Bool
 isPatchIndexDisabled repodir = doesFileExist (repodir </> darcsdir  </> noPatchIndex)
 
 -- | create or update patch index
-createOrUpdatePatchIndexDisk :: (RepoPatch p, ApplyState p ~ Tree)
-                             => Repository p wR wU wT -> IO ()
+createOrUpdatePatchIndexDisk :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                             => Repository rt p wR wU wT -> IO ()
 createOrUpdatePatchIndexDisk repo@(Repo repodir _ _ _)= do
    rmRecursive (repodir </> darcsdir </> noPatchIndex) `catch` \(_ :: IOError) -> return ()
    dpie <- doesPatchIndexExist repodir
@@ -478,7 +481,7 @@ createOrUpdatePatchIndexDisk repo@(Repo repodir _ _ _)= do
 -- | Checks whether a patch index can (and should) be created. If we are not in
 -- an old-fashioned repo, and if we haven't been told not to, then we should
 -- create a patch index if it doesn't already exist.
-canCreatePI :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT
+canCreatePI :: (RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT
             -> IO Bool
 canCreatePI (Repo repodir format _ _) =
     (not . or) <$> sequence [ doesntHaveHashedInventory format
@@ -491,7 +494,7 @@ canCreatePI (Repo repodir format _ _) =
 -- | see if the default is to use patch index or not
 -- | creates Patch index, if it does not exist, and noPatchIndex is not set
 canUsePatchIndex :: (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository p wR wU wT -> IO Bool
+                 => Repository rt p wR wU wT -> IO Bool
 canUsePatchIndex (Repo repodir _ _ _) = do
      piExists <- doesPatchIndexExist repodir
      piDisabled <- isPatchIndexDisabled repodir
@@ -501,8 +504,8 @@ canUsePatchIndex (Repo repodir _ _ _) = do
         (True, True) -> error "patch index exists, and patch index is disabled. run optimize enable-patch-index or disable-patch-index to rectify."
         (False, False) -> return False
 
-createPIWithInterrupt :: (RepoPatch p, ApplyState p ~ Tree)
-                      => Repository p wR wU wT -> IO ()
+createPIWithInterrupt :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                      => Repository rt p wR wU wT -> IO ()
 createPIWithInterrupt repo@(Repo repodir _ _ _) = do
             putStrLn "Creating a patch index, please wait. To stop press Ctrl-C"
             (do
@@ -511,7 +514,7 @@ createPIWithInterrupt repo@(Repo repodir _ _ _) = do
 
 -- | check if patch-index is in sync with repository
 isPatchIndexInSync :: (RepoPatch p, ApplyState p ~ Tree)
-                   => Repository p wR wU wT -> IO Bool
+                   => Repository rt p wR wU wT -> IO Bool
 isPatchIndexInSync (Repo repodir _ _ _) = do
    dpie <- doesPatchIndexExist repodir
    if dpie
@@ -666,17 +669,11 @@ dumpPatchIndex repodir = do
   putStrLn "=============="
   putStrLn $ unlines $ fpSpans2filePaths fpspans infom
 
-
-dumpPatchIndexFiles :: FilePath -> IO ()
-dumpPatchIndexFiles repodir = do
-  (_,_,_,PatchIndex _ _ fpspans infom) <- loadPatchIndex repodir
-  putStr $ unlines $ fpSpans2filePaths fpspans infom
-
 -----------------------------------------------------------------------
 -- Filtering functions based on FilePaths
 -- returns an RL in which the order of patches matters, for annotate to use
-getRelevantSubsequence :: (RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd p)
-                       => Sealed ((RL a) wK) -> Repository p wR wU wR -> [FileName] -> IO (Sealed ((RL a) Origin))
+getRelevantSubsequence :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p)
+                       => Sealed ((RL a) wK) -> Repository rt p wR wU wR -> [FileName] -> IO (Sealed ((RL a) Origin))
 getRelevantSubsequence pxes repository fns = do
    (_, pi@(PatchIndex _ _ _ infom)) <- loadSafePatchIndex repository
    let fids = map (\fn -> evalState (lookupFid fn) pi) fns
@@ -685,29 +682,29 @@ getRelevantSubsequence pxes repository fns = do
    let flpxes = reverseRL $ unsafeUnseal pxes
    return.seal $ keepElems flpxes NilRL pids
 
-  where keepElems :: (RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd p)
+  where keepElems :: (RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p)
                   => FL a wX wY -> RL a wB wX -> S.Set Word32 -> RL a wP wQ
         keepElems NilFL acc _ = unsafeCoerceP acc
         keepElems (x:>:xs) acc pids
-          | (short $ makePatchID $ info x) `S.member` pids = keepElems xs (x:<:acc) pids
+          | (short $ makePatchID $ info x) `S.member` pids = keepElems xs (acc:<:x) pids
           | otherwise                                      = keepElems (unsafeCoerceP xs) acc pids
 
 -- | filter given patches so as to keep only the patches that modify the given files
-filterPatches :: (RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd p) => Repository p wR wU wT -> [FilePath] -> [Sealed2 a] -> IO [Sealed2 a]
+filterPatches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p) => Repository rt p wR wU wT -> [FilePath] -> [Sealed2 a] -> IO [Sealed2 a]
 filterPatches repository fps ops = do
    (_, pi@(PatchIndex _ _ _ infom)) <- loadSafePatchIndex repository
    let fids = concatMap ((\fn -> evalState (lookupFids' fn) pi). fp2fn) fps
        npids = S.unions $ map (touching.fromJust.(`M.lookup` infom)) fids
    return $ filter (flip S.member npids . (\(Sealed2 (PIAP pin _)) -> short $ makePatchID pin)) ops
 
-type PatchFilter p = [FilePath] -> [Sealed2 (PatchInfoAnd p)] -> IO [Sealed2 (PatchInfoAnd p)]
+type PatchFilter rt p = [FilePath] -> [Sealed2 (PatchInfoAnd rt p)] -> IO [Sealed2 (PatchInfoAnd rt p)]
 
 -- | If a patch index is available, filter given patches so as to keep only the patches that
 -- modify the given files. If none is available, return the original input.
 maybeFilterPatches
-    :: (RepoPatch p, ApplyState p ~ Tree)
-    => Repository p wR wU wT
-    -> PatchFilter p
+    :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+    => Repository rt p wR wU wT
+    -> PatchFilter rt p
 maybeFilterPatches repo fps ops = do
     usePI <- canUsePatchIndex repo
     -- in theory we could change the type signature to make this function staged,
@@ -718,7 +715,7 @@ maybeFilterPatches repo fps ops = do
 -----------------------------------------------------------------------
 -- Test patch index
 
-piTest :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+piTest :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT -> IO ()
 piTest repository = do
    (_, PatchIndex rpids fidspans fpspans infom) <- loadSafePatchIndex repository
    let pids = reverse rpids
@@ -764,7 +761,8 @@ piTest repository = do
           isInOrder _ [] = False
 
 -- | Check if patch index can be created and build it with interrupt.
-attemptCreatePatchIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+attemptCreatePatchIndex
+  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT -> IO ()
 attemptCreatePatchIndex repo = do
   canCreate <- canCreatePI repo
   when canCreate $ createPIWithInterrupt repo
