@@ -3,8 +3,10 @@ module Darcs.UI.ApplyPatches
     , StandardPatchApplier(..)
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import System.Exit ( ExitCode ( ExitSuccess ), exitSuccess )
-import Prelude hiding ( catch )
 import System.IO ( hClose, stdout, stderr )
 import Control.Exception
                  ( catch, fromException, SomeException, throwIO )
@@ -36,59 +38,62 @@ import Darcs.Repository
     , setScriptsExecutablePatches
     )
 import Darcs.Repository.Job ( RepoJob(RepoJob) )
-import Darcs.Patch ( RepoPatch, description, PrimOf )
+import Darcs.Patch ( RepoPatch, RepoType, IsRepoType, description, PrimOf )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Witnesses.Ordered
     ( FL, mapFL, nullFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed) )
 
 import Darcs.UI.External ( sendEmail )
-import Darcs.Repository.Lock ( withStdoutTemp, readBinFile )
+import Darcs.Util.Lock ( withStdoutTemp, readBinFile )
 import Darcs.Util.Printer ( vcat, text )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
+
+import GHC.Exts ( Constraint )
 
 data PatchProxy (p :: * -> * -> *) = PatchProxy
 
 -- |This class is a hack to abstract over pull/apply and rebase pull/apply.
 class PatchApplier pa where
 
-    -- |'CarrierType pa p' resolves to either 'p' or 'Rebasing p' 
-    type CarrierType pa (p :: * -> * -> *) :: * -> * -> *
+    type ApplierRepoTypeConstraint pa (rt :: RepoType) :: Constraint
 
     repoJob
         :: pa
         -> [DarcsFlag]
-        -> (forall p wR wU
-               . ( RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree
-                 , RepoPatch (CarrierType pa p), ApplyState (CarrierType pa p) ~ Tree
-                 , ApplyState (PrimOf (CarrierType pa p)) ~ Tree
+        -> (forall rt p wR wU
+               . ( IsRepoType rt, ApplierRepoTypeConstraint pa rt
+                 , RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree
                  )
-              => (PatchProxy p -> Repository (CarrierType pa p) wR wU wR -> IO ()))
+              => (PatchProxy p -> Repository rt p wR wU wR -> IO ()))
         -> RepoJob ()
 
     applyPatches
-        :: forall p wR wU wT wX wZ
-         . (RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
+        :: forall rt p wR wU wT wX wZ
+         . ( ApplierRepoTypeConstraint pa rt, IsRepoType rt
+           , RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree
+           )
         => pa
         -> PatchProxy p
         -> String
         -> [DarcsFlag]
         -> String
-        -> Repository (CarrierType pa p) wR wU wT
-        -> FL (PatchInfoAnd (CarrierType pa p)) wX wT
-        -> FL (PatchInfoAnd (CarrierType pa p)) wX wZ -> IO ()
+        -> Repository rt p wR wU wT
+        -> FL (PatchInfoAnd rt p) wX wT
+        -> FL (PatchInfoAnd rt p) wX wZ -> IO ()
 
 data StandardPatchApplier = StandardPatchApplier
 
 instance PatchApplier StandardPatchApplier where
-    type CarrierType StandardPatchApplier p = p
+    type ApplierRepoTypeConstraint StandardPatchApplier rt = ()
     repoJob StandardPatchApplier _opts f = RepoJob (f PatchProxy)
     applyPatches StandardPatchApplier PatchProxy = standardApplyPatches
 
 standardApplyPatches
-           :: forall p wR wU wT wX wZ . (RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
-           => String -> [DarcsFlag] -> String -> Repository p wR wU wT
-           -> FL (PatchInfoAnd p) wX wT -> FL (PatchInfoAnd p) wX wZ -> IO ()
+           :: forall rt p wR wU wT wX wZ
+            . (IsRepoType rt, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
+           => String -> [DarcsFlag] -> String -> Repository rt p wR wU wT
+           -> FL (PatchInfoAnd rt p) wX wT -> FL (PatchInfoAnd rt p) wX wZ -> IO ()
 standardApplyPatches cmdName opts from_whom repository us' to_be_applied = do
    printDryRunMessageAndExit cmdName
       (verbosity opts)
