@@ -49,7 +49,6 @@ module Darcs.UI.Commands
     , printDryRunMessageAndExit
     , setEnvDarcsPatches
     , setEnvDarcsFiles
-    , formatPath
     , defaultRepo
     , amInHashedRepository
     , amInRepository
@@ -57,11 +56,14 @@ module Darcs.UI.Commands
     , findRepository
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^) )
 import Control.Monad ( when, unless )
 import Data.List ( sort, isPrefixOf )
 import Data.Maybe ( catMaybes )
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 import System.Console.GetOpt ( OptDescr )
 import System.Exit ( exitSuccess )
 import System.IO ( stderr )
@@ -397,7 +399,7 @@ printDryRunMessageAndExit :: (RepoPatch p, ApplyState p ~ Tree)
                           => String
                           -> Verbosity -> Summary -> DryRun -> XmlOutput
                           -> Bool -- interactive
-                          -> FL (PatchInfoAnd p) wX wY
+                          -> FL (PatchInfoAnd rt p) wX wY
                           -> IO ()
 printDryRunMessageAndExit action v s d x interactive patches = do
     when (d == YesDryRun) $ do
@@ -437,7 +439,7 @@ printDryRunMessageAndExit action v s d x interactive patches = do
 -- | Set the DARCS_PATCHES and DARCS_PATCHES_XML environment variables with
 -- info about the given patches, for use in post-hooks.
 setEnvDarcsPatches :: (RepoPatch p, ApplyState p ~ Tree)
-                   => FL (PatchInfoAnd p) wX wY -> IO ()
+                   => FL (PatchInfoAnd rt p) wX wY -> IO ()
 #ifndef WIN32
 setEnvDarcsPatches ps = do
     let k = "Defining set of chosen patches"
@@ -461,6 +463,7 @@ setEnvCautiously e v
     | toobig (10 * 1024) v = return ()
     | otherwise = setEnv e v True
   where
+    -- note: not using (length v) because we want to be more lazy than that
     toobig :: Int -> [a] -> Bool
     toobig 0 _ = True
     toobig _ [] = False
@@ -479,17 +482,6 @@ setEnvDarcsFiles ps =
 setEnvDarcsFiles _ = return ()
 #endif
 
--- | Format a path for screen output, so that the user sees where the path
--- begins and ends. Could (should?) also warn about unprintable characters
--- here.
-formatPath :: String -> String
-formatPath path = "\"" ++ quote path ++ "\""
-  where
-    quote "" = ""
-    quote (c:cs) = if c `elem` ['\\', '"']
-                       then '\\' : c : quote cs
-                       else c : quote cs
-
 defaultRepo :: [DarcsFlag] -> AbsolutePath -> [String] -> IO [String]
 defaultRepo fs = defaultrepo (remoteRepos fs)
 
