@@ -15,15 +15,18 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, PatternGuards #-}
+{-# LANGUAGE CPP #-}
 
 module Darcs.UI.Commands.Apply
     ( apply, applyCmd
     , getPatchBundle -- used by darcsden
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import System.Exit ( exitSuccess )
-import Prelude hiding ( (^), catch )
+import Prelude hiding ( (^) )
 import Control.Monad ( when )
 
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefullyM, info )
@@ -37,7 +40,7 @@ import Darcs.UI.Flags
     , doHappyForwarding, doReverse, verbosity, useCache, dryRun
     , reorder, umask
     , fixUrl, getCc, getSendmailCmd
-    , diffAlgorithm, isUnified, getReply
+    , isUnified, getReply
     )
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import qualified Darcs.UI.Options.All as O
@@ -51,7 +54,7 @@ import Darcs.Repository
     , filterOutConflicts
     )
 import Darcs.Patch.Set ( Origin, newset2RL )
-import Darcs.Patch ( RepoPatch, PrimOf )
+import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Info ( PatchInfo, showPatchInfoUI )
 import Darcs.Patch.Witnesses.Ordered
@@ -64,7 +67,7 @@ import qualified Data.ByteString as B (ByteString, null, init, take, drop)
 import qualified Data.ByteString.Char8 as BC (unpack, last, pack)
 
 import Darcs.Util.Download ( Cachable(Uncachable) )
-import Darcs.Repository.External ( gzFetchFilePS )
+import Darcs.Util.External ( gzFetchFilePS )
 import Darcs.UI.External
     ( sendEmailDoc
     , resendEmail
@@ -74,8 +77,7 @@ import Darcs.UI.Email ( readEmail )
 import Darcs.Patch.Depends ( findUncommon, findCommonWithThem )
 import Darcs.UI.ApplyPatches ( PatchApplier(..), StandardPatchApplier(..), PatchProxy )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(..)
+    ( WhichChanges(..)
     , runSelection
     , selectionContext
     )
@@ -85,7 +87,7 @@ import Darcs.Util.Printer
     ( packedString, vcat, text, empty
     , renderString, RenderMode(..)
     )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 #include "impossible.h"
 
@@ -264,15 +266,15 @@ applyCmd patchApplier (_,o) opts [unfixed_patchesfile] = withRepoLock (dryRun op
 applyCmd _ _ _ _ = impossible
 
 applyCmdCommon
-    :: forall pa p wR wU
+    :: forall rt pa p wR wU
      . ( PatchApplier pa, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree
-       , RepoPatch (CarrierType pa p), ApplyState (CarrierType pa p) ~ Tree 
+       , ApplierRepoTypeConstraint pa rt, IsRepoType rt
        )
     => pa
     -> PatchProxy p
     -> [DarcsFlag]
     -> B.ByteString
-    -> Repository (CarrierType pa p) wR wU wR
+    -> Repository rt p wR wU wR
     -> IO ()
 applyCmdCommon patchApplier patchProxy opts bundle repository = do
   let from_whom = getFrom bundle
@@ -291,8 +293,8 @@ applyCmdCommon patchApplier patchProxy opts bundle repository = do
   let common_i = mapRL info $ newset2RL common
       them_i = mapRL info $ newset2RL them
       required = them_i \\ common_i -- FIXME quadratic?
-      check :: RL (PatchInfoAnd (CarrierType pa p)) wX wY -> [PatchInfo] -> IO ()
-      check (p :<: ps') bad = case hopefullyM p of
+      check :: RL (PatchInfoAnd rt p) wX wY -> [PatchInfo] -> IO ()
+      check (ps' :<: p) bad = case hopefullyM p of
         Nothing | info p `elem` required -> check ps' (info p : bad)
         _ -> check ps' bad
       check NilRL [] = return ()
@@ -316,7 +318,7 @@ applyCmdCommon patchApplier patchProxy opts bundle repository = do
           
   let direction = if doReverse opts then FirstReversed else First
       context = selectionContext direction "apply" (patchSelOpts opts) Nothing Nothing
-  (to_be_applied :> _) <- runSelection (selectChanges their_ps) context
+  (to_be_applied :> _) <- runSelection their_ps context
   applyPatches patchApplier patchProxy "apply" opts from_whom repository us' to_be_applied
 --    see the default (False) for the option
 --    where fixed_opts = if Interactive `elem` opts
@@ -324,7 +326,7 @@ applyCmdCommon patchApplier patchProxy opts bundle repository = do
 --                          else All : opts
 
 getPatchBundle :: RepoPatch p => [DarcsFlag] -> B.ByteString
-                 -> IO (Either String (SealedPatchSet p Origin))
+                 -> IO (Either String (SealedPatchSet rt p Origin))
 getPatchBundle opts fps = do
     let opt_verify = parseFlags O.verify opts
     mps <- verifyPS opt_verify $ readEmail fps
@@ -421,7 +423,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveral flags
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = maybeIsInteractive flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = O.NoSummary -- option not supported, use default
