@@ -25,7 +25,10 @@ module Darcs.UI.Commands.Clone
     , cloneToSSH
     ) where
 
-import Prelude hiding ( (^), catch )
+import Prelude ()
+import Darcs.Prelude
+
+import Prelude hiding ( (^) )
 
 import System.Directory ( doesDirectoryExist, doesFileExist
                         , setCurrentDirectory )
@@ -60,8 +63,8 @@ import Darcs.Repository.Format ( identifyRepoFormat
                                               )
                                , formatHas
                                )
-import Darcs.Repository.Lock ( withTempDir )
-import Darcs.Repository.Ssh ( getSSH, SSHCmd(SCP) )
+import Darcs.Util.Lock ( withTempDir )
+import Darcs.Util.Ssh ( getSSH, SSHCmd(SCP) )
 import Darcs.Repository.Flags
     ( CloneKind(CompleteClone), SetDefault(NoSetDefault), ForgetParent(..) )
 import Darcs.Patch.Bundle ( scanContextFile )
@@ -86,10 +89,8 @@ cloneHelp =
   , "if omitted, it is inferred from the source location."
   , ""
   , "By default Darcs will copy every patch from the original repository."
-  , "This means the copy is completely independent of the original; you can"
-  , "operate on the new repository even when the original is inaccessible."
   , "If you expect the original repository to remain accessible, you can"
-  , "use `--lazy` to avoid copying patches until they are needed (`copy on"
+  , "use `--lazy` to avoid copying patches until they are needed ('copy on"
   , "demand').  This is particularly useful when copying a remote"
   , "repository with a long history that you don't care about."
   , ""
@@ -103,8 +104,8 @@ cloneHelp =
   , ""
   , "When cloning from a remote location, Darcs will look for and attempt"
   , "to use packs created by `darcs optimize http` in the remote repository."
-  , "Packs are single big files that can be downloaded instead of many"
-  , "little files, which makes cloning faster over HTTP."
+  , "Packs are single big files that can be downloaded faster than many"
+  , "little files."
   , ""
   , "Darcs clone will not copy unrecorded changes to the source repository's"
   , "working tree."
@@ -234,7 +235,6 @@ cloneCmd _ opts [inrepodir] = do
                          (withWorkingDir opts)
                          (runPatchIndex opts)
                          (usePacks opts)
-                         (not $ null [p | UpToPattern p <- opts] ) -- --to-match given
                          YesForgetParent
          setCurrentDirectory currentDir
          (scp, args) <- getSSH SCP
@@ -253,7 +253,6 @@ cloneCmd _ opts [inrepodir] = do
                   (withWorkingDir opts)
                   (runPatchIndex opts)
                   (usePacks opts)
-                  (not $ null [p | UpToPattern p <- opts] ) -- --to-match given
                   NoForgetParent
       putInfo opts $ text "Finished cloning."
 
@@ -302,7 +301,7 @@ cloneHelpTag =
   , "`--to-match` options, which exclude patches *after* the first matching"
   , "patch.  Because these options treat the set of patches as an ordered"
   , "sequence, you may get different results after reordering with `darcs"
-  , "optimize`, so tagging is preferred."
+  , "optimize reorder`."
   , ""
   ]
 
@@ -323,7 +322,7 @@ validContextFile opts =
          exists <- doesFileExist ctxFilePath
          if exists
              then do
-                (ps :: PatchSet DummyPatch Origin wX) <-
+                (ps :: PatchSet rt DummyPatch Origin wX) <-
                     scanContextFile ctxFilePath
                 (ps `seq` return $ Right ()) `catch` \(_ :: SomeException) ->
                     return . Left $ "File " ++ ctxFilePath
