@@ -15,11 +15,12 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, MagicHash, OverloadedStrings, DeriveDataTypeable #-}
+{-# LANGUAGE CPP, MagicHash, OverloadedStrings #-}
 
 module Darcs.UI.Commands.Convert ( convert ) where
 
-import Prelude hiding ( (^), readFile, log, lex )
+import Prelude ( lookup )
+import Darcs.Prelude hiding ( readFile, lex )
 
 import System.FilePath.Posix ( (</>) )
 import System.Directory ( setCurrentDirectory, doesDirectoryExist, doesFileExist,
@@ -27,6 +28,7 @@ import System.Directory ( setCurrentDirectory, doesDirectoryExist, doesFileExist
 import System.IO ( stdin )
 import Data.IORef ( newIORef, modifyIORef, readIORef )
 import Data.Char ( isSpace )
+import Control.Arrow ( second, (&&&) )
 import Control.Monad ( when, unless, void, forM_ )
 import Control.Monad.Trans ( liftIO )
 import Control.Monad.State.Strict ( gets, modify )
@@ -47,20 +49,21 @@ import qualified Data.ByteString.Lazy.UTF8 as BLU
 import qualified Data.Attoparsec.ByteString.Char8 as A
 import Data.Attoparsec.ByteString.Char8( (<?>) )
 
-import qualified Storage.Hashed.Tree as T
-import qualified Storage.Hashed.Monad as TM
-import Storage.Hashed.Monad hiding ( createDirectory, exists, rename )
-import Storage.Hashed.Darcs ( hashedTreeIO, darcsAddMissingHashes )
-import Storage.Hashed.Tree( Tree, treeHash, readBlob, TreeItem(..)
-                          , emptyTree, listImmediate, findTree )
-import Storage.Hashed.AnchoredPath( anchorPath, appendPath, floatPath
-                                  , AnchoredPath(..), Name(..)  )
-import Storage.Hashed.Hash( encodeBase16, sha256, Hash(..) )
+import qualified Darcs.Util.Tree as T
+import qualified Darcs.Util.Tree.Monad as TM
+import Darcs.Util.Tree.Monad hiding ( createDirectory, exists, rename )
+import Darcs.Util.Tree.Hashed ( hashedTreeIO, darcsAddMissingHashes )
+import Darcs.Util.Tree( Tree, treeHash, readBlob, TreeItem(..)
+                      , emptyTree, listImmediate, findTree )
+import Darcs.Util.Path( anchorPath, appendPath, floatPath
+                      , parent, anchoredRoot
+                      , AnchoredPath(..), Name(..)
+                      , ioAbsoluteOrRemote, toPath, AbsolutePath )
+import Darcs.Util.Hash( encodeBase16, sha256, Hash(..) )
 
 import Darcs.Util.DateTime ( formatDateTime, fromClockTime, parseDateTime, startOfTime )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.File ( withCurrentDirectory )
-import Darcs.Util.Path ( ioAbsoluteOrRemote, toPath, AbsolutePath )
 import Darcs.Util.Exception ( clarifyErrors )
 import Darcs.Util.Prompt ( askUser )
 import Darcs.Util.Printer ( text, ($$) )
@@ -69,35 +72,45 @@ import Darcs.Util.Workaround ( getCurrentDirectory )
 
 import Darcs.Patch.Depends ( getUncovered )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia, info, hopefully )
-import Darcs.Patch ( Named, showPatch, patch2patchinfo, fromPrim, fromPrims,
-                     infopatch, adddeps, getdeps, effect, patchcontents,
-                     RepoPatch, apply, listTouchedFiles )
+import Darcs.Patch ( IsRepoType, showPatch, fromPrim, fromPrims,
+                     effect,
+                     RepoPatch, apply, listTouchedFiles
+                   , move )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Effect ( Effect )
+import Darcs.Patch.Named
+    ( patch2patchinfo
+    , infopatch, adddeps, getdeps, patchcontents
+    )
+import Darcs.Patch.Named.Wrapped ( WrappedNamed(..) )
+import qualified Darcs.Patch.Named.Wrapped as Wrapped ( getdeps )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..), (=/\=) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..), bunchFL, mapFL, mapFL_FL,
-    concatFL, mapRL, nullFL, (+>+) )
+    concatFL, mapRL, nullFL, (+>+), (+<+)
+    , reverseRL, reverseFL )
 import Darcs.Patch.Witnesses.Sealed ( FlippedSeal(..), Sealed(..), unFreeLeft
                                     ,  flipSeal, unsafeUnsealFlipped )
 import Darcs.Patch.Info ( piRename, piTag, isTag, PatchInfo, patchinfo,
                           piName, piLog, piDate, piAuthor, makePatchname )
-import Darcs.Patch.V1 ( Patch )
-import Darcs.Patch.V2 ( RealPatch )
+import Darcs.Patch.V1 ( RepoPatchV1 )
+import Darcs.Patch.V2 ( RepoPatchV2 )
 import Darcs.Patch.V1.Commute ( publicUnravel )
-import Darcs.Patch.V1.Core ( Patch(PP), isMerger )
-import Darcs.Patch.V2.Real ( mergeUnravelled )
+import Darcs.Patch.V1.Core ( RepoPatchV1(PP), isMerger )
+import Darcs.Patch.V2.RepoPatch ( mergeUnravelled )
 import Darcs.Patch.Prim ( sortCoalesceFL )
+import Darcs.Patch.Prim.Class ( PrimOf )
 import Darcs.Patch.Prim.V1 ( Prim )
+import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), newset2RL, newset2FL )
 import Darcs.Patch.Progress ( progressFL )
 
 import Darcs.Repository.Flags ( UpdateWorking(..), Reorder (..), UseIndex(..), ScanKnown(..)
-                              , AllowConflicts(..), ExternalMerge(..), WantGuiPause(..)
+                              , AllowConflicts(..), ExternalMerge(..), WantGuiPause(..), PatchFormat(..)
                               , Compression(..), DryRun(NoDryRun), DiffAlgorithm(MyersDiff, PatienceDiff) )
 import Darcs.Repository ( Repository, withRepoLock, RepoJob(..), withRepositoryDirectory,
                           createRepository, invalidateIndex,
-                          tentativelyMergePatches, patchSetToPatches,
+                          tentativelyMergePatches,
                           createPristineDirectoryTree,
                           revertRepositoryChanges, finalizeRepositoryChanges,
                           applyToWorking
@@ -111,8 +124,8 @@ import Darcs.Repository.HashedIO ( cleanHashdir )
 import Darcs.Repository.Prefs( FileType(..) )
 import Darcs.Repository.Format(identifyRepoFormat, formatHas, RepoProperty(Darcs2))
 import Darcs.Repository.Motd ( showMotd )
-import Darcs.Repository.Lock ( writeBinFile )
-import Darcs.Repository.External ( fetchFilePS, Cachable(Uncachable) )
+import Darcs.Util.Lock ( writeBinFile )
+import Darcs.Util.External ( fetchFilePS, Cachable(Uncachable) )
 import Darcs.Repository.Diff( treeDiff )
 
 
@@ -124,6 +137,7 @@ import Darcs.UI.Flags
     )
 import Darcs.UI.Commands ( DarcsCommand(..), amInRepository, nodefaults, putInfo
                          , normalCommand, withStdOpts )
+import Darcs.UI.Commands.Util.Tree ( treeHasDir, treeHasFile )
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import qualified Darcs.UI.Options.All as O
 
@@ -189,7 +203,7 @@ convertImportHelp = unlines
  , "Further options are accepted (see `darcs help init`)."
  , ""
  , "To convert a git repo to a new darcs one you may run:"
- , "    $ (cd gitrepo && git fast-export --all) | darcs convert import darcsmirror"
+ , "    $ (cd gitrepo && git fast-export --all -M) | darcs convert import darcsmirror"
  , ""
  , "WARNING: git repositories with branches will produce weird results,"
  , "         use at your own risks."
@@ -375,20 +389,20 @@ toDarcs2 _ opts [inrepodir] = do
   mysimplename <- makeRepoName opts repodir
   createDirectory mysimplename
   setCurrentDirectory mysimplename
-  createRepository False (withWorkingDir opts) (runPatchIndex opts)
+  createRepository PatchFormat2 (withWorkingDir opts) (runPatchIndex opts)
   writeBinFile (darcsdir++"/hashed_inventory") ""
   withRepoLock NoDryRun (useCache opts) NoUpdateWorking (umask opts) $ V2Job $ \repository ->
     withRepositoryDirectory (useCache opts) repodir $ V1Job $ \themrepo -> do
       theirstuff <- readRepo themrepo
-      let patches = mapFL_FL convertNamed $ patchSetToPatches theirstuff
+      let patches = mapFL_FL (convertNamed . hopefully) $ newset2FL theirstuff
           outOfOrderTags = catMaybes $ mapRL oot $ newset2RL theirstuff
               where oot t = if isTag (info t) && info t `notElem` inOrderTags theirstuff
-                            then Just (info t, getdeps $ hopefully t)
+                            then Just (info t, Wrapped.getdeps $ hopefully t)
                             else Nothing
           fixDep p = case lookup p outOfOrderTags of
                      Just d -> p : concatMap fixDep d
                      Nothing -> [p]
-          convertOne :: Patch Prim wX wY -> FL (RealPatch Prim) wX wY
+          convertOne :: RepoPatchV1 Prim wX wY -> FL (RepoPatchV2 Prim) wX wY
           convertOne x | isMerger x = case mergeUnravelled $ publicUnravel x of
                                        Just (FlippedSeal y) ->
                                            case effect y =/\= effect x of
@@ -403,10 +417,12 @@ toDarcs2 _ opts [inrepodir] = do
                                                   fromPrims (effect x)
           convertOne (PP x) = fromPrim x :>: NilFL
           convertOne _ = impossible
-          convertFL :: FL (Patch Prim) wX wY -> FL (RealPatch Prim) wX wY
+          convertFL :: FL (RepoPatchV1 Prim) wX wY -> FL (RepoPatchV2 Prim) wX wY
           convertFL = concatFL . mapFL_FL convertOne
-          convertNamed :: Named (Patch Prim) wX wY -> PatchInfoAnd (RealPatch Prim) wX wY
-          convertNamed n = n2pia $
+          convertNamed :: WrappedNamed ('RepoType 'NoRebase) (RepoPatchV1 Prim) wX wY
+                       -> PatchInfoAnd ('RepoType 'NoRebase) (RepoPatchV2 Prim) wX wY
+          convertNamed (NormalP n)
+                         = n2pia $ NormalP $
                            adddeps (infopatch (convertInfo $ patch2patchinfo n) $
                                               convertFL $ patchcontents n)
                                    (map convertInfo $ concatMap fixDep $ getdeps n)
@@ -415,7 +431,7 @@ toDarcs2 _ opts [inrepodir] = do
           applySome xs = do -- TODO this unsafeCoerce hack is because we don't keep track of the repository state properly
                             -- Really sequence_ $ mapFL applySome below should instead be a repeated add operation -
                             -- there doesn't seem to be any reason we need to do a merge here.
-                            let repository2 = unsafeCoerce# repository :: Repository (RealPatch Prim) wA wB wA
+                            let repository2 = unsafeCoerce# repository :: Repository ('RepoType 'NoRebase) (RepoPatchV2 Prim) wA wB wA
                             Sealed pw <- tentativelyMergePatches repository2 "convert"
                                              YesAllowConflicts NoUpdateWorking
                                              NoExternalMerge NoWantGuiPause
@@ -492,27 +508,28 @@ fastExport _ opts _ = do
     Nothing -> return ()
     Just f  -> writeMarks f newMarks
 
-fastExport' :: (RepoPatch p, ApplyState p ~ Tree) => Repository p r u r -> Marks -> IO Marks
+fastExport' :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+            => Repository rt p r u r -> Marks -> IO Marks
 fastExport' repo marks = do
   putStrLn "progress (reading repository)"
   patchset <- readRepo repo
   marksref <- newIORef marks
   let patches = newset2FL patchset
       tags = inOrderTags patchset
-      mark :: (PatchInfoAnd p) x y -> Int -> TreeIO ()
+      mark :: (PatchInfoAnd rt p) x y -> Int -> TreeIO ()
       mark p n = liftIO $ do putStrLn $ "mark :" ++ show n
                              modifyIORef marksref $ \m -> addMark m n (patchHash p)
       -- apply a single patch to build the working tree of the last exported version
       checkOne :: (RepoPatch p, ApplyState p ~ Tree)
-               => Int -> (PatchInfoAnd p) x y -> TreeIO ()
+               => Int -> (PatchInfoAnd rt p) x y -> TreeIO ()
       checkOne n p = do apply p
                         unless (inOrderTag tags p ||
                                 (getMark marks n == Just (patchHash p))) $
                           fail $ "FATAL: Marks do not correspond: expected " ++
-                                 (show $ getMark marks n) ++ ", got " ++ (BC.unpack $ patchHash p)
+                                 show (getMark marks n) ++ ", got " ++ BC.unpack (patchHash p)
       -- build the working tree of the last version exported by convert --export
       check :: (RepoPatch p, ApplyState p ~ Tree)
-            => Int -> FL (PatchInfoAnd p) x y -> TreeIO (Int,  FlippedSeal( FL (PatchInfoAnd p)) y) 
+            => Int -> FL (PatchInfoAnd rt p) x y -> TreeIO (Int,  FlippedSeal( FL (PatchInfoAnd rt p)) y) 
       check _ NilFL = return (1, flipSeal NilFL)
       check n allps@(p:>:ps)
         | n <= lastMark marks = checkOne n p >> check (next tags n p) ps
@@ -531,8 +548,8 @@ fastExport' repo marks = do
 
 dumpPatches ::  (RepoPatch p, ApplyState p ~ Tree)
             =>  [PatchInfo]
-            -> (forall p0 x0 y0 . (PatchInfoAnd p0) x0 y0 -> Int -> TreeIO ())
-            -> Int -> FL (PatchInfoAnd p) x y -> TreeIO ()
+            -> (forall p0 x0 y0 . (PatchInfoAnd rt p0) x0 y0 -> Int -> TreeIO ())
+            -> Int -> FL (PatchInfoAnd rt p) x y -> TreeIO ()
 dumpPatches _ _ _ NilFL = liftIO $ putStrLn "progress (patches converted)"
 dumpPatches tags mark n (p:>:ps) = do
   apply p
@@ -542,7 +559,7 @@ dumpPatches tags mark n (p:>:ps) = do
              dumpFiles $ map floatPath $ listTouchedFiles p
   dumpPatches tags mark (next tags n p) ps
 
-dumpTag :: (PatchInfoAnd p) x y  -> Int -> TreeIO () 
+dumpTag :: (PatchInfoAnd rt p) x y  -> Int -> TreeIO () 
 dumpTag p n =
   dumpBits [ BLU.fromString $ "progress TAG " ++ cleanTagName p
            , BLU.fromString $ "tag " ++ cleanTagName p -- FIXME is this valid?
@@ -563,10 +580,11 @@ dumpTag p n =
 
 dumpFiles :: [AnchoredPath] -> TreeIO ()
 dumpFiles files = forM_ files $ \file -> do
+  let quotedPath = quotePath $ anchorPath "" file
   isfile <- fileExists file
   isdir <- directoryExists file
   when isfile $ do bits <- readFile file
-                   dumpBits [ BLU.fromString $ "M 100644 inline " ++ anchorPath "" file
+                   dumpBits [ BLU.fromString $ "M 100644 inline " ++ quotedPath
                             , BLU.fromString $ "data " ++ show (BL.length bits)
                             , bits ]
   when isdir $ do -- Always delete directory before dumping its contents. This fixes
@@ -579,9 +597,30 @@ dumpFiles files = forM_ files $ \file -> do
                                   listImmediate $ fromJust $ findTree tt file ]
                   dumpFiles subs
   when (not isfile && not isdir) $ liftIO $ putStrLn $ "D " ++ anchorPath "" file
+  where
+    -- |quotePath escapes and quotes paths containing newlines, double-quotes
+    -- or backslashes.
+    quotePath :: FilePath -> String
+    quotePath path = case foldr escapeChars ("", False) path of
+        (_, False) -> path
+        (path', True) -> quote path'
 
-dumpPatch ::  (forall p0 x0 y0 . (PatchInfoAnd p0) x0 y0 -> Int -> TreeIO ())
-          -> (PatchInfoAnd p) x y -> Int
+    quote str = "\"" ++ str ++ "\""
+
+    escapeChars c (processed, haveEscaped) = case escapeChar c of
+        (escaped, didEscape) ->
+            (escaped ++ processed, didEscape || haveEscaped)
+
+    escapeChar c = case c of
+        '\n' -> ("\\n", True)
+        '\r' -> ("\\r", True)
+        '"'  -> ("\\\"", True)
+        '\\' -> ("\\\\", True)
+        _    -> ([c], False)
+
+
+dumpPatch ::  (forall p0 x0 y0 . (PatchInfoAnd rt p0) x0 y0 -> Int -> TreeIO ())
+          -> (PatchInfoAnd rt p) x y -> Int
           -> TreeIO ()
 dumpPatch mark p n =
   do dumpBits [ BLC.pack $ "progress " ++ show n ++ ": " ++ piName (info p)
@@ -603,7 +642,7 @@ dumpBits = liftIO . BLC.putStrLn . BL.intercalate "\n"
 -- john <john@home> -> john <john@home>
 -- john <john@home  -> john <john@home>
 -- <john>           -> john <unknown>
-patchAuthor :: (PatchInfoAnd p) x y -> String
+patchAuthor :: (PatchInfoAnd rt p) x y -> String
 patchAuthor p
  | null author = unknownEmail "unknown"
  | otherwise = case span (/='<') author of
@@ -628,11 +667,11 @@ patchAuthor p
    emailPad email = "<" ++ email ++ ">"
    mkAuthor name email = name ++ " " ++ email
 
-patchDate :: (PatchInfoAnd p) x y -> String
+patchDate :: (PatchInfoAnd rt p) x y -> String
 patchDate = formatDateTime "%s +0000" . fromClockTime . toClockTime .
   piDate . info
 
-patchMessage :: (PatchInfoAnd p) x y -> BLU.ByteString
+patchMessage :: (PatchInfoAnd rt p) x y -> BLU.ByteString
 patchMessage p = BL.concat [ BLU.fromString (piName $ info p)
                            , case unlines . piLog $ info p of
                                  "" -> BL.empty
@@ -648,12 +687,21 @@ type Content = B.ByteString
 data RefId = MarkId Int | HashId B.ByteString | Inline
            deriving Show
 
+-- Newish (> 1.7.6.1) Git either quotes filenames or has two
+-- non-special-char-containing paths. Older git doesn't do any quoting, so
+-- we'll have to manually try and find the correct paths, when we use the
+-- paths.
+data CopyRenameNames = Quoted B.ByteString B.ByteString
+                     | Unquoted B.ByteString deriving Show
+
 data Object = Blob (Maybe Int) Content
             | Reset Branch (Maybe RefId)
             | Commit Branch Marked AuthorInfo Message
             | Tag Int AuthorInfo Message
             | Modify (Either Int Content) B.ByteString -- (mark or content), filename
             | Gitlink B.ByteString
+            | Copy CopyRenameNames
+            | Rename CopyRenameNames
             | Delete B.ByteString -- filename
             | From Int
             | Merge Int
@@ -662,35 +710,36 @@ data Object = Blob (Maybe Int) Content
             deriving Show
 
 type Ancestors = (Marked, [Int])
-data State = Toplevel Marked Branch
-           | InCommit Marked Ancestors Branch (Tree IO) PatchInfo
-           | Done
+data State p where
+  Toplevel :: Marked -> Branch -> State p
+  InCommit :: Marked -> Ancestors -> Branch -> Tree IO -> RL (PrimOf p) cX cY -> PatchInfo -> State p
+  Done :: State p
 
-instance Show State where
-  show (Toplevel _ _) = "Toplevel"
-  show (InCommit _ _ _ _ _) = "InCommit"
+instance Show (State p) where
+  show Toplevel {} = "Toplevel"
+  show InCommit {} = "InCommit"
   show Done =  "Done"
 
 fastImport :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 fastImport _ opts [outrepo] =
   do createDirectory outrepo
      withCurrentDirectory outrepo $ do
-       createRepository (patchFormat opts == O.PatchFormat1) (withWorkingDir opts) (runPatchIndex opts)
-       withRepoLock NoDryRun (useCache opts) NoUpdateWorking (umask opts) $ V2Job $ \repo -> do
+       createRepository (patchFormat opts) (withWorkingDir opts) (runPatchIndex opts)
+       withRepoLock NoDryRun (useCache opts) NoUpdateWorking (umask opts) $ RepoJob $ \repo -> do
          -- TODO implement --dry-run, which would be read-only?
          marks <- fastImport' repo emptyMarks
          createPristineDirectoryTree repo "." (withWorkingDir opts)
          return marks
 fastImport _ _ _ = fail "I need exactly one output repository."
 
-fastImport' :: forall p r u . (RepoPatch p, ApplyState p ~ Tree) =>
-               Repository p r u r -> Marks -> IO ()
+fastImport' :: forall rt p r u . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) =>
+               Repository rt p r u r -> Marks -> IO ()
 fastImport' repo marks = do
     pristine <- readRecorded repo
     marksref <- newIORef marks
     let initial = Toplevel Nothing $ BC.pack "refs/branches/master"
 
-        go :: State -> B.ByteString -> TreeIO ()
+        go :: State p -> B.ByteString -> TreeIO ()
         go state rest = do (rest', item) <- parseObject rest
                            state' <- process state item
                            case state' of
@@ -704,13 +753,13 @@ fastImport' repo marks = do
                         `appendPath` (Name $ BC.pack $ show (n `mod` 1000))
 
         makeinfo author message tag = do
-          let (name:log) = lines $ BC.unpack message
+          let (name, log) = case BC.unpack message of
+                                      "" -> ("Unnamed patch", [])
+                                      msg -> (head &&& tail) . lines $ msg
               (author'', date'') = span (/='>') $ BC.unpack author
               date' = dropWhile (`notElem` ("0123456789" :: String)) date''
               author' = author'' ++ ">"
-              date = formatDateTime "%Y%m%d%H%M%S" $ case (parseDateTime "%s %z" date') of
-                Just x -> x
-                Nothing -> startOfTime
+              date = formatDateTime "%Y%m%d%H%M%S" $ fromMaybe startOfTime (parseDateTime "%s %z" date')
           liftIO $ patchinfo date (if tag then "TAG " ++ name else name) author' log
 
         addtag author msg =
@@ -718,14 +767,14 @@ fastImport' repo marks = do
              gotany <- liftIO $ doesFileExist $ darcsdir </> "tentative_hashed_pristine"
              deps <- if gotany then liftIO $ getUncovered `fmap` readTentativeRepo repo
                                else return []
-             let ident = NilFL :: FL (RealPatch Prim) cX cX
-                 patch = adddeps (infopatch info_ ident) deps
+             let ident = NilFL :: FL (RepoPatchV2 Prim) cX cX
+                 patch = NormalP (adddeps (infopatch info_ ident) deps)
              void $ liftIO $ addToTentativeInventory (extractCache repo)
                                                      GzipCompression (n2pia patch)
 
         -- processing items
         updateHashes = do
-          let nodarcs = (\(AnchoredPath (Name x:_)) _ -> x /= BC.pack darcsdir)
+          let nodarcs = \(AnchoredPath (Name x:_)) _ -> x /= BC.pack darcsdir
               hashblobs (File blob@(T.Blob con NoHash)) =
                 do hash <- sha256 `fmap` readBlob blob
                    return $ File (T.Blob con hash)
@@ -734,7 +783,31 @@ fastImport' repo marks = do
           modify $ \s -> s { tree = tree' }
           return $ T.filter nodarcs tree'
 
-        process :: State -> Object -> TreeIO State
+        -- Since git doesn't track directores it implicitly deletes
+        -- them when they become empty. We should therefore remove any
+        -- directories that become empty (except the repo-root
+        -- directory!)
+        deleteEmptyParents fp = do
+          let directParent = parent fp
+          unless (directParent == anchoredRoot) $ do
+              parentTree <- flip findTree directParent <$> gets tree
+              case (null . listImmediate) <$> parentTree of
+                      Just True -> do TM.unlink directParent
+                                      deleteEmptyParents directParent
+                      -- Either missing (not possible) or non-empty.
+                      _ -> return ()
+
+        -- generate a Hunk primitive patch from diffing
+        diffCurrent :: State p -> TreeIO (State p)
+        diffCurrent (InCommit mark ancestors branch start ps info_) = do
+          current <- updateHashes
+          Sealed diff <- unFreeLeft `fmap`
+             liftIO (treeDiff PatienceDiff (const TextFile) start current)
+          let newps = ps +<+ reverseFL diff
+          return $ InCommit mark ancestors branch current newps info_
+        diffCurrent _ = error "This is never valid outside of a commit."
+
+        process :: State p -> Object -> TreeIO (State p)
         process s (Progress p) = do
           liftIO $ putStrLn ("progress " ++ BC.unpack p)
           return s
@@ -744,7 +817,7 @@ fastImport' repo marks = do
           modify $ \s -> s { tree = tree' } -- lets dump the right tree, without _darcs
           let root = encodeBase16 $ treeHash tree'
           liftIO $ do
-            putStrLn $ "\\o/ It seems we survived. Enjoy your new repo."
+            putStrLn "\\o/ It seems we survived. Enjoy your new repo."
             B.writeFile (darcsdir </> "tentative_pristine") $
               BC.concat [BC.pack "pristine:", root]
           return Done
@@ -753,7 +826,7 @@ fastImport' repo marks = do
           if Just what == n
              then addtag author msg
              else liftIO $ putStrLn $ "WARNING: Ignoring out-of-order tag " ++
-                             (head $ lines $ BC.unpack msg)
+                             head (lines $ BC.unpack msg)
           return (Toplevel n b)
 
         process (Toplevel n _) (Reset branch from) =
@@ -765,7 +838,7 @@ fastImport' repo marks = do
              return $ Toplevel n branch
 
         process (Toplevel n b) (Blob (Just m) bits) = do
-          TM.writeFile (markpath m) $ (BLC.fromChunks [bits])
+          TM.writeFile (markpath m) (BLC.fromChunks [bits])
           return $ Toplevel n b
 
         process x (Gitlink link) = do
@@ -778,27 +851,61 @@ fastImport' repo marks = do
             addtag author pbranch
           info_ <- makeinfo author message False
           startstate <- updateHashes
-          return $ InCommit mark (previous, []) branch startstate info_
+          return $ InCommit mark (previous, []) branch startstate NilRL info_
 
-        process s@(InCommit _ _ _ _ _) (Modify (Left m) path) = do
+        process s@InCommit {} (Modify (Left m) path) = do
           TM.copy (markpath m) (floatPath $ BC.unpack path)
-          return s
+          diffCurrent s
 
-        process s@(InCommit _ _ _ _ _) (Modify (Right bits) path) = do
+        process s@InCommit {} (Modify (Right bits) path) = do
           TM.writeFile (floatPath $ BC.unpack path) (BLC.fromChunks [bits])
-          return s
+          diffCurrent s
 
-        process s@(InCommit _ _ _ _ _) (Delete path) = do
-          TM.unlink (floatPath $ BC.unpack path)
-          return s
+        process s@InCommit {} (Delete path) = do
+          let floatedPath = floatPath $ BC.unpack path
+          TM.unlink floatedPath
+          deleteEmptyParents floatedPath
+          diffCurrent s
 
-        process (InCommit mark (prev, current) branch start info_) (From from) = do
-          return $ InCommit mark (prev, from:current) branch start info_
+        process (InCommit mark (prev, current) branch start ps info_) (From from) =
+          return $ InCommit mark (prev, from:current) branch start ps info_
 
-        process (InCommit mark (prev, current) branch start info_) (Merge from) = do
-          return $ InCommit mark (prev, from:current) branch start info_
+        process (InCommit mark (prev, current) branch start ps info_) (Merge from) =
+          return $ InCommit mark (prev, from:current) branch start ps info_
 
-        process (InCommit mark ancestors branch start info_) x = do
+        process s@InCommit {} (Copy names) = do
+            (from, to) <- extractNames names
+            TM.copy (floatPath $ BC.unpack from) (floatPath $ BC.unpack to)
+            -- We can't tell Darcs that a file has been copied, so it'll
+            -- show as an addfile.
+            diffCurrent s
+
+        process s@(InCommit mark ancestors branch start _ info_) (Rename names) = do
+          (from, to) <- extractNames names
+          let uFrom = BC.unpack from
+              uTo = BC.unpack to
+              parentDir = parent $ floatPath uTo
+          targetDirExists <- liftIO $ treeHasDir start uTo
+          targetFileExists <- liftIO $ treeHasFile start uTo
+          parentDirExists <-
+              liftIO $ treeHasDir start (anchorPath "" parentDir)
+          -- If the target exists, remove it; if it doesn't, add all
+          -- its parent directories.
+          if targetDirExists || targetFileExists
+              then TM.unlink $ floatPath uTo
+              else unless parentDirExists $ TM.createDirectory parentDir
+          (InCommit _ _ _ _ newPs _) <- diffCurrent s
+          TM.rename (floatPath uFrom) (floatPath uTo)
+          let ps' = newPs :<: move uFrom uTo
+          current <- updateHashes
+          -- ensure empty dirs get deleted
+          deleteEmptyParents (floatPath uFrom)
+          -- run diffCurrent to add the dir deletions prims
+          diffCurrent (InCommit mark ancestors branch current ps' info_)
+
+        -- When we leave the commit, create a patch for the cumulated
+        -- prims.
+        process (InCommit mark ancestors branch _ ps info_) x = do
           case ancestors of
             (_, []) -> return () -- OK, previous commit is the ancestor
             (Just n, list)
@@ -809,11 +916,9 @@ fastImport' repo marks = do
             (Nothing, list) ->
               liftIO $ putStrLn $ "WARNING: Linearising non-linear ancestry " ++ show list
 
-          current <- updateHashes
-          Sealed diff
-                <- unFreeLeft `fmap` (liftIO $ treeDiff PatienceDiff (const TextFile) start current)
-          prims <- return $ fromPrims $ sortCoalesceFL diff
-          let patch = infopatch info_ ((NilFL :: FL p cX cX) +>+ prims)
+          {- current <- updateHashes -} -- why not?
+          (prims :: FL p cX cY)  <- return $ fromPrims $ sortCoalesceFL $ reverseRL ps
+          let patch = NormalP (infopatch info_ ((NilFL :: FL p cX cX) +>+ prims))
           void $ liftIO $ addToTentativeInventory (extractCache repo)
                                                   GzipCompression (n2pia patch)
           case mark of
@@ -827,6 +932,36 @@ fastImport' repo marks = do
           liftIO $ print obj
           fail $ "Unexpected object in state " ++ show state
 
+        extractNames :: CopyRenameNames
+                     -> TreeIO (BC.ByteString, BC.ByteString)
+        extractNames names = case names of
+            Quoted f t -> return (f, t)
+            Unquoted uqNames -> do
+                let spaceIndices = BC.elemIndices ' ' uqNames
+                    splitStr = second (BC.drop 1) . flip BC.splitAt uqNames
+                    -- Reverse the components, so we find the longest
+                    -- prefix existing name.
+                    spaceComponents = reverse $ map splitStr spaceIndices
+                    componentCount = length spaceComponents
+                if componentCount == 1
+                    then return $ head spaceComponents
+                    else do
+                        let dieMessage = unwords
+                                [ "Couldn't determine move/rename"
+                                , "source/destination filenames, with the"
+                                , "data produced by this (old) version of"
+                                , "git, since it uses unquoted, but"
+                                , "special-character-containing paths."
+                                ]
+                            floatUnpack = floatPath . BC.unpack
+                            lPathExists (l,_) =
+                                TM.fileExists $ floatUnpack l
+                            finder [] = error dieMessage
+                            finder (x : rest) = do
+                                xExists <- lPathExists x
+                                if xExists then return x else finder rest
+                        finder spaceComponents
+
     void $ hashedTreeIO (go initial B.empty) pristine $ darcsdir </> "pristine.hashed"
     finalizeRepositoryChanges repo YesUpdateWorking GzipCompression
     cleanRepository repo
@@ -849,6 +984,8 @@ parseObject = next' mbObject
                    <|> p_commit
                    <|> p_tag
                    <|> p_modify
+                   <|> p_rename
+                   <|> p_copy
                    <|> p_from
                    <|> p_merge
                    <|> p_delete
@@ -899,17 +1036,39 @@ parseObject = next' mbObject
         p_hash = lex $ A.takeWhile1 (A.inClass "0123456789abcdefABCDEF")
         p_from = lexString "from" >> From `fmap` p_marked
         p_merge = lexString "merge" >> Merge `fmap` p_marked
-        p_delete = lexString "D" >> Delete `fmap` line
+        p_delete = lexString "D" >> Delete `fmap` p_maybeQuotedName
+        p_rename = do lexString "R"
+                      names <- p_maybeQuotedCopyRenameNames
+                      return $ Rename names
+        p_copy = do lexString "C"
+                    names <- p_maybeQuotedCopyRenameNames
+                    return $ Copy names
         p_modify = do lexString "M"
                       mode <- lex $ A.takeWhile (A.inClass "01234567890")
                       mark <- p_refid
-                      path <- line
+                      path <- p_maybeQuotedName
                       case mark of
                         HashId hash | mode == BC.pack "160000" -> return $ Gitlink hash
                                     | otherwise -> fail ":(("
                         MarkId n -> return $ Modify (Left n) path
                         Inline -> do bits <- p_data
                                      return $ Modify (Right bits) path
+        p_maybeQuotedCopyRenameNames =
+            p_lexTwoQuotedNames <|> Unquoted `fmap` line
+        p_lexTwoQuotedNames = do
+            n1 <- lex p_quotedName
+            n2 <- lex p_quotedName
+            return $ Quoted n1 n2
+        p_maybeQuotedName = lex (p_quotedName <|> line)
+        p_quotedName = do
+          _ <- A.char '"'
+          -- Take until a non-escaped " character.
+          name <- A.scan Nothing
+            (\previous char -> if char == '"' && previous /= Just '\\'
+               then Nothing else Just (Just char))
+          _ <- A.char '"'
+          return $ unescape name
+
 
         next' :: (B.ByteString -> A.Result (Maybe Object)) -> B.ByteString -> TreeIO (B.ByteString, Object)
         next' parser rest =
@@ -927,19 +1086,19 @@ parseObject = next' mbObject
                fail $ "Error parsing stream. " ++ err ++ "\nContext: " ++ show ctx
 
 
-patchHash :: PatchInfoAnd p cX cY -> BC.ByteString
+patchHash :: PatchInfoAnd rt p cX cY -> BC.ByteString
 patchHash p = BC.pack $ show $ makePatchname (info p)
 
-inOrderTag :: (Effect p) => [PatchInfo] -> PatchInfoAnd p wX wZ -> Bool
+inOrderTag :: (Effect p) => [PatchInfo] -> PatchInfoAnd rt p wX wZ -> Bool
 inOrderTag tags p = isTag (info p) && info p `elem` tags && nullFL (effect p)
 
-next :: (Effect p) => [PatchInfo] -> Int ->  PatchInfoAnd p x y -> Int
+next :: (Effect p) => [PatchInfo] -> Int ->  PatchInfoAnd rt p x y -> Int
 next tags n p = if inOrderTag tags p then n else n + 1
 
-inOrderTags :: PatchSet p wS wX -> [PatchInfo]
-inOrderTags (PatchSet _ ts) = go ts
-  where go :: RL(Tagged t1) wT wY -> [PatchInfo]
-        go (Tagged t _ _ :<: ts') = info t : go ts'
+inOrderTags :: PatchSet rt p wS wX -> [PatchInfo]
+inOrderTags (PatchSet ts _) = go ts
+  where go :: RL(Tagged rt t1) wT wY -> [PatchInfo]
+        go (ts' :<: Tagged t _ _) = info t : go ts'
         go NilRL = []
 
 type Marks = M.IntMap BC.ByteString
@@ -960,7 +1119,7 @@ readMarks :: FilePath -> IO Marks
 readMarks p = do lines' <- BC.split '\n' `fmap` BC.readFile p
                  return $ foldl merge M.empty lines'
                `catchall` return emptyMarks
-  where merge set line = case (BC.split ':' line) of
+  where merge set line = case BC.split ':' line of
           [i, hash] -> M.insert (read $ BC.unpack i) (BC.dropWhile (== ' ') hash) set
           _ -> set -- ignore, although it is maybe not such a great idea...
 
@@ -969,3 +1128,21 @@ writeMarks fp m = do removeFile fp `catchall` return () -- unlink
                      BC.writeFile fp marks
   where marks = BC.concat $ map format $ M.assocs m
         format (k, s) = BC.concat [BC.pack $ show k, BC.pack ": ", s, BC.pack "\n"]
+
+-- |unescape turns \r \n \" \\ into their unescaped form, leaving any
+-- other \-preceeded characters as they are.
+unescape :: BC.ByteString -> BC.ByteString
+unescape cs = case BC.uncons cs of
+  Nothing -> BC.empty
+  Just (c', cs') -> if c' == '\\'
+    then case BC.uncons cs' of
+      Nothing -> BC.empty
+      Just (c'', cs'') -> let unescapedC = case c'' of
+                                'r'  -> '\r'
+                                'n'  -> '\n'
+                                '"'  -> '"'
+                                '\\' -> '\\'
+                                x    -> x in
+        BC.cons unescapedC $ unescape cs''
+    else BC.cons c' $ unescape cs'
+
