@@ -27,6 +27,9 @@ module Darcs.UI.Commands.GZCRCs
     , doCRCWarnings
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^) )
 import Control.Monad ( when, unless, forM_ )
 import Control.Monad.Trans ( liftIO )
@@ -49,7 +52,7 @@ import Darcs.Repository ( Repository, withRepository, RepoJob(..) )
 import Darcs.Repository.Cache ( Cache(..), writable, isThisRepo,
                                 hashedFilePath, allHashedDirs )
 import Darcs.Repository.InternalTypes ( extractCache )
-import Darcs.Repository.Lock ( gzWriteAtomicFilePSs )
+import Darcs.Util.Lock ( gzWriteAtomicFilePSs )
 import Darcs.Patch ( RepoPatch )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInRepository )
 import Darcs.UI.Options ( DarcsOption, (^), oid, odesc, ocheck, onormalise, defaultFlags )
@@ -153,7 +156,7 @@ gzcrcsCmd _ opts _ | Check `elem` opts || Repair `elem` opts =
     withRepository (useCache opts) (RepoJob (gzcrcs' opts))
 gzcrcsCmd _ _ _ = error "You must specify --check or --repair for gzcrcs"
 
-gzcrcs' :: (RepoPatch p) => [DarcsFlag] -> Repository p wR wU wT -> IO ()
+gzcrcs' :: (RepoPatch p) => [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
 gzcrcs' opts repo = do
     -- Somewhat ugly IORef use here because it's convenient, would be nicer to
     -- pre-filter the list of locs to check and then decide whether to print
