@@ -23,7 +23,8 @@ module Darcs.UI.Commands.Log
     , changelog, getLogInfo
     ) where
 
-import Prelude hiding ( (^), log, catch )
+import Prelude ()
+import Darcs.Prelude
 
 import Unsafe.Coerce (unsafeCoerce)
 import Data.List ( intersect, sort, nub, find )
@@ -31,20 +32,19 @@ import Data.Maybe ( fromMaybe, fromJust, isJust )
 import Control.Arrow ( second )
 import Control.Exception ( catch, IOException )
 import Control.Monad.State.Strict
-import Control.Applicative ((<$>))
 
 import Darcs.UI.PrintPatch ( showFriendly )
 import Darcs.Patch.PatchInfoAnd ( fmapFLPIAP, hopefullyM, info )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias, findRepository )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
-    ( DarcsFlag(GenContext, HumanReadable,
+    ( DarcsFlag(GenContext,
                 MachineReadable, Count, Interactive,
                 NumberPatches, XMLOutput, Summary,
                 Verbose, Debug, NoPatchIndexFlag)
     , doReverse, showChangesOnlyToFiles
     , useCache, maxCount, umask
-    , verbosity, isUnified, isInteractive, diffAlgorithm, hasSummary
+    , verbosity, isUnified, isInteractive, hasSummary
     , fixSubPaths, getRepourl )
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import qualified Darcs.UI.Options.All as O
@@ -55,7 +55,7 @@ import Darcs.Repository ( PatchSet, PatchInfoAnd,
                           readRepo, unrecordedChanges,
                           withRepoLockCanFail )
 import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(MyersDiff), UpdateWorking(..) )
-import Darcs.Repository.Lock ( withTempDir )
+import Darcs.Util.Lock ( withTempDir )
 import Darcs.Patch.Set ( PatchSet(..), newset2RL )
 import Darcs.Patch.Conflict ( Conflict, CommuteNoConflicts )
 import Darcs.Patch.Format ( PatchListFormat )
@@ -68,8 +68,9 @@ import Darcs.Patch.Show ( ShowPatch )
 import Darcs.Patch.TouchesFiles ( lookTouch )
 import Darcs.Patch.Type ( PatchType(PatchType) )
 import Darcs.Patch.Apply ( Apply, ApplyState )
-import Darcs.Patch ( invert, xmlSummary, description,
-                     effectOnFilePaths, listTouchedFiles )
+import Darcs.Patch ( IsRepoType, invert, xmlSummary, description,
+                     effectOnFilePaths, listTouchedFiles, showPatch )
+import Darcs.Patch.Named.Wrapped ( (:~:)(..) )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(NilFL), RL(..), filterOutFLFL, filterRL,
@@ -93,15 +94,16 @@ import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.UI.SelectChanges ( viewChanges )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Repository.PatchIndex ( PatchFilter, maybeFilterPatches, attemptCreatePatchIndex )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 logHelp :: String
-logHelp =
- "The `darcs log` command lists the patches that constitute the\n" ++
- "current repository or, with `--repo`, a remote repository.  Without\n" ++
- "options or arguments, ALL patches will be listed.\n" ++
- "\n" ++ logHelp' ++
- "\n" ++ logHelp''
+logHelp = unlines
+ [ "The `darcs log` command lists patches of the current repository or,"
+ , "with `--repo`, a remote repository.  Without options or arguments,"
+ , "ALL patches will be listed."
+ , ""]
+ ++ logHelp'
+ ++ logHelp''
 
 logBasicOpts :: DarcsOption a
                 ([O.MatchFlag]
@@ -251,33 +253,32 @@ showLog opts files =
                                  then (reverse xs, b, c)
                                  else (xs, b, c)
 
-
--- FIXME: this prose is unreadable. --twb, 2009-08
 logHelp' :: String
-logHelp' =
- "When given one or more files or directories as arguments, only\n" ++
- "patches which affect those files or directories are listed. This\n" ++
- "includes patches that happened to files before they were moved or\n" ++
- "renamed.\n" ++
- "\n" ++
- "When given a `--from-tag`, `--from-patch` or `--from-match`, only patches\n" ++
- "since that tag or patch are listed.  Similarly, the `--to-tag`,\n" ++
- "`--to-patch` and `--to-match` options restrict the list to older patches.\n" ++
- "\n" ++
- "The `--last` and `--max-count` options both limit the number of patches\n" ++
- "listed.  The former applies BEFORE other filters, whereas the latter\n" ++
- "applies AFTER other filters.  For example `darcs log foo.c\n" ++
- "--max-count 3` will print the last three patches that affect foo.c,\n" ++
- "whereas `darcs log --last 3 foo.c` will, of the last three\n" ++
- "patches, print only those that affect foo.c.\n"
+logHelp' = unlines
+ [ "When given files or directories paths as arguments, only patches which"
+ , "affect those paths are listed.  This includes patches that happened to"
+ , "files before they were moved or renamed."
+ , ""
+ , "When given `--from-tag` or `--from-patch`, only patches since that tag"
+ , "or patch are listed.  Similarly, the `--to-tag` and `--to-patch`"
+ , "options restrict the list to older patches."
+ , ""
+ , "The `--last` and `--max-count` options both limit the number of patches"
+ , "listed.  The former applies BEFORE other filters, whereas the latter"
+ , "applies AFTER other filters.  For example `darcs log foo.c"
+ , "--max-count 3` will print the last three patches that affect foo.c,"
+ , "whereas `darcs log --last 3 foo.c` will, of the last three"
+ , "patches, print only those that affect foo.c."
+ , ""
+ ]
 
-getLogInfo :: forall p wX wY
-                . (Matchable p, ApplyState p ~ Tree)
+getLogInfo :: forall rt p wX wY
+                . (IsRepoType rt, Matchable p, ApplyState p ~ Tree)
                => Maybe Int -> [MatchFlag] -> Bool
                -> Maybe [FilePath]
-               -> PatchFilter p
-               -> PatchSet p wX wY
-               -> IO ( [(Sealed2 (PatchInfoAnd p), [FilePath])]
+               -> PatchFilter rt p
+               -> PatchSet rt p wX wY
+               -> IO ( [(Sealed2 (PatchInfoAnd rt p), [FilePath])]
                      , [(FilePath, FilePath)]
                      , Maybe Doc )
 getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
@@ -300,7 +301,7 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
         sp2s = if secondMatch matchFlags
                then matchSecondPatchset matchFlags ps
                else Sealed ps
-        pf = if haveNonrangeMatch (PatchType :: PatchType p) matchFlags
+        pf = if haveNonrangeMatch (PatchType :: PatchType rt p) matchFlags
              then matchAPatchread matchFlags
              else \_ -> True
 
@@ -309,7 +310,7 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
           | otherwise       = (pfs, renames, doc)
 
         onlyRelated (Sealed2 p, fs) =
-          (Sealed2 $ fmapFLPIAP (filterOutFLFL (unrelated fs)) p, fs)
+          (Sealed2 $ fmapFLPIAP (filterOutFLFL (unrelated fs)) (\_ -> ReflPatch) p, fs)
 
         unrelated fs p
           -- If the change does not affect the patches we are looking at,
@@ -325,12 +326,12 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
 -- "Just n" -- returns at most n patches touching the file (starting from the
 -- beginning of the patch list).
 filterPatchesByNames
-    :: forall p
+    :: forall rt p
      . (Matchable p, ApplyState p ~ Tree)
     => Maybe Int -- ^ maxcount
     -> [FilePath] -- ^ filenames
-    -> [Sealed2 (PatchInfoAnd p)] -- ^ patchlist
-    -> ([(Sealed2 (PatchInfoAnd p),[FilePath])], [(FilePath, FilePath)], Maybe Doc)
+    -> [Sealed2 (PatchInfoAnd rt p)] -- ^ patchlist
+    -> ([(Sealed2 (PatchInfoAnd rt p),[FilePath])], [(FilePath, FilePath)], Maybe Doc)
 filterPatchesByNames maxcount fns patches = removeNonRenames $
     evalState (filterPatchesByNames' fns patches) (maxcount, initRenames) where
         removeNonRenames (ps, renames, doc) = (ps, removeIds renames, doc)
@@ -372,18 +373,16 @@ filterPatchesByNames maxcount fns patches = removeNonRenames $
 (-:-) :: a -> ([a],b,c) -> ([a],b,c)
 x -:- ~(xs,y,z) = (x:xs,y,z)
 
-changelog :: forall p wStart wX
+changelog :: forall rt p wStart wX
            . ( Apply p, ApplyState p ~ Tree, ShowPatch p, IsHunk p
              , PrimPatchBase p, PatchListFormat p
              , Conflict p, CommuteNoConflicts p
              )
-          => [DarcsFlag] -> PatchSet p wStart wX
-          -> ([(Sealed2 (PatchInfoAnd p), [FilePath])], [(FilePath, FilePath)], Maybe Doc)
+          => [DarcsFlag] -> PatchSet rt p wStart wX
+          -> ([(Sealed2 (PatchInfoAnd rt p), [FilePath])], [(FilePath, FilePath)], Maybe Doc)
           -> Doc
 changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
     | Count `elem` opts = text $ show $ length pis_and_fs
-    | MachineReadable `elem` opts =
-        maybe (vsep $ map (unseal2 (showPatchInfo.info)) pis) errorDoc mbErr
     | XMLOutput `elem` opts =
          text "<changelog>"
       $$ vcat created_as_xml
@@ -393,9 +392,11 @@ changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
         mbAppendErr $ vsep (map (number_patch change_with_summary) pis)
     | otherwise = mbAppendErr $ vsep (map (number_patch description') pis)
     where mbAppendErr = maybe id (\err -> ($$ err)) mbErr
-          change_with_summary :: Sealed2 (PatchInfoAnd p) -> Doc
+          change_with_summary :: Sealed2 (PatchInfoAnd rt p) -> Doc
           change_with_summary (Sealed2 hp)
-              | Just p <- hopefullyM hp = showFriendly (verbosity opts) (hasSummary O.NoSummary opts) p
+              | Just p <- hopefullyM hp = if MachineReadable `elem` opts
+                                           then showPatch p
+                                           else showFriendly (verbosity opts) (hasSummary O.NoSummary opts) p
               | otherwise = description hp
                             $$ indent (text "[this patch is unavailable]")
 
@@ -426,46 +427,37 @@ changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
                                   Just n -> text (show n++":") <+> f x
                                   Nothing -> f x
                              else f x
-          get_number :: Sealed2 (PatchInfoAnd p) -> Maybe Int
+          get_number :: Sealed2 (PatchInfoAnd re p) -> Maybe Int
           get_number (Sealed2 y) = gn 1 (newset2RL patchset)
               where iy = info y
-                    gn :: Int -> RL (PatchInfoAnd p) wStart wY -> Maybe Int
-                    gn n (b:<:bs) | seq n (info b) == iy = Just n
+                    gn :: Int -> RL (PatchInfoAnd rt p) wStart wY -> Maybe Int
+                    gn n (bs:<:b) | seq n (info b) == iy = Just n
                                   | otherwise = gn (n+1) bs
                     gn _ NilRL = Nothing
           pis = map fst pis_and_fs
           description' = unseal2 description
 
--- FIXME: this prose is unreadable. --twb, 2009-08
 logHelp'' :: String
-logHelp'' =
- "Three output formats exist.  The default is `--human-readable`.  You can\n" ++
- "also select `--context`, which is the internal format (as seen in patch\n" ++
- "bundles) that can be re-read by Darcs (e.g. `darcs clone --context`).\n" ++
- "\n" ++
- "Finally, there is `--xml-output`, which emits valid XML... unless a the\n" ++
- "patch metadata (author, name or description) contains a non-ASCII\n" ++
- "character and was recorded in a non-UTF8 locale.\n" ++
- "\n" ++
- -- FIXME: can't we just disallow the following usage?
- "Note that while the `--context` flag may be used in conjunction with\n" ++
- "`--xml-output` or `--human-readable`, in neither case will darcs clone be\n" ++
- "able to read the output.  On the other hand, sufficient information\n" ++
- "WILL be output for a knowledgeable human to recreate the current state\n" ++
- "of the repository.\n"
+logHelp'' = unlines
+ [ "Four output formats exist.  The default is `--human-readable`. The slightly"
+ , "different `--machine-readable` format enables to see patch dependencies in"
+ , "non-interactive mode. You can also select `--context`, which is an internal"
+ , "format that can be re-read by Darcs (e.g. `darcs clone --context`)."
+ , ""
+ , "Finally, there is `--xml-output`, which emits valid XML... unless a the"
+ , "patch metadata (author, name or description) contains a non-ASCII"
+ , "character and was recorded in a non-UTF8 locale."
+ ]
 
 logContext :: [DarcsFlag] -> IO ()
 logContext opts = do
   let repodir = fromMaybe "." $ getRepourl opts
   withRepositoryDirectory (useCache opts) repodir $ RepoJob $ \repository -> do
       (_ :> ps') <- contextPatches `fmap` readRepo repository
-      let ps = mapRL (\p -> (seal2 p, [])) ps'
-      let header = if fancy then empty else text "\nContext:\n"
-      let logOutput = changelog opts' emptyset (ps, [], Nothing)
+      let pis = mapRL seal2 ps'
+      let header = text "\nContext:\n"
+      let logOutput = maybe (vsep $ map (unseal2 (showPatchInfo.info)) pis) errorDoc Nothing
       viewDocWith simplePrinters Encode $ header $$ logOutput
-        where opts' = if fancy then opts else MachineReadable : opts
-              fancy = HumanReadable `elem` opts || XMLOutput `elem` opts
-              emptyset = PatchSet NilRL NilRL
 
 -- | changes is an alias for log
 changes :: DarcsCommand [DarcsFlag]
@@ -485,7 +477,6 @@ logPatchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 logPatchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveralOrRange flags
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive False flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = hasSummary O.NoSummary flags
