@@ -24,6 +24,9 @@ module Darcs.UI.Commands.Pull ( -- * Commands.
                                 fetchPatches, revertable
                               ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^) )
 
 import System.Exit ( exitSuccess )
@@ -35,7 +38,6 @@ import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts
     , putInfo
     , setEnvDarcsPatches
-    , formatPath
     , defaultRepo
     , amInHashedRepository
     )
@@ -57,7 +59,8 @@ import Darcs.UI.Flags
     , doReverse, verbosity,  dryRun, umask, useCache, selectDeps
     , remoteRepos, reorder, setDefault
     , isUnified, hasSummary
-    , diffAlgorithm, isInteractive )
+    , isInteractive
+    )
 import Darcs.UI.Options
     ( DarcsOption, (^), odesc, ocheck, onormalise
     , defaultFlags, parseFlags
@@ -81,7 +84,7 @@ import Darcs.Repository
 
 import qualified Darcs.Repository.Cache as DarcsCache
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, hopefully, patchDesc )
-import Darcs.Patch ( RepoPatch, description )
+import Darcs.Patch ( IsRepoType, RepoPatch, description, PrimOf )
 import Darcs.Patch.Bundle( makeBundleN, patchFilename )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Set ( Origin, PatchSet(..), SealedPatchSet )
@@ -97,18 +100,18 @@ import Darcs.Patch.Depends ( findUncommon, findCommonWithThem,
                              newsetIntersection, newsetUnion )
 import Darcs.UI.ApplyPatches ( PatchApplier(..), StandardPatchApplier(..) )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(..)
+    ( WhichChanges(..)
     , runSelection
     , selectionContext
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Util.Exception ( clarifyErrors )
 import Darcs.Util.Printer ( putDocLn, vcat, ($$), text, putDoc )
-import Darcs.Repository.Lock ( writeDocBinFile )
+import Darcs.Util.Lock ( writeDocBinFile )
 import Darcs.Util.Path ( useAbsoluteOrStd, stdOut, AbsolutePath )
 import Darcs.Util.Workaround ( getCurrentDirectory )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Text ( quote )
+import Darcs.Util.Tree( Tree )
 #include "impossible.h"
 
 
@@ -124,11 +127,9 @@ pullHelp :: String
 pullHelp = unlines
  [ "Pull is used to bring patches made in another repository into the current"
  , "repository (that is, either the one in the current directory, or the one"
- , "specified with the `--repodir` option). Pull allows you to bring over all or"
- , "some of the patches that are in that repository but not in this one. Pull"
- , "accepts arguments, which are URLs from which to pull, and when called"
- , "without an argument, pull will use the repository from which you have most"
- , "recently either pushed or pulled."
+ , "specified with the `--repodir` option). Pull accepts arguments, which are"
+ , "URLs from which to pull, and when called without an argument, pull will"
+ , "use the repository specified at `_darcs/prefs/defaultrepo`."
  , ""
  , "The default (`--union`) behavior is to pull any patches that are in any of"
  , "the specified repositories.  If you specify the `--intersection` flag, darcs"
@@ -370,11 +371,11 @@ fetchCmd (_,o) opts repos =
     withRepoLock (dryRun opts) (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $
         fetchPatches o opts repos "fetch" >=> makeBundle opts
 
-fetchPatches :: forall p wR wU . (RepoPatch p, ApplyState p ~ Tree)
+fetchPatches :: forall rt p wR wU . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
              => AbsolutePath -> [DarcsFlag] -> [String] -> String
-             -> Repository p wR wU wR
-             -> IO (SealedPatchSet p Origin,
-                    Sealed ((FL (PatchInfoAnd p)  :\/: FL (PatchInfoAnd p)) wR))
+             -> Repository rt p wR wU wR
+             -> IO (SealedPatchSet rt p Origin,
+                    Sealed ((FL (PatchInfoAnd rt p)  :\/: FL (PatchInfoAnd rt p)) wR))
 fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
   here <- getCurrentDirectory
   repodirs <- (nub . filter (/= here)) `fmap` mapM (fixUrl o) unfixedrepodirs
@@ -384,7 +385,7 @@ fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
   old_default <- getPreflist "defaultrepo"
   when (old_default == repodirs && XMLOutput `notElem` opts) $
       let pulling = if DryRun `elem` opts then "Would pull" else "Pulling"
-      in  putInfo opts $ text $ pulling++" from "++concatMap formatPath repodirs++"..."
+      in  putInfo opts $ text $ pulling++" from "++concatMap quote repodirs++"..."
   (Sealed them, Sealed compl) <- readRepos repository opts repodirs
   addRepoSource (head repodirs) (dryRun opts) (remoteRepos opts) (setDefault False opts)
   mapM_ (addToPreflist "repos") repodirs
@@ -415,16 +416,16 @@ fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
                                  when (reorder opts /= O.Reorder) exitSuccess
   let direction = if doReverse opts then FirstReversed else First
       context = selectionContext direction jobname (pullPatchSelOpts opts) Nothing Nothing
-  (to_be_pulled :> _) <- runSelection (selectChanges psFiltered) context
+  (to_be_pulled :> _) <- runSelection psFiltered context
   return (seal common, seal $ us' :\/: to_be_pulled)
 
 fetchPatches _ _ [] jobname _ = fail $
   "No default repository to " ++ jobname ++ " from, please specify one"
 
-makeBundle :: forall p wR . (RepoPatch p, ApplyState p ~ Tree)
+makeBundle :: forall rt p wR . (RepoPatch p, ApplyState p ~ Tree)
            => [DarcsFlag]
-           -> (SealedPatchSet p Origin,
-               Sealed ((FL (PatchInfoAnd p) :\/: FL (PatchInfoAnd p)) wR))
+           -> (SealedPatchSet rt p Origin,
+               Sealed ((FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wR))
            -> IO ()
 makeBundle opts (Sealed common, Sealed (_ :\/: to_be_fetched)) =
     do
@@ -463,9 +464,9 @@ tuple: the first patchset(s) to be complemented against Rc and then
 the second patchset(s) to be complemented against Rc.
 -}
 
-readRepos :: (RepoPatch p, ApplyState p ~ Tree)
-          => Repository p wR wU wT -> [DarcsFlag] -> [String]
-          -> IO (SealedPatchSet p Origin,SealedPatchSet p Origin)
+readRepos :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+          => Repository rt p wR wU wT -> [DarcsFlag] -> [String]
+          -> IO (SealedPatchSet rt p Origin,SealedPatchSet rt p Origin)
 readRepos _ _ [] = impossible
 readRepos to_repo opts us =
     do rs <- mapM (\u -> do r <- identifyRepositoryFor to_repo (useCache opts) u
@@ -481,7 +482,6 @@ pullPatchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 pullPatchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveral flags
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
