@@ -227,27 +227,29 @@ recordCmd :: (AbsolutePath, AbsolutePath) -> RecordConfig -> [String] -> IO ()
 recordCmd fps cfg args = do
     checkNameIsNotOption (patchname cfg) (isInteractive cfg)
     withRepoLock NoDryRun (useCache cfg) YesUpdateWorking (umask cfg) $ RepoJob $ \(repository :: Repository rt p wR wU wR) -> do
-      files <- if null args then return Nothing
-          else Just . sort <$> fixSubPaths fps args
-      when (files == Just []) $ fail "No valid arguments were given."
       let scan = scanKnown (O.adds (lookfor cfg)) (includeBoring cfg)
-      files' <- traverse (filterExistingPaths repository (verbosity cfg) (useIndex cfg) scan) files
-      when (verbosity cfg /= O.Quiet) $
-          traverse_ (reportNonExisting scan) files'
-      let existing_files = fmap snd files'
-      when (existing_files == Just []) $
-          fail "None of the files you specified exist."
+      existing_files <- do
+        files <- if null args then return Nothing
+                 else Just . sort <$> fixSubPaths fps args
+        when (files == Just []) $ fail "No valid arguments were given."
+        files' <- traverse (filterExistingPaths repository (verbosity cfg) (useIndex cfg) scan) files
+        when (verbosity cfg /= O.Quiet) $
+            traverse_ (reportNonExisting scan) files'
+        let files'' = fmap snd files'
+        when (files'' == Just []) $
+            fail "None of the files you specified exist."
+        return files''
       announceFiles (verbosity cfg) existing_files "Recording changes in"
       debugMessage "About to get the unrecorded changes."
       Sealed replacePs <- if O.replaces (lookfor cfg) == O.YesLookForReplaces
-          then getReplaces (diffingOpts cfg) repository files
+          then getReplaces (diffingOpts cfg) repository existing_files
           else return (Sealed NilFL)
       movesPs <- if O.moves (lookfor cfg) == O.YesLookForMoves
-          then getMovesPs repository files
+          then getMovesPs repository existing_files
           else return NilFL
       changes <- unrecordedChangesWithPatches
                    movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
-                   (diffingOpts cfg) repository files
+                   (diffingOpts cfg) repository existing_files
       debugMessage "I've got unrecorded changes."
       case changes of
           NilFL | not (askDeps cfg) -> do
