@@ -19,20 +19,21 @@
 
 module Darcs.UI.Commands.Rollback ( rollback ) where
 
-import Prelude hiding ( (^), catch )
+import Prelude ()
+import Darcs.Prelude
 
-import Control.Applicative ( (<$>) )
 import Control.Exception ( catch, IOException )
 import Control.Monad ( when )
 import Data.List ( sort )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 import System.Exit ( exitSuccess )
 
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Match ( firstMatch )
 import Darcs.Patch.PatchInfoAnd ( n2pia )
-import Darcs.Patch ( RepoPatch, invert, effect, fromPrims, sortCoalesceFL,
-                     canonize, anonymous, PrimOf )
+import Darcs.Patch ( IsRepoType, RepoPatch, invert, effect, fromPrims, sortCoalesceFL,
+                     canonize, PrimOf )
+import Darcs.Patch.Named.Wrapped ( anonymous )
 import Darcs.Patch.Set ( PatchSet(..), newset2FL )
 import Darcs.Patch.Split ( reversePrimSplitter )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), RL(..), concatFL,
@@ -56,7 +57,7 @@ import Darcs.UI.Options
     , defaultFlags, parseFlags
     )
 import qualified Darcs.UI.Options.All as O
-import Darcs.UI.SelectChanges ( selectChanges, WhichChanges(..),
+import Darcs.UI.SelectChanges ( WhichChanges(..),
                                 selectionContext, selectionContextPrim,
                                 runSelection )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
@@ -65,20 +66,20 @@ import Darcs.Util.Progress ( debugMessage )
 
 rollbackDescription :: String
 rollbackDescription =
- "Apply the inverse of recorded changes to the working copy."
+ "Apply the inverse of recorded changes to the working tree."
 
 rollbackHelp :: String
 rollbackHelp = unlines
     [ "Rollback is used to undo the effects of some changes from patches"
     , "in the repository. The selected changes are undone in your working"
-    , "copy, but the repository is left unchanged. First you are offered a"
+    , "tree, but the repository is left unchanged. First you are offered a"
     , "choice of which patches to undo, then which changes within the"
     , "patches to undo."
     , ""
     , "Before doing `rollback`, you may want to temporarily undo the changes"
-    , "of your working copy (if there are) and save them for later use."
+    , "of your working tree (if there are) and save them for later use."
     , "To do so, you can run `revert`, then run `rollback`, record a patch,"
-    , "and run `unrevert` to restore the saved changes into your working copy."
+    , "and run `unrevert` to restore the saved changes into your working tree."
     ]
 
 rollbackBasicOpts :: DarcsOption a
@@ -116,7 +117,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveralOrLast flags
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps
     , S.summary = O.NoSummary
@@ -154,7 +154,7 @@ rollbackCmd fps opts args = withRepoLock NoDryRun (useCache opts)
                      else Just . sort <$> fixSubPaths fps args
         when (files == Just []) $
             fail "No valid arguments were given."
-        announceFiles files "Rolling back changes in"
+        announceFiles (verbosity opts) files "Rolling back changes in"
         allpatches <- readRepo repository
         let matchFlags = parseFlags O.matchSeveralOrLast opts
         (_ :> patches) <- return $
@@ -164,17 +164,18 @@ rollbackCmd fps opts args = withRepoLock NoDryRun (useCache opts)
         let filesFps = map toFilePath <$> files
             patchCtx = selectionContext LastReversed "rollback" (patchSelOpts opts) Nothing filesFps
         (_ :> ps) <-
-            runSelection (selectChanges patches) patchCtx
+            runSelection patches patchCtx
         exitIfNothingSelected ps "patches"
         setEnvDarcsPatches ps
         let hunkContext = selectionContextPrim Last "rollback" (patchSelOpts opts)
-                              (Just reversePrimSplitter) filesFps Nothing
+                              (Just (reversePrimSplitter (diffAlgorithm opts)))
+                              filesFps Nothing
             hunks = concatFL . mapFL_FL (canonize $ F.diffAlgorithm opts) . sortCoalesceFL . effect $ ps
-        whatToUndo <- runSelection (selectChanges hunks) hunkContext
+        whatToUndo <- runSelection hunks hunkContext
         undoItNow opts repository whatToUndo
 
-undoItNow :: (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
-          => [DarcsFlag] -> Repository p wR wU wT
+undoItNow :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+          => [DarcsFlag] -> Repository rt p wR wU wT
           -> (q :> FL (PrimOf p)) wA wT -> IO ()
 undoItNow opts repo (_ :> prims) = do
     exitIfNothingSelected prims "changes"
