@@ -37,9 +37,7 @@ import Control.Exception ( catch, IOException )
 import Control.Monad ( when, unless, forM_ )
 import Darcs.Util.Tree ( Tree )
 import Data.List ( intercalate, isPrefixOf )
-#ifdef HAVE_HTTP
 import Data.List ( stripPrefix )
-#endif
 import Data.Maybe ( isNothing, fromMaybe )
 
 import Darcs.UI.Commands
@@ -482,7 +480,6 @@ decideOnBehavior opts remote_repo =
     ts -> do announce_recipients ts
              return ts
     where the_targets = collectTargets opts
-#ifdef HAVE_HTTP
           -- the ifdef above is to so that darcs only checks the remote
           -- _darcs/post if we have an implementation of postUrl.  See
           -- our HTTP module for more details
@@ -494,9 +491,6 @@ decideOnBehavior opts remote_repo =
                           return (p++emails)
           readPost = map parseLine . lines where
               parseLine t = maybe (Post t) SendMail $ stripPrefix "mailto:" t
-#else
-          check_post = who_to_email
-#endif
           who_to_email the_remote_repo =
               do email <- (BC.unpack `fmap`
                            fetchFilePS (prefsUrl the_remote_repo++"/email")
