@@ -19,6 +19,9 @@ module Darcs.UI.Commands.ShowAuthors
     ( showAuthors, Spelling, compiledAuthorSpellings, canonizeAuthor, rankAuthors
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^), catch )
 
 import Control.Arrow ( (&&&), (***) )
