@@ -17,48 +17,46 @@
 
 module Darcs.UI.Commands.Tag ( tag, getTags ) where
 
-import Prelude hiding ( (^) )
-import Control.Applicative ( (<$>) )
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Monad ( when )
 import Data.Maybe ( catMaybes )
 
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
-import Darcs.UI.Commands.Record ( getLog )
 import Darcs.UI.Flags
     ( DarcsFlag(AskDeps), getDate, compression, verbosity, useCache, umask, getAuthor
-    , hasAuthor, diffAlgorithm )
+    , hasAuthor )
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import qualified Darcs.UI.Options.All as O
+import Darcs.UI.PatchHeader ( getLog )
 import Darcs.Patch.PatchInfoAnd ( n2pia, hopefully )
 import Darcs.Repository ( withRepoLock, Repository, RepoJob(..), readRepo,
                     tentativelyAddPatch, finalizeRepositoryChanges,
                   )
 import Darcs.Patch
-    ( infopatch, adddeps, Patchy, PrimPatch, PrimOf
-    , RepoPatch
+    ( Patchy, PrimPatch, PrimOf
+    , IsRepoType, RepoPatch
     )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Info ( patchinfo, piTag )
 import Darcs.Patch.Depends ( getUncovered )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info )
-import Darcs.Patch.MaybeInternal ( MaybeInternal(patchInternalChecker), InternalChecker(..) )
-import Darcs.Patch.Named ( patchcontents )
-import Darcs.Patch.Set ( PatchSet(..), emptyPatchSet, appendPSFL, newset2FL )
+import Darcs.Patch.Named.Wrapped ( infopatch, adddeps, runInternalChecker, namedInternalChecker )
+import Darcs.Patch.Set ( PatchSet(..), emptyPatchSet, appendPSFL, newset2FL, patchSetfMap )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), filterOutRLRL, (:>)(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
 import Darcs.UI.SelectChanges
-    ( selectChanges
-    , WhichChanges(..)
+    ( WhichChanges(..)
     , selectionContext
     , runSelection
     , PatchSelectionContext(allowSkipAll)
     )
 import qualified Darcs.UI.SelectChanges as S
-import Darcs.Repository.Util ( patchSetfMap )
 import Darcs.Repository.Flags ( UpdateWorking(..), DryRun(NoDryRun) )
 import Darcs.Util.Path ( AbsolutePath )
 
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 
 import System.IO ( hPutStr, stderr )
 
@@ -153,15 +151,15 @@ tag = DarcsCommand
     , commandParseOptions = onormalise tagOpts
     }
 
-filterNonInternal :: MaybeInternal p => PatchSet p wX wY -> PatchSet p wX wY
+filterNonInternal :: IsRepoType rt => PatchSet rt p wX wY -> PatchSet rt p wX wY
 filterNonInternal =
-  case patchInternalChecker of
+  case namedInternalChecker of
     Nothing -> id
-    Just f -> \(PatchSet ps ts) -> PatchSet (filterOutRLRL (isInternal f . patchcontents . hopefully) ps) ts
+    Just f -> \(PatchSet ts ps) -> PatchSet ts (filterOutRLRL (runInternalChecker f . hopefully) ps)
 
 tagCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 tagCmd _ opts args =
-  withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \(repository :: Repository p wR wU wR) -> do
+  withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \(repository :: Repository rt p wR wU wR) -> do
     date <- getDate (hasPipe opts)
     the_author <- getAuthor (hasAuthor opts) (hasPipe opts)
     patches <- readRepo repository
@@ -198,7 +196,7 @@ tagCmd _ opts args =
                                              " already exists."
                                return ("TAG " ++ name, comment)
 
-getTags :: MaybeInternal p => PatchSet p wW wR -> IO [String]
+getTags :: PatchSet rt p wW wR -> IO [String]
 getTags ps = catMaybes `fmap` patchSetfMap (return . piTag . info) ps
 
 -- This may be useful for developers, but users don't care about
@@ -213,21 +211,20 @@ getTags ps = catMaybes `fmap` patchSetfMap (return . piTag . info) ps
 -- since the last tag, plus that tag itself.
 
 askAboutTagDepends
-     :: forall p wX wY . (RepoPatch p, ApplyState p ~ Tree)
+     :: forall rt p wX wY . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
      => [DarcsFlag]
-     -> FL (PatchInfoAnd p) wX wY
-     -> IO (Sealed (FL (PatchInfoAnd p) wX))
+     -> FL (PatchInfoAnd rt p) wX wY
+     -> IO (Sealed (FL (PatchInfoAnd rt p) wX))
 askAboutTagDepends flags ps = do
   let opts = S.PatchSelectionOptions
              { S.verbosity = verbosity flags
              , S.matchFlags = []
-             , S.diffAlgorithm = diffAlgorithm flags
              , S.interactive = True
              , S.selectDeps = O.PromptDeps
              , S.summary = O.NoSummary
              , S.withContext = O.NoContext
              }
-  (deps:>_) <- runSelection (selectChanges ps) $
+  (deps:>_) <- runSelection ps $
                      ((selectionContext FirstReversed "depend on" opts Nothing Nothing)
                           { allowSkipAll = False })
   return $ Sealed deps
