@@ -25,16 +25,19 @@ module Darcs.UI.Commands.Unrecord
     , matchingHead
     ) where
 
-import Prelude hiding ( (^), catch )
+import Prelude ()
+import Darcs.Prelude
+
+import Prelude hiding ( (^) )
 
 import Control.Exception ( catch, IOException )
-import Control.Monad ( when )
+import Control.Monad ( when, unless )
 import Data.Maybe( isJust, mapMaybe )
 import Data.List ( intercalate )
-import Storage.Hashed.Tree( Tree )
+import Darcs.Util.Tree( Tree )
 import System.Exit ( exitSuccess )
 
-import Darcs.Patch ( RepoPatch, invert, commute, effect )
+import Darcs.Patch ( IsRepoType, RepoPatch, invert, commute, effect )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Bundle ( makeBundleN, contextPatches, minContext )
 import Darcs.Patch.Depends ( findCommonWithThem, newsetUnion )
@@ -53,7 +56,7 @@ import Darcs.Repository ( PatchInfoAnd, withRepoLock, RepoJob(..), Repository,
                           invalidateIndex, unrecordedChanges,
                           identifyRepositoryFor )
 import Darcs.Repository.Flags( UseIndex(..), ScanKnown(..), UpdateWorking(..), DryRun(NoDryRun) )
-import Darcs.Repository.Lock( writeDocBinFile )
+import Darcs.Util.Lock( writeDocBinFile )
 import Darcs.Repository.Prefs ( getDefaultRepoPath )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias
                          , putVerbose, printDryRunMessageAndExit
@@ -67,21 +70,21 @@ import Darcs.UI.Flags
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import Darcs.UI.Options.All ( notInRemoteFlagName )
 import qualified Darcs.UI.Options.All as O
-import Darcs.UI.SelectChanges ( selectChanges, WhichChanges(..),
+import Darcs.UI.SelectChanges ( WhichChanges(..),
                                 selectionContext, runSelection )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Util.English ( presentParticiple )
-import Darcs.Util.Printer ( text, putDoc )
+import Darcs.Util.Printer ( text, putDoc, (<>), (<+>) )
 import Darcs.Util.Progress ( debugMessage )
 
 unrecordDescription :: String
 unrecordDescription =
-    "Remove recorded patches without changing the working copy."
+    "Remove recorded patches without changing the working tree."
 
 unrecordHelp :: String
 unrecordHelp = unlines
  [ "Unrecord does the opposite of record: it deletes patches from"
- , "the repository, without changing the working copy."
+ , "the repository, without changing the working tree."
  , "Deleting patches from the repository makes active changes again"
  , "which you may record or revert later."
  , "Beware that you should not use this command if there is a"
@@ -154,9 +157,9 @@ unrecordCmd _ opts _ =
                     else matchingHead matchFlags allpatches
             let direction = if doReverse opts then Last else LastReversed
                 context = selectionContext direction "unrecord" (patchSelOpts opts) Nothing Nothing
-            (_ :> to_unrecord) <- runSelection (selectChanges patches) context
+            (_ :> to_unrecord) <- runSelection patches context
             when (nullFL to_unrecord) $ do
-                putStrLn "No patches selected!"
+                putInfo opts "No patches selected!"
                 exitSuccess
             putVerbose opts $
                 text "About to write out (potentially) modified patches..."
@@ -165,10 +168,10 @@ unrecordCmd _ opts _ =
             _ <- tentativelyRemovePatches repository (compression opts)
                      YesUpdateWorking to_unrecord
             finalizeRepositoryChanges repository YesUpdateWorking (compression opts)
-            putStrLn "Finished unrecording."
+            putInfo opts "Finished unrecording."
 
-getLastPatches :: RepoPatch p => [MatchFlag] -> PatchSet p Origin wR
-               -> (PatchSet p :> FL (PatchInfoAnd p)) Origin wR
+getLastPatches :: (IsRepoType rt, RepoPatch p) => [MatchFlag] -> PatchSet rt p Origin wR
+               -> (PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin wR
 getLastPatches matchFlags ps = case matchFirstPatchset matchFlags ps of
                                    Sealed p1s -> findCommonWithThem ps p1s
 
@@ -179,7 +182,7 @@ unpullDescription =
 unpullHelp :: String
 unpullHelp = unlines
  [ "Unpull completely removes recorded patches from your local repository."
- , "The changes will be undone in your working copy and the patches"
+ , "The changes will be undone in your working tree and the patches"
  , "will not be shown in your changes list anymore. Beware that if the"
  , "patches are not still present in another repository you will lose"
  , "precious code by unpulling!"
@@ -201,12 +204,12 @@ unpullCmd = genericObliterateCmd "unpull"
 
 obliterateDescription :: String
 obliterateDescription =
-    "Delete selected patches from the repository. (UNSAFE!)"
+    "Delete selected patches from the repository."
 
 obliterateHelp :: String
 obliterateHelp = unlines
  [ "Obliterate completely removes recorded patches from your local"
- , "repository. The changes will be undone in your working copy and the"
+ , "repository. The changes will be undone in your working tree and the"
  , "patches will not be shown in your changes list anymore. Beware that"
  , "you can lose precious code by obliterating!"
  , ""
@@ -312,6 +315,7 @@ genericObliterateCmd :: String
                      -> IO ()
 genericObliterateCmd cmdname _ opts _ =
     let cacheOpt = useCache opts
+        verbOpt = verbosity opts
     in withRepoLock (dryRun opts) cacheOpt YesUpdateWorking (umask opts) $
         RepoJob $ \repository -> do
             -- FIXME we may need to honour --ignore-times here, although this
@@ -331,15 +335,15 @@ genericObliterateCmd cmdname _ opts _ =
                                    else matchingHead matchFlags allpatches
                 nirs -> do
                     (Sealed thems) <-
-                      getNotInRemotePatches cacheOpt repository nirs
+                      getNotInRemotePatches verbOpt cacheOpt repository nirs
                     return $ findCommonWithThem allpatches thems
 
             let direction = if doReverse opts then Last else LastReversed
                 context = selectionContext direction cmdname (patchSelOpts opts) Nothing Nothing
             (kept :> removed) <-
-                runSelection (selectChanges removal_candidates) context
+                runSelection removal_candidates context
             when (nullFL removed) $ do
-                putStrLn "No patches selected!"
+                putInfo opts "No patches selected!"
                 exitSuccess
             case commute (effect removed :> pend) of
                 Nothing -> fail $ "Can't " ++ cmdname
@@ -347,7 +351,7 @@ genericObliterateCmd cmdname _ opts _ =
                                   ++ "unrecorded change."
                 Just (_ :> p_after_pending) -> do
                     printDryRunMessageAndExit "obliterate"
-                      (verbosity opts)
+                      verbOpt
                       (hasSummary O.NoSummary opts)
                       (dryRun opts)
                       (hasXmlOutput opts)
@@ -364,19 +368,21 @@ genericObliterateCmd cmdname _ opts _ =
                     finalizeRepositoryChanges repository
                         YesUpdateWorking (compression opts)
                     debugMessage "Applying patches to working directory..."
-                    _ <- applyToWorking repository (verbosity opts)
+                    _ <- applyToWorking repository verbOpt
                         (invert p_after_pending)
                          `catch` \(e :: IOException) -> fail $
                             "Couldn't undo patch in working dir.\n"
                             ++ show e
-                    putStrLn $ "Finished " ++ presentParticiple cmdname ++ "."
+                    putInfo opts $ "Finished" <+> text (presentParticiple cmdname) <> "."
 
 -- | Get the union of the set of patches in each specified location
-getNotInRemotePatches :: (RepoPatch p, ApplyState p ~ Tree) => O.UseCache
-                      -> Repository p wX wU wT -> [NotInRemoteLocation]
-                      -> IO (SealedPatchSet p Origin)
-getNotInRemotePatches cacheOpt repository nirs = do
-    putStrLn $ "Determining patches not in" ++ pluralExtra ++ ":\n" ++ names
+getNotInRemotePatches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                      => O.Verbosity -> O.UseCache
+                      -> Repository rt p wX wU wT -> [NotInRemoteLocation]
+                      -> IO (SealedPatchSet rt p Origin)
+getNotInRemotePatches verbOpt cacheOpt repository nirs = do
+    unless (verbOpt == O.Quiet) $
+      putStrLn $ "Determining patches not in" ++ pluralExtra ++ ":\n" ++ names
     nirsPaths <- mapM getNotInRemotePath nirs
     newsetUnion `fmap` mapM readNir nirsPaths
   where
@@ -401,23 +407,25 @@ getNotInRemotePatches cacheOpt repository nirs = do
 
 -- | matchingHead returns the repository up to some tag. The tag t is the last
 -- tag such that there is a patch after t that is matched by the user's query.
-matchingHead :: forall p wR. RepoPatch p => [MatchFlag] -> PatchSet p Origin wR
-             -> (PatchSet p :> FL (PatchInfoAnd p)) Origin wR
+matchingHead :: forall rt p wR
+              . (IsRepoType rt, RepoPatch p)
+             => [MatchFlag] -> PatchSet rt p Origin wR
+             -> (PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin wR
 matchingHead matchFlags set =
     case mh set of
         (start :> patches) -> start :> reverseRL patches
   where
-    mh :: forall wX . PatchSet p Origin wX
-       -> (PatchSet p :> RL (PatchInfoAnd p)) Origin wX
-    mh s@(PatchSet x _)
+    mh :: forall wX . PatchSet rt p Origin wX
+       -> (PatchSet rt p :> RL (PatchInfoAnd rt p)) Origin wX
+    mh s@(PatchSet _ x)
         | or (mapRL (matchAPatchread matchFlags) x) = contextPatches s
-    mh (PatchSet x (Tagged t _ ps :<: ts)) =
-        case mh (PatchSet (t :<: ps) ts) of
-            (start :> patches) -> start :> x +<+ patches
+    mh (PatchSet (ts :<: Tagged t _ ps) x) =
+        case mh (PatchSet ts (ps :<: t)) of
+            (start :> patches) -> start :> patches +<+ x
     mh ps = ps :> NilRL
 
 savetoBundle :: (RepoPatch p, ApplyState p ~ Tree) => [DarcsFlag]
-             -> PatchSet p Origin wZ -> FL (PatchInfoAnd p) wZ wT -> IO ()
+             -> PatchSet rt p Origin wZ -> FL (PatchInfoAnd rt p) wZ wT -> IO ()
 savetoBundle opts kept removed@(x :>: _) = do
     let genFullBundle = makeBundleN Nothing kept (mapFL_FL hopefully removed)
     bundle <- if not (minimize opts)
@@ -437,7 +445,6 @@ patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveralOrLast flags
-    , S.diffAlgorithm = diffAlgorithm flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
