@@ -27,7 +27,7 @@ module Darcs.UI.Commands.WhatsNew
 import Prelude ()
 import Darcs.Prelude
 
-import Control.Monad ( void )
+import Control.Monad ( void, when )
 import Control.Monad.Reader ( runReaderT )
 import Control.Monad.State ( evalStateT, liftIO )
 import Darcs.Util.Tree ( Tree )
@@ -73,11 +73,12 @@ import Darcs.UI.Commands
     , commandAlias, nodefaults
     )
 import Darcs.Repository.Resolution ( patchsetConflictResolutions )
-import Darcs.UI.Commands.Util ( announceFiles )
+import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths )
 import Darcs.UI.Flags
     ( DarcsFlag (Summary, LookForAdds, LookForMoves), diffAlgorithm, diffingOpts
     , isUnified, useCache, fixSubPaths
     , verbosity, isInteractive, isUnified, lookForAdds, lookForMoves, lookForReplaces, hasSummary
+    , scanKnown, useIndex
     )
 import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
 import qualified Darcs.UI.Options.All as O
@@ -225,9 +226,16 @@ whatsnewHelp =
 whatsnewCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 whatsnewCmd fps opts args =
    withRepository (useCache opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
-    files <- if null args
-                 then return Nothing
-                 else Just <$> fixSubPaths fps args
+    let scan = scanKnown (O.adds (parseFlags O.lookfor opts)) (parseFlags O.includeBoring opts)
+    existing_files <- do
+      files <- if null args then return Nothing
+               else Just . nubSort <$> fixSubPaths fps args
+      when (files == Just []) $ fail "No valid arguments were given."
+      files' <- traverse (filterExistingPaths repo (verbosity opts) (useIndex opts) scan) files
+      let files'' = fmap snd files'
+      when (files'' == Just []) $
+        fail "None of the files you specified exist."
+      return files''
     let isLookForMoves = lookForMoves opts == O.YesLookForMoves && parseFlags O.summary opts /= Just O.NoSummary
         isLookForAdds = lookForAdds opts == O.YesLookForAdds && parseFlags O.summary opts /= Just O.NoSummary
         isLookForReplaces = lookForReplaces opts == O.YesLookForReplaces
@@ -238,12 +246,12 @@ whatsnewCmd fps opts args =
               | isMachineReadable = (Summary:opts)
               | otherwise         = opts
     movesPs <- if isLookForMoves
-        then getMovesPs repo files
+        then getMovesPs repo existing_files
         else return NilFL
     Sealed replacePs <- if isLookForReplaces
-        then getReplaces (diffingOpts opts) repo files
+        then getReplaces (diffingOpts opts) repo existing_files
         else return (Sealed NilFL)
-    Sealed noLookChanges <- filteredUnrecordedChanges opts' repo files movesPs
+    Sealed noLookChanges <- filteredUnrecordedChanges opts' repo existing_files movesPs
                                                                        (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
     pristine <- readRecorded repo
     -- If we are looking for moves, return the corresponding FL of changes.
@@ -251,14 +259,14 @@ whatsnewCmd fps opts args =
     Sealed unaddedNewPathsPs <- if isLookForAdds
         then do
             -- Use opts not opts', here, since we *do* want to look for adds.
-            Sealed lookChanges <- filteredUnrecordedChanges opts repo files movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
+            Sealed lookChanges <- filteredUnrecordedChanges opts repo existing_files movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
             noLookAddsTree <- applyAddPatchesToPristine noLookChanges pristine
             lookAddsTree <- applyAddPatchesToPristine lookChanges pristine
             ftf <- filetypeFunction
             -- Return the patches that create files/dirs that aren't yet added.
             unFreeLeft <$> treeDiff (diffAlgorithm opts) ftf noLookAddsTree lookAddsTree
         else return (Sealed NilFL)
-    announceFiles (verbosity opts) files "What's new in"
+    announceFiles (verbosity opts) existing_files "What's new in"
     exitOnNoChanges (unaddedNewPathsPs, noLookChanges)
     if maybeIsInteractive opts
       then runInteractive (interactiveHunks pristine) opts' pristine noLookChanges
