@@ -11,7 +11,6 @@ module Darcs.UI.External
     , execPipeIgnoreError
     , pipeDoc
     , pipeDocSSH
-    , maybeURLCmd
     , viewDoc
     , viewDocWith
     , haveSendmail
@@ -27,7 +26,8 @@ module Darcs.UI.External
     , isUTF8Locale
     ) where
 
-import Prelude hiding ( catch )
+import Prelude ()
+import Darcs.Prelude
 
 import Data.Maybe ( isJust, isNothing, maybeToList )
 import Control.Monad ( unless, when, filterM, liftM2, void )
@@ -58,14 +58,14 @@ import Control.Exception
     ( try, finally, catch, IOException )
 import System.IO.Error ( ioeGetErrorType )
 import GHC.IO.Exception ( IOErrorType(ResourceVanished) )
-import Data.Char ( toUpper, toLower )
+import Data.Char ( toLower )
 import Text.Regex
 #if defined (HAVE_MAPI)
 import Foreign.C ( withCString )
 #endif
 #ifdef HAVE_MAPI
 import Foreign.Ptr ( nullPtr )
-import Darcs.Repository.Lock ( canonFilename, writeDocBinFile )
+import Darcs.Util.Lock ( canonFilename, writeDocBinFile )
 #endif
 
 import Darcs.Util.SignalHandler ( catchNonSignal )
@@ -83,12 +83,12 @@ import qualified Data.ByteString as B (ByteString, empty, null, readFile
             ,take, concat, drop, isPrefixOf, singleton, append)
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 
-import Darcs.Repository.Lock
+import Darcs.Util.Lock
     ( withTemp
     , withNamedTemp
     , withOpenTemp
     )
-import Darcs.Repository.Ssh ( getSSH, SSHCmd(..) )
+import Darcs.Util.Ssh ( getSSH, SSHCmd(..) )
 import Darcs.Util.CommandLine ( parseCmd, addUrlencoded )
 import Darcs.Util.Exec ( execInteractive, exec, Redirect(..), withoutNonBlock )
 import Darcs.Util.URL ( SshFilePath, sshUhost )
@@ -122,12 +122,6 @@ darcsProgram = getExecutablePath
 darcsProgram = getProgName
 #endif
 
-maybeURLCmd :: String -> String -> IO (Maybe String)
-maybeURLCmd what url =
-  do let prot = map toUpper $ takeWhile (/= ':') url
-     fmap Just (getEnv ("DARCS_" ++ what ++ "_" ++ prot))
-             `catch` \(_ :: IOException) -> return Nothing
-
 pipeDoc :: RenderMode -> String -> [String] -> Doc -> IO ExitCode
 pipeDoc = pipeDocInternal (PipeToOther simplePrinters)
 
