@@ -15,6 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Flags
     ( -- TODO we want to stop exporting the constructors of DarcsFlag
       -- from here. First need to change all the relevant code over to the
@@ -27,6 +28,7 @@ module Darcs.UI.Flags
     , editDescription
     , diffingOpts
     , diffOpts
+    , scanKnown
     , externalMerge
     , wantGuiPause
     , isInteractive
@@ -92,7 +94,8 @@ module Darcs.UI.Flags
     , applyAs
     ) where
 
-import Prelude hiding ( (^) )
+import Prelude ()
+import Darcs.Prelude
 
 import Data.List ( nub, intercalate )
 import Data.Maybe
@@ -102,7 +105,6 @@ import Data.Maybe
     , catMaybes
     )
 import Control.Monad ( unless )
-import Control.Applicative( (<$>) )
 import System.Directory ( doesDirectoryExist, createDirectory )
 import System.FilePath.Posix ( (</>) )
 
@@ -119,7 +121,7 @@ import Darcs.Util.Prompt
     ( askUser
     , askUserListItem
     )
-import Darcs.Repository.Lock ( writeLocaleFile )
+import Darcs.Util.Lock ( writeLocaleFile )
 import Darcs.Repository.Prefs
     ( getPreflist
     , getGlobal
@@ -138,8 +140,9 @@ import Darcs.Util.Path
     , ioAbsolute
     , makeAbsoluteOrStd
     )
-import Darcs.Util.Printer ( putDocLn, text, ($$) )
+import Darcs.Util.Printer ( putDocLn, ePutDocLn, text, ($$), (<+>) )
 import Darcs.Util.URL ( isValidLocalPath )
+import Darcs.Util.Text ( pathlist )
 
 type Config = [F.DarcsFlag]
 
@@ -158,17 +161,18 @@ minimize = parseFlags O.minimize
 editDescription :: Config -> Bool
 editDescription = parseFlags O.editDescription
 
--- | Non-trivial interaction between options.
-diffOpts :: O.UseIndex -> O.LookForAdds -> Bool -> O.DiffAlgorithm -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
+diffOpts :: O.UseIndex -> O.LookForAdds -> O.IncludeBoring -> O.DiffAlgorithm -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
 diffOpts use_index look_for_adds include_boring diff_alg =
     (use_index, scanKnown look_for_adds include_boring, diff_alg)
-  where
-    scanKnown O.NoLookForAdds _ = O.ScanKnown
-    scanKnown O.YesLookForAdds False = O.ScanAll
-    scanKnown O.YesLookForAdds True = O.ScanBoring
+
+-- | Non-trivial interaction between options.
+scanKnown :: O.LookForAdds -> O.IncludeBoring -> O.ScanKnown
+scanKnown O.NoLookForAdds _ = O.ScanKnown
+scanKnown O.YesLookForAdds O.NoIncludeBoring = O.ScanAll
+scanKnown O.YesLookForAdds O.YesIncludeBoring = O.ScanBoring
 
 diffingOpts :: Config -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-diffingOpts flags = diffOpts (useIndex flags) (lookForAdds flags) False (diffAlgorithm flags)
+diffingOpts flags = diffOpts (useIndex flags) (lookForAdds flags) O.NoIncludeBoring (diffAlgorithm flags)
 
 externalMerge :: Config -> O.ExternalMerge
 externalMerge = parseFlags O.useExternalMerge
@@ -207,7 +211,9 @@ doHappyForwarding :: Config -> Bool
 doHappyForwarding = parseFlags O.happyForwarding
 
 includeBoring :: Config -> Bool
-includeBoring = parseFlags O.includeBoring
+includeBoring cfg = case parseFlags O.includeBoring cfg of
+  O.NoIncludeBoring -> False
+  O.YesIncludeBoring -> True
 
 doAllowCaseOnly :: Config -> Bool
 doAllowCaseOnly = parseFlags O.allowCaseDifferingFilenames
@@ -303,8 +309,13 @@ fixUrl d f = if isValidLocalPath f
                 then toFilePath `fmap` withCurrentDirectory d (ioAbsolute f)
                 else return f
 
--- | @maybeFixSubPaths files@ tries to turn the file paths in its argument into
--- @SubPath@s.
+-- | @maybeFixSubPaths (repo_path, orig_path) file_paths@ tries to turn
+-- @file_paths@ into 'SubPath's, taking into account the repository path and
+-- the original path from which darcs was invoked.
+--
+-- A 'SubPath' is a path /under/ (or inside) the repo path. This does /not/
+-- mean it must exist as a file or directory, nor that the path has been added
+-- to the repository; it merely means that it /could/ be added.
 --
 -- When converting a relative path to an absolute one, this function first tries
 -- to interpret the relative path with respect to the current working directory.
@@ -314,30 +325,24 @@ fixUrl d f = if isValidLocalPath f
 --
 -- It is intended for validating file arguments to darcs commands.
 maybeFixSubPaths :: (AbsolutePath, AbsolutePath) -> [FilePath] -> IO [Maybe SubPath]
-maybeFixSubPaths (r, o) fs = withCurrentDirectory o $ do
+maybeFixSubPaths (r, o) fs = do
   fixedFs <- mapM fixit fs
   let bads = snd . unzip . filter (isNothing . fst) $ zip fixedFs fs
-  unless (null bads) . putStrLn $ "Ignoring non-repository paths: " ++
-    intercalate ", " bads
+  unless (null bads) $
+    ePutDocLn $ text "Ignoring non-repository paths:" <+> pathlist bads
   return fixedFs
  where
-    fixit p = do ap <- ioAbsolute p
+    fixit p = do ap <- withCurrentDirectory o $ ioAbsolute p
                  case makeSubPathOf r ap of
                    Just sp -> return $ Just sp
-                   Nothing -> withCurrentDirectory r $ do
-                     absolutePathByRepodir <- ioAbsolute p
+                   Nothing -> do
+                     absolutePathByRepodir <- withCurrentDirectory r $ ioAbsolute p
                      return $ makeSubPathOf r absolutePathByRepodir
 
--- | @fixSubPaths files@ returns the @SubPath@s for the paths in @files@ that
--- are inside the repository, preserving their order. Paths in @files@ that are
--- outside the repository directory are not in the result.
---
--- When converting a relative path to an absolute one, this function first tries
--- to interpret the relative path with respect to the current working directory.
--- If that fails, it tries to interpret it with respect to the repository
--- directory. Only when that fails does it omit the path from the result.
---
--- It is intended for validating file arguments to darcs commands.
+-- | 'fixSubPaths' is a variant of 'maybeFixSubPaths' that throws out
+-- non-repository paths and duplicates from the result. See there for details.
+-- TODO: why filter out null paths from the input? why here and not in
+-- 'maybeFixSubPaths'?
 fixSubPaths :: (AbsolutePath, AbsolutePath) -> [FilePath] -> IO [SubPath]
 fixSubPaths fps fs = nub . catMaybes <$> maybeFixSubPaths fps
     (filter (not . null) fs)
