@@ -7,6 +7,9 @@ module Darcs.UI.Options
     , optDescr
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.Functor.Compose
 import System.Console.GetOpt
 
