@@ -20,8 +20,7 @@
 
 module Darcs.UI.SelectChanges
     ( -- * Working with changes
-      selectChanges
-    , WhichChanges(..)
+      WhichChanges(..)
     , viewChanges
     , withSelectedPatchFromRepo
     , runSelection
@@ -55,6 +54,9 @@ module Darcs.UI.SelectChanges
     , askAboutDepends
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Prelude hiding ( (^) )
 import Control.Monad ( liftM, unless, when, (>=>) )
 import Control.Monad.Identity ( Identity (..) )
@@ -67,15 +69,14 @@ import Control.Monad.State
     , modify, runStateT
     )
 import Control.Monad.Trans ( liftIO )
-import Data.Char ( toUpper )
 import Data.List ( intercalate, union )
-import Data.Maybe ( isJust, isNothing, catMaybes )
+import Data.Maybe ( isJust, catMaybes )
 import System.Exit ( exitSuccess )
 
 import Darcs.Patch
-    ( Patchy, PrimPatch, RepoPatch, PrimOf
+    ( Patchy, PrimPatch, IsRepoType, RepoPatch, PrimOf
     , commuteFLorComplain, invert
-    , listTouchedFiles, anonymous, fromPrims
+    , listTouchedFiles, fromPrims
     )
 import qualified Darcs.Patch ( thing, things, summary )
 import Darcs.Patch.Apply ( ApplyState )
@@ -95,6 +96,7 @@ import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Patch.Inspect ( PatchInspect )
 import Darcs.Patch.Invert ( Invert )
 import Darcs.Patch.Match ( haveNonrangeMatch, matchAPatch, matchAPatchread )
+import Darcs.Patch.Named.Wrapped ( anonymous )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, n2pia )
 import Darcs.Patch.Set ( PatchSet(..), newset2RL )
 import Darcs.Patch.Show ( ShowPatch )
@@ -119,15 +121,15 @@ import Darcs.Patch.Witnesses.WZipper
 import Darcs.Repository ( Repository, readRepo, readTentativeRepo )
 import Darcs.UI.External ( editText )
 import Darcs.UI.Options.All
-    ( Verbosity(..), Summary(..), DiffAlgorithm(..)
+    ( Verbosity(..), Summary(..)
     , WithContext(..), SelectDeps(..), MatchFlag )
 import Darcs.UI.PrintPatch
     ( printFriendly, printPatch
     , printPatchPager, showFriendly )
-import Darcs.Util.English ( Noun (..), englishNum )
+import Darcs.Util.English ( Noun (..), englishNum, capitalize )
 import Darcs.Util.Printer ( prefix, putDocLn )
 import Darcs.Util.Prompt ( PromptConfig (..), askUser, promptChar )
-import Storage.Hashed.Tree ( Tree )
+import Darcs.Util.Tree ( Tree )
 
 
 -- | When asking about patches, we either ask about them in
@@ -166,7 +168,6 @@ data MatchCriterion p = MatchCriterion
 data PatchSelectionOptions = PatchSelectionOptions
   { verbosity :: Verbosity
   , matchFlags :: [MatchFlag]
-  , diffAlgorithm :: DiffAlgorithm
   , interactive :: Bool
   , selectDeps :: SelectDeps
   , summary :: Summary
@@ -206,8 +207,8 @@ selectionContextPrim whch jn o spl fs p =
      }
 
 -- | A 'PatchSelectionContext' for selecting full patches ('PatchInfoAnd' patches)
-selectionContext :: (RepoPatch p) => WhichChanges -> String -> PatchSelectionOptions -> Maybe (Splitter (PatchInfoAnd p))
-                 -> Maybe [FilePath] -> PatchSelectionContext (PatchInfoAnd p)
+selectionContext :: (IsRepoType rt, RepoPatch p) => WhichChanges -> String -> PatchSelectionOptions -> Maybe (Splitter (PatchInfoAnd rt p))
+                 -> Maybe [FilePath] -> PatchSelectionContext (PatchInfoAnd rt p)
 selectionContext whch jn o spl fs =
  PSC { opts = o
      , splitter = spl
@@ -220,8 +221,8 @@ selectionContext whch jn o spl fs =
      }
 
 -- | A generic 'PatchSelectionContext'.
-selectionContextGeneric :: (RepoPatch p, Invert q)
-                        => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd p))
+selectionContextGeneric :: (IsRepoType rt, RepoPatch p, Invert q)
+                        => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd rt p))
                         -> WhichChanges
                         -> String
                         -> PatchSelectionOptions
@@ -265,13 +266,13 @@ triv :: MatchCriterion p
 triv = MatchCriterion { mcHasNonrange = False, mcFunction = \ _ _ -> True }
 
 -- | 'iswanted' selects patches according to the given match flags
-iswanted :: forall p q
-          . (RepoPatch p, Invert q)
-         => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd p))
+iswanted :: forall rt p q
+          . (IsRepoType rt, RepoPatch p, Invert q)
+         => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd rt p))
          -> [MatchFlag]
          -> MatchCriterion q
 iswanted extract mflags = MatchCriterion
-    { mcHasNonrange = haveNonrangeMatch (PatchType :: PatchType p) mflags
+    { mcHasNonrange = haveNonrangeMatch (PatchType :: PatchType rt p) mflags
     , mcFunction = isWantedMcFunction
     }
   where
@@ -285,32 +286,31 @@ liftR :: Monad m => Reader r a -> ReaderT r m a
 liftR = asks . runReader
 
 -- | runs a 'PatchSelection' action in the given 'PatchSelectionContext'.
-runSelection :: (Patchy p) => PatchSelection p wX wY -> PatchSelectionContext p
+runSelection :: forall p wX wY . (Patchy p, PatchInspect p, ShowPatch p, ApplyState p ~ Tree)
+             => FL p wX wY
+             -> PatchSelectionContext p
              -> IO ((FL p :> FL p) wX wY)
-runSelection = runReaderT
-
--- | Select patches from a @FL@.
-selectChanges :: forall p wX wY . (Patchy p, PatchInspect p, ShowPatch p, ApplyState p ~ Tree) =>
-                                FL p wX wY
-                             -> PatchSelection p wX wY
-selectChanges ps = do
-    whch <- asks whichChanges
-    case whch of
+runSelection ps psc = runReaderT patchSelection psc
+ where
+  patchSelection = do
+    case whichChanges psc of
         First -> normal ps
         Last -> normal ps
         FirstReversed -> reversed ps
         LastReversed -> reversed ps
-  where normal = sc1
-        reversed = return . invert >=> sc1 >=> return . invertC
+  normal fl = sc1 fl
+  reversed fl = do let ifl = invert fl
+                   choices_ <- sc1 ifl
+                   return $ invertC choices_
 
 sc1 :: forall p wX wY . (Patchy p, PatchInspect p, ShowPatch p, ApplyState p ~ Tree) =>
                                 FL p wX wY
                              -> PatchSelection p wX wY
-sc1 =
-    (liftR . patchesToConsider)
-     >=> realSelectChanges
-     >=> (\ps -> do whch <- asks whichChanges ; return $ selectedPatches whch ps)
-     >=> (liftR . canonizeAfterSplitter)
+sc1 fl = do ps <- liftR (patchesToConsider fl)
+            patchChoices_ <- realSelectChanges ps
+            whch <- asks whichChanges
+            let sps = selectedPatches whch patchChoices_
+            liftR (canonizeAfterSplitter sps)
 
 -- | inverses the choices that have been made
 invertC :: (Patchy p) => (FL p :> FL p) wX wY -> (FL p :> FL p) wY wX
@@ -324,7 +324,7 @@ repr LastReversed (Sealed2 p) = Sealed2 (invert p)
 repr Last (Sealed2 p) = Sealed2 p
 repr FirstReversed (Sealed2 p) = Sealed2 (invert p)
 
--- | The equivalent of 'selectChanges' for the @darcs changes@ command
+-- | The equivalent of 'runSelection' for the @darcs log@ command
 viewChanges :: (Patchy p, ShowPatch p, ApplyState p ~ Tree) => PatchSelectionOptions -> [Sealed2 p] -> IO ()
 viewChanges ps_opts = textView ps_opts Nothing 0 []
 
@@ -353,11 +353,11 @@ keysFor = concatMap (map kp)
 
 -- | The function for selecting a patch to amend record. Read at your own risks.
 withSelectedPatchFromRepo ::
-       forall p wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
+       forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
     => String   -- name of calling command (always "amend" as of now)
-    -> Repository p wR wU wT
+    -> Repository rt p wR wU wT
     -> PatchSelectionOptions
-    -> (forall wA . (FL (PatchInfoAnd p) :> PatchInfoAnd p) wA wR -> IO ())
+    -> (forall wA . (FL (PatchInfoAnd rt p) :> PatchInfoAnd rt p) wA wR -> IO ())
     -> IO ()
 withSelectedPatchFromRepo jn repository o job = do
     patchSet <- readRepo repository
@@ -374,14 +374,14 @@ data WithSkipped p wX wY = WithSkipped { _skippedReason :: SkippedReason, skippe
 -- | This ensures that the selected patch commutes freely with the skipped
 -- patches, including pending and also that the skipped sequences has an
 -- ending context that matches the recorded state, z, of the repository.
-wspfr :: forall p wX wY wU. (RepoPatch p, ApplyState p ~ Tree)
+wspfr :: forall rt p wX wY wU. (RepoPatch p, ApplyState p ~ Tree)
       => String
-      -> (forall wA wB . (PatchInfoAnd p) wA wB -> Bool)
-      -> RL (PatchInfoAnd p) wX wY
-      -> FL (WithSkipped (PatchInfoAnd p)) wY wU
-      -> IO (Maybe (FlippedSeal (FL (PatchInfoAnd p) :> PatchInfoAnd p) wU))
+      -> (forall wA wB . (PatchInfoAnd rt p) wA wB -> Bool)
+      -> RL (PatchInfoAnd rt p) wX wY
+      -> FL (WithSkipped (PatchInfoAnd rt p)) wY wU
+      -> IO (Maybe (FlippedSeal (FL (PatchInfoAnd rt p) :> PatchInfoAnd rt p) wU))
 wspfr _ _ NilRL _ = return Nothing
-wspfr jn matches remaining@(p:<:pps) skipped
+wspfr jn matches remaining@(pps:<:p) skipped
     | not $ matches p = wspfr jn matches pps
                             (WithSkipped SkippedAutomatically p :>: skipped)
     | otherwise =
@@ -407,22 +407,21 @@ wspfr jn matches remaining@(p:<:pps) skipped
             'p' -> printPatchPager p >> repeatThis
             'x' -> do putDocLn $ prefix "    " $ Darcs.Patch.summary p
                       repeatThis
-            'q' -> do putStrLn $ jnCapital ++ " cancelled."
+            'q' -> do putStrLn $ (capitalize jn) ++ " cancelled."
                       exitSuccess
             _   -> do putStrLn $ helpFor jn basicOptions advancedOptions
                       repeatThis
-  where jnCapital = toUpper (head jn) : tail jn
-        repeatThis = wspfr jn matches (p:<:pps) skipped
+  where repeatThis = wspfr jn matches (pps:<:p) skipped
         prompt' = "Shall I " ++ jn ++ " this patch?"
         nextPatch = wspfr jn matches pps (WithSkipped SkippedManually p:>:skipped)
-        previousPatch :: RL (PatchInfoAnd p) wX wQ
-                      -> FL (WithSkipped (PatchInfoAnd p)) wQ wU
-                      -> IO (Maybe (FlippedSeal (FL (PatchInfoAnd p) :> PatchInfoAnd p) wU))
+        previousPatch :: RL (PatchInfoAnd rt p) wX wQ
+                      -> FL (WithSkipped (PatchInfoAnd rt p)) wQ wU
+                      -> IO (Maybe (FlippedSeal (FL (PatchInfoAnd rt p) :> PatchInfoAnd rt p) wU))
         previousPatch remaining' NilFL = wspfr jn matches remaining' NilFL
         previousPatch remaining' (WithSkipped sk prev :>: skipped'') =
             case sk of
-                SkippedManually -> wspfr jn matches (prev :<: remaining') skipped''
-                SkippedAutomatically -> previousPatch (prev :<: remaining') skipped''
+                SkippedManually -> wspfr jn matches (remaining' :<: prev) skipped''
+                SkippedAutomatically -> previousPatch (remaining' :<: prev) skipped''
         basicOptions =
                     [[ KeyPress 'y' (jn ++ " this patch")
                      , KeyPress 'n' ("don't " ++ jn ++ " it")
@@ -441,11 +440,9 @@ wspfr jn matches remaining@(p:<:pps) skipped
 -- After selecting with a splitter, the results may not be canonical
 canonizeAfterSplitter :: (FL p :> FL p) wX wY -> Reader (PatchSelectionContext p) ((FL p :> FL p) wX wY)
 canonizeAfterSplitter (x :> y) =
-    do o <- asks opts
-       mspl <- asks splitter
-       let  canonizeIfNeeded = maybe (\_ b -> b) canonizeSplit mspl
-            da = diffAlgorithm o
-       return $ canonizeIfNeeded da x :> canonizeIfNeeded da y
+    do mspl <- asks splitter
+       let canonizeIfNeeded = maybe id canonizeSplit mspl
+       return $ canonizeIfNeeded x :> canonizeIfNeeded y
 
 realSelectChanges :: forall p wX wY.
                      (Patchy p, ShowPatch p, PatchInspect p, ApplyState p ~ Tree)
@@ -512,10 +509,15 @@ patchesToConsider ps =
                 FirstReversed -> TouchesFiles.selectNotTouching
                 LastReversed -> TouchesFiles.deselectNotTouching
           everything = patchChoices ps
-      if isNothing fs && not (mcHasNonrange crit)
-         then return everything
-         else do notUnwanted <- deselectUnwanted everything
-                 return $ deselectNotTouching fs notUnwanted
+      -- first filter patches by matchers
+      -- this often requires to read only metadata
+      notUnwanted <- if not (mcHasNonrange crit)
+                       then return everything
+                       else deselectUnwanted everything
+      -- if there are path restrictions, we need to read patch contents
+      -- of what remains
+      return $ if isJust fs then deselectNotTouching fs notUnwanted
+                            else notUnwanted
 
 -- | Returns the results of a patch selection user interaction
 selectedPatches :: Patchy p => WhichChanges -> PatchChoices p wY wZ
@@ -733,8 +735,7 @@ splitCurrent :: forall p wX wY . Patchy p
              -> InteractiveSelectionM p wX wY ()
 splitCurrent s = do
     FZipper lps_done (lp:>:lps_todo) <- gets lps
-    o <- asks opts
-    case applySplitter s (diffAlgorithm o) (lpPatch lp) of
+    case applySplitter s (lpPatch lp) of
       Nothing -> return ()
       Just (text, parse) ->
           do
@@ -858,9 +859,7 @@ textSelectOne = do
              reprCur = repr whichch (Sealed2 (lpPatch lp))
          (basicOptions,advancedOptions) <- options singleFile
          theSlot <- liftChoices $ patchSlot' lp
-         let
-             the_default = getDefault (whichch == Last || whichch == FirstReversed) theSlot
-             jnCapital = toUpper (head jn) : tail jn
+         let the_default = getDefault (whichch == Last || whichch == FirstReversed) theSlot
          yorn <- promptUser singleFile the_default
          let nextPatch = skipMundane >> showCur
          case yorn of
@@ -893,7 +892,7 @@ textSelectOne = do
                      skipAll
                      return True
                'q' -> liftIO $
-                      do putStrLn $ jnCapital++" cancelled."
+                      do putStrLn $ capitalize jn ++ " cancelled."
                          exitSuccess
                'j' -> skipOne >> showCur >> return False
                'k' -> backOne >> showCur >> return False
@@ -909,7 +908,7 @@ lastQuestion = do
   aThing <- thing
   let (basicOptions, advancedOptions) = optionsLast jn aThing
   yorn <- liftIO . promptChar $
-            PromptConfig { pPrompt = "Do you want to "++jn++" these "++ theThings++"?"
+            PromptConfig { pPrompt = "Do you want to "++capitalize jn++" these "++theThings++"?"
                          , pBasicCharacters = "yglqk"
                          , pAdvancedCharacters = "dan"
                          , pDefault = Just 'y'
@@ -1030,7 +1029,7 @@ skipMundane = do
     boring :> interesting ->
         do
           justDone $ lengthFL boring + numSkipped
-          modify $ \isc -> isc {lps = FZipper (reverseFL boring +<+ reverseFL skipped +<+ lps_done) interesting}
+          modify $ \isc -> isc {lps = FZipper (lps_done +<+ reverseFL skipped +<+ reverseFL boring) interesting}
     where
       show_skipped o jn n ps = do putStrLn $ _nevermind_ jn ++ _these_ n ++ "."
                                   when (verbosity o == Verbose) $
@@ -1055,8 +1054,8 @@ getDefault True InLast   = 'y'
 getDefault False InFirst = 'y'
 getDefault False InLast  = 'n'
 
-askAboutDepends :: forall p wR wU wT wY . (RepoPatch p, ApplyState p ~ Tree)
-                => Repository p wR wU wT -> FL (PrimOf p) wT wY
+askAboutDepends :: forall rt p wR wU wT wY . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                => Repository rt p wR wU wT -> FL (PrimOf p) wT wY
                 -> PatchSelectionOptions
                 -> [PatchInfo] -> IO [PatchInfo]
 askAboutDepends repository pa' ps_opts olddeps = do
@@ -1068,8 +1067,8 @@ askAboutDepends repository pa' ps_opts olddeps = do
   -- FIXME: this code is completely unreadable
   FlippedSeal ps <- return
                       ((case pps of
-                          PatchSet x _ -> FlippedSeal (reverseRL x+>+(pa:>:NilFL)))
-                         :: FlippedSeal (FL (PatchInfoAnd p)) wY)
+                          PatchSet _ x -> FlippedSeal (reverseRL x+>+(pa:>:NilFL)))
+                         :: FlippedSeal (FL (PatchInfoAnd rt p)) wY)
   let (pc, my_lps) = patchChoicesLps ps
       tas = case catMaybes (mapFL (\lp -> if pa `unsafeCompare` lpPatch lp || info (lpPatch lp) `elem` olddeps
                                           then Just (label lp) else Nothing) my_lps) of
@@ -1077,7 +1076,7 @@ askAboutDepends repository pa' ps_opts olddeps = do
                 [] -> error "askAboutDepends: []"
                 tgs -> tgs
   Sealed2 ps' <- return $ case getChoices (forceFirsts tas pc) of _ :> mc :> _ -> Sealed2 $ mapFL_FL lpPatch mc
-  (deps:>_) <- runSelection (selectChanges ps') $
+  (deps:>_) <- runSelection ps' $
                  selectionContext FirstReversed "depend on" ps_opts { matchFlags = [], interactive = True } Nothing Nothing
   return $ olddeps `union` mapFL info deps
 {-
