@@ -3,6 +3,9 @@ module Darcs.Util.Bug
     ( _bug, _bugDoc, _impossible, _fromJust
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.Maybe(fromMaybe)
 import Darcs.Util.Printer ( Doc, errorDoc, text, ($$) )
 
