@@ -1,5 +1,8 @@
 module Darcs.Util.Crypt.SHA256 ( sha256sum ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Crypto.Hash.SHA256 ( hash )
 import Data.ByteString ( ByteString )
 import Data.ByteString.Base16 ( encode )
