@@ -3,10 +3,6 @@
 --  BSD3
 
 {-# LANGUAGE CPP #-}
-#if MIN_VERSION_time(1,5,0)
--- for parseTime, which we need to use while we support time-1.4
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
-#endif
 module Darcs.Util.DateTime
     ( getCurrentTime, toSeconds
     , formatDateTime, fromClockTime, parseDateTime, startOfTime
@@ -20,17 +16,13 @@ import Data.Time.Clock
     ( UTCTime(UTCTime), UniversalTime(ModJulianDate)
     , getModJulianDate, secondsToDiffTime, getCurrentTime
     )
-import Data.Time.Format ( formatTime, parseTime )
+import Data.Time.Format ( formatTime, parseTimeM )
 import Data.Time.LocalTime
     ( utc
     , localTimeToUT1, ut1ToLocalTime
     , localTimeToUTC, utcToLocalTime
     )
-#if MIN_VERSION_time(1,5,0)
 import Data.Time ( defaultTimeLocale )
-#else
-import System.Locale ( defaultTimeLocale )
-#endif
 import System.Time ( ClockTime(TOD) )
 
 toSeconds    :: UTCTime -> Integer
@@ -63,7 +55,7 @@ formatDateTime :: String -> UTCTime -> String
 formatDateTime = formatTime defaultTimeLocale
 
 parseDateTime :: String -> String -> Maybe UTCTime
-parseDateTime = parseTime defaultTimeLocale
+parseDateTime = parseTimeM True defaultTimeLocale
 
 fromClockTime           :: ClockTime -> UTCTime
 fromClockTime (TOD s _) = fromSeconds s
