@@ -60,6 +60,9 @@ module Darcs.Util.Diff.Myers
     , getSlice
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Monad
 import Data.Int
 import Control.Monad.ST
