@@ -20,6 +20,9 @@ module Darcs.Util.Diff.Patience
     ( getChanges
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.List ( sort )
 import Data.Array.Unboxed
 import Data.Array.ST
@@ -27,11 +30,7 @@ import Control.Monad.ST
 import qualified Data.Set as S
 import qualified Data.ByteString as B ( ByteString, elem )
 import qualified Data.ByteString.Char8 as BC ( pack )
-#ifdef USE_LOCAL_DATA_MAP_STRICT
-import qualified Darcs.Data.Map.Strict as M
-#else
 import qualified Data.Map.Strict as M
-#endif
     ( Map, lookup, insertWith, empty, elems )
 import qualified Data.Hashable as H ( hash )
 import Darcs.Util.Diff.Myers (initP, aLen, PArray, getSlice)
@@ -102,7 +101,7 @@ insert h bs hmap = (hunknumber, HunkMapInfo newsize (M.insertWith (\_ o -> (hunk
 --Given a HunkMap, check collisions and return the line with an updated Map
 toHunk' :: HunkMap -> B.ByteString -> (Hunk, HunkMap)
 toHunk' lmap bs | oldbs == Nothing || null oldhunkpair = insert hash bs lmap
-                | otherwise = (fst $ head $ oldhunkpair, lmap)
+                | otherwise = (fst $ head oldhunkpair, lmap)
                     where hash = H.hash bs
                           oldbs = M.lookup hash (getMap lmap)
                           oldhunkpair = filter ((== bs) . snd) $ fromJust oldbs
@@ -111,7 +110,7 @@ listToHunk :: [B.ByteString] -> HunkMap -> ([Hunk], HunkMap)
 listToHunk [] hmap = ([], hmap)
 listToHunk (x:xs) hmap = let (y, hmap') = toHunk' hmap x
                              (ys, hmap'') = listToHunk xs hmap'
-                         in ((y:ys), hmap'')
+                         in (y:ys, hmap'')
 
 --listToHunk :: [B.ByteString] -> HunkMap -> ([Hunk], HunkMap)
 --listToHunk = listToHunk' []
@@ -149,7 +148,7 @@ borings' :: [B.ByteString]
 borings' = map BC.pack ["", "\n", " ", ")", "(", ","]
 
 byparagraph :: [Hunk] -> [[Hunk]]
-byparagraph = reverse . (map reverse) . byparagraphAcc []
+byparagraph = reverse . map reverse . byparagraphAcc []
     where byparagraphAcc xs [] = xs
           byparagraphAcc [] (a:b:c:d)
               | a == nl && c == nl && b == hnull = case d of
@@ -158,7 +157,7 @@ byparagraph = reverse . (map reverse) . byparagraphAcc []
           byparagraphAcc [] (a:as) = byparagraphAcc [[a]] as
           byparagraphAcc (x:xs) (a:b:c:d)
               | a == nl && c == nl && b == hnull = case d of
-                                                   [] -> ((c:b:a:x):xs)
+                                                   [] -> (c:b:a:x):xs
                                                    _  -> byparagraphAcc ([]:((c:b:a:x):xs)) d
           byparagraphAcc (x:xs) (a:as) = byparagraphAcc ((a:x):xs) as
           nl = -1 -- "\n" hunk
@@ -189,27 +188,27 @@ mkdiff :: Ord a =>
           ([a] -> Bool) -> Int -> [a] -> [a] -> [a] -> [(Int,[a],[a])]
 mkdiff b ny (l:ls) (x:xs) (y:ys)
     | l == x && l == y = mkdiff b (ny+1) ls xs ys
-mkdiff boring ny (l:ls) xs ys =
-        if rmd == add
-        then mkdiff boring (ny+length add+1) ls restx resty
-        else if boring rmd && boring add
-             then case lcs rmd add of
-                    [] -> prefixPostfixDiff ny rmd add ++
-                          mkdiff boring (ny+length add+1) ls restx resty
-                    ll -> mkdiff (const False) ny ll rmd add ++
-                          mkdiff boring  (ny+length add+1) ls restx resty
-             else prefixPostfixDiff ny rmd add ++
-                  mkdiff boring (ny+length add+1) ls restx resty
+mkdiff boring ny (l:ls) xs ys
+  | rmd == add = mkdiff boring (ny+length add+1) ls restx resty
+  | boring rmd && boring add =
+      case lcs rmd add of
+        [] -> prefixPostfixDiff ny rmd add ++
+              mkdiff boring (ny+length add+1) ls restx resty
+        ll -> mkdiff (const False) ny ll rmd add ++
+              mkdiff boring  (ny+length add+1) ls restx resty
+  | otherwise = prefixPostfixDiff ny rmd add ++
+                mkdiff boring (ny+length add+1) ls restx resty
     where rmd = takeWhile (/= l) xs
           add = takeWhile (/= l) ys
           restx = drop (length rmd + 1) xs
           resty = drop (length add + 1) ys
 mkdiff _ _ [] [] [] = []
-mkdiff boring ny [] rmd add =
-    if boring rmd && boring add
-    then case lcs rmd add of [] -> prefixPostfixDiff ny rmd add
-                             ll -> mkdiff (const False) ny ll rmd add
-    else prefixPostfixDiff ny rmd add
+mkdiff boring ny [] rmd add
+  | boring rmd && boring add =
+      case lcs rmd add of
+        [] -> prefixPostfixDiff ny rmd add
+        ll -> mkdiff (const False) ny ll rmd add
+  | otherwise = prefixPostfixDiff ny rmd add
 
 prefixPostfixDiff :: Ord a => Int -> [a] -> [a] -> [(Int,[a],[a])]
 prefixPostfixDiff _ [] [] = []
@@ -302,81 +301,79 @@ lcs _ [] = []
 lcs (c1:c1s) (c2:c2s)
     | c1 == c2 = c1: lcs c1s c2s
     | otherwise =
-        reverse $ lcs_simple (reverse (c1:c1s)) (reverse (c2:c2s))
+        reverse $ lcsSimple (reverse (c1:c1s)) (reverse (c2:c2s))
 
-lcs_simple :: Ord a => [a] -> [a] -> [a]
-lcs_simple [] _ = []
-lcs_simple _ [] = []
-lcs_simple s1@(c1:c1s) s2@(c2:c2s)
+lcsSimple :: Ord a => [a] -> [a] -> [a]
+lcsSimple [] _ = []
+lcsSimple _ [] = []
+lcsSimple s1@(c1:c1s) s2@(c2:c2s)
     | c1 == c2 = c1: lcs c1s c2s
-    | otherwise = hunt $ prune_matches s1 $! find_matches s1 s2
+    | otherwise = hunt $ pruneMatches s1 $! findMatches s1 s2
 
-prune_matches :: [a] -> [[Int]] -> [(a, [Int])]
-prune_matches _ [] = []
-prune_matches [] _ = []
-prune_matches (_:cs) ([]:ms) = prune_matches cs ms
-prune_matches (c:cs) (m:ms) = (c,m): prune_matches cs ms
+pruneMatches :: [a] -> [[Int]] -> [(a, [Int])]
+pruneMatches _ [] = []
+pruneMatches [] _ = []
+pruneMatches (_:cs) ([]:ms) = pruneMatches cs ms
+pruneMatches (c:cs) (m:ms) = (c,m): pruneMatches cs ms
 
 type Threshold s a = STArray s Int (Int,[a])
 
 hunt :: [(a, [Int])] -> [a]
 hunt [] = []
 hunt csmatches =
-    runST ( do th <- empty_threshold (length csmatches) l
-               hunt_internal csmatches th
-               hunt_recover th (-1) l )
+    runST ( do th <- emptyThreshold (length csmatches) l
+               huntInternal csmatches th
+               huntRecover th (-1) l )
     where l = maximum (0 : concat (map snd csmatches))
 
-hunt_internal :: [(a, [Int])] -> Threshold s a -> ST s ()
-hunt_internal [] _ = return ()
-hunt_internal ((c,m):csms) th = do
-    hunt_one_char c m th
-    hunt_internal csms th
+huntInternal :: [(a, [Int])] -> Threshold s a -> ST s ()
+huntInternal [] _ = return ()
+huntInternal ((c,m):csms) th = do
+    huntOneChar c m th
+    huntInternal csms th
 
-hunt_one_char :: a -> [Int] ->  Threshold s a -> ST s ()
-hunt_one_char _ [] _ = return ()
-hunt_one_char c (j:js) th = do
-    index_k <- my_bs j th
+huntOneChar :: a -> [Int] ->  Threshold s a -> ST s ()
+huntOneChar _ [] _ = return ()
+huntOneChar c (j:js) th = do
+    index_k <- myBs j th
     case index_k of
       Nothing -> return ()
       Just k -> do
         (_, rest) <- readArray th (k-1)
         writeArray th k (j, c:rest)
-    hunt_one_char c js th
+    huntOneChar c js th
 
 -- This is O(n), which is stupid.
-hunt_recover :: Threshold s a -> Int -> Int -> ST s [a]
-hunt_recover th n limit =
+huntRecover :: Threshold s a -> Int -> Int -> ST s [a]
+huntRecover th n limit =
  do (_, th_max) <- getBounds th
     if n < 0
-       then hunt_recover th th_max limit
-       else if n == 0
+       then huntRecover th th_max limit
+       else if n == 0 || n > th_max
             then return []
-            else if n > th_max
-                 then return []
-                 else do (thn, sn) <- readArray th n
-                         if thn <= limit
-                             then return $ reverse sn
-                             else hunt_recover th (n-1) limit
+            else do (thn, sn) <- readArray th n
+                    if thn <= limit
+                      then return $ reverse sn
+                      else huntRecover th (n-1) limit
 
-empty_threshold :: Int -> Int -> ST s (Threshold s a)
-empty_threshold l th_max = do
+emptyThreshold :: Int -> Int -> ST s (Threshold s a)
+emptyThreshold l th_max = do
   th <- newArray (0,l) (th_max+1, [])
   writeArray th 0 (0, [])
   return th
 
-my_bs :: Int -> Threshold s a -> ST s (Maybe Int)
-my_bs j th = do bnds <- getBounds th
-                my_helper_bs j bnds th
+myBs :: Int -> Threshold s a -> ST s (Maybe Int)
+myBs j th = do bnds <- getBounds th
+               myHelperBs j bnds th
 
-my_helper_bs :: Int -> (Int,Int) -> Threshold s a ->
+myHelperBs :: Int -> (Int,Int) -> Threshold s a ->
                 ST s (Maybe Int)
-my_helper_bs j (th_min,th_max) th =
+myHelperBs j (th_min,th_max) th =
     if th_max - th_min > 1 then do
        (midth, _) <- readArray th th_middle
        if j > midth
-         then my_helper_bs j (th_middle,th_max) th
-         else my_helper_bs j (th_min,th_middle) th
+         then myHelperBs j (th_middle,th_max) th
+         else myHelperBs j (th_min,th_middle) th
     else do
        (minth, _) <- readArray th th_min
        (maxth, _) <- readArray th th_max
@@ -388,32 +385,32 @@ my_helper_bs j (th_min,th_max) th =
 
 
 
-find_matches :: Ord a => [a] -> [a] -> [[Int]]
-find_matches [] [] = []
-find_matches [] (_:bs) = []: find_matches [] bs
-find_matches _ [] = []
-find_matches a b =
-    unzip_indexed $ sort $ find_sorted_matches indexeda indexedb [] []
+findMatches :: Ord a => [a] -> [a] -> [[Int]]
+findMatches [] [] = []
+findMatches [] (_:bs) = []: findMatches [] bs
+findMatches _ [] = []
+findMatches a b =
+    unzipIndexed $ sort $ findSortedMatches indexeda indexedb [] []
     where indexeda = sort $ zip a [1..]
           indexedb = sort $ zip b [1..]
 
-unzip_indexed :: [(Int,[a])] -> [[a]]
-unzip_indexed s = unzip_indexed_helper 1 s
-    where unzip_indexed_helper _ [] = []
-          unzip_indexed_helper thisl ((l,c):rest)
-           | thisl == l = c: unzip_indexed_helper (l+1) rest
-           | otherwise = []: unzip_indexed_helper (thisl+1) ((l,c):rest)
+unzipIndexed :: [(Int,[a])] -> [[a]]
+unzipIndexed s = unzipIndexedHelper 1 s
+    where unzipIndexedHelper _ [] = []
+          unzipIndexedHelper thisl ((l,c):rest)
+           | thisl == l = c: unzipIndexedHelper (l+1) rest
+           | otherwise = []: unzipIndexedHelper (thisl+1) ((l,c):rest)
 
-find_sorted_matches :: Ord a => [(a, Int)] -> [(a, Int)] -> [a] -> [Int]
+findSortedMatches :: Ord a => [(a, Int)] -> [(a, Int)] -> [a] -> [Int]
                              -> [(Int, [Int])]
-find_sorted_matches [] _ _ _ = []
-find_sorted_matches _ [] _ _ = []
-find_sorted_matches ((a,na):as) ((b,nb):bs) aold aoldmatches
+findSortedMatches [] _ _ _ = []
+findSortedMatches _ [] _ _ = []
+findSortedMatches ((a,na):as) ((b,nb):bs) aold aoldmatches
     | [a] == aold = (na, aoldmatches) :
-                    find_sorted_matches as ((b,nb):bs) aold aoldmatches
-    | a > b = find_sorted_matches ((a,na):as) bs aold aoldmatches
-    | a < b = find_sorted_matches as ((b,nb):bs) aold aoldmatches
+                    findSortedMatches as ((b,nb):bs) aold aoldmatches
+    | a > b = findSortedMatches ((a,na):as) bs aold aoldmatches
+    | a < b = findSortedMatches as ((b,nb):bs) aold aoldmatches
 -- following line is inefficient if a line is repeated many times.
-find_sorted_matches ((a,na):as) bs _ _ -- a == b
-      = (na, matches) : find_sorted_matches as bs [a] matches
+findSortedMatches ((a,na):as) bs _ _ -- a == b
+      = (na, matches) : findSortedMatches as bs [a] matches
     where matches = reverse $ map snd $ filter ((==a) . fst) bs
