@@ -51,11 +51,8 @@ import Darcs.Util.Workaround ( renameFile )
 
 #ifdef HAVE_CURL
 import qualified Darcs.Util.Download.Curl as Curl
-#elif defined(HAVE_HTTP)
-import qualified Darcs.Util.Download.HTTP as HTTP
 #else
-import Darcs.Util.Progress ( debugFail )
-import qualified HTTP ( requestUrl, waitNextUrl )
+import qualified Darcs.Util.Download.HTTP as HTTP
 #endif
 
 #include "impossible.h"
@@ -294,20 +291,13 @@ requestUrl = Curl.requestUrl
 waitNextUrl' = Curl.waitNextUrl
 pipeliningEnabled = Curl.pipeliningEnabled
 
-#elif defined(HAVE_HTTP)
+#else
 
 setDebugHTTP = return ()
 requestUrl = HTTP.requestUrl
 waitNextUrl' = HTTP.waitNextUrl
 pipeliningEnabled = return False
 
-#else
-
-setDebugHTTP = debugMessage "URL.setDebugHttp works only with libcurl"
-requestUrl _ _ _ = debugFail "URL.requestUrl: there is no libcurl!"
-waitNextUrl' = debugFail "URL.waitNextUrl': there is no libcurl!"
-pipeliningEnabled = return False
-
 #endif
 
 -- Usage of these environment variables happens in C code, so the
