@@ -36,6 +36,9 @@ module Darcs.Util.IsoDate
     , unsetTime, TimeInterval
     ) where
 
+import Prelude ( (^) )
+import Darcs.Prelude
+
 import Text.ParserCombinators.Parsec
 import System.Time
 import System.IO.Unsafe ( unsafePerformIO )
