@@ -36,33 +36,61 @@
 -- This code was made generic in the element type by Juliusz Chroboczek.
 module Darcs.Util.Printer
     (
-      Printable(..), Doc(Doc,unDoc), Printers, Printers'(..), Printer, Color(..)
+    -- * 'Doc' type and structural combinators
+      Doc(Doc,unDoc)
+    , empty, (<>), (<?>), (<+>), ($$), vcat, vsep, hcat, hsep
+    , minus, newline, plus, space, backslash, lparen, rparen
+    , parens
+    -- * Constructing 'Doc's
+    , text
+    , hiddenText
+    , invisibleText
+    , wrapText, quoted
+    , userchunk, packedString
+    , prefix
+    , hiddenPrefix
+    , insertBeforeLastline
+    , prefixLines
+    , invisiblePS, userchunkPS
+    -- * Rendering
     , RenderMode(..)
+    , renderString, renderStringWith, renderPS, renderPSWith
+    , renderPSs, renderPSsWith
+    -- * Printers
+    , Printers
+    , Printers'(..)
+    , Printer
+    , simplePrinters, invisiblePrinter, simplePrinter
+    -- * Printables
+    , Printable(..)
+    , doc
+    , printable, invisiblePrintable, hiddenPrintable, userchunkPrintable
+    -- * Constructing colored 'Doc's
+    , Color(..)
+    , blueText, redText, greenText, magentaText, cyanText
+    , colorText
+    , lineColor
+    -- * IO
     , hPutDoc,     hPutDocLn,     putDoc,     putDocLn
     , hPutDocWith, hPutDocLnWith, putDocWith, putDocLnWith
     , hPutDocCompr
     , debugDocLn
-    , renderString, renderStringWith, renderPS, renderPSWith
-    , renderPSs, renderPSsWith, lineColor
-    , prefix, insertBeforeLastline, colorText, invisibleText
-    , prefixLines
-    , hiddenText, hiddenPrefix, userchunk, text
-    , printable, wrapText
-    , blueText, redText, greenText, magentaText, cyanText
-    , unsafeText, unsafeBoth, unsafeBothText, unsafeChar
-    , invisiblePS, packedString, unsafePackedString, userchunkPS
-    , simplePrinters, invisiblePrinter, simplePrinter
-    , doc, empty, (<>), (<?>), (<+>), ($$), vcat, vsep, hcat
-    , minus, newline, plus, space, backslash, lparen, rparen
-    , parens
+    , ePutDocLn
     , errorDoc
+    -- * Unsafe constructors
+    , unsafeText, unsafeBoth, unsafeBothText, unsafeChar
+    , unsafePackedString
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Control.Exception ( throwIO, ErrorCall(..) )
 import Data.String ( IsString(..) )
 import Data.List (intersperse)
+import Data.Monoid ( (<>) )
 import GHC.Stack ( currentCallStack )
-import System.IO (Handle, stdout, hPutStr)
+import System.IO (Handle, stdout, stderr, hPutStr)
 import System.IO.Unsafe ( unsafePerformIO )
 import qualified Data.ByteString as B (ByteString, hPut, concat)
 import qualified Data.ByteString.Char8 as BC (unpack, pack, singleton)
@@ -76,36 +104,47 @@ data Printable = S !String
                | PS !B.ByteString
                | Both !String !B.ByteString
 
--- | 'spaceP' is the 'Printable' representation of a space.
+-- | 'Printable' representation of a space
 spaceP :: Printable
 spaceP   = Both " "  (BC.singleton ' ')
 
--- | 'newlineP' is the 'Printable' representation of a newline.
+-- | 'Printable' representation of a newline.
 newlineP :: Printable
 newlineP = S "\n"
 
--- | Minimal 'Doc's representing the common characters 'space', 'newline'
--- 'minus', 'plus', and 'backslash'.
-space, newline, plus, minus, backslash :: Doc
-space     = unsafeBoth " "  (BC.singleton ' ')
-newline   = unsafeChar '\n'
-minus     = unsafeBoth "-"  (BC.singleton '-')
-plus      = unsafeBoth "+"  (BC.singleton '+')
+-- | A 'Doc' representing a space (\" \")
+space :: Doc
+space = unsafeBoth " "  (BC.singleton ' ')
+
+-- | A 'Doc' representing a newline
+newline :: Doc
+newline = unsafeChar '\n'
+
+-- | A 'Doc' representing a \"-\"
+minus :: Doc
+minus = unsafeBoth "-"  (BC.singleton '-')
+
+-- | A 'Doc' representing a \"+\"
+plus :: Doc
+plus = unsafeBoth "+"  (BC.singleton '+')
+
+-- | A 'Doc' representing a \"\\\"
+backslash :: Doc
 backslash = unsafeBoth "\\" (BC.singleton '\\')
 
--- | 'lparen' is the 'Doc' that represents @\"(\"@
+-- | A 'Doc' that represents @\"(\"@
 lparen :: Doc
 lparen = unsafeBoth  "(" (BC.singleton '(')
 
--- | 'rparen' is the 'Doc' that represents @\")\"@
+-- | A 'Doc' that represents @\")\"@
 rparen :: Doc
 rparen = unsafeBoth ")" (BC.singleton ')')
 
--- | @'parens' doc@ returns a 'Doc' with the content of @doc@ put within
--- a pair of parenthesis.
+-- | prop> parens d = lparen <> d <> rparen
 parens :: Doc -> Doc
 parens d = lparen <> d <> rparen
 
+-- | Fail with a stack trace and the given 'Doc' as error message.
 errorDoc :: Doc -> a
 errorDoc x = unsafePerformIO $ do
    stack <- currentCallStack
@@ -129,6 +168,14 @@ putDoc = hPutDoc Encode stdout
 putDocLn :: Doc -> IO ()
 putDocLn = hPutDocLn Encode stdout
 
+-- | 'eputDocLn' puts a doc, followed by a newline to stderr using
+-- 'simplePrinters'. Like putDocLn, it encodes with the user's locale.
+-- This function is the recommended way to output messages that should
+-- be visible to users on the console, but cannot (or should not) be
+-- silenced even when --quiet is in effect.
+ePutDocLn :: Doc -> IO ()
+ePutDocLn = hPutDocLn Encode stderr
+
 -- | 'hputDocWith' puts a doc on the given handle using the given printer.
 hPutDocWith :: Printers -> RenderMode -> Handle -> Doc -> IO ()
 hPutDocWith prs target h d = hPrintPrintables target h (renderWith (prs h) d)
@@ -142,7 +189,7 @@ hPutDocLnWith prs target h d = hPutDocWith prs target h (d <?> newline)
 hPutDoc :: RenderMode -> Handle -> Doc -> IO ()
 hPutDoc = hPutDocWith simplePrinters
 
--- 'hputDocLn' puts a doc, followed by a newline on the given handle using
+-- | 'hputDocLn' puts a doc, followed by a newline on the given handle using
 -- 'simplePrinters'.
 hPutDocLn :: RenderMode -> Handle -> Doc -> IO ()
 hPutDocLn = hPutDocLnWith simplePrinters
@@ -155,11 +202,11 @@ hPutDocCompr target h = gzWriteHandle h . renderPSs target
 debugDocLn :: Doc -> IO ()
 debugDocLn = debugMessage . renderString Standard
 
--- | @'hPrintPrintables' h@ prints a list of 'Printable's to the handle h
+-- | @'hPrintPrintables' h@ prints a list of 'Printable's to the handle @h@
 hPrintPrintables :: RenderMode -> Handle -> [Printable] -> IO ()
 hPrintPrintables target h = mapM_ (hPrintPrintable target h)
 
--- | @hPrintPrintable h@ prints a 'Printable' to the handle h.
+-- | @'hPrintPrintable' h@ prints a 'Printable' to the handle @h@.
 hPrintPrintable :: RenderMode -> Handle -> Printable -> IO ()
 hPrintPrintable Standard h (S ps) = hPutStr h ps
 hPrintPrintable Encode  h (S ps) = B.hPut h (encodeLocale ps)
@@ -168,10 +215,12 @@ hPrintPrintable Encode  h (PS ps) = B.hPut h ps
 hPrintPrintable Standard h (Both _ ps) = B.hPut h ps
 hPrintPrintable Encode  h (Both _ ps) = B.hPut h ps
 
--- | a 'Doc' is a bit of enriched text. 'Doc's get concatanated using
--- '<>', which is right-associative.
+-- | A 'Doc' is a bit of enriched text. 'Doc's are concatenated using
+-- '<>' from class 'Monoid', which is right-associative.
 newtype Doc = Doc { unDoc :: St -> Document }
 
+-- | Together with the language extension OverloadedStrings, this allows to
+-- use string literals where a 'Doc' is expected.
 instance IsString Doc where
    fromString = text
 
@@ -330,6 +379,8 @@ invisiblePS :: B.ByteString -> Doc
 invisiblePS = invisiblePrintable . PS
 
 -- | 'userchunkPS' creates a 'Doc' representing a user chunk from a 'B.ByteString'.
+--
+-- Rrrright. And what, please is that supposed to mean?
 userchunkPS :: B.ByteString -> Doc
 userchunkPS = userchunkPrintable . PS
 
@@ -357,7 +408,6 @@ hiddenText = hiddenPrintable . S
 userchunk :: String -> Doc
 userchunk = userchunkPrintable . S
 
--- | 'blueText' creates a 'Doc' containing blue text from a @String@
 blueText, redText, greenText, magentaText, cyanText :: String -> Doc
 blueText = colorText Blue
 redText = colorText Red
@@ -378,14 +428,23 @@ wrapText n s =
         add_to_line (l:ls) new | length l + length new > n = new:l:ls
         add_to_line (l:ls) new = (l ++ " " ++ new):ls
 
--- | 'printable x' creates a 'Doc' from any 'Printable'.
-printable, invisiblePrintable, hiddenPrintable, userchunkPrintable :: Printable -> Doc
+-- | Creates a 'Doc' from any 'Printable'.
+printable :: Printable -> Doc
 printable x = Doc $ \st -> defP (printers st) x st
 
 mkColorPrintable :: Color -> Printable -> Doc
 mkColorPrintable c x = Doc $ \st -> colorP (printers st) c x st
+
+-- | Creates an invisible 'Doc' from any 'Printable'.
+invisiblePrintable :: Printable -> Doc
 invisiblePrintable x = Doc $ \st -> invisibleP (printers st) x st
+
+-- | Creates a hidden 'Doc' from any 'Printable'.
+hiddenPrintable :: Printable -> Doc
 hiddenPrintable x = Doc $ \st -> hiddenP (printers st) x st
+
+-- | Creates... WTF is a userchunk???
+userchunkPrintable :: Printable -> Doc
 userchunkPrintable x = Doc $ \st -> userchunkP (printers st) x st
 
 -- | 'simplePrinters' is a 'Printers' which uses the set 'simplePriners\'' on any
@@ -417,26 +476,25 @@ simplePrinter x = unDoc $ doc (\s -> x:s)
 invisiblePrinter :: Printer
 invisiblePrinter _ = unDoc empty
 
-infixr 6 <>
+infixr 6 `append`
 infixr 6 <+>
 infixr 5 $$
 
--- | The empty 'Doc'.
+-- | The empty 'Doc'
 empty :: Doc
 empty = Doc $ const Empty
+
 doc :: ([Printable] -> [Printable]) -> Doc
 doc f = Doc $ const $ Document f
 
--- | '(<>)' is the concatenation operator for 'Doc's
-(<>) :: Doc -> Doc -> Doc
--- | @a '<?>' b@ is @a <> b@ if @a@ is not empty, else empty.
-(<?>) :: Doc -> Doc -> Doc
--- | @a '<+>' b@ is @a@ followed by a space, then @b@.
-(<+>) :: Doc -> Doc -> Doc
--- | @a '$$' b@ is @a@ above @b@.
-($$) :: Doc -> Doc -> Doc
--- a then b
-Doc a <> Doc b =
+-- | 'mappend' ('<>') is concatenation, 'mempty' is the 'empty' 'Doc'
+instance Monoid Doc where
+  mempty = empty
+  mappend = append
+
+-- | Concatenation of two 'Doc's
+append :: Doc -> Doc -> Doc
+Doc a `append` Doc b =
    Doc $ \st -> case a st of
                 Empty -> b st
                 Document af ->
@@ -444,7 +502,8 @@ Doc a <> Doc b =
                                          Empty -> s
                                          Document bf -> bf s)
 
--- empty if a empty, else a then b
+-- | @a '<?>' b@ is @a '<>' b@ if @a@ is not empty, else empty
+(<?>) :: Doc -> Doc -> Doc
 Doc a <?> Doc b =
     Doc $ \st -> case a st of
                  Empty -> Empty
@@ -452,7 +511,8 @@ Doc a <?> Doc b =
                                                      Empty -> s
                                                      Document bf -> bf s)
 
--- a then space then b
+-- | @a '<+>' b@ is @a@ followed by a space, then @b@
+(<+>) :: Doc -> Doc -> Doc
 Doc a <+> Doc b =
     Doc $ \st -> case a st of
                  Empty -> b st
@@ -461,7 +521,8 @@ Doc a <+> Doc b =
                                                      Document bf ->
                                                          spaceP:bf s)
 
--- a above b
+-- | @a '$$' b@ is @a@ above @b@
+($$) :: Doc -> Doc -> Doc
 Doc a $$ Doc b =
    Doc $ \st -> case a st of
                 Empty -> b st
@@ -472,17 +533,29 @@ Doc a $$ Doc b =
                         where pf = currentPrefix st
                               sf = lineColorS $ printers st
 
--- | 'vcat' piles vertically a list of 'Doc's.
+-- | Pile 'Doc's vertically
 vcat :: [Doc] -> Doc
 vcat [] = empty
 vcat ds = foldr1 ($$) ds
 
--- | 'vsep' piles vertically a list of 'Doc's leaving a blank line between each.
+-- | Pile 'Doc's vertically, with a blank line in between
 vsep :: [Doc] -> Doc
 vsep [] = empty
 vsep ds = foldr1 ($$) $ intersperse (text "") ds
 
--- | 'hcat' concatenates (horizontally) a list of 'Doc's
+-- | Concatenate 'Doc's horizontally
 hcat :: [Doc] -> Doc
-hcat [] = empty
-hcat ds = foldr1 (<>) ds
+hcat = mconcat
+
+-- | Concatenate 'Doc's horizontally with a space as separator
+hsep :: [Doc] -> Doc
+hsep = foldr (<+>) empty
+
+-- | Quote a string for screen output
+quoted :: String -> Doc
+quoted s = text "\"" <> text (escape s) <> text "\""
+  where
+    escape "" = ""
+    escape (c:cs) = if c `elem` ['\\', '"']
+                       then '\\' : c : escape cs
+                       else c : escape cs
