@@ -2,6 +2,9 @@ module Darcs.Util.Show
     ( appPrec, BSWrapper(..)
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import qualified Data.ByteString as B
 
 appPrec :: Int
