@@ -18,13 +18,13 @@ ALL ask-deps
 touch a
 darcs add a
 echo q | darcs rec -am a0
-darcs ann -p a0
+darcs log -p a0 -v --machine | cat
 echo 1 > a
 echo q | darcs rec -am a1
-darcs ann -p a1
+darcs log -p a1 -v --machine | cat
 echo 2 > a
 echo q | darcs rec -am a2
-darcs ann -p a2
+darcs log -p a2 -v --machine | cat
 
 # add some patches for file 'b'
 # expect no dependency questions for file 'b',
@@ -36,19 +36,21 @@ touch b
 darcs add b
 # test 0
 echo nnnY | tr '[A-Z]' '[a-z]' | darcs rec -am b0
-darcs ann -p b0
+darcs log -p b0 -v --machine | cat
 # test 1
 echo 1 > b
 echo nnyY | tr '[A-Z]' '[a-z]' | darcs rec -am b1
-darcs ann -p b1 | grep '^\[a0'
+darcs log -p b1 -v --machine | cat
+darcs log -p b1 -v --machine | grep '\[a0'
 # test 2
 echo 2 > b
 echo nyY | tr '[A-Z]' '[a-z]' | darcs rec -am b2
-darcs ann -p b2 | grep '^\[a1'
+darcs log -p b2 -v --machine | grep '\[a1'
 # test 3
 echo 3 > b
 echo yY | tr '[A-Z]' '[a-z]' | darcs rec -am b3
-darcs ann -p b3 | grep '^\[a2'
+darcs log -p b3 -v --machine | cat
+darcs log -p b3 -v --machine | grep '\[a2'
 
 cd ..
 rm -rf temp
