@@ -24,7 +24,7 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-. ../tests/lib                  # Load some portability helpers.
+. lib                  # Load some portability helpers.
 
 touch casetest
 test -e CASETEST || exit 200
