@@ -32,4 +32,4 @@ cd repo
 
 echo 'baz' > bar
 
-echo yyyy | darcs amend-rec
+echo yyyy | darcs amend
