@@ -2,7 +2,7 @@
 
 # For issue390: darcs whatsnew somefile" lstats every file in the working copy and pristine/ directory
 
-set -ev
+. ./lib
 
 if ! test -x "$(which strace)"
 then echo skipping test since strace was not found
