@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 # We'd just use `diff -x _darcs -r' if -x was portable.
 diffx () {
@@ -29,10 +29,9 @@ makepristine () {
 
 
 
-rm -rf temp1 temp2 temp3 temp4 temp5
 mkdir temp1
 cd temp1
-darcs init --hashed
+darcs init --darcs-1
 touch foo
 darcs add foo
 darcs rec -m t1 -a -A tester
@@ -49,11 +48,11 @@ darcs changes
 makepristine
 cd ..
 
-darcs get --hashed temp1 temp3
+darcs get temp1 temp3
 cd temp3
 darcs changes
 cp _darcs/hashed_inventory inv
-darcs optimize
+darcs optimize clean
 diff -c inv _darcs/hashed_inventory
 rm inv
 makepristine
@@ -100,7 +99,7 @@ diff -rc temp1/pristine temp3/pristine
 diff -rc temp2/pristine temp3/pristine
 
 cd temp1
-darcs put --hashed ../temp4
+darcs get . ../temp4
 cd ..
 cd temp4
 makepristine
@@ -112,7 +111,7 @@ diff -rc temp2/pristine temp4/pristine
 cd temp1
 darcs tag -A tagger -m atag
 darcs check
-darcs optimize
+darcs optimize clean
 darcs check
 darcs changes | grep t1
 cd ..
@@ -122,11 +121,11 @@ date > foobarpatch
 darcs add foobarpatch
 darcs record -a -A silly -m foobarpatch
 darcs check
-darcs optimize
+darcs optimize clean
 darcs check
 darcs pull -a ../temp1
 darcs check
-darcs optimize --reorder-patches
+darcs optimize reorder
 darcs check
 grep 'Starting with inventory' _darcs/hashed_inventory
 cd ..
@@ -140,24 +139,14 @@ diff -c temp1/_darcs/hashed_inventory temp3/_darcs/hashed_inventory
 cd temp4
 darcs pull -p foobarpatch -a ../temp3
 darcs pull -a ../temp1
-darcs optimize --reorder
+darcs optimize reorder
 darcs check
 darcs push ../temp1
 cd ..
 
 diff temp1/_darcs/hashed_inventory temp4/_darcs/hashed_inventory
 
-rm -rf temp3
-darcs get --old-fashioned temp1 temp3
-
-rm -rf "$HOME"/.darcs/sources
-echo cache:"$HOME"/.darcs/cache > "$HOME"/.darcs/sources
-darcs get --hashed temp3 temp5
-rm -rf temp5
-# check that sources directory was created:
-cd "$HOME"/.darcs/sources && cd -
-
-darcs get temp1 temp5 --hashed --old-fashioned
+darcs get temp1 temp5
 cd temp5
 darcs obliterate --last 3 -a
 darcs pull ../temp1 -a
@@ -165,8 +154,6 @@ darcs obliterate --last 3 -a
 darcs pull ../temp2 -a
 darcs check
 darcs obliterate --last 3 -a
-darcs pull ../temp3 -a
-darcs obliterate --last 3 -a
 darcs pull ../temp4 -a
 cd ..
 
@@ -175,11 +162,6 @@ darcs obliterate --last 3 -a
 darcs pull ../temp5 -a
 cd ..
 
-cd temp3
-darcs obliterate --last 3 -a
-darcs pull ../temp5 -a
-cd ..
-
 cd temp2
 darcs obliterate --last 3 -a
 darcs pull ../temp5 -a
@@ -190,5 +172,3 @@ darcs obliterate --last 3 -a
 darcs pull ../temp5 -a
 cd ..
 
-rm -rf temp1 temp2 temp3 temp4 temp5
-
