@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # A test for a missed resolution, inspired by bug #10 in RT
 
