@@ -1,6 +1,6 @@
 #!/bin/sh
 
-set -ev
+. ./lib
 
 not () { "$@" && exit 1 || :; }
 
@@ -10,7 +10,7 @@ cd temp
 darcs init
 not darcs add a/ 2> err
 cat err
-grep 'does not exist (No such file or directory)' err
+grep 'File a does not exist!' err
 cd ..
 
 rm -rf temp
