@@ -1,5 +1,7 @@
 #!/usr/bin/env bash
 
+#pragma repo-format darcs-2
+
 . lib
 
 mkdir future
@@ -15,11 +17,7 @@ EOF
 cat _darcs/format
 cd ..
 
-if grep 'old-fashioned' .darcs/defaults; then
-    exit 200
-fi
-
-if grep hashed .darcs/defaults; then
+if grep darcs-1 .darcs/defaults; then
     exit 200
 fi
 
