@@ -1,5 +1,8 @@
 #!/usr/bin/env bash
-set -ev
+
+#pragma repo-format darcs-2
+
+. ./lib
 
 rm -rf temp1 temp2
 mkdir temp1
