@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 ### http://bugs.darcs.net/issue458
 ### darcs get --set-scripts-executable ignores umask
-set -ev
+. ./lib
 
 ## Windows doesn't support proper permissions.
 
