@@ -2,7 +2,7 @@
 
 # For issue588, "amend-record --look-for-adds end up with two "addfile" entries"
 
-set -ev
+. ./lib
 
 rm -rf temp1
 darcs init --repodir temp1
@@ -27,6 +27,6 @@ echo y | darcs amend-record --repodir temp1 --all
 
 darcs changes --repodir temp1 --last 1 -v
 
-darcs check --repodir temp1 --no-test
+darcs check --repodir temp1
 
 rm -rf temp1
