@@ -23,10 +23,7 @@ cd tmp_remote
 darcs 'init'
 cd ..
 
-# this is an ugly hack for Cygwin and pwd
-IFS=''
-DIR=`hspwd`
-
+DIR=`pwd`
 # Set up a directory with restrictive permissions
 mkdir -p tmp_restrictive/liberal
 cd tmp_restrictive/liberal
@@ -38,7 +35,7 @@ cd ../..; cd tmp_restrictive/liberal
 touch can_touch
 if [ -e can_touch ]; then
   if hwpwd; then
-    darcs get $DIR/tmp_remote 2> log
+    darcs get "$DIR/tmp_remote" 2> log
     not grep -i 'permission denied' log
   else
     echo "Apparently I can't do `basename $0` on this platform"
@@ -46,7 +43,7 @@ if [ -e can_touch ]; then
 else
   echo "Can't do `basename $0` on this platform"
 fi
-cd $DIR
+cd "$DIR"
 # We have to fix the permissions, just so we can delete it.
 chmod 0755 tmp_restrictive
 
