@@ -11,7 +11,7 @@ check_empty_pending() {
   [ -z _darcs/patches/pending ] || cmp _darcs/patches/pending ../empty_pending
 }
 
-set -ev
+. ./lib
 
 rm -rf temp1 temp2
 mkdir temp1 temp2
