@@ -1,5 +1,8 @@
 #!/usr/bin/env bash
-set -ev
+
+. lib
+
+check_remote_http http://hub.darcs.net/kowey/tabular
 
 rm -rf temp temp2 temp3
 
@@ -15,5 +18,3 @@ darcs pull --tag . -a
 cd ..
 
 diff -u temp2/_darcs/hashed_inventory temp3/_darcs/hashed_inventory
-
-rm -rf temp temp2 temp3
