@@ -23,10 +23,13 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-echo 'Comment this line out and run the script by hand'; exit 200
+# echo 'Comment this line out and run the script by hand'; exit 200
 
-. $(dirname $0)/../lib
-. $(dirname $0)/sshlib
+# . $(dirname $0)/../lib
+# . $(dirname $0)/sshlib
+
+. lib
+. sshlib
 
 # Clean up after previous remote runs
 ${SSH} ${REMOTE} "\
