@@ -1,5 +1,8 @@
 #!/usr/bin/env bash
-set -ev
+
+. lib
+
+check_remote_http http://hub.darcs.net/kowey/tabular
 
 rm -rf temp temp2 temp3
 
@@ -13,7 +16,7 @@ cd temp2
 
 test ! -f _darcs/patches/0000005705-178beaf653578703e32346b4d68c8ee2f84aeef548633b2dafe3a5974d763bf2
 
-darcs annotate -p 'Initial version'
+darcs log -p 'Initial version' -v | cat
 
 test -f _darcs/patches/0000005705-178beaf653578703e32346b4d68c8ee2f84aeef548633b2dafe3a5974d763bf2
 
