@@ -3,7 +3,7 @@
 
 # For issue600, testing optimize --relink 
 
-set -ev
+. ./lib
 
 ## We don't support hard links on Windows.
 
@@ -47,11 +47,11 @@ cp -r x y
 
 ## Now try relinking using darcs.
 rm -rf z
-darcs optimize --verbose --relink --repodir x --sibling y
+darcs optimize relink --verbose --repodir x --sibling y
 rm -rf x/_darcs/patches/pend* y/_darcs/patches/pend*
 if compare x/_darcs/patches y/_darcs/patches
-then echo darcs --relink is working, hard links were done.
-else echo darcs --relink is not working, it did not make any hard links.
+then echo darcs optimize relink is working, hard links were done.
+else echo darcs optimize relink is not working, it did not make any hard links.
      exit 2
 fi
 
