@@ -34,5 +34,5 @@ darcs rec -lam "initial"
 darcs mv file1 file2
 darcs rec -am "move"
 
-darcs show patch-index-test
+darcs show patch-index --verbose
 
