@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 uname | grep "MINGW" > /dev/null && exit 0
 
