@@ -4,23 +4,21 @@
 
 rm -rf temp1 temp2
 
-mkdir temp1
+darcs init temp1
 cd temp1
-darcs init
 
 cd ..
-mkdir temp2
+darcs init temp2
 cd temp2
-darcs init
 
 mkdir one
 cd one
 mkdir two
 cd two
-echo darcs pull should work relative to the current directory
+# darcs pull should work relative to the current directory
 darcs pull -a ../../../temp1 | grep -i 'No remote patches to pull in'
 
-echo -- darcs pull should pull into repo specified with --repo
+# darcs pull should pull into repo specified with --repo
 cd ../..  # now in temp2
 darcs add one;
 darcs record --name uno --all
@@ -42,7 +40,6 @@ else
     chmod a-rwx ./temp1/one # remove all permissions
     not darcs pull --repodir ./temp1 -a 2> err
     chmod u+rwx temp1/one # restore permission
-    cat err
     grep 'permission denied' err
     rm -rf temp1/one
 fi
@@ -55,11 +52,9 @@ not grep temp1 _darcs/prefs/defaultrepo
 #return special message when you try to pull from yourself
 DIR="`pwd`"
 not darcs pull --debug -a "$DIR" 2> out
-cat out
 grep 'Can.t pull from current repository' out
 
 not darcs pull --debug -a . 2> out
-cat out
 grep 'Can.t pull from current repository' out
 
 # and do not update the default repo to be the current di
@@ -73,7 +68,6 @@ not darcs pull --debug 2> err
 grep 'Can.t pull from current repository' err
 
 not darcs pull --debug ../* 2> out
-cat out
 not grep 'Can.t pull from current repository' out
 cd .. # now outside of any repo
 
@@ -121,7 +115,6 @@ darcs record -am newdir
 cd ../temp2
 mkdir newdir
 darcs pull -a --set-default ../temp1 &> out2
-cat out
 grep Backing out2
 grep 'Finished pulling' out2
 grep newdir out2
@@ -130,26 +123,200 @@ cd ..
 rm -rf temp1 temp2
 
 
-# A test for issue662, which triggered:
+# issue662, which triggered:
 #  darcs failed:  Error applying hunk to file ./t.t
 #  Error applying patch to the working directory.
 
-rm -rf tmp;
-darcs init --darcs-1 --repodir=tmp
-touch tmp/t.t
-cd tmp
-darcs add t.t
-echo 'content'>t.t
-darcs record -am 'initial add' --ignore
-echo 'content: remote change'>t.t
+darcs init temp1
+cd temp1
+
+touch t.t
+echo 'content'> t.t
+darcs record -lam 'initial add'
+echo 'content: remote change'> t.t
 darcs record -am 'remote change' --ignore
-darcs get . tmp2
-cd tmp2
-darcs obliterate --last 1 --all;
+cd ..
+darcs clone temp1 temp2
+cd temp2
+darcs obliterate --last 1 --all
 echo 'content: local change'> t.t
-darcs pull -a ../
+darcs pull -a ../temp1
 darcs w -s
 darcs revert -a
+cd ..
+rm -rf temp1 temp2
+
+# pull with conflicts
+
+darcs initialize temp1
+cd temp1
+echo foo > bar
+darcs record -lam addbar
+
+cd ..
+darcs clone temp1 temp2
+cd temp1
+date > bar
+darcs record -a -m datebar
+cd ../temp2
+echo aack >> bar
+darcs record -a -m aackbar
+darcs pull -a
+darcs check
+
+cd ..
+rm -rf temp1 temp2
+
+# pull --union
+
+rm -rf temp1 temp2 temp3
+
+darcs init temp1
+cd temp1
+echo A > A
+darcs record -lam A
+echo B > B
+darcs record -lam B
+
+cd ..
+darcs clone temp1 temp2
+
+cd temp2
+darcs obliterate --last 1 -a
+echo C > C
+darcs record -lam C
+cd ..
+
+darcs init temp3
+cd temp3
+darcs pull -a ../temp1 ../temp2
+
+darcs log > out
+grep A out
+grep B out
+grep C out
+
+cd ..
+rm -rf temp1 temp2 temp3
+
+# pull --intersection
+
+darcs init temp1
+cd temp1
+echo A > A
+darcs record -lam Aismyname
+echo B > B
+darcs record -lam Bismyname
+
+cd ..
+darcs clone temp1 temp2
+cd temp2
+darcs obliterate --last 1 -a
+echo C > C
+darcs record -lam Cismyname
+
+cd ..
+darcs init temp3
+cd temp3
+darcs pull -a --intersection ../temp1 ../temp2
+darcs log > out
+grep Aismyname out
+not grep Bismyname out
+not grep Cismyname out
+
+cd ..
+rm -rf temp1 temp2 temp3
+
+
+# pull --skip-conflicts
+rm -rf R S
+darcs init R
+cd R
+echo 'foo' > foo
+echo 'bar' > bar
+darcs rec -lam 'Add foo and bar'
+cd ..
+
+darcs clone R S
+
+cd R
+echo 'foo2' > foo
+darcs rec -lam 'Change foo (2)'
+echo 'bar2' > bar
+darcs rec -lam 'Change bar (2)'
+cd ..
+
+cd S
+echo 'foo3' > foo
+darcs rec -lam 'Change foo (3)'
+darcs pull --skip-conflicts -a ../R
+test `darcs log --count` -eq 3
+cd ..
+
+cd S
+darcs pull -a ../R
+test `darcs log --count` -eq 4
+cd ..
+rm -rf R S
+
+# bad pending after pull
+
+rm -fr temp1 temp2
+
+darcs init temp1
+cd temp1
+echo abc > A
+echo def > B1
+darcs record -lam patch1
+darcs mv B1 B2
+darcs record -am patch2
+cd ..
+
+darcs init temp2
+cd temp2
+darcs pull -a ../temp1
+not darcs whatsnew
+cd ..
+
+rm -fr temp1 temp2
+
+# issue494: note that in this test, we deliberately select filenames
+# with a backwards sorting order
+darcs init temp1
+cd temp1
+echo abc > b
+darcs record -lam patch1
+darcs mv b a
+echo def > a
+darcs record -am patch2
+cd ..
+
+darcs init temp2
+cd temp2
+darcs pull --all ../temp1
+not darcs whatsnew
+cd ..
+
+rm -fr temp1 temp2
+
+# pull binary
+
+rm -rf temp1 temp2
+
+darcs init temp1
+cd temp1
+printf "%01048576d" 0 > foo
+darcs record -l -a -m xx
+rm foo
+darcs record -a -m yy
+cd ..
+
+darcs init temp2
+cd temp2
+echo yny | darcs pull --set-default ../temp1
+rm foo
+darcs pull -a
+cd ..
+
+rm -rf temp1 temp2
 
-cd ../..
-rm -rf tmp
