@@ -48,3 +48,17 @@ not grep -i d/f/g/2.txt after.lst
 not grep -i d/f/1.txt after.lst
 
 cd ..
+rm -rf temp1
+
+# issue1765 recursive remove on root
+
+darcs init temp1
+cd temp1
+
+mkdir d e                       # Change the working tree.
+echo 'Example content.' > d/f
+darcs record -lam 'Add d/f and e.'
+darcs remove * -r
+
+cd ..
+rm -rf temp1
