@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 # Check that the right patches get sent using --dont-prompt-for-dependencies
 
@@ -21,12 +21,12 @@ echo foo2 > f
 darcs record -Ax -alm foo2
 echo bar2 > b
 darcs record -Ax -alm bar2
-echo y | darcs send ../temp2 -i --dont-prompt-for-dependencies -p foo2 --dry-run > toto
+echo yy | darcs send ../temp2 -i --dont-prompt-for-dependencies -p foo2 --dry-run > toto
 #on the previous line, we get asked about foo2, and we take it
 grep foo2 toto | wc -l | grep 2
 #we don't get asked about foo1, but we take it anyway, so 
 grep foo1 toto | wc -l | grep 1
 #and we don't send bar
-grep bar toto | wc -l | grep 0
+not grep bar toto
 cd ..
 rm -rf temp1 temp2
