@@ -1,23 +1,26 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
+
+rm -rf R
+
+darcs init --repo R
+cd R
 
-darcs init
 echo zig > foo
 darcs add foo
 darcs record -a -m add_foo -A x
+
 echo zag >> foo
 darcs record -a -m mod_foo -A y
+
 echo bar > foo
 darcs record -a -m mod2 -A y
 
 darcs show authors > authors
-grep x authors
-grep y authors
+grep "#1	2	y" authors
+grep "#2	1	x" authors
 
-head -1 authors > first-author
-grep y first-author
+head -1 authors | grep y
 
 darcs show authors --verbose
-darcs show authors --verbose | grep y | wc -l > num-patches
-cat num-patches
-grep 2 num-patches
+[[ $(darcs show authors --verbose | grep y | wc -l) -eq 2 ]]
