@@ -4,8 +4,6 @@
 
 DIR="`pwd`"
 
-abort_windows
-
 # set up the repository
 rm -rf temp1                    # another script may have left a mess.
 mkdir temp1
@@ -13,7 +11,6 @@ cd temp1
 darcs init
 touch foo
 darcs add foo
-# trick: hitting 'enter' in interactive darcs record
-echo n/ | tr / \\012 | darcs record
+echo ny | darcs record
 cd ..
 rm -rf temp1
