@@ -44,7 +44,7 @@ darcs apply $TESTDATA/braced.dpatch
 cd ..
 
 cd braced
-darcs pull -a ../empty | grep 'No remote changes to pull in'
+darcs pull -a ../empty | grep 'No remote patches to pull in'
 
 cd ../empty
-darcs pull -a ../braced | grep 'No remote changes to pull in'
\ No newline at end of file
+darcs pull -a ../braced | grep 'No remote patches to pull in'
